# Copyright 2014 Scratchwork Development LLC. All rights reserved.

class AlertScene < Scene
  def id
    :alert_scene
  end

  def will_appear
  end

  def tick
    calc_buttons
    render
  end

  def button_ids
    [
      :close_button
    ]
  end

  def close_button_selected
    state.next_scene = metadata.source_scene
    state.next_scene_metadata = metadata.source_metadata
  end

  def render
    outputs.primitives << bg_prefab
    outputs.primitives << title_label
    outputs.primitives << text_label
    outputs.primitives << additional_primitives
    outputs.primitives << prefab.button(args, close_button)
  end

  def will_appear_a11y_target
    "layout_title"
  end

  def bg_prefab
    { x: Grid.allscreen_x,
      y: Grid.allscreen_y,
      w: Grid.allscreen_w,
      h: Grid.allscreen_h,
      path: :solid,
      **metadata_background_color,
      a: 255 }
  end

  def metadata_background_color
    metadata.background_color || prefab.white
  end

  def metadata_text_color
    metadata.text_color || prefab.black
  end

  def metadata_text_size_px
    if Grid.landscape?
      metadata.text_size_px || 22
    else
      metadata.text_size_px || 30
    end
  end

  def additional_primitives
    metadata.additional_primitives
  end

  def title_label
    metadata.title ||= ""
    size_px = if Grid.landscape?
                26
              else
                30
              end
    Layout.rect(row: 0, col: [0, Layout.col_count - 1], h: 1)
          .center
          .merge(text: metadata.title,
                 font: "fonts/text.ttf",
                 anchor_x: 0.5,
                 anchor_y: 0.5,
                 size_px: size_px,
                 a11y_id: "layout_title",
                 a11y_text: metadata.title,
                 r: metadata_text_color.r,
                 g: metadata_text_color.g,
                 b: metadata_text_color.b,
                 a11y_trait: :label)
  end

  def close_button
    r = Layout.rect(row: Layout.row_count - 2,
                    col: Layout.col_count.idiv(2) - 1,
                    w: 2,
                    h: 2)

    { **r,
      text: "←",
      size_px: 30,
      **metadata_text_color,
      # font: "fonts/sprite.ttf",
      a11y_id: "alert_dismiss",
      a11y_text: "dismiss",
      a11y_trait: :button }
  end

  def text
    metadata.text ||= ""
    if metadata.text.is_a? Array
      metadata.text
    else
      [metadata.text]
    end
  end

  def text_row
    if Grid.landscape?
      metadata.text_row || 1
    else
      metadata.text_row || 2
    end
  end

  def metadata_text_split_length
    if Grid.landscape?
    metadata.text_split_length || 49
    else
    metadata.text_split_length || 38
    end
  end

  def text_label
    text_with_location = []
    event_label_rect = Layout.rect(row: text_row, col: [0, 11], h: 3)
    starting_x = event_label_rect.x
    starting_y = event_label_rect.y

    anchor_increment = 0
    text.each_with_index do |t, j|
      long_lines_split(t, metadata_text_split_length).each_with_index do |line, i|
        anchor_increment += 1.2
        text_with_location << { x: event_label_rect.x,
                                y: event_label_rect.center.y,
                                font: "fonts/text.ttf",
                                size_px: metadata_text_size_px,
                                anchor_increment: anchor_increment,
                                **metadata_text_color,
                                text: line.strip }
      end
      anchor_increment += 1
    end

    l = text_with_location.length
    text_with_location.each_with_index do |t, i|
      t.x = Grid.w / 2
      t.anchor_x = 0.5

      t.anchor_y = -(l / 2) + t.anchor_increment - 0.5
    end

    if text_with_location[0] && GTK.a11y_enabled?
    text_with_location[0].a11y_id = "world_event_text"
    text_with_location[0].a11y_trait = :label
    text_with_location[0].a11y_text = (text_with_location.map { |s| s.text }.join " \n ")
    end

    text_with_location
  end
end
