# Copyright 2014 Scratchwork Development LLC. All rights reserved.

class CutsceneScene < Scene
  attr :cutscene

  def id
    :cutscene_scene
  end

  def world
    @game.world
  end

  def current_scene
    return { cutscene_class: BuilderCutscene } if @game.world.death_reason == :complete_hard
    return { text: [] } if !world.event

    world.event.scenes[world.event.current_scene]
  end

  def will_appear
    outputs[:ascii_map].background_color = [0, 0, 0, 0]
    outputs[:ascii_map].transient!

    @navigation_color = if Grid.landscape?
                          { r: 0, g: 0, b: 0, a: 255 }
                        else
                          { r: 0, g: 0, b: 0, a: 255 }
                        end
    @cutscene = current_scene[:cutscene_class].new
    if @cutscene.is_a? BuilderCutscene
      queue_music "sounds/hymn.ogg", true
    elsif @cutscene.is_a? DestructionCutscene
      queue_music "sounds/abbot.ogg", true
    elsif @cutscene.is_a? DrillsCutscene
      queue_music "sounds/abbot.ogg", true
    elsif @cutscene.is_a? FirstContactCutscene
      queue_music "sounds/abbot.ogg", true
    elsif @cutscene.is_a? RitualCutscene
      queue_music "sounds/hymn.ogg", true
    elsif @cutscene.is_a? StarGazerCutscene
      queue_music "sounds/hubble.ogg", true
    elsif @cutscene.is_a? MassacreCutscene
      queue_music "sounds/abbot.ogg", true
    elsif @cutscene.is_a? PurgingCutscene
      queue_music "sounds/abbot.ogg", true
    elsif @cutscene.is_a? RebellionCutscene
      queue_music "sounds/hymn.ogg", true
    end

    @cutscene.init_map
    if @metadata && @metadata.x && @metadata.y
      @cutscene.x = @metadata.x
      @cutscene.y = @metadata.y
    end
    update_label_cache
  end

  def update_label_cache
    @cutscene_landmark_labels_cache ||= []
    @cutscene_landmark_labels_cache.clear
    @cutscene_aesthetic_labels_cache ||= []
    @cutscene_aesthetic_labels_cache.clear
    update_landmark_labels
    update_aesthetics_labels
  end

  def max_visible_columns
    45
  end

  def max_visible_rows
    40
  end

  def cutscene_label_size_px
    20
  end

  def tick
    outputs[:ascii_map].background_color = [0, 0, 0, 0]
    outputs[:ascii_map].transient!
    calc
    render
  end

  def calc
    calc_buttons
    calc_keyboard_inputs
    calc_controller_inputs
  end

  def show_navigation_buttons?
    return true if GTK.a11y_enabled?
    return true if GTK.platform?(:mobile)
    return true if inputs.last_active == :mouse
    return false
  end

  def calc_keyboard_inputs
    return if inputs.last_active != :keyboard

    if inputs.keyboard.key_down.enter || inputs.keyboard.key_down.space
      if prefab.notifications.length > 0
        prefab.dismiss_all_notifications
      else
        close_cutscene_button_selected
      end
    end

    if inputs.keyboard.key_down.left
      move_left_button_selected
    elsif inputs.keyboard.key_down.right
      move_right_button_selected
    elsif inputs.keyboard.key_down.up
      move_up_button_selected
    elsif inputs.keyboard.key_down.down
      move_down_button_selected
    end
  end

  def calc_controller_inputs
    return if inputs.last_active != :controller
    if inputs.controller_one.key_down.left
      move_left_button_selected
    elsif inputs.controller_one.key_down.right
      move_right_button_selected
    elsif inputs.controller_one.key_down.up
      move_up_button_selected
    elsif inputs.controller_one.key_down.down
      move_down_button_selected
    end

    if inputs.controller_one.key_down.a || inputs.controller_one.key_down.b
      close_cutscene_button_selected
    end
  end

  def render_navigation_buttons
    outputs.primitives << prefab.button(args, move_up_button)
    outputs.primitives << prefab.button(args, move_down_button)
    outputs.primitives << prefab.button(args, move_left_button)
    outputs.primitives << prefab.button(args, move_right_button)
    outputs.primitives << prefab.button(args, close_cutscene_button)
  end

  def close_cutscene_button_selected
    if @game.world.death_reason == :complete_hard
      state.story_state = :beginning
      state.next_scene = :embark_scene
    else
      world.event.current_scene = current_scene[:when_completed_go_to]
      state.next_scene = :world_event_scene
    end
  end

  def render
    render_a11y
    render_status_labels
    render_navigation_buttons
    render_map
  end

  def will_appear_a11y_target
    :cutscene_title
  end

  def render_a11y
    return if !GTK.a11y_enabled?

    outputs.a11y[:cutscene_title] = {
      x: Grid.w / 2,
      y: Layout.rect(row: 1, h: 1).center.y,
      anchor_x: 0.5,
      anchor_y: 0.5,
      text: @cutscene.title,
      a11y_id: :cutscene_title,
      a11y_text: @cutscene.title,
      a11y_trait: :label
    }

    outputs.primitives << prefab.button(args, a11y_cutscene_button_1)
    outputs.primitives << prefab.button(args, a11y_cutscene_button_2)
    outputs.primitives << prefab.button(args, a11y_cutscene_button_3)
    outputs.primitives << prefab.button(args, a11y_cutscene_button_4)
    outputs.primitives << prefab.button(args, a11y_cutscene_button_5)
    outputs.primitives << prefab.button(args, a11y_cutscene_button_6)
    outputs.primitives << prefab.button(args, a11y_cutscene_close_button)
  end

  def a11y_cutscene_button name, index
    return nil if !GTK.a11y_enabled?
    return nil if !@cutscene.landmarks[name]
    text = @cutscene.landmarks[name][:accessibility_action] || @cutscene.landmarks[name][:title]
    rect = Layout.rect(row: index + 1, col: 0, w: 12, h: 1)
    {
      **rect,
      text: text,
      a11y_id: "cutscene_#{name}".to_sym,
      a11y_text: text,
      a11y_trait: :button
    }
  end

  def show_a11y_cutscene name
    title = @cutscene.landmarks[name][:title]
    message = @cutscene.landmarks[name][:message]

    state.next_scene = :alert_scene
    state.next_scene_metadata = {
      title: title,
      text: message,
      source_scene: :cutscene_scene,
      text_row_offset: 0,
      source_metadata: {
        x: @cutscene.x,
        y: @cutscene.y
      }
    }
  end

  def a11y_cutscene_button_1_selected
    show_a11y_cutscene "1"
  end

  def a11y_cutscene_button_1
    a11y_cutscene_button "1", 1
  end

  def a11y_cutscene_button_2_selected
    show_a11y_cutscene "2"
  end

  def a11y_cutscene_button_2
    a11y_cutscene_button "2", 2
  end

  def a11y_cutscene_button_3_selected
    show_a11y_cutscene "3"
  end

  def a11y_cutscene_button_3
    a11y_cutscene_button "3", 3
  end

  def a11y_cutscene_button_4_selected
    show_a11y_cutscene "4"
  end

  def a11y_cutscene_button_4
    a11y_cutscene_button "4", 4
  end

  def a11y_cutscene_button_5_selected
    show_a11y_cutscene "5"
  end

  def a11y_cutscene_button_5
    a11y_cutscene_button "5", 5
  end

  def a11y_cutscene_button_6_selected
    show_a11y_cutscene "6"
  end

  def a11y_cutscene_button_6
    a11y_cutscene_button "6", 6
  end

  def a11y_cutscene_close_button_selected
    if @game.world.death_reason == :complete_hard
      state.story_state = :beginning
      state.next_scene = :embark_scene
    else
      world.event.current_scene = current_scene[:when_completed_go_to]
      state.next_scene = :world_event_scene
    end
  end

  def a11y_cutscene_close_button
    rect = Layout.rect(row: 8, col: 0, w: 12, h: 1)
    {
      **rect,
      text: "close",
      a11y_id: "cutscene_close".to_sym,
      a11y_text: "close",
      a11y_trait: :button
    }
  end

  def title_label
    lines = @cutscene.title.split("\n")
    lines.map_with_index do |line, i|
      rect = if Grid.landscape?
               Layout.rect(row: 10, col: [0, 5], h: 1)
             else
               Layout.rect(row: 18, col: [0, 5], h: 1)
             end

      rect.merge(y: rect.center.y,
                 text: line.gsub(",", ""),
                 anchor_x: 0,
                 anchor_y: 0.5 + i + 0.3 + i * 0.3,
                 size_px: 30)
    end
  end

  def render_cutscene_message
    outputs.primitives << cutscene_message_title
    outputs.primitives << cutscene_message_text
  end

  def render_status_labels
    outputs.primitives << title_label
  end

  def render_map
    y_offset = if Grid.landscape?
                 600
               else
                 Layout.rect(row: 0, col: 0, h: 18).center.y
               end

    @string_size ||= GTK.calcstringbox(@cutscene_landmark_labels_cache.first.text,
                                     size_px: cutscene_label_size_px,
                                     font: "fonts/sprite.ttf")

    string_w, string_h = @string_size

    screen_w = Grid.w
    screen_h = Grid.h
    map_w = 1280
    map_h = 1280
    x_offset = if Grid.landscape?
                 if $args.inputs.last_active != :mouse && !GTK.a11y_enabled?
                   300
                 else
                   100
                 end
               else
                 0
               end

    outputs[:ascii_map].background_color = [0, 0, 0, 0]
    outputs[:ascii_map].transient!
    outputs[:ascii_map].h = max_visible_rows * cutscene_line_height
    outputs[:ascii_map].primitives << cutscene_landmarks_label
    outputs[:ascii_map].primitives << cutscene_aesthetics_label
    outputs.primitives << { x: x_offset,
                            y: y_offset,
                            w: Grid.w,
                            h: max_visible_rows * cutscene_line_height,
                            anchor_y: 0.5,
                            path: :ascii_map }
  end

  def cutscene_landmarks_label
    @cutscene_landmark_labels_cache
  end

  def cutscene_aesthetics_label
    @cutscene_aesthetic_labels_cache
  end

  def cutscene_line_height
    25.2
  end

  def button_ids
    if GTK.a11y_enabled?
      [
        :a11y_cutscene_button_1,
        :a11y_cutscene_button_2,
        :a11y_cutscene_button_3,
        :a11y_cutscene_button_4,
        :a11y_cutscene_button_5,
        :a11y_cutscene_button_6,
        :a11y_cutscene_close_button
      ]
    else
    [
      :move_up_button,
      :move_down_button,
      :move_left_button,
      :move_right_button,
      :close_cutscene_button
    ]
    end
  end

  def close_message_button
    r = if Grid.orientation == :landscape
          Layout.rect(row: 6, col: 11, w: 2, h: 2)
        else
          Layout.rect(row: 20, col: 5, w: 2, h: 2)
        end

    { **r,
      text: "←",
      size_px: 30,
      # font: "fonts/sprite.ttf",
      a11y_id: "alert_dismiss",
      a11y_text: "dismiss",
      a11y_trait: :button }
  end

  def move_up_button_selected
    @cutscene.move_up
    update_label_cache
    show_cutscene_message_if_needed
  end

  def move_down_button_selected
    @cutscene.move_down
    update_label_cache
    show_cutscene_message_if_needed
  end

  def move_right_button_selected
    @cutscene.move_right
    update_label_cache
    show_cutscene_message_if_needed
  end

  def move_left_button_selected
    @cutscene.move_left
    update_label_cache
    show_cutscene_message_if_needed
  end

  def show_cutscene_message_if_needed
    return if !@cutscene.has_message?
    size_px = if Grid.landscape?
                20
              else
                22
              end

    additional_primitives = if @cutscene.current_message[:focal_point]
                              @cutscene.current_message[:focal_point].map_with_index do |fp, i|
                                r = if Grid.landscape?
                                      { x: 330, y: 440, w: 620, h: 48, center: { x: 640, y: 49 } }
                                    else
                                      Layout.rect(row: 11 - (@cutscene.current_message[:focal_point].length / 2).to_i, col: 0, w: 12)
                                    end
                                {
                                  x: r.center.x,
                                  y: r.y,
                                  text: fp,
                                  size_px: size_px,
                                  anchor_x: 0.5,
                                  anchor_y: 1 + (i * 1.2),
                                  font: "fonts/sprite.ttf",
                                }
                              end
                            else
                              nil
                            end

    state.next_scene = :alert_scene
    state.next_scene_metadata = {
      title: @cutscene.current_message[:title],
      text: @cutscene.current_message[:message],
      source_scene: :cutscene_scene,
      additional_primitives: additional_primitives,
      text_row_offset: 0,
      source_metadata: {
        x: @cutscene.x,
        y: @cutscene.y
      }
    }
  end

  def update_landmark_labels
    (max_visible_rows + 1).times
      .map { |i| cutscene_landmarks_for start_y + i }
      .each_with_index.map do |l, i|
      if l && !@cutscene_landmark_labels_cache[i]
        @cutscene_landmark_labels_cache[i] =
          { x: Layout.rect(row: 0, col: 0).x + 8,
            y: Layout.rect(row: 0.5, col: 0, h: 0).y - cutscene_line_height * (i + 1) - 24,
            text: l,
            **text_color,
            size_px: cutscene_label_size_px,
            font: "fonts/sprite.ttf",
            anchor_x: 0 }
      end
    end
  end

  def update_aesthetics_labels
    (max_visible_rows + 1).times
      .map { |i| cutscene_aesthetics_for start_y + i }
      .each_with_index.map do |l, i|
      if l && !@cutscene_aesthetic_labels_cache[i]
        @cutscene_aesthetic_labels_cache[i] =
          { x: Layout.rect(row: 0, col: 0).x + 8,
            y: Layout.rect(row: 0.5, col: 0, h: 0).y - cutscene_line_height * (i + 1) - 24,
            text: l,
            **prefab.really_light_gray,
            size_px: cutscene_label_size_px,
            font: "fonts/sprite.ttf",
            anchor_x: 0 }
      end
    end
  end

  def start_y
    if Grid.landscape?
      start_y = 0
    else
      start_y = (@cutscene.y - (max_visible_rows / 2)).to_i

      if start_y + max_visible_rows > @cutscene.height
        start_y = @cutscene.height - max_visible_rows
      end
    end
    start_y
  end

  def start_x
    if Grid.landscape?
      start_x = 0
    else
      start_x = (@cutscene.x - (max_visible_columns / 2)).to_i
      if start_x < 0
        start_x = 0
      elsif start_x + (max_visible_columns) > @cutscene.width
        start_x = @cutscene.width - max_visible_columns
      end
    end
    start_x
  end

  def cutscene_landmarks_for row
    str = ""
    return nil if @cutscene_landmark_labels_cache[row]
    max_visible_columns.times do |i|
      if (@cutscene.x == start_x + i && @cutscene.y == row)
        str.concat "#{@cutscene.movement_char}"
      else
        candidate_char = @cutscene.map[[start_x + i, row]]
        if @cutscene.landmarks[candidate_char]
          candidate_char = @cutscene.landmarks[candidate_char][:char]
          str.concat candidate_char if candidate_char
        else
          str.concat " "
        end
      end
    end
    str
  end

  def cutscene_aesthetics_for row
    str = ""
    return nil if @cutscene_aesthetic_labels_cache[row]
    max_visible_columns.times do |i|
      if (@cutscene.x == start_x + i && @cutscene.y == row)
        str.concat " "
      else
        candidate_char = @cutscene.map[[start_x + i, row]]
        if @cutscene.landmarks[candidate_char]
          str.concat " "
        else
          str.concat candidate_char if candidate_char
        end
      end
    end
    str
  end

  def move_up_button
    return nil if !show_navigation_buttons?

    if Grid.landscape?
      bt_row = 8
      bt_col = 20

      Layout.rect(row: bt_row, col: bt_col, w: 2, h: 2)
            .merge(text: "↑", **@navigation_color)
    else
      bt_row = 7
      bt_col = 19.5
      bt_row += 12
      bt_col -= 12

      Layout.rect(row: bt_row, col: bt_col, w: 2.5, h: 2.5)
            .merge(text: "↑")
    end
  end

  def move_down_button
    return nil if !show_navigation_buttons?
    if Grid.landscape?
      bt_row = 10
      bt_col = 20

      Layout.rect(row: bt_row, col: bt_col, w: 2, h: 2)
            .merge(text: "↓", **@navigation_color)
    else
      bt_row = 9.5
      bt_col = 19.5
      bt_row += 12
      bt_col -= 12

      Layout.rect(row: bt_row, col: bt_col, w: 2.5, h: 2.5)
            .merge(text: "↓")
    end
  end

  def mark_admiral_cutscene_if_needed
    if @cutscene.is_a? StarGazerCutscene
      @game.world.last_admiral_cutscene_visited = :star_gazer_cutscene
    elsif @cutscene.is_a? PurgingCutscene
      @game.world.last_admiral_cutscene_visited = :purging_cutscene
    elsif @cutscene.is_a? DestructionCutscene
      @game.world.last_admiral_cutscene_visited = :destruction_cutscene
    end
  end

  def close_cutscene_button
    mark_admiral_cutscene_if_needed
    return nil if !show_navigation_buttons?
    if Grid.landscape?
      bt_row = 8
      bt_col = 22
      Layout.rect(row: bt_row, col: bt_col, w: 2, h: 2)
            .merge(text: "close", **@navigation_color, size_px: 20)
    else
      bt_row = 7
      bt_col = 22
      bt_row += 12
      bt_col -= 12

      Layout.rect(row: bt_row, col: bt_col, w: 2.5, h: 2.5)
            .merge(text: "close")
    end
  end

  def move_right_button
    return nil if !show_navigation_buttons?
    if Grid.landscape?
      bt_row = 10
      bt_col = 22

      Layout.rect(row: bt_row, col: bt_col, w: 2, h: 2)
            .merge(text: "→", **@navigation_color)
    else
      bt_row = 9.5
      bt_col = 22
      bt_row += 12
      bt_col -= 12

      Layout.rect(row: bt_row, col: bt_col, w: 2.5, h: 2.5)
            .merge(text: "→")
    end
  end

  def move_left_button
    return nil if !show_navigation_buttons?
    if Grid.landscape?
      bt_row = 10
      bt_col = 18

      Layout.rect(row: bt_row, col: bt_col, w: 2, h: 2)
            .merge(text: "←", **@navigation_color)
    else
      bt_row = 9.5
      bt_col = 17
      bt_row += 12
      bt_col -= 12

      Layout.rect(row: bt_row, col: bt_col, w: 2.5, h: 2.5)
            .merge(text: "←")
    end
  end

  def controller_instructions accept_char
    "[DIRECTIONAL]: move around. [#{accept_char}]: back."
  end

  def keyboard_instructions
    "[DIRECTIONAL]: move around. [ENTER/SPACE]: back."
  end
end
