# Copyright 2014 Scratchwork Development LLC. All rights reserved.

class EmbarkScene < Scene
  attr :status_labels

  def id
    :embark_scene
  end

  def world
    @game.world
  end

  def will_appear
    @status_labels = []
    @first_time_ever_playing_cache = nil

    if state.previous_scene == :game_over_scene
      @status_labels = []
    end

    @confirm_reset_at = -1000
    queue_music "sounds/embark.ogg", true
  end

  def tick
    if state.tick_count == 0
      @status_labels = []
      state.story_state = :beginning
      set_selection! :ellipses_button
      @game.load
      if first_time_ever_playing?
        start_dream_sequence
      elsif @game.has_save?
        state.story_state = :ask_to_continue
        set_selection! :continue_current_journey_button
      end
    end

    if inputs.keyboard.key_down.ac_back
      GTK.request_quit
    end

    render
  end

  def render
    return if first_time_ever_playing?
    calc_buttons
    outputs.primitives << title_label
    outputs.primitives << description_label
    render_buttons
    render_labels

    if inputs.keyboard.key_down.escape || inputs.keyboard.key_down.p
      settings_button_selected
    end

    if inputs.controller_one.key_down.start
      settings_button_selected
    end
  end

  def render_labels
    outputs.primitives << @status_labels
    outputs.primitives << god_mode_status_label
  end

  def ellipses_button_selected
    if @game.mind_games.current_message
      state.next_scene = :alert_scene
      state.next_scene_metadata = {
        title: "",
        text: @game.mind_games.message,
        source_scene: :embark_scene,
        background_color: prefab.black,
        text_color: @game.mind_games.tutorial_mode? ? prefab.white : { r: 255, g: 0, b: 0 },
        text_size_px: 35,
        text_split_length: 29
      }
      @game.mind_games.mark_message_as_seen
    else
      state.story_state = :start
      @game.reset

      outputs.a11y[:button_selected_notification] = {
        a11y_id: "button_selected_notification",
        a11y_notification_target: :description_label,
        a11y_trait: :notification
      }
    end
  end

  def its_never_over_button
    if Grid.orientation == :landscape
      row = 5
      col = [6, 17]
    else
      row = 4 + 6
      col = [0, 11]
    end

    Layout.rect(row: row, col: col, h: 1.5)
          .merge(text: "it's never over!",
                 a11y_id: "its_never_over_button",
                 a11y_text: "it's never over!",
                 a11y_trait: :button)
  end

  def button_ids
    if state.story_state == :beginning
      [
        :ellipses_button,
        :settings_button,
        :hey_listen_button,
        :thank_you_button
      ]
    elsif state.story_state == :start
      [
        :its_never_over_button,
        :settings_button,
        :hey_listen_button,
        :thank_you_button
      ]
    elsif state.story_state == :have_to_make_it_to_the_ship
      [
        :fill_flask_button,
        :settings_button,
        :hey_listen_button,
        :thank_you_button
      ]
    elsif state.story_state == :flask_filled
      [
        :take_food_button,
        :refuse_food_button,
        :settings_button,
        :hey_listen_button,
        :thank_you_button
      ]
    elsif state.story_state == :food_taken
      [
        :take_compass_button,
        :settings_button,
        :hey_listen_button,
        :thank_you_button
      ]
    elsif state.story_state == :embark
      if @game.world.praying_unlocked
        [
          :embark_button,
          :settings_button,
          :pray_button,
          :hey_listen_button,
          :thank_you_button
        ]
      else
        [
          :embark_button,
          :settings_button,
          :hey_listen_button,
          :thank_you_button
        ]
      end
    elsif state.story_state == :ask_to_continue
      [
        :continue_current_journey_button,
        :reset_current_journey_button,
        :settings_button,
        :hey_listen_button,
        :thank_you_button
      ]
    else
      [:settings_button, :hey_listen_button, :thank_you_button]
    end
  end

  def thank_you_button_selected
    state.next_scene = :modal_scene

    modal_width = if Grid.landscape?
                    [6, 17]
                  else
                    [0, 11]
                  end

    message_size_px = if Grid.landscape?
                        22
                      else
                        30
                      end

    message_wrap_length = if Grid.landscape?
                            54
                          else
                            39
                          end

    start_row = if Grid.landscape?
                  3
                else
                  4
                end

    state.modal_scene_labels = {
      title: Layout.rect(row: 0, col: modal_width, h: 1)
                   .center
                   .merge(text: "thank you",
                          font: "fonts/text.ttf",
                          a11y_id: :modal_title,
                          a11y_trait: :label,
                          a11y_text: "thank you",
                          size_px: 30,
                          anchor_x: 0.5,
                          anchor_y: 0.5),
      text: prefab.labels(lines: [
                            "thank you for letting this humble experience into your life.",
                            "indie game developers such as myself rely heavily on word of mouth recommendations and reviews.",
                            "hope you enjoy the dev commentary.",
                            "- Amir",
                            "ps: dev commentary located in settings"
                          ],
                          rect: { row: start_row, col: modal_width },
                          wrap_length: message_wrap_length,
                          size_px: message_size_px,
                          a11y_id: :modal_text)
    }

    state.modal_scene_buttons = {
      a_dark_room_button: Layout.rect(row: Layout.row_count - 4, col: modal_width, h: 2)
                                .merge(text: "leave review",
                                       a11y_id: "leave_review_button",
                                       a11y_trait: :button,
                                       a11y_text: "leave review")
                                .merge(metadata: {
                                         callback: lambda {
                                           if GTK.platform?(:steam)
                                             GTK.openurl "steam://store/3072490"
                                           elsif GTK.platform?(:ios)
                                             GTK.openurl "itms-apps://itunes.apple.com/app/id736683061"
                                           elsif GTK.platform?(:android)
                                             GTK.openurl "https://play.google.com/store/apps/details?id=net.amirrajan.theensign"
                                           else
                                             GTK.openurl "https://store.steampowered.com/app/2460660/A_Dark_Room/"
                                           end
                                         }
                                       }),
      close_button: Layout.rect(row: Layout.row_count - 2, col: modal_width, h: 2)
                          .merge(text: "close",
                                 a11y_id: "close_button",
                                 a11y_trait: :button,
                                 a11y_text: "close")
                          .merge(metadata: {
                                   callback: lambda {
                                     state.next_scene = :embark_scene
                                   }
                                 })
    }
  end

  def hey_listen_button_selected
    state.next_scene = :modal_scene

    modal_width = if Grid.landscape?
                    [6, 17]
                  else
                    [0, 11]
                  end

    state.modal_scene_labels = {
      title: Layout.rect(row: 0, col: 0, w: Layout.col_count, h: 1)
                   .center
                   .merge(text: "hey listen!",
                          font: "fonts/text.ttf",
                          a11y_id: :modal_title,
                          a11y_text: "hey listen!",
                          a11y_trait: :label,
                          size_px: 30,
                          anchor_x: 0.5,
                          anchor_y: 0.5),
      text: $root_scene.prefab.labels(lines: [
                                        "it's strongly recommended that \"a dark room\" is played first.",
                                        "\"the ensign\" is a brutally difficult pre-sequel and covers the events that lead up to lighting the fire.",
                                        "good luck, death awaits.",
                                      ],
                                      rect: { row: 3, col: modal_width },
                                      wrap_length: 39,
                                      a11y_id: :modal_text)
    }

    state.modal_scene_buttons = {
      a_dark_room_button: Layout.rect(row: Layout.row_count - 4, col: modal_width, h: 2)
                                .merge(text: "get a dark room",
                                       id: :get_a_dark_room,
                                       a11y_id: :get_a_dark_room,
                                       a11y_text: "get a dark room",
                                       a11y_trait: :button)
                                .merge(metadata: {
                                         callback: lambda {
                                           if GTK.platform?(:steam)
                                             GTK.openurl "steam://store/2460660"
                                           elsif GTK.platform?(:ios)
                                             GTK.openurl "itms-apps://itunes.apple.com/app/id736683061"
                                           elsif GTK.platform?(:android)
                                             GTK.openurl "https://play.google.com/store/apps/details?id=com.yourcompany.adarkroom"
                                           else
                                             GTK.openurl "https://store.steampowered.com/app/2460660/A_Dark_Room/"
                                           end
                                         }
                                       }),
      close_button: Layout.rect(row: Layout.row_count - 2, col: modal_width, h: 2)
                          .merge(text: "close",
                                 id: :close_button,
                                 a11y_id: :close_button,
                                 a11y_text: "close",
                                 a11y_trait: :button)
                          .merge(metadata: {
                                   callback: lambda {
                                     state.next_scene = :embark_scene
                                   }
                                 })
    }
  end

  def hey_listen_button
    return if @game.cleared_game_once?
    bt_rect = if Grid.orientation == :landscape
                Layout.rect(row: 11, col: 12, h: 1, w: 4)
              else
                Layout.rect(row: 22, col: 4, h: 1, w: 4)
              end

    bt_rect.merge(text: "hey listen!",
                  a11y_id: "hey_listen_button",
                  a11y_text: "hey listen!",
                  a11y_trait: :button)
  end

  def thank_you_button
    return if !@game.cleared_game_once?
    bt_rect = if Grid.orientation == :landscape
                Layout.rect(row: 11, col: 12, h: 1, w: 4)
              else
                Layout.rect(row: 22, col: 4, h: 1, w: 4)
              end

    bt_rect.merge(text: "thank you",
                  a11y_id: "thank_you_button",
                  a11y_text: "thank you",
                  a11y_trait: :button)
  end

  def pray_button_selected
    state.next_scene = :god_mode_scene
  end

  def refuse_food_button_selected
    @game.world.hard_mode = true
    @game.stores[:compass] = 0
    @game.world.stores[:food] = 0
    state.story_state = :embark
    outputs.a11y[:button_selected_notification] = {
      a11y_id: "button_selected_notification",
      a11y_notification_target: :description_label,
      a11y_trait: :notification
    }
  end

  def refuse_food_button
    return if !@game.world.killed_at_least_one_defector

    if Grid.orientation == :landscape
      row = 6.5
      col = [6, 17]
    else
      row = 4 + 7.5
      col = [0, 11]
    end

    Layout.rect(row: row, col: col, h: 1.5)
          .merge(text: "refuse food",
                 a11y_id: "refuse_food_button",
                 a11y_text: "refuse food",
                 a11y_trait: :button)
  end

  def god_mode_status_label
    return nil if state.story_state != :embark

    if state.god_mode_submitted
      m = state.god_mode_multipliers || @game.world.default_multipliers

      god_mode_string = "(D: #{m[:durability]}, P: #{m[:poison]}, H: #{m[:hunger]}, T: #{m[:thirst]}, V: #{m[:vision]}, E: #{m[:enemy_damage]})"
    else
      god_mode_string = ""
    end

    c = if Layout.landscape?
          6
        else
          0
        end

    r = if Grid.landscape?
          7.95
        else
          13
        end

    Layout.rect(row: r, col: c, w: 12, h: 1)
          .center.merge(text: god_mode_string,
                        font: "fonts/text.ttf",
                        font_px: 30,
                        anchor_x: 0.5,
                        anchor_y: 0.2)
  end

  def pray_button
    if Grid.orientation == :landscape
      row = 6.5
      col = [6, 17]
    else
      row = 4 + 7.5
      col = [0, 11]
    end

    Layout.rect(row: row, col: col, h: 1.5)
          .merge(text: "pray",
                 a11y_id: "pray_button",
                 a11y_text: "pray",
                 a11y_trait: :button)
  end

  def clear_areas
    return if @game.world.stores[:food] == 0 #this means they refused the food

    if @game.world.cleared_tannery_once
      @game.world.teleport_to_landmark :tannery
      @game.world.reveal_map 5
      @game.world.event.on_clear if @game.world.event
      @game.world.move_count = 0
    end

    if @game.world.cleared_iron_mine_once
      @game.world.teleport_to_landmark :iron_mine
      @game.world.reveal_map 6
      @game.world.event.on_clear if @game.world.event
      @game.world.move_count = 0
    end

    if @game.world.cleared_armoury_once
      @game.world.teleport_to_landmark :armoury
      @game.world.reveal_map 7
      @game.world.stores[:bullets] = 20
      rifle = Rifle.new
      rifle.current_durability = rifle.durability
      @game.world.weapon_stores << rifle
      @game.world.event.on_clear if @game.world.event
      @game.world.teleport_to_landmark :ship
      @game.world.reveal_map 8
      @game.world.move_count = 0
    end

    @game.world.teleport_to_landmark :swamp
  end

  def embark_button_selected
    world.begin_journey

    if (world.hard_mode?)
      d = if state.god_mode_submitted
            state.god_mode_multipliers
          else
            @game.world.default_multipliers
          end

      world.durability_multiplier = d[:durability]
      world.poison_multiplier = d[:poison]
      world.hunger_multiplier = d[:hunger]
      world.thirst_multiplier = d[:thirst]
      world.base_vision_radius = d[:vision]
      world.enemy_damage_multiplier = d[:enemy_damage]
    else
      d = @game.world.default_multipliers
      world.durability_multiplier = d[:durability]
      world.poison_multiplier = d[:poison]
      world.hunger_multiplier = d[:hunger]
      world.thirst_multiplier = d[:thirst]
      world.base_vision_radius = d[:vision]
      world.enemy_damage_multiplier = d[:enemy_damage]
    end

    clear_areas
    world.move_in_random_direction if @game.world.stores[:food] > 0
    state.next_scene = :world_scene
    state.god_mode_submitted = false
    notify_compass
  end

  def continue_current_journey_button
    if Grid.orientation == :landscape
      row = 5
      col = [6, 17]
    else
      row = 4 + 6
      col = [0, 11]
    end

    Layout.rect(row: row, col: col, h: 1.5)
          .merge(text: "continue current journey",
                 a11y_id: "embark_button",
                 a11y_text: "continue current journey",
                 a11y_trait: :button)
  end

  def reset_current_journey_button_selected
    if @confirm_reset_at.elapsed_time > 300
      @confirm_reset_at = Kernel.tick_count
    else
      begin
        @game.load
        @game.world.cause_death :starvation, false
      rescue
        @game.reset
      end

      @game.save
      state.next_scene = :alert_scene
      state.next_scene_metadata = {
        title: "",
        text: "the hand of god turns back time.",
        source_scene: :embark_scene
      }
      state.story_state = :beginning
    end
  end

  def reset_current_journey_button
    if Grid.orientation == :landscape
      row = 6.5
      col = [6, 17]
    else
      row = 5.5 + 6
      col = [0, 11]
    end

    @confirm_reset_at ||= -1000

    if @confirm_reset_at.elapsed_time < 300
      Layout.rect(row: row, col: col, h: 1.5)
            .merge(text: "sure?",
                   a11y_id: "reset_current_journey_button",
                   a11y_text: "sure?",
                   a11y_trait: :button)
    else
      Layout.rect(row: row, col: col, h: 1.5)
            .merge(text: "reset current journey",
                   a11y_id: "reset_current_journey_button",
                   a11y_text: "reset current journey",
                   a11y_trait: :button)
    end
  end

  def embark_button
    return if first_time_ever_playing?
    if Grid.orientation == :landscape
      row = 5
      col = [6, 17]
    else
      row = 4 + 6
      col = [0, 11]
    end

    Layout.rect(row: row, col: col, h: 1.5)
          .merge(text: "embark",
                 a11y_id: "embark_button",
                 a11y_text: "embark",
                 a11y_trait: :button)
  end

  def its_never_over_button_selected
    state.story_state = :have_to_make_it_to_the_ship

    outputs.a11y[:button_selected_notification] = {
      a11y_id: "button_selected_notification",
      a11y_notification_target: :description_label,
      a11y_trait: :notification
    }
  end

  def take_food_button
    if Grid.orientation == :landscape
      row = 5
      col = [6, 17]
    else
      row = 4 + 6
      col = [0, 11]
    end

    Layout.rect(row: row, col: col, h: 1.5)
          .merge(text: "take food",
                 a11y_id: "take_food_button",
                 a11y_text: "take food",
                 a11y_trait: :button)
  end

  def take_food_button_selected
    state.story_state = :food_taken
    @game.world.add_food 2

    r = if Grid.orientation == :landscape
          Layout.rect(row: 7.5, col: 6, w: 1, h: 1)
        else
          Layout.rect(row: 13, col: 0, w: 1, h: 1)
        end

    size_px = if Layout.landscape?
                26
              else
                30
              end


    @status_labels << r.merge(text: "food: 2",
                              font: "fonts/text.ttf",
                              size_px: size_px,
                              anchor_y: @status_labels.length + 1)

    outputs.a11y[:button_selected_notification] = {
      a11y_id: "button_selected_notification",
      a11y_notification_target: :description_label,
      a11y_trait: :notification
    }
  end

  def fill_flask_button
    if Grid.orientation == :landscape
      row = 5
      col = [6, 17]
    else
      row = 4 + 6
      col = [0, 11]
    end

    Layout.rect(row: row, col: col, h: 1.5)
          .merge(text: "fill flask",
                 a11y_id: "fill_flask_button",
                 a11y_text: "fill flask",
                 a11y_trait: :button)
  end

  def fill_flask_button_selected
    state.story_state = :flask_filled

    r = if Grid.orientation == :landscape
          Layout.rect(row: 7.5, col: 6, w: 1, h: 1)
        else
          Layout.rect(row: 13, col: 0, w: 1, h: 1)
        end

    size_px = if Layout.landscape?
                26
              else
                30
              end

    @status_labels << r.merge(text: "water: #{@game.world.container[:max]}",
                              size_px: size_px,
                              font: "fonts/text.ttf")

    outputs.a11y[:button_selected_notification] = {
      a11y_id: "button_selected_notification",
      a11y_notification_target: :description_label,
      a11y_trait: :notification
    }
  end

  def ellipses_button
    return if first_time_ever_playing?
    if Grid.orientation == :landscape
      row = 5
      col = [6, 17]
    else
      row = 4 + 6
      col = [0, 11]
    end

    Layout.rect(row: row, col: col, h: 1.5)
          .merge(text: "...",
                 a11y_id: "ellipses_button",
                 a11y_text: "...",
                 a11y_trait: :button)
  end

  def main_game_gear_text
    and_laser = ""
    and_laser = ", katana" if @game.world.deaths_by_defector > 1

    return "take compass#{and_laser}" if !@game.world.cleared_tannery_once

    return "take compass, steel gear#{and_laser}" if @game.world.cleared_armoury_once

    return "take compass, iron gear#{and_laser}" if @game.world.cleared_iron_mine_once

    return "take compass, leather gear#{and_laser}" if @game.world.cleared_tannery_once
  end

  def take_compass_button
    if Grid.orientation == :landscape
      row = 5
      col = [6, 17]
    else
      row = 4 + 6
      col = [0, 11]
    end

    Layout.rect(row: row, col: col, h: 1.5)
          .merge(text: main_game_gear_text,
                 a11y_id: "take_compass_button",
                 a11y_text: main_game_gear_text,
                 a11y_trait: :button)
  end

  def take_compass_button_selected
    state.story_state = :embark
    @game.world.enable_compass

    if @game.world.deaths_by_defector > 1
      katana = Katana.new
      katana.current_durability = katana.durability
      @game.world.weapon_stores << katana
    end
    # compass
    # leather

    r = if Grid.orientation == :landscape
          Layout.rect(row: 7.5, col: 6, w: 1, h: 1)
        else
          Layout.rect(row: 13, col: 0, w: 1, h: 1)
        end

    size_px = if Layout.landscape?
                26
              else
                30
              end

    @status_labels << r.merge(text: "hp: #{game.world.current_hp}/#{game.world.max_hp.to_i}",
                              font: "fonts/text.ttf",
                              size_px: size_px,
                              anchor_y: @status_labels.length + 1)

    unicode_arrow = case game.world.compass_direction
                    when "NE"
                      "↗"
                    when "NW"
                      "↖"
                    when "SE"
                      "↘"
                    when "SW"
                      "↙"
                    when "N"
                      "↑"
                    when "S"
                      "↓"
                    when "E"
                      "→"
                    when "W"
                      "←"
                    else
                      "spins"
                    end

    @status_labels << r.merge(text: "compass: #{unicode_arrow} (#{@game.world.compass_direction})",
                              font: "fonts/text.ttf",
                              size_px: size_px,
                              anchor_y: @status_labels.length + 1)

    outputs.a11y[:button_selected_notification] = {
      a11y_id: "button_selected_notification",
      a11y_notification_target: :description_label,
      a11y_trait: :notification
    }
  end

  def description_label
    if Grid.orientation == :landscape
      col = [6, 17]
      row = 1
    else
      col = [0, 11]
      row = 0 + 5
    end

    description_label_rect = Layout.rect(row: row, col: col, h: 3)

    text_with_location = []
    starting_x = description_label_rect.x
    starting_y = description_label_rect.y

    anchor_increment = 0
    text = if state.story_state == :beginning
             ["the admiral kneels from exhaustion. says the war is lost. says that wanderer destruction ends here."]
           elsif state.story_state == :start
             ["the wanderers cannot be defeated. the admiral's resolve has weakened. pathetic."]
           elsif state.story_state == :have_to_make_it_to_the_ship
             ["have to make it to the starship and call for reinforcements. can't go thirsty. this swamp water will have to do."]
           elsif state.story_state == :flask_filled
             ["the admiral reluctantly extends a hand. a few meager scraps of food."]
           elsif state.story_state == :food_taken
             if @game.world.deaths_by_defector == 0
               ["he says to follow the compass."]
             else
               ["he hands over his gear, says to follow the compass."]
             end
           elsif state.story_state == :embark
             ["death awaits. their humanity, this world, makes them strong."]
           elsif state.story_state == :ask_to_continue
             ["continue current journey, or restart altogether? if this is the first time, either option will start a new game. good luck."]
           else
             raise "unknown story state: #{state.story_state}"
           end

    text.each_with_index do |t, j|
      long_lines_split(t, 38).each_with_index do |line, i|
        anchor_increment += 1.1
        text_with_location << { x: description_label_rect.center.x + 4,
                                y: description_label_rect.center.y,
                                size_px: 30,
                                font: "fonts/text.ttf",
                                anchor_increment: anchor_increment,
                                text: line }
      end
      anchor_increment += 1
    end

    l = text_with_location.length
    text_with_location.each_with_index do |t, i|
      t.anchor_x = 0.5
      t.anchor_y = -(l / 2) + t.anchor_increment - 0.5
    end

    if GTK.a11y_enabled?
      text_with_location[0].a11y_id = "description_label"
      text_with_location[0].a11y_trait = :label
      text_with_location[0].a11y_text = text_with_location.map { |t| t.text }.join "\n"
    end

    text_with_location
  end

  def will_appear_a11y_target
    :title_label
  end

  def title_label
    title_rect = if Grid.orientation == :landscape
                   Layout.rect(row: 0, col: [6, 17], h: 1)
                 else
                   Layout.rect(row: 0, col: [0, 11], h: 1)
                 end

    title_rect.center
              .merge(text: "a murky swamp",
                     font: "fonts/text.ttf",
                     a11y_id: :title_label,
                     a11y_text: "a murky swamp",
                     a11y_trait: :label,
                     anchor_x: 0.5,
                     anchor_y: 0.5,
                     size_px: 30)
  end

  def start_dream_sequence
    begin
      @game.load
    rescue
      @game.reset
      @game.save
    end
    @game.world.reset
    @game.world.landmarks.clear
    @game.world.final_boss = true
    @game.world.is_defector_dead = false
    @game.world.defector_x = @game.world.half_width + [6, -6].sample
    @game.world.defector_y = @game.world.half_height + [6, -6].sample
    @game.world.init_event
    @game.world.water = 10
    @game.world.stores[:food] = 10
    @game.world.is_dream_sequence = true
    state.next_scene = :world_scene
  end

  def settings_button
    return if first_time_ever_playing?
    bt_rect = if Grid.orientation == :landscape
                Layout.rect(row: 11, col: 8, h: 1, w: 4)
              else
                Layout.rect(row: 23, col: 4, h: 1, w: 4)
              end

    bt_rect.merge(text: "settings",
                  a11y_id: "room_settings_button",
                  a11y_text: "settings",
                  a11y_trait: :button)
  end

  def settings_button_selected
    state.next_scene = :settings_scene
  end

  def first_time_ever_playing?
    return @first_time_ever_playing_cache == :yes if @first_time_ever_playing_cache
    @first_time_ever_playing_cache = :no
    return false if @game.just_woke_up_from_nightmare
    return false if (@game.total_defector_deaths || 0) > 0
    return false if (@game.world.water_deaths    || 0) > 0
    return false if (@game.world.food_deaths     || 0) > 0
    return false if (@game.world.battle_deaths   || 0) > 0

    @game.load

    return false if @game.just_woke_up_from_nightmare
    return false if @game.total_defector_deaths > 0
    return false if @game.world.water_deaths > 0
    return false if @game.world.food_deaths > 0
    return false if @game.world.battle_deaths > 0
    @first_time_ever_playing_cache = :yes
    return true
  end

  def continue_current_journey_button_selected
    state.next_scene = :world_scene
    notify_compass
  end

  def notify_compass
    if @game.stores[:compass] == 1
      compass_dir_text = case game.world.compass_direction
                         when "NE"
                           "points ↗ (NE)"
                         when "NW"
                           "points ↖ (NW)"
                         when "SE"
                           "points ↘ (SE)"
                         when "SW"
                           "points ↙ (SW)"
                         when "N"
                           "points ↑ (N)"
                         when "S"
                           "points ↓ (S)"
                         when "E"
                           "points → (E)"
                         when "W"
                           "points ← (W)"
                         else
                           "spins"
                         end
      text = "the compass #{compass_dir_text}"
      queue_notification :compass_direction, text
    end
  end

  def keyboard_instructions
    "#{default_keyboard_instructions} [ESC]: settings"
  end

  def controller_instructions accept_char
    "#{default_controller_instructions} [START]: settings"
  end
end
