# Copyright 2014 Scratchwork Development LLC. All rights reserved.

class Action < Hash
  attr_accessor :attack_delay, :current_cooldown, :weapon, :consumes

  def damage
    @weapon.damage
  end

  def initialize stores, options
    @stores = stores
    @attack_delay = options[:attack_delay]
    @current_cooldown = options[:attack_delay]
    @current_mini_cooldown = 0
    @weapon = options[:weapon]
    @consumes = options[:consumes]
    @durability_multiplier = options[:durability_multiplier] || 1
  end

  def action_name
    @weapon.name
  end

  def durability
    @weapon.durability
  end

  def durability_percentage
    @weapon.durability_percentage
  end

  def reset_cooldown
    @current_cooldown = @attack_delay
  end

  def current_durability
    @weapon.current_durability
  end

  def has_durability?
    @weapon.has_durability?
  end

  def melee?
    return false if @consumes

    return false if damage == 0

    true
  end

  def can_use?
    @current_cooldown >= @attack_delay and
      (melee? or
       has_consumable? or
       action_name == :stealth or
       action_name == :food)
  end

  def has_consumable?
    (@stores[@consumes] || 0) > 0
  end

  def used missed
    @current_cooldown = 0

    decrement_consumption

    decrement_durability if !missed
  end

  def broken?
    @weapon.broken?
  end

  def decrement_durability
    return if !has_durability?

    return if broken?

    weapon.current_durability -= 1 * @durability_multiplier
    weapon.current_durability = 0 if weapon.current_durability < 0

    if(broken?)
      @weapon.damage = 1
    end
  end

  def decrement_consumption
    return if !@consumes

    return if !@stores[@consumes]

    @stores[@consumes] -= 1
  end

  def tick
    # not used mini tick is used instead
    # return if can_use?

    # @current_cooldown += 1
  end

  def mini_tick
    @current_cooldown = @attack_delay if @current_cooldown > @attack_delay
    return if can_use?

    @current_cooldown += 0.016666666
  end

  def percentage
    @current_cooldown.fdiv(@attack_delay)
  end
end
