# Copyright 2014 Scratchwork Development LLC. All rights reserved.

class Game
  attr_accessor :stores, :world, :tick_count,
                :perks, :ship_thrusters, :ship_hull,
                :ship_base_thrusters, :fastest_completion_ticks, :mind_games,
                :at_beginning, :total_tick_count, :story_line, :total_defector_deaths,
                :just_woke_up_from_nightmare, :view_state, :last_battle_tick_result, :active_module,
                :cleared_hard_mode, :cleared_game_once

  def initialize
    @ship_base_thrusters = 0
    @fastest_completion_ticks = 0
    @tick_count = 0
    @battle_tick_count = 0
    @ship_thrusters = 0
    @ship_hull = 0
    @perks = []
    @stores = Stores.new
    @world = World.new self
    @mind_games = MindGames.new self
    @story_line = StoryLine.new self
    @at_beginning = true
    @total_tick_count = 0
    @total_defector_deaths = 0
  end

  def has_save?
    GameState.has_save?
  end

  def process_mind_games
    mind_games.process
  end

  def save
    GameState.save self
  end

  def save_synchronously
    GameState.save self
  end

  def load
    GameState.load self
  end

  def ship_hull_multiplier
    return 2 if Accessibility.accessibility_enabled?
    return 1
  end

  def has_upgrade? upgrade
    @stores[upgrade] == 1
  end

  def has_perk? name
    @perks.include? name
  end

  def upgrade_thrusters
    return if (@world.stores[:alien_alloy] || 0) <= 0

    @ship_thrusters += 1

    @world.stores[:alien_alloy] -= 1
  end

  def upgrade_hull
    return if (@world.stores[:alien_alloy] || 0) <= 0

    @ship_hull += 1

    @world.stores[:alien_alloy] -= 1
  end

  def add_perk name
    return if @perks.include? name

    @perks << name
  end

  def cleared_game_once?
    return @world.cleared_game_once || @world.cleared_hard_mode
  end

  def reset
    @ship_thrusters = 0
    @ship_hull = 0
    @tick_count = 0
    @at_beginning = true
    @stores.clear
    @world.reset
  end

  def exited
    @world.exited
  end

  def one_minute_passed!
    @tick_count += 1
    @total_tick_count += 1
    @story_line.process
  end
end
