# Copyright 2014 Scratchwork Development LLC. All rights reserved.

class GameState
  def self.get_ios_user_defaults_keys
    [
      "ship_thrusters", "ship_hull", "tick_count", "total_tick_count",
      "total_defector_deaths", "stores", "world", "at_beginning",
      "x", "y", "asthetics", "landmarks",
      "world_stores", "swamp_stores", "world_weapon_stores", "swamp_weapon_stores",
      "cleared", "aborted", "visited", "water", "hunger_countdown",
      "current_hp", "punch_count", "admiral_visits", "death_reason",
      "percentage_cleared_this_time", "percentage_uncovered_this_time", "hunger_countdown", "fight_delay",
      "fight_move", "food_deaths", "water_deaths", "battle_deaths", "deaths_by_defector",
      "kills_of_defector", "defector_x", "defector_y", "is_defector_dead", "hard_mode",
      "final_boss", "defeated_final_boss", "durability_multiplier", "poison_multiplier",
      "hunger_multiplier", "thirst_multiplier", "base_vision_radius", "enemy_damage_multiplier",
      "thruster_stores", "thruster_weapon_stores", "shield_stores", "shield_weapon_stores",
      "admiral_stores", "admiral_weapon_stores", "has_opened_chest", "cleared_tannery_once",
      "cleared_iron_mine_once", "cleared_armoury_once", "visited_starship_once", "message",
      "message_history", "story_line_message", "story_line_message_history", "cleared_game_once",
      "cleared_hard_mode", "killed_at_least_one_defector",
    ]
  end

  def self.user_default_raw_value_to_stores_hash value
    stores_hash = {}
    value.each do |k, v|
      stores_hash[k.to_sym] = v
    end
    stores_hash
  end

  def self.get_quadrant_mappings
    quadrant_mappings = {}
    60.times do |x|
      60.times do |y|
        quadrant_mappings[[x,y]] = {
          original_loc: { x: x, y: y },
          original_quadrant: { x: x.idiv(5), y: y.idiv(5) },
          original_quadrant_loc: { x: x % 5, y: y % 5 }
        }
      end
    end

    quadrant_mappings.each do |k, qm|
      new_quadrant = { x: qm.original_quadrant.x + 3, y: qm.original_quadrant.y - 2 }
      if new_quadrant.y < 0 && new_quadrant.x <= 5
        new_quadrant.x -= 3
        new_quadrant.y += 2
      elsif new_quadrant.y < 0 && new_quadrant.x > 5
        new_quadrant.x += 6
        new_quadrant.y += 2
      elsif new_quadrant.y > 7 && new_quadrant.x <= 5
        new_quadrant.x -= 3
        new_quadrant.y -= 2
      elsif new_quadrant.y > 7 && new_quadrant.x > 5
        new_quadrant.x += 6
        new_quadrant.y -= 2
      end

      qm.new_quadrant = new_quadrant
      qm.new_loc = {
        x: (qm.new_quadrant.x * 5) + qm.original_quadrant_loc.x,
        y: (qm.new_quadrant.y * 5) + qm.original_quadrant_loc.y
      }
    end

    quadrant_mappings
  end

  def self.user_default_raw_value_to_location quadrent_mappings, value
    loc = {}
    value.each do |k, v|
      x, y = k.split(",").map { |s| s.to_i }
      new_x = quadrent_mappings[[x, y]][:new_loc][:x]
      new_y = quadrent_mappings[[x, y]][:new_loc][:y]
      if v.is_a? String
        loc[[new_x, new_y]] = v.to_sym
      end
    end
    loc
  end

  def self.user_default_raw_value_to_weapon_stores value
    weapon_stores = []
    name_to_class = {
      "knife" => :Knife,
      "iron_sword" => :IronSword,
      "steel_sword" => :SteelSword,
      "katana" => :Katana,
      "rifle" => :Rifle,
      "laser" => :Laser,
      "grenade" => :Grenade,
      "masamune" => :Masamune,
      "muramasa" => :Muramasa,
    }
    value.each do |v|
      id, name, durability, current_durability, damage, order = v.split(",")
      next if !name_to_class[name]
      weapon = Object.const_get(name_to_class[name]).new
      weapon.id = id
      weapon.name = name.to_sym
      weapon.durability = durability.to_i
      weapon.current_durability = current_durability.to_i
      weapon.damage = damage.to_i
      weapon.order = order.to_i
      weapon_stores << weapon
    end
    weapon_stores
  end

  def self.apply_user_defaults user_defaults, game
    quadrent_mappings = get_quadrant_mappings
    game.ship_thrusters = user_defaults["ship_thrusters"]
    game.ship_hull = user_defaults["ship_hull"]
    game.tick_count = user_defaults["tick_count"]
    game.total_tick_count = user_defaults["total_tick_count"]
    game.total_defector_deaths = user_defaults["total_defector_deaths"]

    # stores
    game.stores.clear
    user_default_raw_value_to_stores_hash(user_defaults["stores"]).each do |k, v|
      game.stores[k] = v
    end

    # fog of war
    game.world.clear
    user_default_raw_value_to_location(get_quadrant_mappings, user_defaults["world"]).each do |k, v|
      game.world[k] = v
    end

    game.at_beginning = 0
    world_x = user_defaults["x"]
    world_y = user_defaults["y"]
    new_world_x = quadrent_mappings[[world_x, world_y]][:new_loc][:x]
    new_world_y = quadrent_mappings[[world_x, world_y]][:new_loc][:y]
    game.world.x = new_world_x
    game.world.y = new_world_y
    game.world[[new_world_x, new_world_y]] = :you

    # aesthetics
    game.world.aesthetics.clear
    game.world.generate_aesthetics

    game.world.aesthetics.each do |(x, y), tile|
      if x >= game.world.width || y >= game.world.height || x < 0 || y < 0
        game.world.aesthetics.delete([x, y])
      end
    end

    game.world.aesthetics.each do |k, v|
      if game.world[k] != :fog && game.world[k] != :visible
        game.world[k] = :visible
      end
    end

    # landmarks
    game.world.landmarks.clear
    user_default_raw_value_to_location(get_quadrant_mappings, user_defaults["landmarks"]).each do |k, v|
      game.world.landmarks[k] = v
    end

    game.world.landmarks.each do |(x, y), v|
      if x >= game.world.width || y >= game.world.height
        game.world.landmarks.delete([x, y])
      end
    end

    # world stores
    game.world.stores.clear
    user_default_raw_value_to_stores_hash(user_defaults["world_stores"]).each do |k, v|
      game.world.stores[k] = v
    end

    # world weapon stores
    game.world.weapon_stores.clear
    user_default_raw_value_to_weapon_stores(user_defaults["world_weapon_stores"]).each do |w|
      game.world.weapon_stores << w
    end

    # swamp stores
    game.world.swamp_stores.clear
    user_default_raw_value_to_stores_hash(user_defaults["swamp_stores"]).each do |k, v|
      game.world.swamp_stores[k] = v
    end

    # swamp weapon stores
    game.world.swamp_weapon_stores.clear
    user_default_raw_value_to_weapon_stores(user_defaults["swamp_weapon_stores"]).each do |w|
      game.world.swamp_weapon_stores << w
    end

    # cleared locations
    game.world.cleared.clear
    user_default_raw_value_to_location(get_quadrant_mappings, user_defaults["cleared"]).each do |k, v|
      game.world.cleared[k] = v
    end

    # aborted locations
    game.world.aborted.clear
    user_default_raw_value_to_location(get_quadrant_mappings, user_defaults["aborted"]).each do |k, v|
      game.world.aborted[k] = v
    end

    # visited locations
    game.world.visited.clear
    user_default_raw_value_to_location(get_quadrant_mappings, user_defaults["visited"]).each do |k, v|
      game.world.visited[k] = v
    end

    game.world.water = user_defaults["water"]
    game.world.hunger_countdown = user_defaults["hunger_countdown"]
    game.world.current_hp = user_defaults["current_hp"]
    game.world.punch_count = user_defaults["punch_count"]

    if user_defaults["admiral_visits"].to_s.length > 0
      game.world.admiral_visits = user_defaults["admiral_visits"].to_sym
    else
      game.world.admiral_visits = :stars
    end

    if user_defaults["death_reason"].to_s.length > 0
      game.world.death_reason = user_defaults["death_reason"].to_sym
    else
      game.world.death_reason = nil
    end

    game.world.percentage_cleared_this_time = user_defaults["percentage_cleared_this_time"]
    game.world.percentage_uncovered_this_time = user_defaults["percentage_uncovered_this_time"]
    game.world.hunger_countdown = user_defaults["hunger_countdown"]
    game.world.fight_delay = user_defaults["fight_delay"]
    game.world.fight_move = user_defaults["fight_move"]
    game.world.food_deaths = user_defaults["food_deaths"]
    game.world.water_deaths = user_defaults["water_deaths"]
    game.world.battle_deaths = user_defaults["battle_deaths"]
    game.world.deaths_by_defector = user_defaults["deaths_by_defector"]
    game.world.kills_of_defector = user_defaults["kills_of_defector"]
    if user_defaults["is_defector_dead"] == 1
      game.world.is_defector_dead = true
      game.world.defector_x = nil
      game.world.defector_y = nil
    else
      game.world.is_defector_dead = false
      defector_x = user_defaults["defector_x"]
      defector_y = user_defaults["defector_y"]
      new_defector_x = quadrent_mappings[[defector_x, defector_y]][:new_loc][:x]
      new_defector_y = quadrent_mappings[[defector_x, defector_y]][:new_loc][:y]
      game.world.defector_x = new_defector_x
      game.world.defector_y = new_defector_y
    end

    if user_defaults["hard_mode"] == 1
      game.world.hard_mode = true
    else
      game.world.hard_mode = false
    end

    if user_defaults["final_boss"] == 1
      game.world.final_boss = true
    else
      game.world.final_boss = false
    end

    if user_defaults["defeated_final_boss"] == 1
      game.world.defeated_final_boss = true
    else
      game.world.defeated_final_boss = false
    end

    game.world.durability_multiplier = user_defaults["durability_multiplier"]
    game.world.poison_multiplier = user_defaults["poison_multiplier"]
    game.world.hunger_multiplier = user_defaults["hunger_multiplier"]
    game.world.thirst_multiplier = user_defaults["thirst_multiplier"]
    game.world.base_vision_radius = user_defaults["base_vision_radius"]
    game.world.enemy_damage_multiplier = user_defaults["enemy_damage_multiplier"]

    # thruster stores
    game.world.thruster_stores.clear
    user_default_raw_value_to_stores_hash(user_defaults["thruster_stores"]).each do |k, v|
      game.world.thruster_stores[k] = v
    end

    # thruster weapon stores
    game.world.thruster_weapon_stores.clear
    user_default_raw_value_to_weapon_stores(user_defaults["thruster_weapon_stores"]).each do |w|
      game.world.thruster_weapon_stores << w
    end

    # shield stores
    game.world.shield_stores.clear
    user_default_raw_value_to_stores_hash(user_defaults["shield_stores"]).each do |k, v|
      game.world.shield_stores[k] = v
    end

    # shield weapon stores
    game.world.shield_weapon_stores.clear
    user_default_raw_value_to_weapon_stores(user_defaults["shield_weapon_stores"]).each do |w|
      game.world.shield_weapon_stores << w
    end

    # admiral stores
    game.world.admiral_stores.clear
    user_default_raw_value_to_stores_hash(user_defaults["admiral_stores"]).each do |k, v|
      game.world.admiral_stores[k] = v
    end

    # admiral weapon stores
    game.world.admiral_weapon_stores.clear
    user_default_raw_value_to_weapon_stores(user_defaults["admiral_weapon_stores"]).each do |w|
      game.world.admiral_weapon_stores << w
    end

    if user_defaults["has_opened_chest"] == 1
      game.world.has_opened_chest = true
    else
      game.world.has_opened_chest = false
    end

    if user_defaults["cleared_tannery_once"] == 1
      game.world.cleared_tannery_once = true
    else
      game.world.cleared_tannery_once = false
    end

    if user_defaults["cleared_iron_mine_once"] == 1
      game.world.cleared_iron_mine_once = true
    else
      game.world.cleared_iron_mine_once = false
    end

    if user_defaults["cleared_armoury_once"] == 1
      game.world.cleared_armoury_once = true
    else
      game.world.cleared_armoury_once = false
    end

    if user_defaults["visited_starship_once"] == 1
      game.world.visited_starship_once = true
    else
      game.world.visited_starship_once = false
    end

    game.mind_games.message = user_defaults["message"]
    game.mind_games.message_history = user_defaults["message_history"]

    game.story_line.message = user_defaults["story_line_message"]
    game.story_line.message_history = user_defaults["story_line_message_history"]

    if user_defaults["cleared_game_once"] == 1
      game.world.cleared_game_once = true
    else
      game.world.cleared_game_once = false
    end

    if user_defaults["cleared_hard_mode"] == 1
      game.world.cleared_hard_mode = true
    else
      game.world.cleared_hard_mode = false
    end

    if user_defaults["killed_at_least_one_defector"] == 1
      game.world.killed_at_least_one_defector = true
    else
      game.world.killed_at_least_one_defector = false
    end
  end

  def self.get_ios_user_defaults
    return {} if !$gtk.platform?(:ios)

    user_defaults = $gtk.ffi_misc.ios_get_standardUserDefaults(get_ios_user_defaults_keys)
    user_defaults["ship_thrusters"] ||= 0                      # DONE int
    user_defaults["ship_hull"] ||= 0                           # DONE int
    user_defaults["tick_count"] ||= 0                          # DONE int
    user_defaults["total_tick_count"] ||= 0                    # DONE int
    user_defaults["total_defector_deaths"] ||= 0               # DONE int
    user_defaults["stores"] ||= {}                             # DONE Hash<string, int> to Hash<Symbol, int>
    user_defaults["world"] ||= {}                              # DONE Hash<string, string> to Hash<Tuple<int, int>, symbol>
    user_defaults["x"] ||= 0                                   # DONE int
    user_defaults["y"] ||= 0                                   # DONE int
    user_defaults["asthetics"] ||= {}                          # DONE Hash<string, string> to Hash<Tuple<int, int>, symbol>
    user_defaults["landmarks"] ||= {}                          # DONE Hash<string, string> to Hash<Tuple<int, int>, symbol>
    user_defaults["world_stores"] ||= {}                       # DONE Hash<string, int> to Hash<Symbol, int>
    user_defaults["world_weapon_stores"] ||= []                # DONE Array<string> to Array<Weapon>
    user_defaults["swamp_stores"] ||= {}                       # DONE Hash<string, int> to Hash<Symbol, int>
    user_defaults["swamp_weapon_stores"] ||= []                # DONE Array<CSV> to Array<Weapon> CVS format: "id,name,durability,current_durability,damage,order"
    user_defaults["cleared"] ||= {}                            # DONE Hash<string, string> to Hash<Tuple<int, int>, symbol>
    user_defaults["aborted"] ||= {}                            # DONE Hash<string, string> to Hash<Tuple<int, int>, symbol>
    user_defaults["visited"] ||= {}                            # DONE Hash<string, string> to Hash<Tuple<int, int>, symbol>
    user_defaults["water"] ||= 0                               # DONE int
    user_defaults["hunger_countdown"] ||= 0                    # DONE int
    user_defaults["current_hp"] ||= 0                          # DONE int
    user_defaults["punch_count"] ||= 0                         # DONE int
    user_defaults["admiral_visits"] ||= "stars"                # DONE String to Symbol
    user_defaults["death_reason"] ||= ""                       # DONE String to Symbol
    user_defaults["percentage_cleared_this_time"] ||= 0        # DONE int
    user_defaults["percentage_uncovered_this_time"] ||= 0      # DONE int
    user_defaults["hunger_countdown"] ||= 0                    # DONE int
    user_defaults["fight_delay"] ||= 0                         # DONE int
    user_defaults["fight_move"] ||= 0                          # DONE int
    user_defaults["food_deaths"] ||= 0                         # DONE int
    user_defaults["water_deaths"] ||= 0                        # DONE int
    user_defaults["battle_deaths"] ||= 0                       # DONE int
    user_defaults["deaths_by_defector"] ||= 0                  # DONE int
    user_defaults["kills_of_defector"] ||= 0                   # DONE int
    user_defaults["defector_x"] ||= 0                          # DONE int
    user_defaults["defector_y"] ||= 0                          # DONE int
    user_defaults["is_defector_dead"] ||= 1                    # DONE int to bool
    user_defaults["hard_mode"] ||= 0                           # DONE int to bool
    user_defaults["final_boss"] ||= 0                          # DONE int to bool
    user_defaults["defeated_final_boss"] ||= 0                 # DONE int to bool
    user_defaults["durability_multiplier"] ||= 1               # DONE int
    user_defaults["poison_multiplier"] ||= 0                   # DONE int
    user_defaults["hunger_multiplier"] ||= 1                   # DONE int
    user_defaults["thirst_multiplier"] ||= 1                   # DONE int
    user_defaults["base_vision_radius"] ||= 2                  # DONE int
    user_defaults["enemy_damage_multiplier"] ||= 1             # DONE int
    user_defaults["thruster_stores"] ||= {}                    # DONE Hash<string, int> to Hash<Symbol, int>
    user_defaults["thruster_weapon_stores"] ||= []             # DONE Array<string> to Array<Weapon>
    user_defaults["shield_stores"] ||= {}                      # DONE Hash<string, int> to Hash<Symbol, int>
    user_defaults["shield_weapon_stores"] ||= []               # DONE Array<string> to Array<Weapon>
    user_defaults["admiral_stores"] ||= {}                     # DONE Hash<string, int> to Hash<Symbol, int>
    user_defaults["admiral_weapon_stores"] ||= []              # DONE Array<string> to Array<Weapon>
    user_defaults["has_opened_chest"] ||= 1                    # DONE int to bool
    user_defaults["cleared_tannery_once"] ||= 0                # DONE int to bool
    user_defaults["cleared_iron_mine_once"] ||= 0              # DONE int to bool
    user_defaults["cleared_armoury_once"] ||= 0                # DONE int to bool
    user_defaults["visited_starship_once"] ||= 0               # DONE int to bool
    user_defaults["message"] ||= ""                            # DONE String
    user_defaults["message_history"] ||= []                    # DONE Array<string>
    user_defaults["story_line_message"] ||= ""                 # DONE String
    user_defaults["story_line_message_history"] ||= []         # DONE Array<string>
    user_defaults["cleared_game_once"] ||= 0                   # DONE int to bool
    user_defaults["cleared_hard_mode"] ||= 0                   # DONE int to bool
    user_defaults["killed_at_least_one_defector"] ||= 0        # DONE int to bool
    user_defaults
  end

  def self.needs_migration? user_defaults
    return false if has_save?
    return false unless GTK.platform?(:ios)
    return true if (user_defaults["total_defector_deaths"] || 0) > 0
    return true if (user_defaults["water_deaths"]  || 0) > 0
    return true if (user_defaults["food_deaths"]   || 0) > 0
    return true if (user_defaults["battle_deaths"] || 0) > 0
    return true if (user_defaults["killed_at_least_one_defector"] || 0) == 1
    return false
  end

  def self.migrate_save_data_if_needed game
    return unless GTK.platform?(:ios)
    return if has_save?
    user_defaults = get_ios_user_defaults
    return unless needs_migration? user_defaults
    apply_user_defaults user_defaults, game
    save game
  rescue Exception => e
    # no op if migration failed
  end

  def self.load game
    migrate_save_data_if_needed game

    return unless has_save?

    content = GTK.read_file "saves/game_data.txt"

    raw_data = {}
    content.each_line do |line|
      key, value = line.strip.split(" ", 2)
      raw_data[key] = value || ""
    end

    $args.audio.volume = raw_data["volume"].to_f if raw_data["volume"]
    game.ship_thrusters = raw_data["game.ship_thrusters"].to_i
    game.ship_hull = raw_data["game.ship_hull"].to_i
    game.tick_count = raw_data["game.tick_count"].to_i
    game.total_tick_count = raw_data["game.total_tick_count"].to_i
    game.total_defector_deaths = raw_data["game.total_defector_deaths"].to_i
    stores_deserialize game.stores, raw_data["game.stores"]
    locs_deserialize game.world.to_h, raw_data["game.world"]
    game.at_beginning = raw_data["game.at_beginning"] == "true"
    game.world.last_admiral_cutscene_visited = (raw_data["game.world.last_admiral_cutscene_visited"] || :none).to_sym
    game.world.x = raw_data["game.world.x"].to_i
    game.world.y = raw_data["game.world.y"].to_i
    game.world.move_count = raw_data["game.world.move_count"].to_i
    locs_deserialize game.world.aesthetics, raw_data["game.world.aesthetics"]
    locs_deserialize game.world.landmarks, raw_data["game.world.landmarks"]
    stores_deserialize game.world.stores, raw_data["game.world.stores"]
    stores_deserialize game.world.swamp_stores, raw_data["game.world.swamp_stores"]
    weapon_stores_deserialize game.world.weapon_stores, raw_data["game.world.weapon_stores"]
    weapon_stores_deserialize game.world.swamp_weapon_stores, raw_data["game.world.swamp_weapon_stores"]
    locs_deserialize game.world.cleared, raw_data["game.world.cleared"]
    locs_deserialize game.world.aborted, raw_data["game.world.aborted"]
    locs_deserialize game.world.visited, raw_data["game.world.visited"]
    game.world.water = raw_data["game.world.water"].to_i
    game.world.hunger_countdown = raw_data["game.world.hunger_countdown"].to_i
    game.world.current_hp = raw_data["game.world.current_hp"].to_i
    game.world.punch_count = raw_data["game.world.punch_count"].to_i
    game.world.admiral_visits = raw_data["game.world.admiral_visits"].to_sym
    game.world.death_reason = raw_data["game.world.death_reason"].to_sym
    game.world.percentage_cleared_this_time = raw_data["game.world.percentage_cleared_this_time"].to_i
    game.world.percentage_uncovered_this_time = raw_data["game.world.percentage_uncovered_this_time"].to_i
    game.world.hunger_countdown = raw_data["game.world.hunger_countdown"].to_i
    game.world.fight_delay = raw_data["game.world.fight_delay"].to_i
    game.world.fight_move = raw_data["game.world.fight_move"].to_i
    game.world.food_deaths = raw_data["game.world.food_deaths"].to_i
    game.world.water_deaths = raw_data["game.world.water_deaths"].to_i
    game.world.battle_deaths = raw_data["game.world.battle_deaths"].to_i
    game.world.deaths_by_defector = raw_data["game.world.deaths_by_defector"].to_i
    game.world.kills_of_defector = raw_data["game.world.kills_of_defector"].to_i
    game.world.defector_x = raw_data["game.world.defector_x"].to_i
    game.world.defector_y = raw_data["game.world.defector_y"].to_i
    game.world.is_defector_dead = raw_data["game.world.is_defector_dead"] == "true"
    game.world.hard_mode = raw_data["game.world.hard_mode"] == "true"
    game.world.final_boss = raw_data["game.world.final_boss"] == "true"
    game.world.praying_unlocked = raw_data["game.world.praying_unlocked"] == "true"
    game.world.defeated_final_boss = raw_data["game.world.defeated_final_boss"] == "true"
    game.world.durability_multiplier = (raw_data["game.world.durability_multiplier"] || game.default_multipliers[:durability]).to_i
    game.world.poison_multiplier = (raw_data["game.world.poison_multiplier"] || game.default_multipliers[:poison]).to_i
    game.world.hunger_multiplier = (raw_data["game.world.hunger_multiplier"] || game.default_multipliers[:hunger]).to_i
    game.world.thirst_multiplier = (raw_data["game.world.thirst_multiplier"] || game.default_multipliers[:thirst]).to_i
    game.world.base_vision_radius = (raw_data["game.world.base_vision_radius"] || game.default_multipliers[:vision]).to_i
    game.world.enemy_damage_multiplier = (raw_data["game.world.enemy_damage_multiplier"] || game.default_multipliers[:enemy_damage]).to_i
    stores_deserialize game.world.thruster_stores, raw_data["game.world.thruster_stores"]
    weapon_stores_deserialize game.world.thruster_weapon_stores, raw_data["game.world.thruster_weapon_stores"]
    stores_deserialize game.world.shield_stores, raw_data["game.world.shield_stores"]
    weapon_stores_deserialize game.world.shield_weapon_stores, raw_data["game.world.shield_weapon_stores"]
    stores_deserialize game.world.admiral_stores, raw_data["game.world.admiral_stores"]
    weapon_stores_deserialize game.world.admiral_weapon_stores, raw_data["game.world.admiral_weapon_stores"]
    game.world.has_opened_chest = raw_data["game.world.has_opened_chest"] == "true"
    game.world.cleared_tannery_once = raw_data["game.world.cleared_tannery_once"] == "true"
    game.world.cleared_iron_mine_once = raw_data["game.world.cleared_iron_mine_once"] == "true"
    game.world.cleared_armoury_once = raw_data["game.world.cleared_armoury_once"] == "true"
    game.world.visited_starship_once = raw_data["game.world.visited_starship_once"] == "true"
    game.mind_games.message = raw_data["game.mind_games.message"]
    strings_deserialize game.mind_games.message_history, raw_data["game.mind_games.message_history"]
    game.story_line.message = raw_data["game.story_line.message"]
    strings_deserialize game.story_line.message_history, raw_data["game.story_line.message_history"]
    game.world.cleared_game_once = raw_data["game.world.cleared_game_once"] == "true"
    game.world.cleared_hard_mode = raw_data["game.world.cleared_hard_mode"] == "true"
    game.world.killed_at_least_one_defector = raw_data["game.world.killed_at_least_one_defector"] == "true"
  end

  def self.save game
    GTK.write_file "saves/game_data.txt", ""

    raw_data = {}
    raw_data["volume"] = $args.audio.volume
    raw_data["game.ship_thrusters"] = game.ship_thrusters
    raw_data["game.ship_hull"] = game.ship_hull
    raw_data["game.tick_count"] = game.tick_count
    raw_data["game.total_tick_count"] = game.total_tick_count
    raw_data["game.total_defector_deaths"] = game.total_defector_deaths
    raw_data["game.stores"] = stores_serialize game.stores
    raw_data["game.world"] = locs_serialize game.world.to_h
    raw_data["game.at_beginning"] = game.at_beginning
    raw_data["game.world.last_admiral_cutscene_visited"] = game.world.last_admiral_cutscene_visited
    raw_data["game.world.x"] = game.world.x
    raw_data["game.world.y"] = game.world.y
    raw_data["game.world.aesthetics"] = locs_serialize game.world.aesthetics
    raw_data["game.world.landmarks"] = locs_serialize game.world.landmarks
    raw_data["game.world.stores"] = stores_serialize game.world.stores
    raw_data["game.world.swamp_stores"] = stores_serialize game.world.swamp_stores
    raw_data["game.world.weapon_stores"] = weapon_stores_serialize game.world.weapon_stores
    raw_data["game.world.swamp_weapon_stores"] = weapon_stores_serialize game.world.swamp_weapon_stores
    raw_data["game.world.cleared"] = locs_serialize game.world.cleared
    raw_data["game.world.aborted"] = locs_serialize game.world.aborted
    raw_data["game.world.visited"] = locs_serialize game.world.visited
    raw_data["game.world.water"] = game.world.water
    raw_data["game.world.move_count"] = game.world.move_count
    raw_data["game.world.hunger_countdown"] = game.world.hunger_countdown
    raw_data["game.world.current_hp"] = game.world.current_hp
    raw_data["game.world.punch_count"] = game.world.punch_count
    raw_data["game.world.admiral_visits"] = game.world.admiral_visits
    raw_data["game.world.death_reason"] = game.world.death_reason
    raw_data["game.world.percentage_cleared_this_time"] = game.world.percentage_cleared_this_time
    raw_data["game.world.percentage_uncovered_this_time"] = game.world.percentage_uncovered_this_time
    raw_data["game.world.hunger_countdown"] = game.world.hunger_countdown
    raw_data["game.world.fight_delay"] = game.world.fight_delay
    raw_data["game.world.fight_move"] = game.world.fight_move
    raw_data["game.world.food_deaths"] = game.world.food_deaths
    raw_data["game.world.water_deaths"] = game.world.water_deaths
    raw_data["game.world.battle_deaths"] = game.world.battle_deaths
    raw_data["game.world.deaths_by_defector"] = game.world.deaths_by_defector
    raw_data["game.world.kills_of_defector"] = game.world.kills_of_defector
    raw_data["game.world.defector_x"] = game.world.defector_x
    raw_data["game.world.defector_y"] = game.world.defector_y
    raw_data["game.world.is_defector_dead"] = game.world.is_defector_dead
    raw_data["game.world.praying_unlocked"] = game.world.praying_unlocked
    raw_data["game.world.hard_mode"] = game.world.hard_mode
    raw_data["game.world.final_boss"] = game.world.final_boss
    raw_data["game.world.defeated_final_boss"] = game.world.defeated_final_boss
    raw_data["game.world.durability_multiplier"] = game.world.durability_multiplier
    raw_data["game.world.poison_multiplier"] = game.world.poison_multiplier
    raw_data["game.world.hunger_multiplier"] = game.world.hunger_multiplier
    raw_data["game.world.thirst_multiplier"] = game.world.thirst_multiplier
    raw_data["game.world.base_vision_radius"] = game.world.base_vision_radius
    raw_data["game.world.enemy_damage_multiplier"] = game.world.enemy_damage_multiplier
    raw_data["game.world.thruster_stores"] = stores_serialize game.world.thruster_stores
    raw_data["game.world.thruster_weapon_stores"] = weapon_stores_serialize game.world.thruster_weapon_stores
    raw_data["game.world.shield_stores"] = stores_serialize game.world.shield_stores
    raw_data["game.world.shield_weapon_stores"] = weapon_stores_serialize game.world.shield_weapon_stores
    raw_data["game.world.admiral_stores"] = stores_serialize game.world.admiral_stores
    raw_data["game.world.admiral_weapon_stores"] = weapon_stores_serialize game.world.admiral_weapon_stores
    raw_data["game.world.has_opened_chest"] = game.world.has_opened_chest
    raw_data["game.world.cleared_tannery_once"] = game.world.cleared_tannery_once
    raw_data["game.world.cleared_iron_mine_once"] = game.world.cleared_iron_mine_once
    raw_data["game.world.cleared_armoury_once"] = game.world.cleared_armoury_once
    raw_data["game.world.visited_starship_once"] = game.world.visited_starship_once
    raw_data["game.mind_games.message"] = game.mind_games.message
    raw_data["game.mind_games.message_history"] = strings_serialize game.mind_games.message_history
    raw_data["game.story_line.message"] = game.story_line.message
    raw_data["game.story_line.message_history"] = strings_serialize game.story_line.message_history
    raw_data["game.world.cleared_game_once"] = game.world.cleared_game_once
    raw_data["game.world.cleared_hard_mode"] = game.world.cleared_hard_mode
    raw_data["game.world.killed_at_least_one_defector"] = game.world.killed_at_least_one_defector

    raw_data.each do |k, v|
      GTK.append_file "saves/game_data.txt", "#{k} #{v}\n"
    end

    true
  end

  def self.has_save?
    GTK.stat_file "saves/game_data.txt"
  end

  def self.locs_serialize hash
    hash.keys.map do |k|
      "#{k}=>#{hash[k]}"
    end.join("|")
  end

  def self.locs_deserialize target, s
    target.clear
    s.split("|").each do |s|
      k, v = s.split("=>")
      x, y = k.gsub("[", "").gsub("]", "").split(",").map { |s| s.to_i }
      if v == "true" || v == "false"
        v = v == "true"
      else
        v = v.to_sym
      end
      target[[x, y]] = v
    end
  end

  def self.stores_deserialize target, s
    target.clear
    s.split("|").each do |s|
      k, v = s.split("=>")
      target[k.to_sym] = v.to_i
    end
  end

  def self.stores_serialize hash
    hash.keys.map do |k|
      "#{k}=>#{hash[k]}"
    end.join("|").strip
  end

  def self.weapon_stores_serialize array
    array.map do |w|
      w.to_h.map do |k, v|
        "#{k}=>#{v}"
      end.join "|"
    end.join(",").strip
  end

  def self.strings_serialize strings
    strings.join("|").strip
  end

  def self.weapon_stores_deserialize target, s
    target.clear
    weapons = s.split(",").map do |w|
      kvps = w.split("|").map do |kvp|
        kvp.split "=>"
      end
      h = Hash[kvps]
      c = Object.const_get h["type"].to_sym
      weapon = c.new
      weapon.id = h["id"]
      weapon.name = h["name"].to_sym
      weapon.durability = h["durability"].to_i
      weapon.current_durability = h["current_durability"].to_i
      weapon.damage = h["damage"].to_i
      weapon.order = h["order"].to_i
      weapon
    end

    weapons.each do |w|
      target << w
    end
  end

  def self.strings_deserialize target, s
    target.clear
    ss = s.split("|")
    ss.each do |s|
      target << s
    end
  end
end
