# Copyright 2014 Scratchwork Development LLC. All rights reserved.

class MiniWorld < Hash
  attr_accessor :map, :x, :y, :event, :width, :height, :landmarks, :event, :enemies, :ship_defectors_killed, :fight_stacks,
                :moves_taken, :ticks, :end_game_sequence, :artifact_worn

  def initialize game
    @game = game
    @landmarks = {}
    init_map
    @ship_defectors_killed = 0
    @fight_stacks = 0
    @enemies = []
    @moves_taken = 0
    @ticks = 0
    @end_game_sequence = false
    artifact_worn = false
    @tempx = 39
    @tempy = 37

    @game.save
  end

  def init_map
    @map = { }

    w = 0
    h = 0

    layout.each do |row|
      row.each_char do |column|
        if(column == movement_char)
          @x = w
          @y = h
          @map[[w,h]] = " "
        elsif event_map.include? column
          @landmarks[[w,h]] = column
          @map[[w,h]] = " "
        else
          @map[[w,h]] = column
        end
        w += 1
      end
      @width = w
      w = 0
      h += 1
    end

    @height = h
  end

  def blown_map
    @map = { }

    w = 0
    h = 0

    blown_layout.each do |row|
      row.each_char do |column|
        if(column == movement_char)
          @x = w
          @y = h
          @map[[w,h]] = " "
        elsif event_map.include? column
          @landmarks[[w,h]] = column
          @map[[w,h]] = " "
        else
          @map[[w,h]] = column
        end
        w += 1
      end
      @width = w
      w = 0
      h += 1
    end

    @height = h
    @x = @tempx
    @y = @tempy
  end

  def blown_map_save_builder
    @map = { }

    w = 0
    h = 0

    blown_layout_save_builder.each do |row|
      row.each_char do |column|
        if(column == movement_char)
          @x = w
          @y = h
          @map[[w,h]] = " "
        elsif event_map.include? column
          @landmarks[[w,h]] = column
          @map[[w,h]] = " "
        else
          @map[[w,h]] = column
        end
        w += 1
      end
      @width = w
      w = 0
      h += 1
    end

    @height = h
    @x = @tempx
    @y = @tempy
  end

  def meat_cool_down= value
    @game.world.meat_cool_down = value
  end

  def meat_cool_down
    @game.world.meat_cool_down
  end

  def meat_delay
    @game.world.meat_delay
  end

  def tick_meat
    @game.world.tick_meat
  end

  def init_actions
    @game.world.init_actions
  end

  def meat_heal
    @game.world.meat_heal
  end

  def stores
    @game.world.stores
  end

  def heal_after_battle
    @game.world.heal_after_battle
  end

  def available_actions
    @game.world.available_actions
  end

  def enemy_damage_multiplier
    @game.world.enemy_damage_multiplier
  end

  def current_hp
    @game.world.current_hp
  end

  def max_hp
    @game.world.max_hp
  end

  def punch_level
    @game.world.punch_level
  end

  def can_eat_meat?
    @game.world.can_eat_meat?
  end

  def eat_meat
    @game.world.eat_meat
  end

  def dead?
    @game.world.dead?
  end

  def in_store_area?
    @game.world.in_store_area?
  end

  def has_event?
    #@event
    !@event.nil?
  end

  def event
    @event
  end

  def landmark_at? x, y
    @landmarks[[x,y]]
  end

  def carry
    @game.world.carry
  end

  def drop_stores key, target
    @game.world.drop_stores key, target
  end

  def drop_weapon weapon, target
    @game.world.drop_weapon weapon, target
  end

  def in_store_area?
    @game.world.in_store_area?
  end

  def inventory_weight
    @game.world.inventory_weight
  end

  def rusted?
    @game.world.rusted?
  end

  def stores
    @game.world.stores
  end

  def take_stores key, target
    @game.world.take_stores key, target
  end

  def take_weapon weapon, target
    @game.world.take_weapon weapon, target
  end

  def weapon_stores
    @game.world.weapon_stores
  end

  def move_up
    move :y, -1
  end

  def move_down
    move :y, 1
  end

  def move_left
    move :x, -1
  end

  def move_right
    move :x, 1
  end

  def wall_at? x, y
    @map[[x, y]] != " "
  end

  def walls
    @map.map do |k, v|
      if v != " "
        k
      else
        nil
      end
    end.compact
  end

  def move_enemies
    @enemies.each do |e|
      if @ticks < 10
        if [1, 2, 3, 4].sample != 1 #1/4 chance to not move
          move_towards_player e
        end
      elsif @ticks < 20
        if [1, 2, 3, 4, 5, 6, 7, 8].sample != 1 #1/8 chance to not move
          move_towards_player e
        end
      else
        if [1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16].sample != 1
          move_towards_player e
        end
      end
    end
  end

  def add_random_enemy
    @temp_enemies = []
    case @enemies.count
    when 0
      @temp_enemies = [
        {
          char: "&",
          x: 17,
          y: 3
        },{
          char: "&",
          x: 22,
          y: 3
        },{
          char: "&",
          x: 25,
          y: 3
        },{
          char: "&",
          x: 21,
          y: 3
        },{
          char: "&",
          x: 22,
          y: 3
        }
      ]
    when 1
      @temp_enemies = [
        {
          char: "&",
          x: @enemies[0][:x],
          y: @enemies[0][:y]
        },{
          char: "&",
          x: 22,
          y: 3
        },{
          char: "&",
          x: 25,
          y: 3
        },{
          char: "&",
          x: 21,
          y: 3
        },{
          char: "&",
          x: 22,
          y: 3
        }
      ]
    when 2
      @temp_enemies = [
        {
          char: "&",
          x: @enemies[0][:x],
          y: @enemies[0][:y]
        },{
          char: "&",
          x: @enemies[1][:x],
          y: @enemies[1][:y]
        },{
          char: "&",
          x: 25,
          y: 3
        },{
          char: "&",
          x: 21,
          y: 3
        },{
          char: "&",
          x: 22,
          y: 3
        }
      ]
    when 3
      @temp_enemies = [
        {
          char: "&",
          x: @enemies[0][:x],
          y: @enemies[0][:y]
        },{
          char: "&",
          x: @enemies[1][:x],
          y: @enemies[1][:y]
        },{
          char: "&",
          x: @enemies[2][:x],
          y: @enemies[2][:y]
        },{
          char: "&",
          x: 21,
          y: 3
        },{
          char: "&",
          x: 22,
          y: 3
        }
      ]
    when 4
      @temp_enemies = [
        {
          char: "&",
          x: @enemies[0][:x],
          y: @enemies[0][:y]
        },{
          char: "&",
          x: @enemies[1][:x],
          y: @enemies[1][:y]
        },{
          char: "&",
          x: @enemies[2][:x],
          y: @enemies[2][:y]
        },{
          char: "&",
          x: @enemies[3][:x],
          y: @enemies[3][:y]
        },{
          char: "&",
          x: 22,
          y: 3
        }
      ]
    when 5
      @temp_enemies = @enemies
    else
      @temp_enemies = @enemies
    end
    @enemies = @temp_enemies

  end

  def add_enemy enemy
    @temp_enemies = []
    case @enemies.count
    when 1
      @temp_enemies = [
        {
          char: "&",
          x: @enemies[0][:x],
          y: @enemies[0][:y]
        },{
          char: "&",
          x: enemy[:x],
          y: enemy[:y]
        },{
          char: "&",
          x: 25,
          y: 3
        },{
          char: "&",
          x: 21,
          y: 3
        },{
          char: "&",
          x: 22,
          y: 3
        }
      ]
    when 2
      @temp_enemies = [
        {
          char: "&",
          x: @enemies[0][:x],
          y: @enemies[0][:y]
        },{
          char: "&",
          x: @enemies[1][:x],
          y: @enemies[1][:y]
        },{
          char: "&",
          x: enemy[:x],
          y: enemy[:y]
        },{
          char: "&",
          x: 21,
          y: 3
        },{
          char: "&",
          x: 22,
          y: 3
        }
      ]
    when 3
      @temp_enemies = [
        {
          char: "&",
          x: @enemies[0][:x],
          y: @enemies[0][:y]
        },{
          char: "&",
          x: @enemies[1][:x],
          y: @enemies[1][:y]
        },{
          char: "&",
          x: @enemies[2][:x],
          y: @enemies[2][:y]
        },{
          char: "&",
          x: enemy[:x],
          y: enemy[:y]
        },{
          char: "&",
          x: 22,
          y: 3
        }
      ]
    when 4
      @temp_enemies = [
        {
          char: "&",
          x: @enemies[0][:x],
          y: @enemies[0][:y]
        },{
          char: "&",
          x: @enemies[1][:x],
          y: @enemies[1][:y]
        },{
          char: "&",
          x: @enemies[2][:x],
          y: @enemies[2][:y]
        },{
          char: "&",
          x: @enemies[3][:x],
          y: @enemies[3][:y]
        },{
          char: "&",
          x: enemy[:x],
          y: enemy[:y]
        }
      ]
    when 5
      @temp_enemies = @enemies
    else
      @temp_enemies = @enemies
    end
    @enemies = @temp_enemies
  end

  def check_duplicate_enemies at_x, at_y
    has_duplicate = false
    count = 0
    @enemies.each do |enemy|
      if(at_x == enemy[:x] && at_y == enemy[:y])
        count += 1
        if count > 1
          has_duplicate = true
          @enemies.delete(enemy)
          add_enemy enemy
        end
      end
    end
    if has_duplicate == true
    end
  end

  def move_towards_player enemy
    first_ordinal = [:x, :y].sample
    other_ordinal = :y
    other_ordinal = :x if first_ordinal == :y

    new_loc = towards_center_to(
      { x: @x, y: @y },
      { x: enemy[:x], y: enemy[:y] },
      first_ordinal
    )

    if(wall_at? new_loc[:x], new_loc[:y])
      new_loc = towards_center_to(
        { x: @x, y: @y },
        { x: enemy[:x], y: enemy[:y] },
        other_ordinal
      )

      return if(wall_at? new_loc[:x], new_loc[:y])
      return if @enemies.find { |e| e[:x] == new_loc[:x] && e[:y] == new_loc[:y] }
      return if new_loc[:y] < enemy[:y] && new_loc[:y] < 21

      enemy[:x] = new_loc[:x]
      enemy[:y] = new_loc[:y]
    elsif new_loc[first_ordinal] == enemy[first_ordinal]
      new_loc = towards_center_to(
        { x: @x, y: @y },
        { x: enemy[:x], y: enemy[:y] },
        other_ordinal
      )

      return if(wall_at? new_loc[:x], new_loc[:y])
      return if @enemies.find { |e| e[:x] == new_loc[:x] && e[:y] == new_loc[:y] }
      return if new_loc[:y] < enemy[:y] && new_loc[:y] < 21

      enemy[:x] = new_loc[:x]
      enemy[:y] = new_loc[:y]
    else
      return if @enemies.find { |e| e[:x] == new_loc[:x] && e[:y] == new_loc[:y] }
      return if new_loc[:y] < enemy[:y] && new_loc[:y] < 21

      enemy[:x] = new_loc[:x]
      enemy[:y] = new_loc[:y]

      new_loc = towards_center_to(
        { x: @x, y: @y },
        { x: enemy[:x], y: enemy[:y] },
        other_ordinal
      )

      return if(wall_at? new_loc[:x], new_loc[:y])
      return if @enemies.find { |e| e[:x] == new_loc[:x] && e[:y] == new_loc[:y] }
      return if new_loc[:y] < enemy[:y] && new_loc[:y] < 21

      enemy[:x] = new_loc[:x]
      enemy[:y] = new_loc[:y]
    end
  end

  def towards_center_to(destination, from, axis)
    new_x = from[:x]
    new_y = from[:y]


    if axis == :x
      if(new_x != destination[:x])
        if(new_x > destination[:x])
          new_x -= 1
        else
          new_x += 1
        end
      end
      #rng move adjacent
      new_y = [from[:y], from[:y], from[:y], from[:y], [from[:y] - 1, from[:y] + 1].sample].sample
    end

    if axis == :y
      if(new_y != destination[:y])
        if(new_y > destination[:y])
          new_y -= 1
        else
          new_y += 1
        end
      end
      new_x = [from[:x], from[:x], from[:x], from[:x], [from[:x] - 1, from[:x] + 1].sample].sample
    end

    { x: new_x, y: new_y }
  end

  def move direction, value
    new_x = @x
    new_y = @y
    new_x = @x + value if direction == :x
    new_y = @y + value if direction == :y
    return if wall_at? new_x, new_y
    @moves_taken += 1
    if @end_game_sequence
    #  puts "#{@ticks}"
    else
      #  puts "#{@moves_taken}"
    end
    @x = new_x
    @y = new_y
    @tempx = new_x
    @tempy = new_y
    move_enemies
    init_event
    @enemies.each do |e|
      check_duplicate_enemies(e[:x], e[:y])
    end
  end

  def init_event
    @enemies.each do |enemy|
      if (enemy[:x] == @x && enemy[:y] == @y)
        @fight_stacks += 1
        @enemies.delete(enemy)

        @event = ShipDefectorEvent.new @game
        @ship_defectors_killed += 1
        #puts "defectors fought: #{@ship_defectors_killed}"
        add_random_enemy
        return
      end
    end

    landmark = @landmarks[[x,y]]

    event_class = event_map[landmark]
    if event_class
      @event = event_class.new @game
    end
  end

  def start_endgame_sequence
    @end_game_sequence = true
    @artifact_worn = false
    @ticks = 2 #ensures player cannot sneak into 'dead' zone in between map switches
    end_game_process
  end

  def game_tick
    return unless @end_game_sequence
    return if @event
    @ticks += 1
    end_game_process
  end

  def end_game_message
    @end_game_message
  end

  def end_game_sequence_complete?
    @ticks >= 142
  end

  def end_game_process
    if(@artifact_worn && @ticks < 132)#does not have builder pendant
      @ticks = 142 #jumps to 3 moves before explosion
    end

    if (@ticks == 4)
      @end_game_message = "???"
    elsif (@ticks == 6)
      @end_game_message = "the engines remain silent."
    elsif (@ticks == 7 && !@enemies_spawned)
      blown_map
      spawn_enemies
      @landmarks = {[21, 3]=>"V", [39, 3]=>"B", [49, 8]=>"M", [30, 17]=>"L", [49, 17]=>"O", [39, 23]=>"X"}
      @enemies_spawned = true
    elsif (@ticks == 8)
      @end_game_message = "explosions ring through the ship. defectors spill out the blast doors."
    elsif (@ticks == 10)
      @end_game_message = "a trap. the defectors close in, screaming for the admiral's head."
    elsif (@ticks == 12)
      @end_game_message = "self destruct sequence initiated."
    elsif (@ticks == 24)
      @end_game_message = "detonation in: 10."
    elsif (@ticks == 36)
      @end_game_message = "detonation in: 9."
    elsif (@ticks == 48)
      @end_game_message = "detonation in: 8."
    elsif (@ticks == 60)
      @end_game_message = "detonation in: 7."
    elsif (@ticks == 72)
      @end_game_message = "detonation in: 6."
    elsif (@ticks == 84)
      @end_game_message = "detonation in: 5."
    elsif (@ticks == 96)
      @end_game_message = "detonation in: 4."
    elsif (@ticks == 108)
      @end_game_message = "detonation in: 3."
    elsif (@ticks == 120)
      @end_game_message = "detonation in: 2."
    elsif (@ticks == 132)
      @end_game_message = "detonation in: 1."
    elsif (@ticks == 144)
      if (@artifact_worn)
        food_refused = (@game.stores[:compass] || 0) == 0
        if food_refused
          @game.world.cleared_game_once = true
          @game.world.cleared_hard_mode = true
          @game.world.cause_death :complete_hard #yes, it's a hack, don't screw with this, you'll cause memory leaks
          @game.save
        else
          @game.world.cleared_game_once = true
          @game.world.cause_death :complete #yes, it's a hack, don't screw with this, you'll cause memory leaks
          @game.save
        end
      end
    elsif (@ticks > 144)
      #@space_screen = new_space_screen
      #popup @space_screen #blows up (error wise)

      if(!@artifact_worn)
        @game.world.cause_death :crashed
      end
    end
  end

  def end_game?
    if(@artifact_worn && @ticks > 144)
      return true
    else
      return false
    end
  end
  def location_for_landmark char
    @landmarks.find { |k, v| v == char }[0]
  end

  def describe_terrain
    walls.find_all do |(wall_x, wall_y)|
      loc = { x: wall_x, y: wall_y }
      Geometry.distance({ x: @x, y: @y }, loc) <= 3
    end.sort_by do |(wall_x, wall_y)|
      loc = { x: wall_x, y: wall_y }
      Geometry.distance({ x: @x, y: @y }, loc)
    end.map do |(wall_x, wall_y)|
      loc = { x: wall_x, y: wall_y }
      "wall: #{desc_location loc}"
    end
  end

  def describe_landmark loc
    descriptions = {
      "X" => "blast doors",
      "O" => "observatory",
      "M" => "medical bay",
      "B" => "builder's quarters",
      "V" => "reinforced blast doors",
      "S" => "shields",
      "T" => "thrusters",
      "L" => "launch controls",
      "A" => "admiral's quarters"
    }

    if loc.is_a? Array
      loc = { x: loc[0], y: loc[1] }
    end

    entry = @landmarks[[loc[:x], loc[:y]]]
    description = descriptions[entry] || entry.to_s.gsub("_", " ")
    "#{description}, #{desc_location loc}"
  end

  def describe_landmarks
    landmarks.sort_by do |k, v|
      loc = { x: k[0], y: k[1] }
      Geometry.distance({ x: @x, y: @y }, loc)
    end.map do |k, v|
      loc = { x: k[0], y: k[1] }
      describe_landmark loc
    end
  end

  #def describe_defector
  #return "" if @is_defector_dead
  #  return "" if self[[@defector_x, @defector_y]] != :visible
  #  return "defector: #{desc_location({ x: @defector_x, y: @defector_y })}"
  #end

  def describe_defectors
    message = ""
    @enemies.sort_by do |e|
      Geometry.distance({ x: @x, y: @y }, e)
    end.each do |e|
      message += "defector: #{desc_location({ x: e[:x], y: e[:y]})},\n"
      #message.join("defector: #{desc_location({ x: e[:x], y: e[:y]})},")
      #puts "#{message}"
    end
    #puts "all enemies done:\n#{message}"

    #return "defector: #{desc_location()}"
    return message
  end

  def desc_location loc
    if loc.is_a? Array
      loc = { x: loc[0], y: loc[1] }
    end
    distance_x = @x - loc[:x]
    distance_y = @y - loc[:y]
    direction_y = "north"
    direction_y = "south" if distance_y < 0
    direction_x = "west"
    direction_x = "east" if distance_x < 0
    return "current location" if distance_x == 0 and distance_y == 0
    return "#{distance_x.abs} #{clicks distance_x} #{direction_x}" if distance_y == 0
    return "#{distance_y.abs} #{clicks distance_y} #{direction_y}" if distance_x == 0
    return "#{distance_y.abs} #{clicks distance_y} #{direction_y}, #{distance_x.abs} #{clicks distance_x} #{direction_x}"
  end

  def clicks distance
    return "click" if distance.abs == 1
    return "clicks"
  end
end
