# Copyright 2014 Scratchwork Development LLC. All rights reserved.

class ArmouryEvent < WorldEvent
  def initialize game
    @game = game
    @title = "the armoury"
    @combat_tick = 0
    super @game.world.stores, @game.world.weapon_stores, @game.world.weapon_items, @game.world.enemy_damage_multiplier
  end

  def action_on_exit
    :kill_in_battle
  end

  def generates_road?
    true
  end

  def init_scenes
    leave = { :next_scene => :end }
    clear = { :next_scene => :clear }

    @scenes = {
      :start => {
        :text => [
          "veterans of the war patrol the perimeter, rifles slung over their shoulders."
        ],
        :options => {
          :attack => {:next_scene => { 1 => :a1 } },
          :leave => leave
        }
      },
      :a1 => {
        :text => ["a soldier, alerted, opens fire."],
        :combat => true,
        :enemy => :soldier,
        :ranged => true,
        :damage => 8,
        :hit => 0.8,
        :attack_delay => 2,
        :health => 50,
        :max_health => 50,
        :loot => {
          :food => { :min => 1, :max => 5, :chance => 1.0 },
        },
        :options => {
          :continue => {
            :next_scene => { 1 => :a2 }
          },
        }
      },
      :a2 => {
        :text => ["a second soldier joins the fight."],
        :combat => true,
        :enemy => :soldier,
        :ranged => true,
        :damage => 8,
        :hit => 0.8,
        :attack_delay => 2,
        :health => 50,
        :max_health => 50,
        :loot => {
          :food => { :min => 1, :max => 5, :chance => 1.0 },
        },
        :options => {
          :continue => {
            :next_scene => { 1 => :a3 }
          },
        }
      },
      :a3 => {
        :text => ["a grizzled soldier attacks, drawing a katana."],
        :combat => true,
        :enemy => :veteran,
        :damage => 9,
        :hit => 0.8,
        :attack_delay => 2,
        :health => 65,
        :max_health => 65,
        :loot => {
          :katana => { :min => 1, :max => 1, :chance => 1.0 },
          :food => { :min => 1, :max => 5, :chance => 1.0 }
        },
        :options => {
          :continue => {
            :next_scene => { 1 => :mine }
          },
        }
      },
      :mine =>  {
        :text => [
          "weapons lay in the store room.",
          "steel armour glistens in the starlight through broken windows.",
        ],
        :loot => {
          :bullets => { :min => 10, :max => 20, :chance => 1.0 },
          :rifle => { :min => 1, :max => 3, :chance => 1.0 }
        },
        :options => {
          :continue => {
            :next_scene => { 1 => :cleared }
          },
        }
      },
      :cleared => {
        :text => [
          "the military presence has been cleared.",
          "a convoy and water tank, now, free to take.",
        ],
        :options => { :leave => clear }
      }
    }
  end
end
