# Copyright 2014 Scratchwork Development LLC. All rights reserved.

class CityEvent < WorldEvent
  def initialize game
    @game = game
    @title = "a ruined city"
    @combat_tick = 0
    super @game.world.stores, @game.world.weapon_stores, @game.world.weapon_items, @game.world.enemy_damage_multiplier
  end

  def action_on_exit
    :aborted
  end

  def becomes_outpost?
    true
  end

  def init_scenes
    @scenes = {
      :start => {
        :text => [
          "a battered highway sign stands guard at the entrance to this once-great city.",
          "the towers that still stand jut from the landscape like the ribcage of an ancient beast.",
        ],
        :options => {
          :enter => {
            :next_scene => { 0.33 => :a1, 0.33 => :a2, 1 => :a3 }
          },
          :leave => leave
        }
      },
      :a1 => {
        :storyline => true,
        :text => [
          "the streets are empty. the smell of the admiral's genocide still lingers in the air.",
        ],

        :options => {
          :tower => {
            :next_scene => { 1 => :b1 }
          },
          :streets => {
            :next_scene => { 1 => :b2 }
          },

          :leave => leave
        }
      },
      :a2 => {
        :text => [
          "orange traffic cones are set across the street, faded and cracked.",
          "lights flash through the alleys between buildings. a wanderer has been strung as a warning."
        ],
        :options => {

          :through_cones => {
            :next_scene => { 1 => :b3 }
          },
          :alleyways => {
            :next_scene => { 1 => :b4 }
          },

          :leave => leave
        }
      },
      :a3 => {
        :text => [
          "they are trying to rebuild. trying to move on with life.",
          "faces, darkened by soot and blood, stare out from crooked huts.",
        ],
        :options => {

          :town_square => {
            :next_scene => { 1 => :b6 }
          },
          :slums => {
            :next_scene => { 1 => :b5 }
          },

          :leave => leave
        }
      },
      :b1 => {
        :text => [
          "the old tower seems mostly intact.",
          "the shell of a burned out car blocks the entrance.",
          "most of the windows at ground level are busted anyway."
        ],
        :options => {

          :enter => {
            :next_scene => { 1 => :c1 }
          },
          :back => {
            :next_scene => { 1 => :c2 }
          },

          :leave => leave
        }
      },
      :b2 => {
        :text => ["a huge lizard scrambles up out of the darkness of an old metro station."],
        :combat => true,
        :enemy => :lizard,
        :damage => 5,
        :hit => 0.8,
        :attack_delay => 2,
        :health => 20,
        :max_health => 20,
        :loot => {
          :eyes => { :min => 1, :max => 2, :chance => 0.5 },
          :flesh => { :min => 1, :max => 3, :chance => 1.0 }
        },
        :options => {
          :continue => {
            :next_scene => { 0.5 => :c2, 1 => :c3 }
          },
          :run => aborted
        }
      },
      :b3 => {
        :text => ["the sniper's shot echoes in the empty street."],
        :combat => true,
        :enemy => :sniper,
        :damage => 60,
        :hit => 0.8,
        :attack_delay => 5,
        :health => 45,
        :max_health => 45,
        :ranged => true,
        :loot => {
          :food => { :min => 1, :max => 5, :chance => 1.0 },
          :bullets => { :min => 1, :max => 5, :chance => 1.0 },
          :rifle => { :min => 1, :max => 1, :chance => 1.0 }
        },
        :options => {
          :continue => {
            :next_scene => { 0.5 => :c4, 1 => :c5 }
          },
          :run => aborted
        }
      },
      :b4 => {
        :text => ["the soldier steps out from between the buildings, rifle raised."],
        :combat => true,
        :enemy => :soldier,
        :ranged => true,
        :damage => 16,
        :hit => 0.8,
        :attack_delay => 2,
        :health => 50,
        :max_health => 50,
        :loot => {
          :food => { :min => 5, :max => 10, :chance => 1.0 },
          :bullets => { :min => 1, :max => 5, :chance => 0.5 },
          :rifle => { :min => 1, :max => 1, :chance => 0.2 }
        },
        :options => {
          :continue => {
            :next_scene => { 0.5 => :c5, 1 => :c6 }
          },
          :run => aborted
        }
      },
      :b5 => {
        :text => ["a frail man stands defiantly, blocking the path."],
        :combat => true,
        :enemy => :frail_man,
        :damage => 1,
        :hit => 0.8,
        :attack_delay => 2,
        :health => 10,
        :max_health => 10,
        :loot => {
          :food => { :min => 1, :max => 5, :chance => 1.0 },
          :knife => { :min => 1, :max => 1, :chance => 0.2 }
        },
        :options => {
          :continue => {
            :next_scene => { 0.5 => :c7, 1 => :c8 }
          },
          :run => aborted
        }
      },
      :b6 => {
        :text => [
          "nothing but downcast eyes.",
          "the war broke the will of these humans."
        ],
        :options => {
          :continue => {
            :next_scene => { 0.5 => :c8, 1 => :c9 }
          },
          :leave => leave
        }
      },
      :c1 => {
        :text => ["a thug is waiting on the other side of the wall."],
        :combat => true,
        :enemy => :thug,
        :damage => 3,
        :hit => 0.8,
        :attack_delay => 2,
        :health => 30,
        :max_health => 30,
        :loot => {
          :steel_sword => { :min => 1, :max => 1, :chance => 0.5 },
          :food => { :min => 1, :max => 3, :chance => 1.0 },
        },
        :options => {
          :continue => {
            :next_scene => { 0.5 => :d1, 1 => :d2 }
          },
          :run => aborted
        }
      },
      :c2 => {
        :text => ["a snarling beast jumps out from behind a car."],
        :combat => true,
        :enemy => :beast,
        :damage => 2,
        :hit => 0.8,
        :attack_delay => 1,
        :health => 30,
        :max_health => 30,
        :loot => {
          :flesh => { :min => 1, :max => 5, :chance => 1.0 },
          :eyes => { :min => 1, :max => 2, :chance => 0.8 },
          :teeth => { :min => 1, :max => 5, :chance => 0.5 }
        },
        :options => {
          :continue => {
            :next_scene => { 1 => :d2 }
          },
          :run => aborted
        }
      },
      :c3 => {
        :text => [
          "street above the subway platform is blown away.",
          "lets some starlight down into the dusty haze.",
          "a sound comes from the tunnel, just ahead."
        ],
        :options => {
          :enter => {
            :next_scene => { 0.5 => :d2, 1 => :d3 },
            :next_scene_cost => { :torch => 1 },
          },
          :leave => leave
        }
      },
      :c4 => {
        :text => [
          "looks like a camp of sorts up ahead.",
          "rusted chainlink is pulled across an alleyway.",
          "fires burn in the courtyard beyond."
        ],
        :options => {
          :enter => {
            :next_scene => { 0.5 => :d4, 1 => :d5 }
          },
          :leave => leave
        }
      },
      :c5 => {
        :text => [
          "more voices can be heard ahead.",
          "they must be here for a reason."
        ],
        :options => {
          :enter => {
            :next_scene => { 1 => :d5 }
          },
          :leave => leave
        }
      },
      :c6 => {
        :text => [
          "the sound of gunfire carries on the wind.",
          "the street ahead glows with firelight."
        ],
        :options => {
          :enter => {
            :next_scene => { 0.5 => :d5, 1 => :d6 }
          },
          :leave => leave
        }
      },
      :c7 => {
        :text => [
          "more squatters are crowding around now.",
          "someone throws a stone in defiance to the wanderer race."
        ],
        :options => {
          :attack => {
            :next_scene => { 0.5 => :d7, 1 => :d8 }
          },
          :leave => leave
        }
      },
      :c8 => {
        :text => [
          "an improvised shop is set up on the sidewalk.",
          "the owner stands by, stoic. the war has numbed his emotions."
        ],
        :loot => {
          :steel_sword => { :min => 1, :max => 1, :chance => 1.0 },
          :rifle => { :min => 1, :max => 1, :chance => 0.5 },
          :bullets => { :min => 1, :max => 8, :chance => 0.25 },
          :alien_alloy => { :min => 1, :max => 1, :chance => 0.5 }
        },
        :options => {
          :continue => {
            :next_scene => { 1 => :d8 }
          },
          :run => aborted
        }
      },
      :c9 => {
        :text => [
          "strips of cured meat hang drying by the side of the street.",
          "the people back away, avoiding eye contact."
        ],
        :loot => {
          :food => { :min => 5, :max => 10, :chance => 1 }
        },
        :options => {
          :continue => {
            :next_scene => { 0.5 => :d8, 1 => :d9 }
          },
          :run => aborted
        }
      },
      :d1 => {
        :text => ["a large bird nests at the top of the stairs."],
        :combat => true,
        :enemy => :bird,
        :damage => 5,
        :hit => 0.7,
        :attack_delay => 1,
        :health => 45,
        :max_health => 45,
        :loot => {
          :flesh => { :min => 1, :max => 3, :chance => 1.0 }
        },
        :options => {
          :continue => {
            :next_scene => { 0.5 => :end1, 1 => :end2 }
          },
          :run => aborted
        }
      },
      :d2 => {
        :text => [
          "the debris is denser here.",
          "maybe some useful stuff in the rubble."
        ],
        :loot => {
          :bullets => { :min => 1, :max => 5, :chance => 0.5 },
          :alien_alloy => { :min => 1, :max => 1, :chance => 0.5 },
        },
        :options => {
          :continue => {
            :next_scene => { 1 => :end2 }
          },
          :run => aborted
        }
      },
      :d3 => {
        :text => ["a swarm of rats rushes up the tunnel."],
        :combat => true,
        :enemy => :rats,
        :plural => true,
        :damage => 1,
        :hit => 0.8,
        :attack_delay => 0.25,
        :health => 60,
        :max_health => 60,
        :loot => {
          :eyes => { :min => 1, :max => 2, :chance => 0.8 },
          :teeth => { :min => 1, :max => 2, :chance => 1.0 }
        },
        :options => {
          :continue => {
            :next_scene => { 0.5 => :end2, 1 => :end3 }
          },
          :run => aborted
        }
      },
      :d4 => {
        :text => ["a large man attacks, drawing a katana."],
        :combat => true,
        :enemy => :veteran,
        :damage => 3,
        :hit => 0.8,
        :attack_delay => 2,
        :health => 45,
        :max_health => 45,
        :loot => {
          :katana => { :min => 1, :max => 1, :chance => 1.0 },
          :food => { :min => 1, :max => 5, :chance => 1.0 }
        },
        :options => {
          :continue => {
            :next_scene => { 0.5 => :end4, 1 => :end5 }
          },
          :run => aborted
        }
      },
      :d5 => {
        :text => ["a second soldier opens fire."],
        :combat => true,
        :enemy => :soldier,
        :ranged => true,
        :damage => 8,
        :hit => 0.8,
        :attack_delay => 2,
        :health => 50,
        :max_health => 50,
        :loot => {
          :food => { :min => 1, :max => 5, :chance => 1.0 },
          :bullets => { :min => 1, :max => 5, :chance => 0.5 },
          :rifle => { :min => 1, :max => 1, :chance => 0.2 }
        },
        :options => {
          :continue => {
            :next_scene => { 1 => :end5 }
          },
          :run => aborted
        }
      },
      :d6 => {
        :text => ["a masked soldier rounds the corner, gun drawn."],
        :combat => true,
        :enemy => :commando,
        :ranged => true,
        :damage => 3,
        :hit => 0.9,
        :attack_delay => 2,
        :health => 55,
        :max_health => 55,
        :loot => {
          :rifle => { :min => 1, :max => 1, :chance => 0.5 },
          :bullets => { :min => 1, :max => 5, :chance => 1.0 },
          :food => { :min => 1, :max => 5, :chance => 0.8 }
        },
        :options => {
          :continue => {
            :next_scene => { 0.5 => :end5, 1 => :end6 }
          },
          :run => aborted
        }
      },
      :d7 => {
        :text => ["the crowd surges forward."],
        :combat => true,
        :enemy => :squatters,
        :plural => true,
        :damage => 2,
        :hit => 0.7,
        :attack_delay => 0.5,
        :health => 40,
        :max_health => 40,
        :loot => {
          :teeth => { :min => 1, :max => 1, :chance => 1.0 },
          :flesh => { :min => 1, :max => 2, :chance => 1.0 },
          :eyes => { :min => 1, :max => 2, :chance => 0.8 }
        },
        :options => {
          :continue => {
            :next_scene => { 0.5 => :end7, 1 => :end8 }
          },
          :run => aborted
        }
      },
      :d8 => {
        :text => ["a young man lashes out."],
        :combat => true,
        :enemy => :youth,
        :damage => 6,
        :hit => 0.7,
        :attack_delay => 1,
        :health => 45,
        :max_health => 45,
        :loot => {
          :teeth => { :min => 1, :max => 2, :chance => 1.0 }
        },
        :options => {
          :continue => {
            :next_scene => { 1 => :end8 }
          },
          :run => aborted
        }
      },
      :d9 => {
        :text => ["a squatter stands firmly in the doorway of a small hut."],
        :combat => true,
        :enemy => :squatter,
        :damage => 3,
        :hit => 0.8,
        :attack_delay => 2,
        :health => 20,
        :max_health => 20,
        :loot => {
          :knife => { :min => 1, :max => 1, :chance => 1.0 },
          :teeth => { :min => 1, :max => 2, :chance => 0.5 }
        },
        :options => {
          :continue => {
            :next_scene => { 0.5 => :end8, 1 => :end9 }
          },
          :run => aborted
        }
      },
      :end1 => {
        :text => [
          "bird must have liked shiney things.",
          "some good stuff woven into its nest."
        ],
        :loot => {
          :bullets => { :min => 5, :max => 10, :chance => 1.0 },
          :bolas => { :min => 2, :max => 6, :chance => 0.8 },
          :alien_alloy => { :min => 1, :max => 1, :chance => 0.5 }
        },
        :options => {
          :leave => clear
        }
      },
      :end2 => {
        :text => [
          "not much here.",
          "scavengers must have gotten to this place already."
        ],
        :loot => {
          :torch => { :min => 1, :max => 5, :chance => 1.0 },
          :food => { :min => 1, :max => 5, :chance => 0.5 }
        },
        :options => {
          :leave => clear
        }
      },
      :end3 => {
        :text => [
          "the tunnel opens up at another platform.",
          "the walls are scorched from the war.",
          "bodies and supplies from both sides litter the ground."
        ],
        :loot => {
          :rifle => { :min => 1, :max => 1, :chance => 0.8 },
          :bullets => { :min => 1, :max => 5, :chance => 1.0 },
          :laser => { :min => 1, :max => 1, :chance => 0.5 },
          :battery => { :min => 1, :max => 5, :chance => 0.8 },
          :alien_alloy => { :min => 1, :max => 1, :chance => 0.5 }
        },
        :options => {
          :leave => clear
        }
      },
      :end4 => {
        :text => [
          "the small military outpost is well supplied.",
          "arms and munitions from the war, are neatly arranged on the store-room floor.",
        ],
        :loot => {
          :rifle => { :min => 1, :max => 1, :chance => 1 },
          :bullets => { :min => 1, :max => 10, :chance => 1 },
          :grenade => { :min => 1, :max => 5, :chance => 0.8 }
        },
        :options => {
          :leave => clear
        }
      },
      :end5 => {
        :text => [
          "searching the bodies yields a few supplies.",
          "more soldiers will be on their way.",
          "time to move on."
        ],
        :loot => {
          :rifle => { :min => 1, :max => 1, :chance => 1 },
          :bullets => { :min => 1, :max => 10, :chance => 1 },
          :food => { :min => 1, :max => 5, :chance => 0.8 }
        },
        :options => {
          :leave => clear
        }
      },
      :end6 => {
        :text => [
          "the small settlement has clearly been burning a while.",
          "the bodies of the wanderers that lived here are still visible in the flames.",
          "still time to rescue a few supplies."
        ],
        :loot => {
          :laser => { :min => 1, :max => 1, :chance => 0.5 },
          :battery => { :min => 1, :max => 5, :chance => 0.5 },
          :food => { :min => 1, :max => 10, :chance => 1 },
          #:radar => { :min => 1, :max => 1, :chance => 1 },
        },
        :options => {
          :leave => clear
        }
      },
      :end7 => {
        :text => [
          "the remaining settlers flee from the violence, their belongings forgotten.",
          "there's not much, but some useful things can still be found."
        ],
        :loot => {
          :steel_sword => { :min => 1, :max => 1, :chance => 0.8 },
          :battery => { :min => 1, :max => 5, :chance => 0.5 },
          :food => { :min => 1, :max => 10, :chance => 1 }
        },
        :options => {
          :leave => clear
        }
      },
      :end8 => {
        :text => [
          "the young settler was carrying a canvas sack.",
          "it contains travelling gear, and a few trinkets.",
          "there's nothing else here."
        ],
        :loot => {
          :steel_sword => { :min => 1, :max => 1, :chance => 0.8 },
          :bolas => { :min => 2, :max => 6, :chance => 0.8 },
          :food => { :min => 1, :max => 10, :chance => 1 }
        },
        :options => {
          :leave => clear
        }
      },
      :end9 => {
        :text => [
          "inside the hut, a child cries.",
          "a few belongings rest against the walls.",
          "there's nothing else here."
        ],
        :loot => {
          :rifle => { :min => 1, :max => 1, :chance => 0.8 },
          :bullets => { :min => 1, :max => 5, :chance => 1.0 },
          :bolas => { :min => 2, :max => 6, :chance => 0.8 },
          :alien_alloy => { :min => 1, :max => 1, :chance => 0.5 }
        },
        :options => {
          :leave => clear
        }
      }
    }
  end
end
