# Copyright 2014 Scratchwork Development LLC. All rights reserved.

class DefectorEvent < WorldEvent
  def initialize game
    @game = game
    @title = "a defector"
    @combat_tick = 0
    super @game.world.stores, @game.world.weapon_stores, @game.world.weapon_items, @game.world.enemy_damage_multiplier
  end

  def action_on_exit
    :kill_in_battle
  end

  def replenishes_water?
    true
  end

  def init_scenes
    replenish_water

    @scenes = {
      :start => {
        :storyline => true,
        :text => [
          ['scars cover his body from the war.'],
          ['allegiance to the humans marked by cauterized wounds.'],
          ['he rushes in at blinding speeds, fueled by an unwavering resolve.'],
        ].sample,
        :options => {
          :engage => {
            :next_scene => { 1 => :engage }
          },
        },
      },
      :engage => {
        :text => ["the predator attacks relentlessly."],
        :combat => true,
        :enemy => :defector,
        :damage => 8,
        :hit => 0.8,
        :attack_delay => 1,
        :health => 0,
        :max_health => 0,
        :loot => {
          :laser => { :min => 1, :max => 1, :chance => 0.2 },
          :battery => { :min => 8, :max => 12, :chance => 0.3 },
          :food => { :min => 1, :max => 5, :chance => 1.0 },
          :eyes => { :min => 1, :max => 2, :chance => 0.4 }
        },
        :options => { :leave => clear }
      }
    }

    min = @game.world.kills_of_defector - 2
    max = @game.world.kills_of_defector
    boost = (min..max).to_a.sample * 5
    boost = 79 if boost > 79
    @scenes[:engage][:health] = 20 + boost
    @scenes[:engage][:max_health] = 20 + boost
  end
end

class ImpossibleDefectorEvent < WorldEvent
  def initialize game
    @game = game
    @title = "a defector"
    @combat_tick = 0
    super @game.world.stores, @game.world.weapon_stores, @game.world.weapon_items, @game.world.enemy_damage_multiplier
  end

  def replenishes_water?
    true
  end

  def action_on_exit
    :kill_in_battle
  end

  def init_scenes
    replenish_water

    @scenes = {
      :start => {
        :storyline => true,
        :text => [
          ['strengthened by occultist artifacts. teeth filed crudely into fangs.'],
          ['driven by rage and resentment of the war. screams a deafening battle cry on approach.'],
          ['occultist artifacts pushing him towards insanity. maddened by power overwhelming.'],
        ].sample,
        :options => {
          :engage => {
            :next_scene => { 1 => :engage }
          },
        },
      },
      :engage => {
        :text => ["the predator attacks relentlessly."],
        :combat => true,
        :enemy => :defector,
        :damage => 8,
        :hit => 0.8,
        :attack_delay => 1,
        :health => 666,
        :max_health => 666,
        :loot => {
          :laser => { :min => 1, :max => 1, :chance => 1.0 },
          :battery => { :min => 8, :max => 12, :chance => 1.0 },
          :food => { :min => 5, :max => 10, :chance => 1.0 },
          :eyes => { :min => 1, :max => 2, :chance => 0.4 }
        },
        :options => {
          :continue => { :next_scene => { 1 => :bodies } }
        }
      },
      :bodies => {
        :storyline => true,
        :text => ["never forget what it's like to accomplish the impossible."],
        :options => {
          :continue => clear
        }
      }
    }
  end
end
