# Copyright 2014 Scratchwork Development LLC. All rights reserved.

class ManAndBoyEvent < WorldEvent
  def initialize game
    @game = game
    @title = "a house"
    @combat_tick = 0
    super @game.world.stores, @game.world.weapon_stores, @game.world.weapon_items, @game.world.enemy_damage_multiplier
  end

  def replenishes_water?
    true
  end

  def thoughts_message
    "nobody wants to be here and nobody wants to leave."
  end

  def action_on_exit
    :aborted
  end

  def init_scenes
    @scenes = {
      :start => {
        :text => [
          'an old house remains here, once white siding yellowed and peeling.',
          'the door hangs open.'
        ],
        :options => {
          :go_inside => {
            :next_scene => { 1 => :man_and_boy }
          },
          :leave => aborted
        },
      },
      :man_and_boy => {
        :storyline => true,
        :title => "a man and boy",
        :text => [
          "eyes filled with contempt, a man and boy rise to their feet. they move slowly back from their supplies."
        ],
        :options => {
          :take_supplies => {
            :next_scene => { 1 => :take_supplies }
          },
          :leave => aborted
        },
      },
      :take_supplies => {
        :storyline => true,
        :text => [
          "the boy clenches his fists at his side, white knuckled."
        ],
        :loot => {
          :food => { :min => 7, :max => 10, :chance => 1.0 },
          :torch => { :min => 1, :max => 3, :chance => 1.0 },
        },
        :options => {
          :continue => {
            :next_scene => { 1 => :continue }
          }
        },
      },
      :continue => {
        :storyline => true,
        :text => [
          "a blade hangs at the man's side. loose rags cover his face, stained a ruddy brown."
        ],
        :options => {
          :engage => {
            :next_scene => { 1 => :engage }
          },
          :leave => clear
        },
      },
      :engage => {
        :text => ["calmly, the man draws his blade. this isn't his first battle."],
        :combat => true,
        :enemy => :squatter,
        :damage => 8,
        :hit => 0.8,
        :attack_delay => 2,
        :health => 40,
        :max_health => 40,
        :loot => {
          :katana => { :min => 1, :max => 1, :chance => 1.0 },
        },
        :options => {
          :continue => { :next_scene => { 1 => :boy } }
        }
      },
      :boy => {
        :text => ['the boy attacks, blinded by rage.'],
        :combat => true,
        :enemy => :squatter,
        :damage => 2,
        :hit => 0.8,
        :attack_delay => 1,
        :health => 20,
        :max_health => 20,
        :loot => {
          :knife => { :min => 1, :max => 1, :chance => 1.0 },
        },
        :options => {
          :continue => { :next_scene => { 1 => :revolver } }
        }
      },
      :revolver => {
        :storyline => true,
        :text => ['a rucksack on the corpse. inside, a revolver with two bullets.'],
        :loot => {
          :rifle => { :min => 1, :max => 1, :chance => 1.0 },
          :bullets => { :min => 2, :max => 2, :chance => 1.0 },
        },
        :options => {
          :continue => clear
        }
      }
    }
  end
end
