# Copyright 2014 Scratchwork Development LLC. All rights reserved.

class MansionEvent < WorldEvent
  def initialize game
    @game = game
    @title = "an old house"
    @combat_tick = 0
    @weapon_loot = []
    @loot = { :food => 10 }
    super @game.world.stores, @game.world.weapon_stores, @game.world.weapon_items, @game.world.enemy_damage_multiplier
  end

  def action_on_exit
    :aborted
  end

  def replenishes_water?
    true
  end

  def thoughts_message
    "feral howls fill the air."
  end

  def init_scenes
    replenish_water

    @scenes = {
      :start => {
        :text => [
          'an old house remains here, once white siding yellowed and peeling.',
          'the door hangs open.'
        ],
        :options => {
          :go_inside => { :next_scene => :foyer },
          :leave => aborted
        }
      },
      :foyer => {
        :storyline => true,
        :title => "foyer",
        :text => [
          'the circular foyer presents two large rooms. dirt covers its marble floors.',
        ],
        :options => {
          :study => { :next_scene => :study },
          :formal => { :next_scene => :formal },
          :leave => aborted
        }
      },
      :formal => {
        :storyline => true,
        :title => "formal dining",
        :text => [
          "a broken dining table. claw marks etched deeply into the aged wood. a portrait of an elderly man lies on the ground, surrounded by shattered glass."
        ],
        :options => {
          :foyer => { :next_scene => :foyer },
          :conservatory => { :next_scene => :conservatory },
          :parlour => { :next_scene => :parlour },
        }
      },
      :study => {
        :storyline => true,
        :title => "study",
        :text => [
          'a ransacked desk, turned on its side. oil paintings decorate the walls, depicting battles between wanderers and humans.',
        ],
        :options => {
          :foyer => { :next_scene => :foyer },
          :parlour => { :next_scene => :parlour },
          :bedroom => { :next_scene => :bedroom },
        }
      },
      :conservatory => {
        :storyline => true,
        :title => "conservatory",
        :text => [
          'the glass room is in immaculate condition. pleiades and orion shimmer in the eternal night.',
        ],
        :options => {
          :formal => { :next_scene => :formal },
        }
      },
      :parlour => {
        :storyline => true,
        :title => "living quarters",
        :text => [
          'everything has been shredded. a basement door lies in the middle of the room, splintered and broken.',
        ],
        :options => {
          :study => { :next_scene => :study },
          :formal => { :next_scene => :formal },
          :basement => { :next_scene => :basement },
          :kitchen => { :next_scene => :kitchen },
        }
      },
      :basement => {
        :storyline => true,
        :title => "basement",
        :text => [
          'a dimly lit room. bones of wanderers strewn about, picked clean.',
        ],
        :options => {
          :parlour => { :next_scene => :parlour },
        }
      },
      :kitchen => {
        :storyline => true,
        :title => "kitchen",
        :text => [
          'a broken window leading out. fresh blood paints the floor.',
        ],
        :options => {
          :larder => { :next_scene => :larder },
          :parlour => { :next_scene => :parlour },
          :bedroom => { :next_scene => :bedroom },
          :leave => clear,
        }
      },
      :bedroom => {
        :storyline => true,
        :title => "bedroom",
        :text => [
          'an aristocrat in a wheelchair, turned into a meal. ragged flesh hangs off of blood stained bones.',
        ],
        :options => {
          :study => { :next_scene => :study },
          :kitchen => { :next_scene => :kitchen },
        }
      },
      :larder => {
        :text => [
          "a cool larder with tilted shelves.",
        ],
        :loot => { },
        :options => {
          :continue => {
            :next_scene => { 1 => :kitchen }
          }
        },
      }
    }
  end

  def loot_sample
    @loot
  end

  def weapon_sample
    @weapon_loot
  end
end
