# Copyright 2014 Scratchwork Development LLC. All rights reserved.

class MazeEvent < WorldEvent
  def initialize game
    @game = game
    @title = "a damp cave"
    @combat_tick = 0
    super @game.world.stores, @game.world.weapon_stores, @game.world.weapon_items
  end

  def action_on_exit
    :aborted
  end

  def becomes_outpost?
    true
  end

  def init_scenes
    @scenes = {
      :start => {
        :text => [ "the mouth of the cave is wide and dark.", "can't see what's inside." ],
        :options => {
          :go_inside => { :next_scene => :go_inside },
          :leave => leave
        }
      }
    }

    @scenes[:entrance] = {
      :text => [ "a dark labyrinth awaits." ],
      :options => {
        :leave => leave
      }
    }

    generate_left_path @scenes, :entrance
  end

  def generate_left_path scenes, key
    id = GTK.create_uuid
    scenes[key][:options][:left_path] = { :next_scene => id }
    scenes[id] = { :text => [ "" ], }
    scenes[id][:options] = { }
    scenes[id][:options][:back] = { :next_scene => key }
  end
end


#      :go_inside => {
#        :cost => { :torch => 1 },
#        :text => [ "the earth here is split, as if bearing an ancient wound." ],
#        :options => {
#          :left_path => {
#            :next_scene => { 1 => :a1 }
#          },
#          :middle_path => {
#            :next_scene => { 1 => :a2 }
#          },
#          :right_path => {
#            :next_scene => { 1 => :a3 }
#          },
#        }
#      },
#      :a1 => {
#        :text => [ "a startled beast defends its home." ],
#        :combat => true,
#        :enemy => :beast,
#        :damage => 1,
#        :health => 5,
#        :max_health => 5,
#        :attack_delay => 1,
#        :hit => 0.8,
#        :loot => {
#          :eyes => { :min => 1, :max => 2, :chance => 0.8 },
#          :teeth => { :min => 1, :max => 5, :chance => 1.0 },
#        },
#        :options => {
#          :continue => {
#            :next_scene => { 0.5 => :b1, 1.0 => :b2 }
#          },
#          :run => aborted
#        }
#      },
#      :a2 => {
#				:text => [ 'the cave narrows a few feet in.', "the walls are moist and moss-covered." ],
#
#        :options => {
#          :left_path => {
#            :next_scene => { 1 => :b2 }
#          },
#          :right_path => {
#            :next_scene => { 1 => :b3 }
#          },
#          :leave => leave
#        },
#			},
#			:a3 => {
#				:text => [
#          'the remains of an old camp sits just inside the cave.',
#          'bedrolls, torn and blackened, lay beneath a thin layer of dust.',
#          'a wanderer stayed here in hiding.'
#        ],
#				:loot => {
#					:food => { :min => 1, :max => 5, :chance => 1 },
#					:torch => { :min => 1, :max => 5, :chance => 1 },
#				},
#
#				:options => {
#          :continue => {
#            :next_scene => { 0.5 => :b3, 1.0 => :b4 }
#          },
#          :run => aborted
#				}
#			},
#			:b1 => {
#				:text => [
#          'the body of a wanderer lies in a small cavern.',
#          "rot's been to work on it, and some of the pieces are missing.",
#          "can't tell what left it here."
#        ],
#				:loot => {
#					:iron_sword => { :min => 1, :max => 1, :chance => 1 },
#					:food => { :min => 1, :max => 5, :chance => 0.8 },
#					:torch => { :min => 1, :max => 3, :chance => 0.5 }
#				},
#				:options => {
#					:continue => {
#						:next_scene => { 1 => :b2 }
#					},
#          :run => aborted
#				}
#			},
#			:b2 => {
#				:text => [ 'the torch sputters and dies in the damp air.', 'the darkness is absolute.' ],
#				:options => {
#					:light => {
#						:next_scene => { 1 => :light }
#					},
#          :leave => leave
#				}
#			},
#      :b3 => {
#        :text => [ "a startled beast defends its home." ],
#        :combat => true,
#        :enemy => :beast,
#        :damage => 1,
#        :health => 5,
#        :max_health => 5,
#        :attack_delay => 1,
#        :hit => 0.8,
#        :loot => {
#          :eyes => { :min => 1, :max => 2, :chance => 0.8 },
#          :teeth => { :min => 1, :max => 2, :chance => 1.0 },
#        },
#        :options => {
#          :continue => {
#            :next_scene => { 1 => :c2 }
#          },
#          :run => aborted
#        }
#      },
#      :b4 => {
#        :text => [ "a cave lizard attacks." ],
#        :combat => true,
#        :enemy => :cave_lizard,
#        :damage => 3,
#        :health => 6,
#        :max_health => 6,
#        :attack_delay => 2,
#        :hit => 0.8,
#        :loot => {
#          :flesh => { :min => 1, :max => 3, :chance => 1.0 },
#          :eyes => { :min => 1, :max => 2, :chance => 0.8 },
#        },
#        :options => {
#          :continue => {
#            :next_scene => { 1 => :c2 }
#          },
#          :run => aborted
#        }
#      },
#			:light => {
#        :cost => { :torch => 1 },
#        :text => [ "a startled beast defends its home." ],
#				:combat => true,
#				:enemy => :beast,
#				:damage => 3,
#				:hit => 0.8,
#				:attack_delay => 2,
#				:health => 10,
#				:max_health => 10,
#				:loot => {
#					:eyes => { :min => 1, :max => 2, :chance => 0.8 },
#					:teeth => { :min => 1, :max => 3, :chance => 1 }
#				},
#				:options => {
#          :continue => {
#            :next_scene => { 0.5 => :end1, 1.0 => :end2 }
#          },
#          :run => aborted
#				}
#			},
#			:c2 => {
#        :text => [ "a cave lizard attacks." ],
#				:combat => true,
#				:enemy => :lizard,
#				:damage => 4,
#				:hit => 0.8,
#				:attack_delay => 2,
#				:health => 10,
#				:max_health => 10,
#				:loot => {
#					:flesh => { :min => 1, :max => 3, :chance => 1 },
#					:eyes => { :min => 1, :max => 2, :chance => 0.8 }
#				},
#				:options => {
#          :continue => {
#            :next_scene => { 0.7 => :end2, 1.0 => :end3 }
#          },
#          :run => aborted
#				}
#			},
#			:end1 => {
#				:text => [ 'the nest of a large animal lies at the back of the cave.' ],
#				:loot => {
#          :iron_sword => { :min => 1, :max => 1, :chance => 1 },
#          :bolas => { :min => 1, :max => 3, :chance => 1.0 }
#				},
#        :options => { :leave => clear }
#			},
#			:end2 => {
#				:text => [ 'an abandoned supply cache. must have been forgotten about during the conflict.' ],
#        :loot => {
#					:torch => { :min => 1, :max => 2, :chance => 1 },
#					:iron_sword => { :min => 1, :max => 1, :chance => 0.3 },
#					:food => { :min => 5, :max => 10, :chance => 1 },
#					:steel_sword => { :min => 1, :max => 1, :chance => 0.3 },
#					:bolas => { :min => 1, :max => 3, :chance => 1.0 }
#				},
#        :options => { :leave => clear }
#			},
#			:end3 => {
#				:text => [ 'an old case is wedged behind a rock, covered in a thick layer of dust.' ],
#        :loot => {
#          :steel_sword => { :min => 1, :max => 1, :chance => 1 },
#          :bolas => { :min => 1, :max => 3, :chance => 1.0 }
#        },
#        :options => { :leave => clear }
#			}
# end
