# Copyright 2014 Scratchwork Development LLC. All rights reserved.

class RitualEvent < WorldEvent
  def initialize game
    @game = game
    @title = "a house"
    @combat_tick = 0
    super @game.world.stores, @game.world.weapon_stores, @game.world.weapon_items, @game.world.enemy_damage_multiplier
  end

  def replenishes_water?
    true
  end

  def action_on_exit
    :clear
  end

  def init_scenes
    @scenes = {
      :start => {
        :text => [
          'an old house remains here, once white siding yellowed and peeling.',
          'the door hangs open.'
        ],
        :options => {
          :go_inside => {
            :next_scene => { 1 => :man_and_wanderer }
          },
          :leave => clear
        },
      },
      :man_and_wanderer => {
        :storyline => true,
        :title => "a ritual",
        :text => [
          "a man stands over a kneeling wanderer, wielding a ceremonial blade and a scalding hot iron.",
        ],
        :options => {
          :observe => {
            :next_scene => { 1 => :observe }
          }
        },
      },
      :observe => {
        :storyline => true,
        :text => [
          "the man swiftly severs the wanderer's limbs, save two.",
          "carefully cauterizing each wound."
        ],
        :options => {
          :continue => {
            :next_scene => { 1 => :continue }
          }
        },
      },
      :continue => {
        :storyline => true,
        :text => [
          "the human and the newly minted defector stand, ready to fight."
        ],
        :options => {
          :engage => {
            :next_scene => { 1 => :engage }
          },
          :back_away => {
            :next_scene => { 1 => :back_away }
          },
        },
      },
      :back_away  => {
        :storyline => true,
        :text => [
          "the human extends his hand. presents the iron, covered in coagulated blood."
        ],
        :loot => {
          :cauter => { :min => 1, :max => 1, :chance => 1.0 }
        },
        :options => {
          :continue => clear
        },
      },
      :engage => {
        :text => ["they attack, spirit unwavering."],
        :combat => true,
        :enemy => :squatter_and_defector,
        :damage => 16,
        :hit => 0.8,
        :attack_delay => 1,
        :health => 80,
        :max_health => 80,
        :loot => {
          :cauter => { :min => 1, :max => 1, :chance => 1.0 },
          :katana => { :min => 1, :max => 1, :chance => 1.0 },
          :laser => { :min => 1, :max => 1, :chance => 1.0 },
          :battery => { :min => 10, :max => 10, :chance => 1.0 },
          :food => { :min => 10, :max => 20, :chance => 1.0 }
        },
        :options => {
          :continue => { :next_scene => { 1 => :bodies } }
        }
      },
      :bodies => {
        :storyline => true,
        :text => ["the admiral spoke the truth, the war is lost. no way to sustain the onslaught of their combined numbers."],
        :options => {
          :continue => clear
        }
      }
    }
  end
end
