# Copyright 2014 Scratchwork Development LLC. All rights reserved.

class ShipDefectorEvent < WorldEvent
  def initialize game
    @game = game
    @title = "a defector"
    @combat_tick = 0
    super @game.world.stores, @game.world.weapon_stores, @game.world.weapon_items, @game.world.enemy_damage_multiplier
  end

  def action_on_exit
    :kill_in_battle
  end

  def replenishes_water?
    true
  end

  def init_scenes
    replenish_water

    @scenes = {
      :start => {
        :storyline => true,
        :text => [
          ['scars cover his body from the war.'],
          ['allegiance to the humans marked by cauterized wounds.'],
          ['he rushes in at blinding speeds, fueled by an unwavering resolve.'],
        ].sample,
        :options => {
          :engage => {
            :next_scene => { 1 => :engage }
          },
        },
      },
      :engage => {
        :text => ["the predator attacks relentlessly."],
        :combat => true,
        :enemy => :defector,
        :damage => 8,
        :hit => 0.8,
        :attack_delay => 1,
        :health => 0,
        :max_health => 0,
        :loot => {},
        :options => { :leave => leave}
      }
    }

    @scenes[:engage][:health] = 79
    @scenes[:engage][:max_health] = 79
  end
end

class ShipDefectorRoomEvent < WorldEvent
  def initialize game
    @game = game
    if $mini_world.end_game_sequence != true
      @title = "blast doors"
    else
      @title = "lower level"
    end
    @combat_tick = 0
    super @game.world.stores, @game.world.weapon_stores, @game.world.weapon_items, @game.world.enemy_damage_multiplier
  end

  def action_on_exit
    :kill_in_battle
  end

  def replenishes_water?
    true
  end

  def init_scenes
    replenish_water

    if $mini_world.end_game_sequence != true
      @scenes = {
        :start => {
          :storyline => true,
          :text => [
            ['the blast doors leading to the lower floor are sealed shut.'],
          ],
          :options => {
            leave: leave
          },
        }
      }
    else
      @scenes = {
        :start => {
          :storyline => true,
          :text => [
            ['a lone defector stands resolute. stares hatefully.'],
          ],
          :options => {
            :engage => {
              :next_scene => { 1 => :engage }
            },
          },
        },
        :engage => {
          :text => ["the predator attacks relentlessly."],
          :combat => true,
          :enemy => :defector,
          :damage => 12,
          :hit => 0.9,
          :attack_delay => 1,
          :health => 199,
          :max_health => 199,
          :loot => {},
          :options => { :leave => leave}
        }
      }

      @scenes[:engage][:health] = 199
      @scenes[:engage][:max_health] = 199
    end
  end
end
