# Copyright 2014 Scratchwork Development LLC. All rights reserved.

class ShipEvent < WorldEvent
  def initialize game
    @game = game
    @title = "a ship"
    @combat_tick = 0
    super @game.world.stores, @game.world.weapon_stores, @game.world.weapon_items, @game.world.enemy_damage_multiplier
    @end_game_sequence = false
    @end_game_ticks = 0
    @reinforced_defector_killed = false
    @time_remaining_message = ''
    @prev_scene = :start
    @loot = {}
    @weapon_loot = []
    @artifact_worn = false

    @game.save
  end

  def on_clear
    @game.world.event = nil
  end

  def replenishes_water?
    true
  end

  def thoughts_message
    "feral howls fill the air."
  end

  def init_scenes
    replenish_water

    @scenes = {
      start: {
        text: ['south east wing'],
        options: {
          head_to_north_east_wing: :se3a,
          head_to_observatory: :se1a,
          head_to_launch_controls: :se4a,
          head_to_blast_doors: :se2a,
        }
      },
      se1a: {
        text: ['ship corridor'],
        options: {
          continue_to_observatory: :observatory,
          go_back: :start
        }
      },
      se1b: {
        text: ['ship corridor'],
        options: {
          continue: :start,
          go_back: :observatory
        }
      },
      observatory: {
        text: ['observatory'],
        options: {
          observe_history: :observe,
          head_to_south_east_wing: :se1b,
        }
      },
      observe: {
        text: ['the annals of history, through the eyes of the great ones'],
        options: {
          observe_history: :observe2,
          back_to_observatory: :observatory
        }
      },
      observe2: {
        text: ['knowledge unlimited'],
        options: {
          back: :observe
        }
      },
      se2a: {
        text: ['ship corridor'],
        options: {
          continue_to_blast_doors: :blast_doors,
          go_back: :start
        }
      },
      se2b: {
        text: ['ship corridor'],
        options: {
          continue: :start,
          go_back: :blast_doors
        }
      },
      blast_doors: {
        text: ['blast doors'],
        options: {
          head_to_south_east_wing: :se2b,
          open_doors: :blast_doors2,
        }
      },
      blast_doors2: {
        text: ['the blast doors slowly open'],
        options: {
          leave_ship: leave,
          stay: :blast_doors
        }
      },
      se3a: {
        text: ['eastern ship corridor'],
        options: {
          continue_to_north_east_wing: :e1a,
          go_back: :start
        }
      },
      e1a: {
        text: ['eastern ship corridor'],
        options: {
          continue_to_north_east_wing: :ne2a,
          go_back: :se3a
        }
      },
      ne2a: {
        text: ['eastern ship corridor'],
        options: {
          continue_to_north_east_wing: :north_east_wing,
          go_back: :e1a
        }
      },
      se3b: {
        text: ['eastern ship corridor'],
        options: {
          continue_to_south_east_wing: :start,
          go_back: :e1b
        }
      },
      e1b: {
        text: ['eastern ship corridor'],
        options: {
          continue_to_south_east_wing: :se3b,
          go_back: :ne2b
        }
      },
      ne2b: {
        text: ['eastern ship corridor'],
        options: {
          continue_to_south_east_wing: :e1b,
          go_back: :north_east_wing
        }
      },
      north_east_wing: {
        text: ['north east wing'],
        options: {
          head_to_north_west_wing: :ne4a,
          head_to_south_east_wing: :ne2b,
          head_to_medical_bay: :ne1a,
          "head to builder's room" => :ne3a
        }
      },
      ne1a: {
        text: ['ship corridor'],
        options: {
          continue_to_medical_bay: :medical_bay,
          go_back: :north_east_wing
        }
      },
      ne1b: {
        text: ['ship corridor'],
        options: {
          continue: :north_east_wing,
          go_back: :medical_bay
        }
      },
      medical_bay: {
        text: ['medical bay'],
        options: {
          recover_health: :medical_bay2,
          head_to_north_east_wing: :ne1b,
        }
      },
      medical_bay2: {
        text: ['flesh is sutured, bones are unbroken'],
        options: {
          go_back: :medical_bay
        }
      },
      ne3a: {
        text: ['ship corridor'],
        options: {
          "continue to builder's room" => :builder_room,
          go_back: :north_east_wing
        }
      },
      ne3b: {
        text: ['ship corridor'],
        options: {
          continue: :north_east_wing,
          go_back: :builder_room
        }
      },
      builder_room: {
        text: ['strange markings cover the walls.',
               'a rose, etched in the middle.'],
        options: {
          #TODO: flush out builder room (button? to save yourself)
          touch_rose: :pickup,
          head_to_north_east_wing: :ne3b
        }
      },
      pickup:{
        text: ['the rose is cold to the touch.',
               'nothing happens.'],
        options: {
          step_back: :builder_room
        }
      },
      wear:{
        text: [
          #'strange markings cover the walls.',
          #    "\na rose, etched in the middle.\n",
          #    'it glows brightly as you approach it.',
          'the rose feels warm and comforting.',
          "\na calm aura emanating.\n",
          'the aura provides protection against all harm.'
        ],
        options:{
          #step_back: :builder_room
          step_back: leave
        }
      },
      ne4a: {
        text: ['northern ship corridor'],
        options: {
          continue_to_north_west_wing: :n1a,
          go_back: :north_east_wing
        }
      },
      n1a: {
        text: ['northern ship corridor'],
        options: {
          continue_to_north_west_wing: :nw1a,
          go_back: :ne4a
        }
      },
      nw1a: {
        text: ['northern ship corridor'],
        options: {
          continue_to_north_west_wing: :north_west_wing,
          go_back: :n1a
        }
      },
      nw1b: {
        text: ['northern ship corridor'],
        options: {
          continue_to_north_east_wing: :n1b,
          go_back: :north_west_wing
        }
      },
      n1b: {
        text: ['northern ship corridor'],
        options: {
          continue_to_north_east_wing: :ne4b,
          go_back: :nw1b
        }
      },
      ne4b: {
        text: ['northern ship corridor'],
        options: {
          continue_to_north_east_wing: :north_east_wing,
          go_back: :n1b
        }
      },
      north_west_wing: {
        text: ['north west wing'],
        options: {
          head_to_north_east_wing: :nw1b,
          head_to_south_west_wing: :nw2b,
          head_to_reinforced_blast_doors: :nw3a,
          head_to_shield_bay: :nw4a
        }
      },
      nw3a: {
        text: ['ship corridor'],
        options: {
          continue_to_reinforced_blast_doors: :reinforced_blast_doors,
          go_back: :north_west_wing
        }
      },
      nw3b: {
        text: ['ship corridor'],
        options: {
          continue: :north_west_wing,
          go_back: :reinforced_blast_doors
        }
      },
      :reinforced_blast_doors => {
        :text => ['reinforced blast doors'],
        :options => {
          :open_doors => :reinforced_blast_doors2,
          :head_to_north_west_wing => :nw3b
        }
      },
      reinforced_blast_doors2: {
        text: ["the doors won't open. a dark silhouette is seen on the other side."],
        options: {
          go_back: :reinforced_blast_doors
        }
      },
      reinforced_blast_doors3: {
        text: ["the doors open slowly. a battered defector emerges from the room. no where to run. prepare yourself."],
        options: {
          fight: :reinforced_defector
        }
      },
      reinforced_defector: {
        :text => ['a scarred defector'],
        :combat => true,
        :enemy => :scarred_defector,
        :damage => 0,
        :damage => 14,
        :hit => 0.8,
        :attack_delay => 1,
        #:health => [65, 66, 67, 68, 69].sample,
        :health => 1,
        :max_health => 179,
        :loot => {},
        :options => {
          :continue => {:next_scene => { 1 => :reinforced_blast_doors}}
        },
        :stunned => false,
        :stun_countdown => 0
      },
      nw4a: {
        text: ['ship corridor'],
        options: {
          continue_to_shield_bay: :shield_bay,
          go_back: :north_west_wing
        }
      },
      nw4b: {
        text: ['ship corridor'],
        options: {
          continue: :north_west_wing,
          go_back: :shield_bay
        }
      },
      shield_bay: {
        text: ['shield bay'],
        options: {
          upgrade_shields: :upgrade_shields,
          head_to_north_west_wing: :nw4b
        }
      },
      upgrade_shields: {
        text: [
          'show the current percentage of shield upgrades'
        ],
        loot: { },
        white_listed_loot: [:alien_alloy],
        loot_title: 'the hull',
        loot_drop_command: 'install',
        loot_take_command: 'remove',
        loot_action: '...',
        continue_command: 'back',
        white_listed_weapons: [:alien_alloy],
        :options => {
          :leave => :shield_bay
        }
      },
      nw2b: {
        text: ['western ship corridor'],
        options: {
          continue_to_south_west_wing: :w1b,
          go_back: :north_west_wing
        }
      },
      w1b: {
        text: ['western ship corridor'],
        options: {
          continue_to_south_west_wing: :sw3b,
          go_back: :nw2b
        }
      },
      sw3b: {
        text: ['western ship corridor'],
        options: {
          continue_to_south_west_wing: :south_west_wing,
          go_back: :w1b
        }
      },
      nw2a: {
        text: ['western ship corridor'],
        options: {
          continue_to_north_west_wing: :north_west_wing,
          go_back: :w1a
        }
      },
      w1a: {
        text: ['western ship corridor'],
        options: {
          continue_to_north_west_wing: :nw2a,
          go_back: :sw3a
        }
      },
      sw3a: {
        text: ['western ship corridor'],
        options: {
          continue_to_north_west_wing: :w1a,
          go_back: :south_west_wing
        }
      },
      south_west_wing: {
        text: ['south west wing'],
        options: {
          head_to_north_west_wing: :sw3a,
          head_to_engine_room: :sw1a,
          head_to_launch_controls: :sw4a,
          "head to admiral's quarters" => :sw2a
        }
      },
      sw1a: {
        text: ['ship corridor'],
        options: {
          continue_to_engine_room: :engine_room,
          go_back: :south_west_wing
        }
      },
      sw1b: {
        text: ['ship corridor'],
        options: {
          continue: :south_west_wing,
          go_back: :engine_room
        }
      },
      engine_room: {
        text: ['engine room'],
        options: {
          upgrade_thrusters: :upgrade_thrusters,
          head_to_south_west_wing: :sw1b
        }
      },
      upgrade_thrusters: {
        text: [
          'show the current percentage of thruster upgrades'
        ],
        loot: { },
        white_listed_loot: [:alien_alloy],
        loot_title: 'the engine',
        loot_drop_command: 'install',
        loot_take_command: 'remove',
        loot_action: '...',
        continue_command: 'back',
        white_listed_weapons: [:alien_alloy],
        :options => {
          :leave => :engine_room
        }
      },
      sw2a: {
        text: ['ship corridor'],
        options: {
          "continue to admiral's quarters" => :admirals_quarters,
          go_back: :south_west_wing
        }
      },
      sw2b: {
        text: ['ship corridor'],
        options: {
          continue: :south_west_wing,
          go_back: :admirals_quarters
        }
      },
      admirals_quarters: {
        text: ["admiral's quarters"],
        options: {
          enter_room: :admirals_quarters2,
          head_to_south_west_wing: :sw2b
        }
      },
      :admirals_quarters2 => {
        :text => ['a cold presence chills to the bone.',
                  'a chest found on the floor.'],
        :options => {
          #:open_chest => {:next_scene => { 1 => :unlock_chest}},
          :open_chest => {:next_scene => { 1 => :loot_chest}},
          :go_back => :admirals_quarters
        }
      },
      :unlock_chest => {
        #:cost => { :admiral_key => 1},
        :text => [
          #'the chest. radianting with energy. contents unparalleled.'
          'armour and weapons are found.'
        ],
        :options => {
          :open_chest => {:next_scene => { 1 => :loot_chest}}
        }
      },
      :loot_chest => {
        :text => [
          #"ammunition and weapons are found in the chest.\n",
          #"underneath lies two special weapons, \nMurasame and Masamune."
          'armour and weapons are found.'
        ],
        :loot => {
          #:food => { :min => 5, :max => 10, :chance => 1},
          #:bullet => { :min => 25, :max => 50, :chance => 1},
          #:battery => { :min => 15, :max => 25, :chance => 1},
          #:grenade => { :min => 5, :max => 10, :chance => 1},
          #:bolas => { :min => 10, :max => 15, :chance => 1},
          #:knife => { :min => 1, :max => 1, :chance => 1},
          #:iron_sword => { :min => 1, :max => 1, :chance => 1},
          #:steel_sword => { :min => 1, :max => 1, :chance => 1},
          #:katana => { :min => 1, :max => 1, :chance => 1},
          #:rifle  => { :min => 1, :max => 1, :chance => 1},
          #:laser  => { :min => 1, :max => 1, :chance => 1}
        },
        loot_title: "admiral's chest",
        loot_action: 'loot chest',
        continue_command: 'back',
        :options => {
          :leave => :admirals_quarters
        }
      },
      sw4a: {
        text: ['southern ship corridor'],
        options: {
          continue_to_launch_controls: :launch_controls,
          go_back: :south_west_wing
        }
      },
      sw4b: {
        text: ['southern ship corridor'],
        options: {
          continue_to_south_west_wing: :south_west_wing,
          go_back: :launch_controls
        }
      },
      launch_controls: {
        text: ['launch controls'],
        options: {
          initiate_lift_off: :launch,
          head_to_south_west_wing: :sw4b,
          head_to_south_east_wing: :se4b
        }
      },
      launch: {
        text: ['commencing liftoff'],
        options: {
          back: :launch_controls,
          #fight: :defector_battle, #testing
          #loot: :loot_body #testing
        },
      },
      se4a: {
        text: ['southern ship corridor'],
        options: {
          continue_to_launch_controls: :launch_controls,
          go_back: :start
        }
      },
      se4b: {
        text: ['southern ship corridor'],
        options: {
          continue_to_south_east_wing: :start,
          go_back: :launch_controls
        }
      },
      defector_battle: {
        text: ['a defector'],
        :combat => true,
        :enemy => :defector,
        :damage => 9,
        #:damage => 0,
        :hit => 0.8,
        :attack_delay => 1,
        #:health => [65, 66, 67, 68, 69].sample,
        :health => 1,
        :max_health => 69,
        :loot => {},
        options: {
          continue: :start
        },
        :stunned => false,
        :stun_countdown => 0
      },
      :loot_body => {
        :text => ['an empty soul'],
        :loot => {
          :food => { :min => 2, :max => 6, :chance => 1 }
        },
        :options => {
          :continue => { :next_scene => { 1 => :launch_controls} }
        }
      }
    }

  end

  def loot_sample
    if @current_scene == :upgrade_thrusters
      return @game.world.thruster_stores
    elsif @current_scene == :upgrade_shields
      return @game.world.shield_stores
    elsif @current_scene == :loot_chest
      return @game.world.admiral_stores
    else
      @loot
    end
  end

  def weapon_sample
    if @current_scene == :upgrade_thrusters
      return @game.world.thruster_weapon_stores
    elsif @current_scene == :upgrade_shields
      return @game.world.shield_weapon_stores
    elsif @current_scene == :loot_chest
      return @game.world.admiral_weapon_stores
    else
      @weapon_loot
    end
  end

  def time_remaining
    if @end_game_sequence
      if @end_game_ticks < 1
        @time_remaining_message = 'liftoff sequence engaged.'
      elsif @end_game_ticks < 2
        @time_remaining_message = 'an explosion rocks the ship. defectors swarm from all ends.'
      elsif @end_game_ticks < 4
        @time_remaining_message = 'critical malfunction! self destruct sequence initated in ...'
      elsif @end_game_ticks < 6
        @time_remaining_message = 'self destruct sequence initated in 10..'
      elsif @end_game_ticks < 8
        @time_remaining_message = 'self destruct sequence initated in 9..'
      elsif @end_game_ticks < 10
        @time_remaining_message = 'self destruct sequence initated in 8..'
      elsif @end_game_ticks < 12
        @time_remaining_message = 'self destruct sequence initated in 7..'
      elsif @end_game_ticks < 14
        @time_remaining_message = 'self destruct sequence initated in 6..'
      elsif @end_game_ticks < 16
        @time_remaining_message = 'self destruct sequence initated in 5..'
      elsif @end_game_ticks < 18
        @time_remaining_message = 'self destruct sequence initated in 4..'
      elsif @end_game_ticks < 20
        @time_remaining_message = 'self destruct sequence initated in 3..'
      elsif @end_game_ticks < 22
        @time_remaining_message = 'self destruct sequence initated in 2..'
      elsif @end_game_ticks < 24
        @time_remaining_message = 'self destruct sequence initated in 1..'
      else
        @time_remaining_message = "the ship explodes. a sparkle from the builder's room is all that is left."
      end
      return @time_remaining_message
    end
  end

  def scene_texts
    if (@current_scene == :se1a || @current_scene == :se1b ||
        @current_scene == :se2a || @current_scene == :se2b ||
        @current_scene == :sw2a || @current_scene == :sw2b ||
        @current_scene == :nw3a || @current_scene == :nw3b ||
        @current_scene == :ne1a || @current_scene == :ne1b ||
        @current_scene == :ne3a || @current_scene == :ne3b )
      @scene_text = 'ship corridor'
    elsif (@current_scene == :ne4a || @current_scene == :ne4b ||
           @current_scene == :n1a || @current_scene == :n1b ||
           @current_scene == :nw1a || @current_scene == :nw1b )
      @scene_text = 'northern ship corridor'
    elsif (@current_scene == :nw2a || @current_scene == :nw2b ||
           @current_scene == :w1a || @current_scene == :w1b ||
           @current_scene == :sw3a || @current_scene == :sw3b )
      @scene_text = 'western ship corridor'
    elsif (@current_scene == :sw4a || @current_scene == :sw4b ||
           @current_scene == :se4a || @current_scene == :se4b )
      @scene_text = 'southern ship corridor'
    elsif (@current_scene == :medical_bay2 )
      @scene_text = 'flesh is sutured, bones are unbroken.'
    elsif @current_scene == :start
      @scene_text = 'south east wing'
    elsif @current_scene == :north_east_wing
      @scene_text = 'north east wing'
    elsif @current_scene == :north_west_wing
      @scene_text = 'north west wing'
    elsif @current_scene == :south_west_wing
      @scene_text = 'south west wing'
    end
  end

  def scene_changed
    if (@end_game_sequence &&
        (@current_scene == :se1a || @current_scene == :se1b ||
         @current_scene == :se2a || @current_scene == :se2b ||
         @current_scene == :se4a || @current_scene == :se4b ||
         @current_scene == :ne1a || @current_scene == :ne1b ||
         @current_scene == :ne3a || @current_scene == :ne3b ||
         @current_scene == :ne4a || @current_scene == :ne4b ||
         @current_scene == :n1a || @current_scene == :n1b ||
         @current_scene == :nw1a || @current_scene == :nw1b ||
         @current_scene == :nw2a || @current_scene == :nw2b ||
         @current_scene == :nw3a || @current_scene == :nw3b ||
         @current_scene == :w1a || @current_scene == :w1b ||
         @current_scene == :sw2a || @current_scene == :sw2b ||
         @current_scene == :sw3a || @current_scene == :sw3b ||
         @current_scene == :sw4a || @current_scene == :sw4b ||
         @current_scene == :start || @current_scene == :north_east_wing ||
         @current_scene == :south_west_wing || @current_scene == :north_west_wing ||
         @current_scene == :medical_bay2))

      @end_game_ticks += 1

      @scenes[@current_scene][:text] = ["#{scene_texts}" + "\n#{time_remaining}"]

      if(@end_game_ticks > 24)
        if(!@artifact_worn)
          @scenes[@current_scene][:text] = "the ship explodes. a sparkle from the builder's room is all that is left."
          @game.world.cause_death :exploded
        end
      end
    end

    if @end_game_sequence
      if (@prev_scene == :w1a && @current_scene == :nw2a) ||
         (@prev_scene == :north_west_wing && @current_scene == :nw2b) ||
         (@prev_scene == :w1b && @current_scene == :sw3b) ||
         (@prev_scene == :south_west_wing && @current_scene == :sw3a) ||
         (@prev_scene == :south_west_wing && @current_scene == :sw4a) ||
         (@prev_scene == :n1a && @current_scene == :nw1a) ||
         (@prev_scene == :north_west_wing && @current_scene == :nw1b) ||
         (@prev_scene == :north_east_wing && @current_scene == :ne4a) ||
         (@prev_scene == :n1b && @current_scene == :ne4b) ||
         (@prev_scene == :medical_bay && @current_scene == :ne1b) ||
         (@prev_scene == :north_east_wing && @current_scene == :ne3a) ||
         (@prev_scene == :nw1b && @current_scene == :n1b)

        if (@prev_scene == :south_west_wing && @current_scene == :sw4a)
          @scenes[@current_scene][:options].delete(:continue_to_launch_controls)
        elsif (@prev_scene == :south_west_wing && @current_scene == :sw3a) || (@prev_scene == :w1a && @current_scene == :nw2a)
          @scenes[@current_scene][:options].delete(:continue_to_north_west_wing)
        elsif (@prev_scene == :north_west_wing && @current_scene == :nw2b) ||
              (@prev_scene == :w1b && @current_scene == :sw3b)
          @scenes[@current_scene][:options].delete(:continue_to_south_west_wing)
        elsif (@prev_scene == :n1a && @current_scene == :nw1a) ||     (@prev_scene == :north_east_wing && @current_scene == :ne4a)
          @scenes[@current_scene][:options].delete(:continue_to_north_west_wing)
        elsif (@prev_scene == :north_west_wing && @current_scene == :nw1b) ||
              (@prev_scene == :n1b && @current_scene == :ne4b) ||
              (@prev_scene == :nw1b && @current_scene == :n1b)
          @scenes[@current_scene][:options].delete(:continue_to_north_east_wing)
        elsif (@prev_scene == :medical_bay && @current_scene == :ne1b)
          @scenes[@current_scene][:options].delete(:continue)
        elsif (@prev_scene == :north_east_wing && @current_scene == :ne3a)
          @scenes[@current_scene][:options].delete("continue to builder's room")
        end

        @scenes[@current_scene][:options] = {
          :fight => {:next_scene => {1 => :defector_battle}}
        }
      end
    end

    if @current_scene == :launch
      if !@end_game_sequence
        if($game.world.thruster_stores[:alien_alloy] != nil && $game.world.shield_stores[:alien_alloy] == nil)
          if ($game.world.thruster_stores[:alien_alloy] < 10)
            @scenes[:launch][:text] = ['the ship is not ready for liftoff! needs more shields and thrusters.']
          else
            @scenes[:launch][:text] = ['ship needs more shields.']
          end
        elsif($game.world.thruster_stores[:alien_alloy] == nil && $game.world.shield_stores[:alien_alloy] != nil)
          if ($game.world.shield_stores[:alien_alloy] < 10)
            @scenes[:launch][:text] = ['the ship is not ready for liftoff! needs more shields and thrusters.']
          else
            @scenes[:launch][:text] = ['ship needs more thrusters.']
          end
        elsif (($game.world.thruster_stores[:alien_alloy] != nil && $game.world.shield_stores[:alien_alloy] != nil))
          if ($game.world.thruster_stores[:alien_alloy] >= 10 && $game.world.shield_stores[:alien_alloy] >= 10)
            @scenes[:launch][:text] = ['ready for liftoff! the ship starts to ascend the heavens!']
            begin_end_game_sequence
          elsif ($game.world.thruster_stores[:alien_alloy] >= 10 && $game.world.shield_stores[:alien_alloy] < 10)
            @scenes[:launch][:text] = ['ship needs more shields']
          elsif ($game.world.thruster_stores[:alien_alloy] < 10 && $game.world.shield_stores[:alien_alloy] >= 10)
            @scenes[:launch][:text] = ['ship needs more thrusters']
          elsif ($game.world.thruster_stores[:alien_alloy] < 10 && $game.world.shield_stores[:alien_alloy] < 10)
            @scenes[:launch][:text] = ['the ship is not ready for liftoff! needs more shields and thrusters.']
          end
        end
      else
        @scenes[:launch][:text] = ['the ship, ascending the heavens']
      end
    elsif @current_scene == :observe2
      #TODO: flush out rest of observatory narrative elements
      @scenes[:observe2][:text] = [
        ['death comes for all'],
        ['none are safe'],
        ['despair']
      ].sample
    elsif @current_scene == :medical_bay2
      $game.world.current_hp = $game.world.max_hp
      if @end_game_sequence
        @end_game_ticks += 2
        @scenes[@current_scene][:text] = ["#{scene_texts}" + "\n#{time_remaining}"]
      end
    elsif @current_scene == :reinforced_defector
      #@reinforced_defector_killed = true
      @scenes[:reinforced_blast_doors][:options].delete (:open_doors)
      @scenes[:reinforced_blast_doors][:options].delete (:head_to_north_west_wing)
      @scenes[:reinforced_blast_doors][:options] = {
        :head_to_north_west_wing => {:next_scene => { 1 => :nw3b}}
      }
      @scenes[:reinforced_blast_doors][:text] = ['a defeated defector.']
    elsif @current_scene == :defector_battle
      @scenes[:defector_battle][:health] = [65,66,67,68,69].sample
      @scenes[:defector_battle][:options].delete(:continue)
      @scenes[:defector_battle][:options] = {
        :continue => { :next_scene => { 1 => @prev_scene} }
      }

      @scenes[@prev_scene][:options].delete(:fight)

      if (@prev_scene == :sw4a)
        @scenes[@prev_scene][:options] = {
          :continue_to_launch_controls => {
            :next_scene => { 1 => :launch_controls}},
          :go_back => {
            :next_scene => { 1 => :south_west_wing}}
        }
      elsif (@prev_scene == :sw3b)
        @scenes[@prev_scene][:options] = {
          :continue_to_south_west_wing => {
            :next_scene => { 1 => :south_west_wing}},
          :go_back => {
            :next_scene => { 1 => :w1b}}
        }
      elsif (@prev_scene == :nw2a)
        @scenes[@prev_scene][:options] = {
          :continue_to_north_west_wing => {
            :next_scene => { 1 => :north_west_wing}},
          :go_back => {
            :next_scene => { 1 => :w1a}}
        }
      elsif (@prev_scene == :nw2b)
        @scenes[@prev_scene][:options] = {
          :continue_to_south_west_wing => {
            :next_scene => { 1 => :w1b}},
          :go_back => {
            :next_scene => { 1 => :north_west_wing}}
        }
      elsif (@prev_scene == :sw3a)
        @scenes[@prev_scene][:options] = {
          :continue_to_north_west_wing => {
            :next_scene => { 1 => :w1a}},
          :go_back => {
            :next_scene => { 1 => :south_west_wing}}
        }
      elsif (@prev_scene == :nw1a)
        @scenes[@prev_scene][:options] = {
          :continue_to_north_west_wing => {
            :next_scene => { 1 => :north_west_wing}},
          :go_back => {
            :next_scene => { 1 => :n1a}}
        }
      elsif (@prev_scene == :ne4a)
        @scenes[@prev_scene][:options] = {
          :continue_to_north_west_wing => {
            :next_scene => { 1 => :n1a}},
          :go_back => {
            :next_scene => { 1 => :north_east_wing}}
        }
      elsif (@prev_scene == :nw1b)
        @scenes[@prev_scene][:options] = {
          :continue_to_north_east_wing => {
            :next_scene => { 1 => :n1b}},
          :go_back => {
            :next_scene => { 1 => :north_west_wing}}
        }
      elsif (@prev_scene == :ne4b)
        @scenes[@prev_scene][:options] = {
          :continue_to_north_east_wing => {
            :next_scene => { 1 => :north_east_wing}},
          :go_back => {
            :next_scene => { 1 => :n1b}}
        }
      elsif (@prev_scene == :n1b)
        @scenes[@prev_scene][:options] = {
          :continue_to_north_east_wing => {
            :next_scene => { 1 => :ne4b}},
          :go_back => {
            :next_scene => { 1 => :nw1b}}
        }
      elsif (@prev_scene == :ne1b)
        @scenes[@prev_scene][:options] = {
          :continue_to_north_east_wing => {
            :next_scene => { 1 => :north_east_wing}},
          :go_back => {
            :next_scene => { 1 => :medical_bay}}
        }
      elsif (@prev_scene == :ne3a)
        @scenes[@prev_scene][:options] = {
          "continue to builder's room" => {
            :next_scene => { 1 => :builder_room}},
          :go_back => {
            :next_scene => { 1 => :north_east_wing}}
        }
      end
    elsif @current_scene == :admirals_quarters
      if $game.world.has_opened_chest
        #$game.world.stores[:admiral_key] = 1
      end
    elsif @current_scene == :loot_chest
      #if !$game.world.u_weapons_unlocked
      if (!$game.world.has_opened_chest)
        #$game.world.u_weapons_unlocked = true
        $game.world.admiral_weapon_stores << Masamune.new
        $game.world.admiral_weapon_stores << Murasame.new
        @game.stores[:war_mantle] = 1
        @game.stores[:leather_armour] = 1
        @game.stores[:iron_armour] = 1
        @game.stores[:steel_armour] = 1
      end
      $game.world.has_opened_chest = true
    elsif @current_scene == :wear
      @artifact_worn = true
      $game.world.blind_artifact_worn = true
    elsif @current_scene == :builder_room
      if(@artifact_worn)
        $game.world.blind_artifact_worn = true
        #@space_screen = new_space_screen #broken
        #popup @space_screen
      end
    end

    #TODO : part of builder's room
    #if @current_scene == :wear
    #    $mini_world.artifact_worn = true
    #    $mini_world.blown_map_save_builder
    #    $mini_world_screen.draw_map
    #end
    @prev_scene = @current_scene
  end

  #def new_space_screen
  #  create SpaceScreen
  #end

  def begin_end_game_sequence
    @end_game_sequence = true
    @end_game_ticks = 0

    @scenes[:start][:options].delete (:head_to_north_east_wing)
    @scenes[:north_east_wing][:options].delete (:head_to_south_east_wing)
    @scenes[:south_west_wing][:options].delete (:head_to_engine_room)
    @scenes[:north_west_wing][:options].delete (:head_to_shield_bay)

    @scenes[:blast_doors][:options].delete (:open_doors)
    @scenes[:blast_doors][:text] = ["the doors won't budge. there must be another way off"]

    @scenes[:launch_controls][:options].delete (:initiate_lift_off)
    @scenes[:launch_controls][:text] = ['the ship is ascending the heavens']

    @scenes[:reinforced_blast_doors][:options].delete (:open_doors)
    @scenes[:reinforced_blast_doors][:options].delete (:head_to_north_west_wing)
    @scenes[:reinforced_blast_doors][:options] = {
      :open_doors => {:next_scene => { 1 => :reinforced_blast_doors3}} ,
      :head_to_north_west_wing => {:next_scene => { 1 => :nw3b}}
    }

    @scenes[:reinforced_defector][:health] = [@scenes[:reinforced_defector][:max_health] - 2, @scenes[:reinforced_defector][:max_health] - 4, @scenes[:reinforced_defector][:max_health] - 6, @scenes[:reinforced_defector][:max_health] - 8, @scenes[:reinforced_defector][:max_health] - 5].sample

    @scenes[:builder_room][:options].delete(:touch_rose)
    @scenes[:builder_room][:options].delete(:head_to_north_east_wing)
    @scenes[:builder_room][:options] = {
      :touch_rose => {:next_scene => { 1 => :wear}},
      :head_to_north_east_wing => {:next_scene => { 1 => :ne3b}}
    }
  end

end
