# Copyright 2014 Scratchwork Development LLC. All rights reserved.

class SwampEvent < WorldEvent
  def initialize game
    @game = game
    @title = "a murky swamp"
    @combat_tick = 0
    super @game.world.stores,
          @game.world.weapon_stores,
          @game.world.weapon_items,
          @game.world.enemy_damage_multiplier
  end

  def replenishes_water?
    true
  end

  def action_on_exit
    :end
  end

  def on_clear_replace_with
    :bottomless_box
  end

  def carrying_compass?
    @game.stores[:compass] == 1
  end

  def init_scenes
    replenish_water

    compass = {
      :text => [
        "the wanderer compass reads #{@game.world.compass_direction}."
      ],
      :options => {
        :back => { :next_scene => :start }
      }
    }

    @scenes = {
      :start => {
        :options => {
          :stores => { :next_scene => { 1 => :stores } },
          :cook => { :next_scene => { 1 => :cook } },
        }
      },
      :stores => {
        :text => [
          "supplies will be safe here."
        ],
        :loot => { },
        :options => {
          :continue => { :next_scene => { 1 => :start } }
        }
      },
      :cook => {
        :text => [
          "meager scraps, poorly prepared."
        ],
        :cost => { :flesh => 3 },
        :loot => {
          :food => { :min => 5, :max => 10, :chance => 1.0 },
        },
        :options => {
          :continue => { :next_scene => { 1 => :start } }
        },
      },
    }

    if !carrying_compass?
      @scenes[:start][:options][:compass] = { :next_scene => :compass }
      @scenes[:compass] = compass
    end

    @scenes[:start][:options][:leave] = leave

    continuation = {
      :sever_limbs => {
        :storyline => true,
        :text => [
          "he speaks of a builder, forgives her for the rebellion she led against him."
        ],
        :options => {
          :cauterize_wounds => { :next_scene => :cauterize_wounds }
        },
      },
      :cauterize_wounds => {
        :storyline => true,
        :text => [
          "tears fill his eyes. he remembers each pull of the trigger. her falling to the ground. wishes he could take it all back."
        ],
        :options => {
          :leave => clear
        },
      }
    }

    if [:destruction, :humanity].include? @game.world.admiral_visits and armoury_cleared? and @game.story_line.cutscene_3_unlocked?
      if(cauter? and sword?)
        @scenes[:start][:storyline] = true
        @scenes[:start][:text] = [
          "the admiral speaks of wanderer destruction. tries desperately to find peace. he speaks with absolute resolve. asks for humanity. kneels."
        ]

        @scenes[:start][:options] = {
          :sever_limbs => { :next_scene => { 1 => :sever_limbs } },
          :stores => { :next_scene => { 1 => :stores } },
          :cook => { :next_scene => { 1 => :cook } },
          :leave => leave
        }

        @scenes[:sever_limbs] = continuation[:sever_limbs]
        @scenes[:cauterize_wounds] = continuation[:cauterize_wounds]

        add_destruction_cutscene
      else
        @scenes[:start][:storyline] = true
        @scenes[:start][:text] = [
          "the admiral speaks of wanderer destruction. tries desperately to find peace."
        ]

        add_destruction_cutscene
      end

      @game.world.admiral_visits = :humanity
    elsif [:peace, :destruction].include? @game.world.admiral_visits and iron_mine_cleared? and @game.story_line.cutscene_2_unlocked?
      @scenes[:start][:storyline] = true
      @scenes[:start][:text] = [
        "the admiral speaks of purging the planet. looks upon the scorched landscape with resentful eyes."
      ]

      add_purging_cutscene

      @game.world.admiral_visits = :destruction
    elsif [:stars, :peace].include? @game.world.admiral_visits and tannery_cleared? and @game.story_line.cutscene_1_unlocked?
      @scenes[:start][:storyline] = true
      @scenes[:start][:text] = [
        "the admiral gazes up at the stars. speaks of wanderer's thirst to conquer."
      ]

      add_star_gazer_cutscene

      @game.world.admiral_visits = :peace
    elsif
      @scenes[:start][:text] = [
        "the admiral rests beneath a canopy, wincing from the wounds of battle."
      ]

      if(@game.world.admiral_visits == :beginning)
        @game.world.admiral_visits = :stars
      elsif(@game.world.admiral_visits == :whincing)
        @game.world.admiral_visits = :stars
      elsif(@game.world.admiral_visits == :still_whincing) #leaving this guard clause to make sure I don't break backwards compatiblity
        @game.world.admiral_visits = :stars
      end
    end

    # add_debug_cutscenes
  end

  def add_debug_cutscenes
    @scenes[:start][:options] = { }

    [StarGazerCutscene,
     PurgingCutscene,
     DestructionCutscene,
     FirstContactCutscene,
     DrillsCutscene,
     MassacreCutscene,
     RebellionCutscene,
     RitualCutscene].each do |c|
      @scenes[c.to_s.to_sym] = {
        cutscene: true,
        cutscene_class: c,
        when_completed_go_to: :start
      }

      @scenes[:start][:options][c.to_s.to_sym] = {
        :next_scene => c.to_s.to_sym
      }
    end

  end

  def add_destruction_cutscene
    @scenes[:destruction] = {
      cutscene: true,
      cutscene_class: DestructionCutscene,
      when_completed_go_to: :start
    }

    @scenes[:start][:options][:destruction] = { :next_scene => :destruction }
    @scenes[:start][:options].delete(:leave)
    @scenes[:start][:options][:leave] = leave
  end

  def add_purging_cutscene
    @scenes[:the_purging] = {
      cutscene: true,
      cutscene_class: PurgingCutscene,
      when_completed_go_to: :start
    }

    @scenes[:start][:options][:the_purging] = { :next_scene => :the_purging }
    @scenes[:start][:options].delete(:leave)
    @scenes[:start][:options][:leave] = leave
  end

  def add_star_gazer_cutscene
    @scenes[:star_gazer] = {
      cutscene: true,
      cutscene_class: StarGazerCutscene,
      when_completed_go_to: :start
    }

    @scenes[:start][:options][:star_gazer] = { :next_scene => :star_gazer }
    @scenes[:start][:options].delete(:leave)
    @scenes[:start][:options][:leave] = leave
  end

  def iron_mine_cleared?
    @game.world.named_event_cleared? :iron_mine
  end

  def armoury_cleared?
    @game.world.named_event_cleared? :armoury
  end

  def tannery_cleared?
    @game.world.named_event_cleared? :tannery
  end

  def cauter?
    (@game.world.stores[:cauter] || 0) > 0
  end

  def katana?
    @game.world
         .weapon_stores
         .find_all do |w|
           (w.name == :katana || w.name == :murasame || w.name == :masamune) &&
           w.durability_percentage > 0
         end.any?
  end

  def iron_sword?
    @game.world
         .weapon_stores
         .find_all { |w| w.name == :iron_sword && w.durability_percentage > 0 }
         .any?
  end

  def steel_sword?
    @game.world
         .weapon_stores
         .find_all { |w| w.name == :steel_sword && w.durability_percentage > 0 }
         .any?
  end

  def sword?
    katana? || iron_sword? || steel_sword?
  end

  def loot_sample
    if @current_scene == :cook
      results = super
      @game.world.swamp_stores[:food] ||= 0
      @game.world.swamp_stores[:food] += results[:food]
    end

    return @game.world.swamp_stores
  end

  def weapon_sample
    return @game.world.swamp_weapon_stores
  end
end
