# Copyright 2014 Scratchwork Development LLC. All rights reserved.

class TownEvent < WorldEvent
  def initialize game
    @game = game
    @title = "a deserted town"
    @combat_tick = 0
    super @game.world.stores, @game.world.weapon_stores, @game.world.weapon_items, @game.world.enemy_damage_multiplier
  end

  def becomes_outpost?
    true
  end

  def action_on_exit
    :aborted
  end

  def book_list
    @book_list ||= [
      {
        :text => [
          "the annals speak of the fleet's arrival. no regard was given to the habitants.",
          "any attempts to communicate, to understand, were ignored."
        ],
        :cutscene_name => :first_contact,
        :cutscene_class => FirstContactCutscene
      },
      {
        :text => [
          "the annals speak of the wanderer drills. how effortlessly they were being constructed by the occultists.", "how quickly they depleted the planet's resources.",
        ],
        :cutscene_name => :the_drills,
        :cutscene_class => DrillsCutscene
      },
      {
        :text => [
          "the annals speak of the admiral. they speak of the massacres he lead. the cities he razed."
        ],
        :cutscene_name => :the_massacre,
        :cutscene_class => MassacreCutscene
      },
      {
        :text => [
          "the annals speak of a high ranking occultist, a builder. they speak of her pleas to the admiral.",
          "they speak of the rebellion she led against him."
        ],
        :cutscene_name => :the_rebellion,
        :cutscene_class => RebellionCutscene
      },
      {
        :text => [
          "the annals speak of the ritual. they speak of wanderers pledging their allegiance to the humans... kneeling... accepting humanity.",
        ],
        :cutscene_name => :the_ritual,
        :cutscene_class => RitualCutscene
      }
    ]
  end

  def set_random_book
    selected_book = book_list.sample
    @scenes[selected_book[:cutscene_name]] = {
      cutscene: true,
      cutscene_class: selected_book[:cutscene_class],
      when_completed_go_to: :take_book
    }

    @scenes[:take_book] = {
      :storyline => true,
      :text => selected_book[:text],
      :options => { },
    }

    @scenes[:take_book][:options][selected_book[:cutscene_name]] = { :next_scene => selected_book[:cutscene_name] }
    @scenes[:take_book][:options][:exit_room] = {
      :next_scene => {
        0.5 => :b1, 1 => :b2
      }
    }
  end

  def init_scenes
    @scenes = {
      :start => {
        :text => [
          'a small suburb lays ahead, empty houses scorched and peeling.',
          "broken streetlights stand, rusting. light hasn't graced this place in a long time."
        ],
        :options => {
          :schoolhouse => {
            :next_scene => { 1 => :schoolhouse },
            :next_scene_cost => { :torch => 1 }
          },
          :streets => {
            :next_scene => { 1 => :a2 }
          },

          :leave => leave
        }
      },
      :schoolhouse => {
        :text => [
          "where the windows of the schoolhouse aren't shattered, they're blackened with soot.",
          'the double doors creak endlessly in the wind.'
        ],
        :options => {
          :enter => {
            :next_scene => {
              0.99 => :library,
              0.1 => :b1,
              1 => :b2
            },
          }
        }
      },
      :library => {
        :storyline => true,
        :text => [
          "through the vestibule lies a hexagonal room. books flicker in the torch light.",
          "the annals of war line the walls. most of them have been burned.",
        ],
        :options => {
          :take_book => { :next_scene => :take_book },
          :leave => leave
        }
      },
      :a2 => {
        :text => ['ambushed on the street.'],
        :combat => true,
        :enemy => :thug,
        :damage => 4,
        :hit => 0.8,
        :attack_delay => 2,
        :health => 30,
        :max_health => 30,
        :loot => {
          :knife => { :min => 1, :max => 1, :chance => 1 },
          :food => { :min => 1, :max => 5, :chance => 0.5 }
        },
        :options => {
          :continue => {
            :next_scene => { 0.5 => :b3, 1 => :b4 }
          },
          :run => aborted
        }
      },
      :b1 => {
        :text => ['a small cache of supplies is tucked inside a rusting locker.' ],
        :loot => {
          :food => { :min => 1, :max => 5, :chance => 1 },
          :torch => { :min => 1, :max => 3, :chance => 0.8 },
          :bullets => { :min => 1, :max => 5, :chance => 0.3 }
        },
        :options => {
          :continue => {
            :next_scene => { 0.5 => :c1, 1 => :c2 }
          },
          :run => aborted
        }
      },
      :b2 => {
        :text => ['a scavenger waits just inside the door.'],
        :combat => true,
        :enemy => :scavenger,
        :damage => 4,
        :hit => 0.8,
        :attack_delay => 2,
        :health => 30,
        :max_health => 30,
        :loot => {
          :knife => { :min => 1, :max => 1, :chance => 1.0 },
          :food => { :min => 1, :max => 5, :chance => 0.5 }
        },
        :options => {
          :continue => {
            :next_scene => { 0.5 => :c2, 1 => :c3 },
          },
          :run => aborted
        }
      },
      :b3 => {
        :text => ['a beast stands alone in an overgrown park.'],
        :combat => true,
        :enemy => :beast,
        :damage => 3,
        :hit => 0.8,
        :attack_delay => 1,
        :health => 25,
        :max_health => 25,
        :loot => {
          :teeth => { :min => 1, :max => 5, :chance => 1 },
          :eyes => { :min => 1, :max => 2, :chance => 1 }
        },
        :options => {
          :continue => {
            :next_scene => { 0.5 => :c4, 1 => :c5 }
          },
          :run => aborted
        }
      },
      :b4 => {
        :text => [
          'an overturned caravan is spread across the pockmarked street.',
          "it's been picked over by scavengers, but there's still some things worth taking."
        ],
        :loot => {
          :food => { :min => 1, :max => 5, :chance => 1.0 },
          :torch => { :min => 1, :max => 3, :chance => 0.5 },
          :bullets => { :min => 1, :max => 5, :chance => 0.3 }
        },
        :options => {
          :continue => {
            :next_scene => { 0.5 => :c5, 1 => :c6 },
          },
          :run => aborted
        }
      },
      :c1 => {
        :text => ['a thug moves out of the shadows.'],
        :combat => true,
        :enemy => :thug,
        :damage => 4,
        :hit => 0.8,
        :attack_delay => 2,
        :health => 30,
        :max_health => 30,
        :loot => {
          :knife => { :min => 1, :max => 1, :chance => 1.0 },
          :food => { :min => 1, :max => 5, :chance => 0.5 }
        },
        :options => {
          :continue => { :next_scene => { 1 => :d1 } },
          :run => aborted
        }
      },
      :c2 => {
        :text => ['a beast charges out of a ransacked classroom.'],
        :combat => true,
        :enemy => :beast,
        :damage => 3,
        :hit => 0.8,
        :attack_delay => 1,
        :health => 25,
        :max_health => 25,
        :loot => {
          :teeth => { :min => 1, :max => 5, :chance => 1 },
          :eyes => { :min => 1, :max => 2, :chance => 1 }
        },
        :options => {
          :continue => {
            :next_scene => { 1 => :d1 }
          },
          :run => aborted
        }
      },
      :c3 => {
        :text => [
          'through the large gymnasium doors, footsteps can be heard.',
          'the torchlight casts a flickering glow down the hallway.',
          'the footsteps stop.'
        ],
        :options => {
          :continue => {
            :next_scene => { 1 => :d1 }
          },
          :leave => leave
        }
      },
      :c4 => {
        :text => ['another beast, drawn by the noise, leaps out of a copse of trees.'],
        :combat => true,
        :enemy => :beast,
        :damage => 4,
        :hit => 0.8,
        :attack_delay => 1,
        :health => 25,
        :max_health => 25,
        :loot => {
          :teeth => { :min => 1, :max => 5, :chance => 1 },
          :eyes => { :min => 1, :max => 2, :chance => 1 }
        },
        :options => {
          :continue => {
            :next_scene => { 1 => :d2 }
          },
          :run => aborted
        }
      },
      :c5 => {
        :text => [
          "something's causing a commotion a ways down the road.",
          "a fight, maybe."
        ],
        :options => {
          :continue => {
            :next_scene => { 1 => :d2 }
          },
          :leave => leave
        }
      },
      :c6 => {
        :text => [
          'a small basket of food is hidden under a park bench, with a note attached.',
          "can't read the words."
        ],
        :loot => {
          :food => { :min => 1, :max => 5, :chance => 1 }
        },
        :options => {
          :continue => {
            :next_scene => { 1 => :d2 }
          },
          :run => aborted
        }
      },
      :d1 => {
        :text => ['a panicked scavenger bursts through the door, screaming.'],
        :combat => true,
        :enemy => :scavenger,
        :damage => 5,
        :hit => 0.8,
        :attack_delay => 2,
        :health => 30,
        :max_health => 30,
        :loot => {
          :food => { :min => 1, :max => 5, :chance => 1 },
          :steel_sword => { :min => 1, :max => 1, :chance => 0.8 }
        },
        :options => {
          :continue => {
            :next_scene => { 0.5 => :end1, 1 => :end2 }
          },
          :run => aborted
        }
      },
      :d2 => {
        :text => ["a man stands over a dead wanderer. notices he's not alone."],
        :combat => true,
        :enemy => :vigilante,
        :damage => 6,
        :hit => 0.8,
        :attack_delay => 2,
        :health => 30,
        :max_health => 30,
        :loot => {
          :food => { :min => 1, :max => 5, :chance => 1 },
          :steel_sword => { :min => 1, :max => 1, :chance => 0.5 },
        },
        :options => {
          :continue => {
            :next_scene => { 0.5 => :end3, 1 => :end4 }
          },
          :run => aborted
        }
      },
      :end1 => {
        :text => [
          'scavenger had a small camp in the school.',
          'collected scraps spread across the floor like they fell from heaven.'
        ],
        :loot => {
          :steel_sword => { :min => 1, :max => 1, :chance => 1 },
          :food => { :min => 5, :max => 10, :chance => 1 },
          :bolas => { :min => 2, :max => 6, :chance => 0.8 }
        },
        :options => { :leave => clear }
      },
      :end2 => {
        :text => [
          "scavenger'd been looking for supplies in here, it seems.",
          "a shame to let what he'd found go to waste."
        ],
        :loot => {
          :food => { :min => 5, :max => 10, :chance => 1 },
          :knife => { :min => 1, :max => 1, :chance => 1 }
        },
        :options => { :leave => clear }
      },
      :end3 => {
        :text => [
          "beneath the wanderer's rags, clutched in one of its many hands, a glint of steel.",
          "worth killing for, it seems."
        ],
        :loot => {
          :rifle => { :min => 1, :max => 1, :chance => 1 },
          :bullets => { :min => 1, :max => 5, :chance => 1 },
          #:radar => { :min => 1, :max => 1, :chance => 1.0 },
        },
        :options => { :leave => clear }
      },
      :end4 => {
        :text => [
          "eye for an eye seems fair.",
          "always worked before, at least.",
          "picking the bones finds some useful trinkets."
        ],
        :loot => {
          :food => { :min => 5, :max => 10, :chance => 1 },
          :torch => { :min => 1, :max => 5, :chance => 1 },
          :bolas => { :min => 2, :max => 6, :chance => 0.8 }
        },
        :options => { :leave => clear }
      }
    }

    set_random_book

    @scenes
  end
end
