# Copyright 2014 Scratchwork Development LLC. All rights reserved.

class ShipMiniWorld < MiniWorld
  def movement_char
    "@"
  end

  def spawn_enemies
    return if @enemies_spawned
    @enemies_spawned = true
    blast_door_loc   =  location_for_landmark "V"
    builder_room_loc =  location_for_landmark "B"
    medical_bay_loc  =  location_for_landmark "M"
    shield_bay_loc   =  location_for_landmark "S"
    engine_room_loc  =  location_for_landmark "T"
    observatory_loc  =  location_for_landmark "O"
    return if !blast_door_loc
    return if !builder_room_loc
    return if !medical_bay_loc
    return if !shield_bay_loc
    return if !engine_room_loc
    return if !observatory_loc
    @enemies = [
      {
        char: "&",
        x: blast_door_loc[0],
        y: shield_bay_loc[1] +
        distance_scalar(shield_bay_loc[1], engine_room_loc[1]) },
      {
        char: "&",
        x: builder_room_loc[0],
        y: medical_bay_loc[1] +
        distance_scalar(observatory_loc[1], medical_bay_loc[1]) - 1},
      {
        char: "&",
        x: blast_door_loc[0] +
        (distance_scalar(blast_door_loc[0], builder_room_loc[0]) * rand).to_i - 2,
        y: shield_bay_loc[1] +
        (distance_scalar(shield_bay_loc[1], engine_room_loc[1]) * rand).to_i - 2
      },
      {
        char: "&",
        x: blast_door_loc[0] +
        (distance_scalar(blast_door_loc[0], builder_room_loc[0]) * rand).to_i - 1,
        y: medical_bay_loc[1] +
        (distance_scalar(observatory_loc[1], medical_bay_loc[1]) * rand).to_i - 1
      },
      {
        char: "&",
        x: blast_door_loc[0] +
        (distance_scalar(blast_door_loc[0], builder_room_loc[0]) * rand).to_i - 2,
        y: medical_bay_loc[1] +
        (distance_scalar(observatory_loc[1], medical_bay_loc[1]) * rand).to_i - 2
      },
    ]
  end

  def distance_scalar i_one, i_two
    (i_one - i_two).abs.fdiv(2).to_i
  end

  def location_for_landmark char
    (@landmarks.find { |k, v| v == char } || " ")[0]
  end

  def event_map
    {
      "X" => LeaveShipEvent,
      "O" => ObservatoryEvent,
      "M" => MedicalBayEvent,
      "B" => BuilderRoomEvent,
      "V" => ShipDefectorRoomEvent,
      "S" => ShieldsEvent,
      "T" => ThrustersEvent,
      "L" => LiftoffEvent,
      "A" => AdmiralRoomEvent
    }
  end

  def layout
    [
      "                +---------+       +---------+               ",
      "                |         |       |         |               ",
      "                |         |       |         |               ",
      "                |    V    |       |    B    |               ",
      "                |         |       |         |               ",
      "      +---------+         +-------+         +---------+     ",
      "      |                                               |     ",
      "      |                                               |     ",
      "      |    S                                     M    |     ",
      "      |                                               |     ",
      "      |                                               |     ",
      "      +---------+         +-------+        +----------+     ",
      "                |         |       |        |                ",
      "                |         |       |        |                ",
      "      +---------+         +-------+        +----------+     ",
      "      |                                               |     ",
      "      |                                               |     ",
      "      |    T                  L                  O    |     ",
      "      |                                               |     ",
      "      |                                               |     ",
      "      +---------+         +-------+         +---------+     ",
      "                |         |       |         |               ",
      "                |         |       |    @    |               ",
      "                |    A    |       |    X    |               ",
      "                |         |       |         |               ",
      "                +---------+       +---------+               ",
    ]
  end

  def blown_layout
    [
      "           &   #--#------#+  ,    +---------+               ",
      "  *     ,    , *.     &   |       |         |          ,    ",
      "     .          * &       |       |         |               ",
      "        . /  -  |    V    /    .  |    B    |               ",
      "      *  /, *  .#         |  ,    |         |               ",
      "   +-- #-  - +,*          +-------+         +---------+     ",
      "   | &   *# .    #                                    |     ",
      "   |       ,  , * #                                   |     ",
      "  | __- S  * #                                   M    |     ",
      "  |+     & .*  #                                      |     ",
      " |/  , /. # ,*,  *            ,/,                     |     ",
      " /  .   `-.-,--+* -*      +--/--/-+___     +----------+     ",
      "      ,   . , #  #//      |.      *   ^/*-/   -     .       ",
      "  _-/`  *   *   . |/      /+   #     __     /    #          ",
      "   * /        #    --+     -------+|/  *^/-+----------+     ",
      "  - #          #.,*/                                  |     ",
      "  #,            ,              @                      |     ",
      "   *            /              L                 O    |     ",
      "  /&           |_                                     |     ",
      "   /#         /||                                     |     ",
      "  +-` #  __ _.`, __--__   +-------+         +---------+     ",
      "    #, #   _   //      -/|   -    |         |               ",
      "     *  *   --   ,   .   |     /  |         |               ",
      "   .   +  *    ,          |     . |    X    |               ",
      "          ##       .   ,   +      |         |               ",
      "      -        _   -              +---------+               ",
    ]
  end

  def blown_layout_save_builder
    [
      "           &   #--#------#+  ,    +---------+               ",
      "  *     ,    , *.     &   |       | _-v-v-_ |          ,    ",
      "     .          * &       |       |v       v|               ",
      "        . /  -  |    V    /    .  |    B    |               ",
      "      *  /, *  .#         |  ,    |^_  @  _^|               ",
      "   +-- #-  - +,*          +-------+ ^^-_-^^ +---------+     ",
      "   | &   *# .    #                  & & & &           |     ",
      "   |       ,  , * #                  & & &            |     ",
      "  | __-    * #                                   M    |     ",
      "  |+     & .*  #                                      |     ",
      " |/  , /. # ,*,  *            ,/,                     |     ",
      " /  .   `-.-,--+* -*      +--/--/-+___     +----------+     ",
      "      ,   . , #  #//      |.      *   ^/*-/   -     .       ",
      "  _-/`  *   *   . |/      /+   #     __     /    #          ",
      "   * /        #    --+     -------+|/  *^/-+----------+     ",
      "  - #          #.,*/                                  |     ",
      "  #,            ,    +         @                      |     ",
      "   *            /   +#+                          O    |     ",
      "  /&           |_    +                                |     ",
      "   /#         /||                                     |     ",
      "  +-` #  __ _.`, __--__   +-------+         +---------+     ",
      "    #, #   _   //      -/|   -    |         |               ",
      "     *  *   --   ,   .   |     /  |         |               ",
      "   .   +  *    ,          |     . |    X    |               ",
      "          ##       .   ,   +      |         |               ",
      "      -        _   -              +---------+               ",
    ]
  end
end
