# Copyright 2014 Scratchwork Development LLC. All rights reserved.

class Asteroid
  attr_accessor :x, :y, :speed

  def initialize x, y, speed
    @x = x
    @y = y
    @speed = speed
    @is_new = true
  end

  def move
    if !@is_new
      @y += speed
    else
      @is_new = false
    end
  end
end

class Space
  @@stub_roll_value = nil

  attr_accessor :altitude, :current_tick, :asteroids, :next_barrage

  def frames_per_second
    60
  end

  def ticks_in_millisecond
    (1000 / frames_per_second).floor
  end

  def to_ticks milliseconds
    (milliseconds / ticks_in_millisecond).floor
  end

  def initialize
    @next_barrage = 100000
    @current_tick = 0
    @altitude = 0
    @asteroids = []
    @width = 300
    @height = 400
    new_asteroids
  end

  def self.stub_roll with_value
    @@stub_roll_value = with_value
  end

  def roll
    @@stub_roll_value || rand
  end

  def random_x
    (@width.to_f * roll).floor
  end

  def remove_asteroid asteroid
    asteroids.delete_at(asteroids.index(asteroid))
  end

  def new_asteroids
    return if @altitude > 200

    @asteroids << Asteroid.new(random_x, -20, random_speed)

    #Harder
    if @altitude > 10
      @asteroids << Asteroid.new(random_x, -20, random_speed)
    end

    #HARDER
    if @altitude > 20
      @asteroids << Asteroid.new(random_x, -20, random_speed)
    end

    ##HAAAAAARDERRRRR!!!!1
    if @altitude > 30
      @asteroids << Asteroid.new(random_x, -20, random_speed)
    end

    if @altitude > 40
      @asteroids << Asteroid.new(random_x, -20, random_speed)
    end

    if @altitude > 50
      @asteroids << Asteroid.new(random_x, -20, random_speed)
    end

    if @altitude > 60
      @asteroids << Asteroid.new(random_x, -20, random_speed)
    end

    set_next_barrage
  end

  def collision? rect1, rect2, skinnier_by = 0
    box1 = {
      :top => rect1.origin.y,
      :left => rect1.origin.x,
      :right => rect1.origin.x + (rect1.size.width - skinnier_by),
      :bottom => rect1.origin.y + rect1.size.height
    }

    box2 = {
      :top => rect2.origin.y,
      :left => rect2.origin.x,
      :right => rect2.origin.x + (rect2.size.width - skinnier_by),
      :bottom => rect2.origin.y + rect2.size.height
    }

    return false if (box1[:right] < box2[:left])
    return false if (box1[:left] > box2[:right])
    return false if (box1[:bottom] < box2[:top])
    return false if (box1[:top] > box2[:bottom])

    return true
  end

  def altitude_change_after
    to_ticks(1000)
  end

  def base_speed
    1500.0 * 2.0
  end

  def random_speed
    slowest = base_speed * 0.65

    random_speed = roll * slowest

    milliseconds_to_reach_bottom = base_speed - random_speed

    ticks_to_reach_bottom = to_ticks(milliseconds_to_reach_bottom)

    dy = @height.to_f / ticks_to_reach_bottom.to_f

    dy
  end

  def can_eject?
    @altitude > 60
  end

  def completed?
    @altitude > 200 and @asteroids.count == 0
  end

  def tick
    altitude_changed = false

    @current_tick += 1

    if @current_tick % altitude_change_after == 0
      @altitude += 1
      altitude_changed = true
    end

    new_asteroids if @next_barrage < @current_tick

    move_all_asteroids

    altitude_changed
  end

  def move_all_asteroids
    @asteroids.each { |a| a.move }
  end

  def set_next_barrage
    next_milliseconds = (1000 - (@altitude * 10))
    ticks = to_ticks(next_milliseconds)
    @next_barrage = @current_tick + ticks
  end
end
