# Copyright 2014 Scratchwork Development LLC. All rights reserved.

class StoryLine
  attr_accessor :message_history, :message

  def initialize game
    @game = game
    @message = ""
    @message_history = []
  end

  def tutorial_mode?
    world.tutorial_mode?
  end

  def swamp_message?
    self.current_message == cutscene_1_message ||
      self.current_message == cutscene_2_message ||
      self.current_message == cutscene_3_message
  end

  def current_message
    return false if !@message
    return false if @message == ""
    return @message if !@message_history.include? @message

    false
  end

  def reset
    @message = nil
    @message_history.clear
  end

  def mark_message_as_seen
    @message_history << @message if !seen? @message
    @message = nil
  end

  def world
    @game.world
  end

  def havent_cleared_tannery?
    !world.mine_cleared? :tannery
  end

  def havent_cleared_iron_mine?
    !world.mine_cleared? :iron_mine
  end

  def seen? message
    return true if message == ""
    return true if !message
    @message_history.include? message
  end

  def havent_found? landmark
    !world.has_visited? landmark and has_compass?
  end

  def queue_message m
    return if @message and !seen? @message
    return if seen? m

    @message = m
  end

  def world
    @game.world
  end

  def havent_cleared_armoury?
    !world.mine_cleared? :armoury
  end

  def havent_visisted_ship?
    !world.has_visited? :ship
  end

  def havent_upgraded_ship?
    !(@game.ship_thrusters > 1 || @game.ship_hull > 1)
  end

  def cutscene_1_unlocked?
    seen? cutscene_1_message
  end

  def cutscene_2_unlocked?
    seen? cutscene_2_message
  end

  def cutscene_3_unlocked?
    seen? cutscene_3_message
  end

  def cutscene_1_message
    "his screams subside. the cold wind carries only silence."
  end

  def cutscene_2_message
    "saw visions of the admiral, beneath the swamp's canopy."
  end

  def cutscene_3_message
    "the admiral screams, cries of utter despair fill the air."
  end

  def process force = false
    if @game.tick_count >= 0
      queue_message "the admiral screams in pain, heard from miles away."
    end

    if @game.tick_count >= 1
      queue_message "a week passes by. have to make it to the ship."
    end

    return if havent_cleared_tannery?

    if @game.tick_count >= 2
      queue_message cutscene_1_message
    end

    return if havent_cleared_iron_mine?

    if @game.tick_count >= 3
      queue_message "a month passes by. this war isn't over. the fleet will prevail."
    end

    if @game.tick_count >= 6
      queue_message "two months pass. stars uncountable fill the sky."
    end

    if @game.tick_count >= 7
      queue_message "the seven sisters rise and set, chased by orion."
    end

    return if @game.world.kills_of_defector == 0

    if @game.tick_count >= 9
      queue_message "the defectors continue to hunt. continue to seek revenge."
    end

    if @game.tick_count >= 12
      queue_message "three months turn into four. this cold planet continues its aimless journey through starlight."
    end

    if @game.tick_count >= 13
      queue_message cutscene_2_message
    end

    return if havent_cleared_armoury?

    if @game.tick_count >= 15
      queue_message "why haven't reinforcements arrived yet?! what's taking so long?!"
    end

    if @game.tick_count >= 18
      queue_message "the defectors haven't forgotten the war. haven't forgotten the admiral's crimes."
    end

    if @game.tick_count >= 21
      queue_message "what right do the humans have to this planet?!"
    end

    if @game.tick_count >= 24
      queue_message "the seven sisters have risen and set two hundred times. still hunted by orion."
    end

    return if havent_visisted_ship?

    if @game.tick_count >= 27
      queue_message "everything's crumbling. the ship. this world. should've heeded the admiral's warnings."
    end

    if @game.tick_count >= 29
      queue_message "none of this would have happened if the scouts overlooked this world."
    end

    if @game.tick_count >= 31
      queue_message "how can this world survive much longer without a mother star?"
    end

    if @game.tick_count >= 32
      queue_message cutscene_3_message
    end

    if @game.tick_count >= 34
      queue_message "the atmosphere will continue to degrade. the temperature will drop. everything will die."
    end

    if @game.tick_count >= 36
      queue_message "a year passes by. no sign of reinforcements. the only ally is starlight."
    end

    if @game.tick_count >= 38
      queue_message "the defectors continue the hunt. the humans continue to rebuild."
    end

    if @game.tick_count >= 40
      queue_message "this war is lost. have to escape this world."
    end
  end
end
