# Copyright 2014 Scratchwork Development LLC. All rights reserved.

class Weapon
  attr_accessor :name, :durability, :current_durability, :id, :damage, :order

  def initialize(id,
                 name,
                 damage,
                 order,
                 durability = nil,
                 current_durability = nil)
    @id = id
    @name = name
    @durability = durability
    @damage = damage
    @current_durability = (durability * rand).ceil if @durability
    @current_durability = current_durability if current_durability and @durability
    @order = order
  end

  def durability_percentage
    return 100 if !has_durability?
    ((@current_durability.to_f / @durability.to_f) * 100.0).round
  end

  def has_durability?
    !@durability.nil?
  end

  def broken?
    has_durability? and current_durability <= 0
  end

  def serialize
    "#{id},#{name},#{durability || "none"},#{current_durability || "none"},#{damage},#{order}"
  end

  def requires_ammo?
    false
  end

  def ammo_key
    nil
  end

  def to_h
    {
      id: id,
      name: name,
      durability: durability,
      current_durability: current_durability,
      damage: damage,
      order: order,
      type: self.class.name
    }
  end
end

class Food < Weapon
  def initialize
    super GTK.create_uuid, :food, 0, -1
  end
end

class Fists < Weapon
  def initialize
    super GTK.create_uuid, :fists, 1, 0
  end
end

class Knife < Weapon
  def initialize
    super GTK.create_uuid, :knife, 2, 1, 30
  end
end

class IronSword < Weapon
  def initialize
    super GTK.create_uuid, :iron_sword, 4, 2, 20
  end
end

class SteelSword < Weapon
  def initialize
    super GTK.create_uuid, :steel_sword, 6, 3, 30
  end
end

class Katana < Weapon
  def initialize
    super GTK.create_uuid, :katana, 8, 4, 16
  end
end

class Rifle < Weapon
  def initialize
    super GTK.create_uuid, :rifle, 5, 5, 22
  end

  def requires_ammo?
    true
  end

  def ammo_key
    :bullets
  end
end

class Laser < Weapon
  def initialize
    super GTK.create_uuid, :laser, 8, 6, 19
  end

  def requires_ammo?
    true
  end

  def ammo_key
    :battery
  end
end

class Grenade < Weapon
  def initialize
    super GTK.create_uuid, :grenade, 15, 7
  end

  def requires_ammo?
    true
  end

  def ammo_key
    :grenade
  end
end

class Bolas < Weapon
  def initialize
    super GTK.create_uuid, :bolas, 0, 8
  end

  def requires_ammo?
    true
  end

  def ammo_key
    :bolas
  end
end

class Stealth < Weapon
  def initialize
    super GTK.create_uuid, :stealth, 0, 11
  end
end

class Masamune < Weapon
  def initialize
    super GTK.create_uuid, :masamune, 25, 9, 50, 50
  end
end

class Murasame < Weapon
  def initialize
    super GTK.create_uuid, :murasame, 25, 10, 50, 50
  end
end
