# Copyright 2014 Scratchwork Development LLC. All rights reserved.

class World < Hash
  @@stub_roll_value = nil
  @@override_aesthetics = nil
  @@disable_landmarks = false

  def radius
    raise "no radius"
  end

  def initialize game = nil
    @last_admiral_cutscene_visited = :none
    @game = game
    @move_count = 0
    @aesthetics = Hash.new
    @landmarks = Hash.new
    @cleared = Hash.new
    @aborted = Hash.new
    @visited = Hash.new
    @cleared_tannery_once = false
    @cleared_iron_mine_once = false
    @cleared_armoury_once = false
    @cleared_game_once = false
    @cleared_hard_mode = false
    @visited_starship_once = false
    @food_deaths = 0
    @water_deaths = 0
    @battle_deaths = 0
    @is_defector_dead = true
    @final_boss = false
    @defeated_final_boss = false
    @kills_of_defector = 0
    @thoughts_message = ""
    @u_weapons_unlocked = false
    @has_opened_chest = false
    @blind_artifact_worn = false
    @killed_at_least_one_defector = false
    @punch_count = 0
    @deaths_by_defector = 0
    @width = 90
    @height = 40
    @half_width = (@width / 2).to_i
    @half_height = (@height / 2).to_i
    reset
  end

  def deaths_by_defector
    @deaths_by_defector || 0
  end

  def aesthetics
    @aesthetics[[45, 20]] ||= :forest
    @aesthetics
  end

  def aesthetics= value
    @aesthetics = value
  end

  def cleared_hard_mode?
    return @cleared_hard_mode
  end

  def default_multipliers
    {
      durability: 1,
      poison: 0,
      hunger: 1,
      thirst: 1,
      vision: 2,
      enemy_damage: 1
    }
  end

  def multipliers_are_default?
    return current_multipliers == default_multipliers
  end

  def current_multipliers
    {
      durability: @durability_multiplier,
      poison: @poison_multiplier,
      hunger: @hunger_multiplier,
      thirst: @thirst_multiplier,
      vision: @base_vision_radius,
      enemy_damage: @enemy_damage_multiplier
    }
  end

  def describe_terrain
    closest_terrains = []

    terrains = {
      :forest => "forest",
      :field => "fields",
      :barrens => "barrens",
    }

    terrains.keys.each do |k|
      closest_terrain = closest(k)
      if(closest_terrain.count == 4 and distance_to(closest_terrain.first[:x], closest_terrain.first[:y]) == 1)
        closest_terrains << "#{terrains[k]}: to the north, south, east, and west"
      else
        closest_terrain.map do |loc|
          closest_terrains << "#{terrains[k]}: #{desc_location(loc)}"
        end
      end
    end

    closest_terrains
  end

  def describe_roads
    closest(:road).map do |loc|
      "roads: #{desc_location(loc)}"
    end
  end

  def add_food count
    @stores[:food] ||= 0
    @stores[:food] += count
  end

  def move_defector
    return if @is_defector_dead
    @previous_defector_visible = @defector_visible

    defector_currently_visible = is_defector_visible?

    new_loc = towards_center_to({ x: @x, y: @y }, { x: @defector_x, y: @defector_y })
    if(@defector_x == new_loc[:x])
      @defector_y = new_loc[:y]
    elsif(@defector_y == new_loc[:y])
      @defector_x = new_loc[:x]
    else
      single_direction = [:x, :y].sample
      if(single_direction == :x)
        @defector_x = new_loc[:x]
      else
        @defector_y = new_loc[:y]
      end
    end

    if first_encounter? && rand > 0.2 && defector_distance > 2
      new_loc = towards_center_to({ x: @x, y: @y }, { x: @defector_x, y: @defector_y })

      if(@defector_x == new_loc[:x])
        @defector_y = new_loc[:y]
      elsif(@defector_y == new_loc[:y])
        @defector_x = new_loc[:x]
      else
        single_direction = [:x, :y].sample
        if(single_direction == :x)
          @defector_x = new_loc[:x]
        else
          @defector_y = new_loc[:y]
        end
      end
    end

    if is_defector_visible? && !defector_currently_visible
      @defector_visible = true
    end
  end

  def exited
    return if !@event
    return if @event.is_a? TE::MiniWorld

    action_on_exit = @event.action_on_exit

    if(in_battle?)
      cause_death :battle
    elsif(action_on_exit == :kill_in_battle)
      cause_death :battle
    elsif(action_on_exit == :clear)
      @event.on_clear
    elsif(action_on_exit == :end)
    #do nothing
    elsif(action_on_exit == :aborted)
      @event.on_aborted
    end

    @game.save
  end

  def in_battle?
    return false if !@event

    return @event.in_battle?
  end

  def enable_compass
    @game.stores[:compass] = 1
  end

  def describe_unexplored
    result = []
    closest_terrain = []

    to_filter = fog_locations

    #cradinal directions
    #N,NE,E,SE,S,SW,W,NW

    [:N, :S, :E, :W].each do |dir|
      closest = to_filter
                  .find_all { |loc| compass_direction_loc(loc) == dir }
                  .sort_by { |loc| distance_to_loc loc }
                  .first

      closest_terrain << closest if closest
    end

    if closest_terrain.count < 4
      [:NE, :SE, :NW, :SW].each do |dir|
        closest = to_filter
                    .find_all { |loc| compass_direction_loc(loc) == dir  }
                    .sort_by { |loc| distance_to_loc loc }
                    .first

        closest_terrain << closest if closest
      end
    end

    closest_terrain.map do |loc|
      result << desc_location(loc)
    end.first

    result
  end

  def closest name
    to_filter = visible_locations

    smallest_distance = to_filter.find_all { |loc|
      @aesthetics[[loc[:x], loc[:y]]] == name
    }.map { |loc|
      distance_to loc[:x], loc[:y]
    }.sort.first

    if(smallest_distance)
      return to_filter.find_all { |loc|
        @aesthetics[[loc[:x], loc[:y]]] == name and
          distance_to(loc[:x], loc[:y]) == smallest_distance
      }
    else
      return []
    end
  end

  def describe_landmarks landmarks_to_describe = nil
    landmarks_to_describe ||= visible_landmarks
    landmarks_to_describe.sort_by { |loc|
      distance_to(loc[:x], loc[:y])
    }.reject { |loc|
      @cleared[[loc[:x], loc[:y]]]
    }.map { |loc|
      describe_landmark loc
    }
  end

  def describe_landmark loc
    descriptions = {
      :borehole => "bore hole",
      :close_house => "house",
      :morality_house => "house",
      :cowering_family => "house",
      :mother_and_baby => "house",
      :man_and_boy => "house",
      :mansion => "house",
      :a_small_child => "house",
      :firelit_room => "house",
      :suicide_house => "house",
      :builder => "defector",
      :ritual => "house",
      :bottomless_box => "swamp",
      :close_cave => "cave",
      :zero_zero => "house",
      :lozenge_lounge => "house"
    }

    entry = @landmarks[[loc[:x], loc[:y]]]
    description = descriptions[entry] || entry.to_s.gsub("_", " ")
    "#{description}, #{desc_location loc}"
  end

  def describe_defector
    return "defector: location unknown" if @is_defector_dead
    return "defector: location unknown" if self[[@defector_x, @defector_y]] != :visible
    return "defector: #{desc_location({ x: @defector_x, y: @defector_y })}"
  end

  def desc_location loc
    distance_x = @x - loc[:x]
    distance_y = @y - loc[:y]
    direction_y = "north"
    direction_y = "south" if distance_y < 0
    direction_x = "west"
    direction_x = "east" if distance_x < 0
    return "current location" if distance_x == 0 and distance_y == 0
    return "#{distance_x.abs} #{clicks distance_x} #{direction_x}" if distance_y == 0
    return "#{distance_y.abs} #{clicks distance_y} #{direction_y}" if distance_x == 0
    return "#{distance_y.abs} #{clicks distance_y} #{direction_y}, #{distance_x.abs} #{clicks distance_x} #{direction_x}"
  end

  def clicks distance
    return "click" if distance.abs == 1
    return "clicks"
  end

  def cleared?
    fog_locations.count == 0
  end

  def city_visible?
    visible_locations.any? { |loc| @landmarks[[loc[:x], loc[:y]]] == :city }
  end

  def can_embark?
    @current_embark_ticks >= @embark_after
  end

  def tick
    return if !@game.stores[:compass]

    @current_embark_ticks += 1 if @current_embark_ticks < @embark_after
  end

  def vision_radius
    # return 20 # GODMODE
    return 10 if first_encounter?
    (@base_vision_radius || 2) + (2 * (@stores[:sight] || 0))
  end

  def is_defector_visible?
    return false if @is_defector_dead

    self[[@defector_x, @defector_y]] == :visible and defector_distance <= vision_radius * 6
  end

  def defector_distance
    return nil if !@defector_x || !@defector_y
    distance_to(@defector_x, @defector_y)
  end

  def is_defector_very_near?
    defector_distance == 1
  end

  def height
    # !!! FIXME needs to be 16:9
    @height
  end

  def width
    # !!! FIXME needs to be 16:9
    @width
  end

  def half_width
    @half_width
  end

  def half_height
    @half_height
  end

  def reset
    @current_hp = max_hp

    @god_mode = false
    @previous_defector_visible = false
    @defector_visible = false
    @embark_after = 90
    @current_embark_ticks = 0
    @meat_cool_down = meat_delay
    @meat_delay = 0
    self.clear
    @aesthetics.clear
    @landmarks.clear
    @cleared.clear
    @aborted.clear
    @visited.clear
    @vision_radius = 2
    @width = 90
    @height = 40
    @half_width = (@width / 2).to_i
    @half_height = (@height / 2).to_i

    @width.times do |x|
      @height.times do |y|
        if x == @half_width and y == @half_height
          self[[x, y]] = :you
          @x = x
          @y = y
        else
          self[[x, y]] = :fog
        end
      end
    end

    init_fight
    init_stores
    init_terrain_probabilities
    init_landmark_definitions
    init_defector
    reset_multipliers
    generate_aesthetics
    generate_landmarks if !@@disable_landmarks
    reveal_map

    @stores[:food] ||= 0
  end

  def reset_multipliers
    @enemy_damage_multiplier = default_multipliers[:enemy_damage]
    @durability_multiplier = default_multipliers[:durability]
    @poison_multiplier = default_multipliers[:poison]
    @hunger_multiplier = default_multipliers[:hunger]
    @thirst_multiplier = default_multipliers[:thirst]
    @base_vision_radius = default_multipliers[:vision]
  end

  def init_defector
    @defector_x = 0
    @defector_y = 0
    @is_defector_dead = true
  end

  def embark_items
    [
      :food, :knife, :torch,
      :iron_sword, :steel_sword, :rifle,
      :bullets, :laser, :battery,
      :grenade, :bolas, :katana, :jewel
    ]
  end

  def weapon_items
    [
      :knife, :iron_sword, :steel_sword,
      :rifle, :laser, :katana
    ]
  end

  def init_fight
    @fight_move = 0
    @fight_delay = 6
  end

  def landmark_locations landmark
    found = nil
    @landmarks.keys.each do |key|
      if @landmarks[key] == landmark
        found = key
      end
    end
    found
  end

  def mine_cleared? mine
    cleared[landmark_locations(mine)]
  end

  def named_event_cleared? event_name
    cleared[landmark_locations(event_name)]
  end

  def named_event_aborted? event_name
    aborted[landmark_locations(event_name)]
  end

  def named_event_completed? event_name
    named_event_cleared?(event_name) and !named_event_aborted?(event_name)
  end

  def radar_ticks
    loc = uncleared_boreholes_by_distance.first

    return "----" if !loc

    distance = distance_to loc[0], loc[1]

    return "#{distance.to_s} clicks"
  end

  def poisoned?
    (@poison_multiplier || 0) > 0
  end

  def rusted?
    (@durability_multiplier || 1) > 1
  end

  def in_store_area?
    @event.is_a?(SwampEvent) ||
      @event.is_a?(BottomlessBoxEvent)
  end

  def directions_from_direction_tool
    return radar_ticks if directions_tool == :locket

    compass_direction
  end

  def compass_direction
    return "spins" if @landmarks.keys.length == 0
    if !mine_cleared? :tannery
      compass_direction_string landmark_locations(:tannery)
    elsif !mine_cleared? :iron_mine
      compass_direction_string landmark_locations(:iron_mine)
    elsif !mine_cleared? :armoury
      compass_direction_string landmark_locations(:armoury)
    elsif !named_event_cleared? :ship
      compass_direction_string landmark_locations(:ship)
    else
      "spins"
    end
  end

  def compass_direction_string destination
    return "spins" if !destination

    north_south_or_none = ""
    north_south_or_none = "N" if y > destination[1]
    north_south_or_none = "S" if y < destination[1]
    east_west_or_none = ""
    east_west_or_none = "W" if x > destination[0]
    east_west_or_none = "E" if x < destination[0]

    return "spins" if north_south_or_none == "" and east_west_or_none == ""
    return "#{north_south_or_none}#{east_west_or_none}"
  end

  def directions_tool
    return :locket if (@stores[:radar] || 0) > 0

    :compass
  end

  def compass_direction_loc loc
    loc = { :x => loc[0], :y => loc[1] } if loc.is_a? Array

    distance_x = @x - loc[:x]
    distance_y = @y - loc[:y]
    north = y > loc[:y]
    south = y < loc[:y]
    east = x < loc[:x]
    west = x > loc[:x]

    return :N if distance_x == 0 and north
    return :NE if north and east
    return :E if distance_y == 0 and east
    return :SE if south and east
    return :S if distance_x == 0 and south
    return :SW if south and west
    return :W if distance_y == 0 and west
    return :NW if north and west
  end

  def enemy_presence
    return "none, while on roads (the # signs)" if @aesthetics[[x,y]] == :road
    return "low" if @fight_move == 0
    return "medium" if @fight_move <= (@fight_delay / 2)
    return "high" if @fight_move <= @fight_delay
    return "very high" if @fight_move > @fight_delay
  end

  def enemy_presence_symbol
    return :road if @aesthetics[[x,y]] == :road
    return :low if @fight_move == 0
    return :medium if @fight_move <= (@fight_delay / 2)
    return :high if @fight_move <= @fight_delay
    return :very_very_high if @fight_move + 4 > @fight_delay
    return :very_high if @fight_move > @fight_delay
  end

  def fog_locations
    locations_for :fog
  end

  def visible_locations
    locations_for :visible
  end

  def visible_landmarks
    @landmarks.keys.find_all { |k|
      self[[k[0], k[1]]] == :visible
    }.map { |k| { :x => k[0], :y => k[1] } }
  end

  def locations_for type
    self.keys
        .map { |k| { :x => k[0], :y => k[1] } }
        .find_all { |k| self[[k[:x], k[:y]]] == type }
  end

  def reveal_random
    random_x = 10
    random_y = 10

    random_fog = fog_locations.sample

    if random_fog
      random_x = random_fog[:x]
      random_y = random_fog[:y]
    end

    area = 5

    start_x = random_x - area
    start_y = random_y - area

    end_x = random_x + area
    end_y = random_y + area

    (start_x..end_x).each do |x|
      (start_y..end_y).each do |y|
        current_value = self[[x,y]]
        if current_value == :fog and distance_to(x, y, random_x, random_y) <= area
          self[[x,y]] = :visible
        end
      end
    end

    [random_x, random_y]
  end

  def self.stub_roll with_value
    @@stub_roll_value = with_value
  end

  def self.disable_landmarks
    @@disable_landmarks = true
  end

  def self.enable_landmarks
    @@disable_landmarks = false
  end

  def self.override_aesthetics aesthetic
    @@override_aesthetics = aesthetic
  end

  def roll
    @@stub_roll_value || rand
  end

  def take_stores store, from
    move_stores store, from, @stores
  end

  def take_weapon weapon, from
    return if(!from.include? weapon)
    @weapon_stores << weapon
    from.delete weapon
  end

  def drop_weapon weapon, to
    return if(!@weapon_stores.include? weapon)
    to << weapon
    @weapon_stores.delete weapon
  end

  def unload_stores store, to
    @stores[store].times do
      drop_stores store, to
    end
  end

  def drop_stores store, to
    move_stores store, @stores, to

    @stores.delete(store) if @stores[store] == 0 and store != :food
  end

  def move_stores store, from, to
    return if !from[store] or from[store] == 0
    to[store] ||= 0
    from[store] -= 1
    to[store] += 1
  end

  def init_stores
    @stores = Stores.new
    if @swamp_stores.nil?
      @swamp_stores = Stores.new
    elsif hard_mode?
      @swamp_stores = Stores.new
    end

    @swamp_weapon_stores = [ ]
    @stores[:food] = 0
    @weapon_stores = [ ]

    begin
      @stores.delete(:admiral_key)
    rescue
    end


    # @swamp_stores[:alien_alloy] = 50 # GODMODE

    @has_opened_chest = false
    @admiral_stores = Stores.new
    @admiral_weapon_stores = [ ]
    @shield_stores = Stores.new
    @shield_weapon_stores = []
    @thruster_stores = Stores.new
    @thruster_weapon_stores = []

  end

  def god_mode!
    @god_mode = true
    @stores.clear
    @stores[:food] = 10
    @stores[:grenade] = 5
    @stores[:stealth] = 1
    @stores[:bolas] = 5
    @stores[:bullets] = 20
    @stores[:battery] = 20
    @stores[:torch] = 5
    @stores[:eyes] = 5
    @stores[:teeth] = 5
    @stores[:flesh] = 5
    @stores[:torch] = 5
    @stores[:health] = 1
    @stores[:sight] = 1
    @stores[:scope] = 1
    @stores[:melee] = 1
    @stores[:dodge] = 1
    @weapon_stores.clear
    @weapon_stores << Knife.new
    @weapon_stores << IronSword.new
    @weapon_stores << SteelSword.new
    @weapon_stores << Katana.new
    @weapon_stores << Rifle.new
    @weapon_stores << Laser.new
    @weapon_stores << Knife.new
    @weapon_stores << IronSword.new
    @weapon_stores << SteelSword.new
    @weapon_stores << Katana.new
    @weapon_stores << Rifle.new
    @weapon_stores << Laser.new
    @weapon_stores << Masamune.new
    @weapon_stores << Murasame.new
    self.keys.each do |k|
      self[k] = :visible
    end
  end

  def disable_god_mode!
    @god_mode = false
  end

  def toggle_god_mode!
    if @god_mode
      disable_god_mode!
    else
      god_mode!
    end
  end

  def self.consumables
    {
      :shoot => :bullets,
      :blast => :battery,
      :lob => :grenade,
      :stun => :bolas
    }
  end

  def init_actions
    available_actions
  end

  def available_action_ids
    available_actions.map { |a| a.action_name }
  end

  def actions_hash
    r = {}
    available_actions.each do |a|
      r[a.action_name] = a
    end
    r
  end

  def available_actions
    @all_weapon_names ||= [
      :knife,
      :iron_sword,
      :steel_sword,
      :katana,
      :rifle,
      :laser,
      :murasame,
      :masamune
    ]

    result = []

    result << Action.new(@stores, attack_delay: 5, weapon: Food.new)

    result << Action.new(@stores, attack_delay: 2, weapon: Fists.new)

    @all_weapon_names.each do |weapon_name|
      if(has_weapon? weapon_name)
        selected_weapon = most_durable_attack(weapon_name)
        result << selected_weapon if selected_weapon
      end
    end

    if((@stores[:grenade] || 0) > 0)
      result << Action.new(@stores, attack_delay: 5, weapon: Grenade.new, consumes: World.consumables[:lob])
    end

    if((@stores[:bolas] || 0) > 0)
      result << Action.new(@stores, attack_delay: 15, weapon: Bolas.new, consumes: World.consumables[:stun])
    end

    if((@stores[:stealth] || 0) > 0)
      result << Action.new(@stores, attack_delay: 5, weapon: Stealth.new)
    end

    @actions = result

    result
  end

  def actions
    @actions
  end

  def has_weapon? weapon_name
    @weapon_stores.any? { |w| w.name == weapon_name }
  end

  def most_durable_attack weapon_name
    weapon_to_equip =  most_durable_weapon(weapon_name)

    if(weapon_name == :knife)
      return Action.new(@stores, attack_delay: 2, damage: 2, weapon: weapon_to_equip, durability_multiplier: (@durability_multiplier || 1))
    elsif(weapon_name == :iron_sword)
      return Action.new(@stores, attack_delay: 2, damage: 4, weapon: weapon_to_equip, durability_multiplier: (@durability_multiplier || 1))
    elsif(weapon_name == :steel_sword)
      return Action.new(@stores, attack_delay: 2, damage: 6, weapon: weapon_to_equip, durability_multiplier: (@durability_multiplier || 1))
    elsif(weapon_name == :katana)
      return Action.new(@stores, attack_delay: 2, damage: 8, weapon: weapon_to_equip, durability_multiplier: (@durability_multiplier || 1))
    elsif(weapon_name == :rifle)
      return Action.new(@stores, attack_delay: 1, damage: 5, weapon: weapon_to_equip, consumes: World.consumables[:shoot], durability_multiplier: (@durability_multiplier || 1))
    elsif(weapon_name == :laser)
      return Action.new(@stores, attack_delay: 1, damage: 8, weapon: weapon_to_equip, consumes: World.consumables[:blast], durability_multiplier: (@durability_multiplier || 1))
    elsif(weapon_name == :murasame)
      return Action.new(@stores, attack_delay: 2, damage: 25, weapon: weapon_to_equip, durability_multiplier: (@durability_multiplier || 1))
    elsif(weapon_name == :masamune)
      return Action.new(@stores, attack_delay: 2, damage: 25, weapon: weapon_to_equip, durability_multiplier: (@durability_multiplier || 1))
    end
  end

  def most_durable_weapon weapon_name
    @weapon_stores
      .find_all { |w| w.name == weapon_name }
      .sort_by { |w| w.current_durability }
      .reverse
      .first
  end

  def can_eat_meat?
    @stores[:food] > 0
  end

  def meat_delay
    5
  end

  def meat_heal
    10 + (@stores[:health] || 0) * 5
  end

  def heal_after_battle
    @meat_cool_down = meat_delay

    return false if !can_eat_meat?

    eat_meat

    return true
  end

  def eat_meat
    return if !can_eat_meat?

    #return if @current_hp > (max_hp - 15) # for testing purposes. will not over heal unless health is at least 15 less than max health
    @current_hp += meat_heal

    @current_hp = max_hp if @current_hp > max_hp

    @stores[:food] -= 1
  end


  def uncleared_boreholes
    borehole_locations = []
    @landmarks.keys.each do |loc|
      if (@landmarks[loc] == :borehole || @landmarks[loc] == :battlefield) and !@cleared[loc]
        borehole_locations << [loc[0], loc[1]]
      end
    end

    borehole_locations
  end

  def uncleared_boreholes_by_distance
    uncleared_boreholes.sort_by do |loc|
      distance_to loc[0], loc[1]
    end
  end

  def init_debug_landmark_definitions
    @landmark_definitions = {
      :tannery => { :num => 1, :min_radius => 3, :max_radius => 6 },
      :iron_mine => { :num => 1, :min_radius => 3, :max_radius => 6 },
      :armoury => { :num => 1, :min_radius => 3, :max_radius => 6 },
      :house => { :num => 1, :min_radius => 3, :max_radius => 6 },
      :close_house => { :num => 1, :min_radius => 3, :max_radius => 6 },
      :cave => { :num => 1, :min_radius => 3, :max_radius => 6 },
      :close_cave => { :num => 1, :min_radius => 3, :max_radius => 6 },
      :town => { :num => 1, :min_radius => 3, :max_radius => 6 },
      :city => { :num => 1, :min_radius => 3, :max_radius => 6 },
      :ship => {:num => 1, :min_radius => 3, :max_radius => 6 },
      :borehole => {:num => 1, :min_radius => 3, :max_radius => 6 },
      :battlefield => { :num => 1, :min_radius => 3, :max_radius => 6 },
      :merchant => { :num => 1, :min_radius => 3, :max_radius => 6 },
      :thieves => { :num => 1, :min_radius => 3, :max_radius => 6 },
      :master => { :num => 1, :min_radius => 3, :max_radius => 6 },

      :cowering_family => { :num => 1, :min_radius => 3, :max_radius => 6 },
      :mother_and_baby => { :num => 1, :min_radius => 3, :max_radius => 6 },
      :man_and_boy => { :num => 1, :min_radius => 3, :max_radius => 6 },
      :mansion => { :num => 1, :min_radius => 3, :max_radius => 6 },
      :a_small_child => { :num => 1, :min_radius => 3, :max_radius => 6 },
      :firelit_room => { :num => 1, :min_radius => 3, :max_radius => 6 },
      :suicide_house => { :num => 1, :min_radius => 3, :max_radius => 6 },
      :ritual => { :num => 1, :min_radius => 3, :max_radius => 6 },
    }
  end

  def init_real_landmark_definitions
    magic_radius = 30
    @landmark_definitions = {
      :tannery => { :num => 1, :min_radius => 6, :max_radius => 6 },
      :iron_mine => { :num => 1, :min_radius => 9, :max_radius => 9 },
      :armoury => { :num => 1, :min_radius => 24, :max_radius => 24 },
      :house => { :num => 22, :min_radius => 4, :max_radius => magic_radius * 1.5 },
      :close_house => { :num => 2, :min_radius => 2, :max_radius => 4 },
      :cave => { :num => 9, :min_radius => 4, :max_radius => 15 },
      :close_cave => { :num => 2, :min_radius => 4, :max_radius => 4 },
      :town => { :num => 20, :min_radius => 10, :max_radius => 25 },
      :city => { :num => 20, :min_radius => 20, :max_radius => magic_radius * 1.5 },
      :ship => {:num => 1, :min_radius => 30, :max_radius => 30 },
      :borehole => {:num => 10, :min_radius => 18, :max_radius => magic_radius * 1.5 },
      :battlefield => { :num => 10, :min_radius => 18, :max_radius => magic_radius * 1.5 },
      :merchant => { :num => 5, :min_radius => 10, :max_radius => 25 },
      :thieves => { :num => 1, :min_radius => 15, :max_radius => 15 },
      :master => { :num => 1, :min_radius => 15, :max_radius => 15 },

      :cowering_family => { :num => 1, :min_radius => 15, :max_radius => 32 },
      :mother_and_baby => { :num => 1, :min_radius => 15, :max_radius => 32 },
      :man_and_boy => { :num => 1, :min_radius => 15, :max_radius => 32 },
      :mansion => { :num => 1, :min_radius => 15, :max_radius => 32 },
      :a_small_child => { :num => 1, :min_radius => 15, :max_radius => 32 },
      :firelit_room => { :num => 1, :min_radius => 15, :max_radius => 32 },
      :suicide_house => { :num => 1, :min_radius => 15, :max_radius => 32 },
      :ritual => { :num => 1, :min_radius => 20, :max_radius => 26 },
    }
  end

  def init_landmark_definitions
    init_real_landmark_definitions
  end

  def black_sheep_wall
    reveal_map 100
  end

  def abort_landmark x, y
    aborted[[x, y]] = true
  end

  def clear_landmark x, y
    landmark_name = @landmarks[[x,y]]

    set_tentative_clears

    if(landmark_name == :tannery)
      @game.stores[:leather_armour] = 1
      @game.stores[:rucksack] = 1
      @game.stores[:waterskin] = 1

      #when the tannery is cleared once, immediately place the defector
      place_defector 39, 39 if !@cleared_tannery_once

      @cleared_tannery_once_tentative = true
    end

    if(landmark_name == :iron_mine)
      @game.stores[:leather_armour] = 1
      @game.stores[:rucksack] = 1
      @game.stores[:waterskin] = 1
      @game.stores[:iron_armour] = 1
      @game.stores[:cask] = 1
      @game.stores[:wagon] = 1

      @cleared_iron_mine_once_tentative = true
    end

    if(landmark_name == :armoury)
      @game.stores[:leather_armour] = 1
      @game.stores[:rucksack] = 1
      @game.stores[:waterskin] = 1
      @game.stores[:iron_armour] = 1
      @game.stores[:cask] = 1
      @game.stores[:wagon] = 1
      @game.stores[:steel_armour] = 1
      @game.stores[:convoy] = 1
      @game.stores[:water_tank] = 1

      @cleared_armoury_once_tentative = true
    end

    #maybe remove? move to admiral room.
    #  if(landmark_name == :builder)
    #    @game.stores[:war_mantle] = 1
    #    @game.stores[:leather_armour] = 1
    #    @game.stores[:iron_armour] = 1
    #    @game.stores[:steel_armour] = 1
    #  end

    if(landmark_name == :zero_zero)
      @defector_x = 3
      @defector_y = 3
      @final_boss = true
    end

    if(@defector_x == x and defector_y == y)
      if(@final_boss)
        @defeated_final_boss = true
      end

      @final_boss = false
      @kills_of_defector += 1
      @killed_at_least_one_defector = true
      place_defector 39, 39
      return
    end

    cleared[[x,y]] = true

    @game.save
  end

  def total_deaths
    @food_deaths + @water_deaths + @battle_deaths
  end

  def tutorial_mode?
    !@cleared_tannery_once
  end

  def tannery_event?
    return false if !@event

    @event.class == TanneryEvent
  end

  def sniper_event?
    return false if !@event

    @event.class == SniperEvent
  end

  def defector_event?
    return false if !@event_on_death

    return true if @event_on_death == DefectorEvent
    return true if @event_on_death == ImpossibleDefectorEvent

    return false
  end

  def cause_death reason, tick_mind_games = true
    return if @death_caused

    if reason == :battle
      @battle_deaths += 1
      if @event_on_death == DefectorEvent
        @deaths_by_defector += 1
      elsif @event_on_death == ImpossibleDefectorEvent
        @game.total_defector_deaths += 1 # impossible defector deaths
        if tutorial_mode?
          @game.just_woke_up_from_nightmare = true
        end
        tick_mind_games = false
      end
    elsif reason == :dehydration
      @water_deaths += 1
    elsif reason == :starvation
      @food_deaths +=1
    elsif reason == :poisoned
      @food_deaths +=1
    elsif reason == :exploded
      #TODO have tracker for end game credits/achievements?
    end

    @death_reason = reason
    @percentage_uncovered_this_time = percentage_uncovered
    @percentage_cleared_this_time = percentage_cleared
    @current_hp = 0
    @current_embark_ticks = 0
    @death_caused = true
    @game.process_mind_games if tick_mind_games
  end

  def reveal_entire_map
    self.keys.each do |k|
      self[k] = :visible
    end
  end

  def in_range_of_swamp greater_than_x, less_than_x, greater_than_y, less_than_y
    dx = (@half_width  - @x).abs
    dy = (@half_height - @y).abs

    if (dx / @half_width.to_f) > (dy / half_height.to_f)
      (dx >= greater_than_x && dx <= less_than_x)
    else
      (dy >= greater_than_y && dy <= less_than_y)
    end
  end

  def food
    @stores[:food] || 0
  end

  def decrement_hp amount
    @current_hp -= amount

    if @current_hp <= 0
      @event_on_death = @event.class
      cause_death :battle
    end
  end

  def base_carry_space
    # return 700 # GODMODE
    if @god_mode
      900
    else
      100
    end
  end

  def base_water
    10
  end

  def base_hp
    25
  end

  def max_hp
    if @game.has_upgrade? :war_mantle
      return base_hp + 55
    elsif @game.has_upgrade? :steel_armour
      return base_hp + 35
    elsif @game.has_upgrade? :iron_armour
      return base_hp + 15
    elsif @game.has_upgrade? :leather_armour
      return base_hp + 5
    else
      return base_hp
    end
  end

  def armour
    if @game.has_upgrade? :war_mantle
      return :war_mantle
    elsif @game.has_upgrade? :steel_armour
      return :steel_armour
    elsif @game.has_upgrade? :iron_armour
      return :iron_armour
    elsif @game.has_upgrade? :leather_armour
      return :leather_armour
    else
      return :rags
    end
  end

  def carry
    if @game.has_upgrade? :convoy
      return { :name => :convoy, :max => base_carry_space + 600 }
    elsif @game.has_upgrade? :wagon
      return { :name => :wagon, :max => base_carry_space + 300 }
    elsif @game.has_upgrade? :rucksack
      return { :name => :rucksack, :max => base_carry_space + 100 }
    else
      return { :name => :bag, :max => base_carry_space }
    end
  end

  def container
    if @game.has_upgrade? :water_tank
      return { :name => :water_tank, :max => base_water + 50 }
    elsif @game.has_upgrade? :cask
      return { :name => :cask, :max => base_water + 20 }
    elsif @game.has_upgrade? :waterskin
      return { :name => :waterskin, :max => base_water + 10 }
    else
      return { :name => :flask, :max => base_water }
    end
  end

  def inventory_weight
    stores_weight + weapon_stores_weight
  end

  def stores_weight
    @stores.keys.map do |key|
      @stores[key] * weight_for(key)
    end.inject(:+) || 0
  end

  def weapon_stores_weight
    @weapon_stores.map do |weapon|
      weight_for(weapon.name)
    end.inject(:+) || 0
  end

  def ticks_til_hunger
    2
  end

  def nullify_event
    @event = nil
  end

  def move_in_random_direction
    send [:move_up, :move_down, :move_left, :move_right].sample
    @event = nil
  end

  def set_tentative_clears
    @cleared_tannery_once = true if(@cleared_tannery_once_tentative)
    @cleared_iron_mine_once = true if(@cleared_iron_mine_once_tentative)
    @cleared_armoury_once = true if(@cleared_armoury_once_tentative)
  end

  def begin_journey opts = { }
    @game.just_woke_up_from_nightmare = false
    @last_admiral_cutscene_visited = :none
    @is_dream_sequence = false
    @cleared_tannery_once_tentative = false
    @cleared_iron_mine_once_tentative = false
    @cleared_armoury_once_tentative = false
    @death_caused = false
    @durability_multiplier = 1
    @poison_multiplier = 0
    @hunger_multiplier = 1
    @thirst_multiplier = 1
    @base_vision_radius = 2
    @enemy_damage_multiplier = 1
    @admiral_visits = :beginning
    @current_hp = max_hp
    @fight_move = 0
    @death_reason = nil
    @percentage_uncovered_this_time = 0
    @percentage_cleared_this_time = 0
    @last_aesthetics_state = @aesthetics.clone
    @event = nil
    @final_boss = false
    @defeated_final_boss = false
    @kills_of_defector = 0
    @thoughts_message = ""
    @is_defector_dead = true
    @game.at_beginning = false
    @game.story_line.reset
    reset_hunger_countdown
    reset_water_countdown
    reset_outposts

    reveal_map 5 if deaths_by_defector > 0
    @aesthetics ||= {}

    if @aesthetics.length == 0
      generate_aesthetics
    end

    if @cleared_tannery_once
      place_defector 39, 39
    end

    if false #for debugging
      @swamp_stores[:food] = 100
      @swamp_stores[:sight] = 5
      @swamp_stores[:alien_alloy] = 50
      @game.stores[:convoy] = 1
    end

    @move_count = 0
  end

  def hard_mode?
    @hard_mode
  end

  def place_impossible_defector?
    final_boss
  end

  def reset_outposts
    @cleared.keys.each do |key|
      @cleared.delete(key) if @landmarks[key] == :outpost
    end
  end

  def reset_hunger_countdown
    @stores[:food] ||= 0
    @hunger_countdown = ticks_til_hunger
  end

  def max_water
    container[:max]
  end

  def reset_water_countdown
    @water = container[:max]
  end

  def eat_or_starve
    # return # GODMODE
    return if @god_mode
    return if first_encounter?
    return if (@hunger_multiplier || 1) == 0
    return if landmark_at? x, y

    if @hunger_countdown > 0
      @hunger_countdown -= 1
    else
      if @stores[:food] <= 0
        cause_death :starvation
      else
        @stores[:food] -= 1 * (@hunger_multiplier || 1)
        reset_hunger_countdown
      end
    end

    refill_all
  end

  def drink_or_dehydrate
    return if @god_mode
    return if first_encounter?
    return if landmark_at? x, y

    if @water > 0
      @water -= 1 * (@thirst_multiplier || 1)
    else
      cause_death :dehydration
    end

    refill_all
  end

  def refill_all #delete later (and at eat_or_starve & drink_or_dehydrate)
    test = false #set false to disable
    if(test)
      @stores[:food] = 5
      @stores[:torch] = 5
      reset_water_countdown
      current_hp = max_hp * 5
    end
  end

  def water_consumption
    1
  end

  def weight_for key
    weights = {
      :knife => 10,
      :iron_sword => 30,
      :steel_sword => 50,
      :rifle => 50,
      :bullets => 1,
      :battery => 2,
      :laser => 20,
      :bolas => 2,
      :eyes => 2,
      :flesh => 2,
      :teeth => 2,
      :leather => 2,
      :torch => 5,
      :cauter => 1
    }

    weights[key] || 10
  end

  def dead?
    @current_hp <= 0
  end

  def has_event?
    !@event.nil?
  end

  def event
    @event
  end

  def init_terrain_probabilities
    @terrain_probs = { :forest => 0.15, :field => 0.35, :barrens => 0.5 }
    @terrain_consistancy = 0.5
  end

  def landmark_at_loc? loc
    landmark_at? loc[:x], loc[:y]
  end

  def landmark_at? x, y
    !!@landmarks[[x, y]]
  end

  def landmark_near? x, y
    #there is a small chance that a landmark
    #can't be placed at any location
    #that still gives spaces between other
    #landmarks, so there is a 20% chance that
    #it'll do simpler space detection to keep
    #from having an inifinite loop happen
    #yes, I died a little bit inside doing this

    if(roll < 0.2)
      landmark_at?(x, y)
    else
      landmark_at?(x - 1, y - 1) ||
        landmark_at?(x - 0, y - 1) ||
        landmark_at?(x + 1, y - 1) ||

        landmark_at?(x - 1, y + 0) ||
        landmark_at?(x - 0, y + 0) ||
        landmark_at?(x + 1, y + 0) ||

        landmark_at?(x - 1, y + 1) ||
        landmark_at?(x - 0, y + 1) ||
        landmark_at?(x + 1, y + 1)
    end
  end

  def at_landmark?(name)
    @landmarks[[x,y]] == name
  end

  def place_defector_within_sight
    farthest_visible = visible_locations.sort_by { |v| distance_to_loc(v) }.reverse.first
    @defector_x = farthest_visible[:x]
    @defector_y = farthest_visible[:y]
    @is_defector_dead = false
  end

  def place_defector min_radius, max_radius
    r = (roll * (max_radius - min_radius)).floor + min_radius
    x_dist = (roll * r).floor
    y_dist = r - x_dist
    x_dist = -x_dist if(roll < 0.5)
    y_dist = -y_dist if(roll < 0.5)
    x = @half_height + x_dist
    x = 0 if(x < 0)
    x = @half_width * 2 if(x > @half_width * 2)
    y = @half_height + y_dist
    y = 0 if(y < 0)
    y = @half_height * 2 if(y > @half_height * 2)
    x = (@half_width * 2) - 1 if x >= (@half_width * 2)
    y = (@half_height * 2) - 1 if y >= (@half_height * 2)
    @defector_x = x
    @defector_y = y
    @is_defector_dead = false
  end

  def generate_landmark min_radius, max_radius, landmark
    x = (@width / 2).to_i
    y = (@height / 2).to_i
    while landmark_near?(x, y)
      r = (roll * (max_radius - min_radius)).floor + min_radius
      x_dist = (roll * r).floor
      y_dist = r - x_dist
      x_dist = -x_dist if(roll < 0.5)
      y_dist = -y_dist if(roll < 0.5)
      x = @half_width + x_dist
      x = 0 if(x < 0)
      x = @half_width * 2 if(x > @half_width * 2)
      y = @half_height + y_dist
      y = 0 if(y < 0)
      y = @half_height * 2 if(y > @half_height * 2)
      x = (@half_width * 2) - 1 if x >= (@half_width * 2)
      y = (@half_height * 2) - 1 if y >= (@half_height * 2)
    end

    @landmarks[[x, y]] = landmark
  end

  def ship_cleared?
    named_event_cleared? :ship
  end

  def generate_landmarks
    @swamp_location = nil
    @landmarks[[@half_width,@half_height]] = :swamp

    @landmark_definitions.keys.each do |k|
      landmark = @landmark_definitions[k]

      landmark[:num].times do |i|
        generate_landmark landmark[:min_radius], landmark[:max_radius], k
      end
    end

    @landmarks[[@half_width,@half_height]] = nil if @@disable_landmarks

    place_builder

    place_zero_zero

    place_lozenge
  end

  def place_zero_zero
    @landmarks[[0, 0]] = :zero_zero
  end

  def place_lozenge
    @landmarks[[2, 1]] = :lozenge_lounge
  end

  def place_builder
    location = landmark_locations(:ship)

    builder = { x: location[0], y: location[1] }

    while(landmark_at?(builder[:x], builder[:y]))
      builder = towards_center(builder)
    end

    @landmarks[[builder[:x], builder[:y]]] = :builder
  end

  def place_merchant
    location = landmark_locations(:armoury)

    merchant = { x: location[0], y: location[1] }

    6.times { merchant = towards_center(merchant) }

    while(landmark_at?(merchant[:x], merchant[:y]))
      merchant = towards_center(merchant)
    end

    @landmarks[[merchant[:x], merchant[:y]]] = :merchant
  end

  def towards_center_to(destination, from)
    new_x = from[:x]
    new_y = from[:y]

    if(new_x != destination[:x])
      if(new_x > destination[:x])
        new_x -= 1
      else
        new_x += 1
      end
    end

    if(new_y != destination[:y])
      if(new_y > destination[:y])
        new_y -= 1
      else
        new_y += 1
      end
    end

    { x: new_x, y: new_y }
  end

  def towards_center(loc)
    towards_center_to({ x: (@width / 2).to_i, y: (@height / 2).to_i }, loc)
  end

  def away_from_center(loc)
    new_x = loc[:x]
    new_y = loc[:y]

    if(new_x > (@width / 2).to_i)
      new_x += 1
    else
      new_x -= 1
    end

    if(new_y > (@height / 2).to_i)
      new_y += 1
    else
      new_y -= 1
    end

    { x: new_x, y: new_y }
  end

  def generate_aesthetics
    @half_width.times do |r|
      r = r + 1
      (r * 8).times do |t|
        x = nil
        y = nil

        if t < (2 * r)
          x = @half_width - r + t
          y = @half_height - r
        elsif t < (4 * r)
          x = @half_width + r
          y = @half_height - (3 * r) + t
        elsif t < (6 * r)
          x = @half_width + (5 * r) - t
          y = @half_height + r
        else
          x = @half_width - r
          y = @half_height + (7 * r) - t
        end

        x = x.to_i
        y = y.to_i

        @aesthetics[[x,y]] = choose_tile(x, y)
      end
    end
  end

  def choose_tile x, y
    return @@override_aesthetics if @@override_aesthetics

    adjacent = [
      @aesthetics[[x, y - 1]],
      @aesthetics[[x, y + 1]],
      @aesthetics[[x + 1, y]],
      @aesthetics[[x - 1, y]]
    ]

    #todo: use radius instead of using the number 30
    return :forest if x == 44 && y == 19
    return :forest if x == 45 && y == 19
    return :forest if x == 46 && y == 19

    return :forest if x == 44 && y == 20
    return :forest if x == 45 && y == 20
    return :forest if x == 46 && y == 20

    return :forest if x == 44 && y == 21
    return :forest if x == 45 && y == 21
    return :forest if x == 46 && y == 21

    chances = { }
    non_sticky = 1
    adjacent.each do |i|
      if !i.nil?
        cur = chances[i] || 0
        chances[i] = cur + @terrain_consistancy
        non_sticky -= @terrain_consistancy
      end
    end

    @terrain_probs.keys.each do |t|
      cur = chances[t] || 0
      cur += @terrain_probs[t] * non_sticky
      chances[t] = cur
    end

    sorted = chances.sort_by { |tile, chance| chance }.each

    c = 0
    dice_roll = roll
    result = nil

    sorted.each do |entry|
      value = entry.last

      c += value

      result = entry.first.to_sym if dice_roll < c and result.nil?
    end

    result || :barrens
  end

  def revealed_locs_last_move
    @revealed_locs_last_move
  end

  def reveal_map override = nil
    @revealed_locs_last_move = []
    value_to_use = override || vision_radius
    start_x = @x - value_to_use
    start_y = @y - value_to_use

    end_x = @x + value_to_use
    end_y = @y + value_to_use

    (start_x..end_x).each do |x|
      (start_y..end_y).each do |y|
        current_value = self[[x,y]]
        if current_value == :fog and distance_to(x, y) <= value_to_use
          @revealed_locs_last_move << [x,y]
          self[[x,y]] = :visible
        end
      end
    end
  end

  def self.event_map
    {
      :cave => CaveEvent,
      :close_cave => CaveEvent,
      :house => HouseEvent,
      :close_house => SafeHouseEvent,
      :mother_and_baby => MotherAndBabyEvent,
      :man_and_boy => ManAndBoyEvent,
      :mansion => MansionEvent,
      :a_small_child => SmallChildEvent,
      :zero_zero => ZeroZeroEvent,
      :bottomless_box => BottomlessBoxEvent,
      :ship => ShipMiniWorld, #handled in init_event(checks accesibility)
      :firelit_room => FireLitRoomEvent,
      :suicide_house => SuicideHouse,
      :ritual => RitualEvent,
      :cowering_family => CoweringFamilyEvent,
      :iron_mine => IronMineEvent,
      :merchant => MerchantEvent,
      :master => TheMasterEvent,
      :thieves => ThievesGuildEvent,
      :tannery => TanneryEvent,
      :armoury => ArmouryEvent,
      :swamp => SwampEvent,
      #:swamp => ShipMiniWorld,
      #:swamp => ShipEvent,
      :town => TownEvent,
      :city => CityEvent,
      :builder => BuilderEvent,
      :battlefield => BattlefieldEvent,
      :borehole => BoreholeEvent,
      :outpost => OutpostEvent,
      :lozenge_lounge => LozengeLoungeEvent
    }
  end

  def end_game?
    if(@blind_artifact_worn)
      return true
    else
      return false
    end
  end

  def teleport_to_landmark landmark
    loc = landmark_locations landmark
    teleport_to_loc loc[0], loc[1]
  end

  def teleport_to_loc x, y
    self[[@x, @y]] = :visible
    @x = x
    @y = y
    self[[@x, @y]] = :you
    init_event false
  end

  def has_visited? landmark
    @visited.keys.any? { |k| @visited[k] == landmark }
  end

  def init_event should_save = true
    if(@defector_x == @x and @defector_y == @y)
      if(place_impossible_defector?)
        @event = ImpossibleDefectorEvent.new @game
        @event.god_mode = @god_mode
      else
        @event = DefectorEvent.new @game
        @event.god_mode = @god_mode
      end
      return
    end

    return if event_cleared? x, y

    landmark = @landmarks[[x,y]]

    if x == @half_width && y == @half_height
      set_tentative_clears
    end

    if landmark == :ship
      @visited_starship_once = true
      @visited[[x,y]] = landmark if !@visited[[x,y]]
    end

    event_class = World.event_map[landmark]

    if landmark == :ship
      #if landmark == :swamp
      event_class = ShipMiniWorld
    end

    if event_class
      @visited[[x,y]] = landmark if !@visited[[x,y]]
      @event = event_class.new @game
      @event.god_mode = @god_mode
      if !@event.is_a? MiniWorld
        @move_count += @event.scenes.length
      end
    end
  end

  def self.easy_encounter_range
    [0, 10]
  end

  def self.easy_encounters
    if first_encounter?
      World.encounters
           .select do |h|
        h[:range] == World.easy_encounter_range
      end.map { |h| h[:event] }.first
    else
      World.encounters .select do |h|
        @aesthetics[[x, y]] == h[:aesthetics] and
          in_range_of_swamp h[:range][0] * @half_width,
                            h[:range][1] * @half_width,
                            h[:range][0] * @half_height,
                            h[:range][1] * @half_height
      end.map { |h| h[:event] }.first
    end
  end

  def self.encounters
    easy   = [0.00, 0.30]
    medium = [0.30, 0.60]
    hard   = [0.60, 1.00]

    [
      {
        :event => SnarlingBeastEvent,
        :aesthetics => :forest,
        :range => easy
      },
      {
        :event => StrangeBirdEvent,
        :aesthetics => :field,
        :range => easy
      },
      {
        :event => GauntManEvent,
        :aesthetics => :barrens,
        :range => easy
      },
      {
        :event => ManEaterEvent,
        :aesthetics => :forest,
        :range => medium
      },
      {
        :event => HugeLizardEvent,
        :aesthetics => :field,
        :range => medium
      },
      {
        :event => ScavengerEvent,
        :aesthetics => :barrens,
        :range => medium
      },
      {
        :event => FeralTerrorEvent,
        :aesthetics => :forest,
        :range => hard
      },
      {
        :event => SniperEvent,
        :aesthetics => :field,
        :range => hard
      },
      {
        :event => SoldierEvent,
        :aesthetics => :barrens,
        :range => hard
      },
    ]
  end

  def fight_chance
    return 0.60 if first_encounter?
    return 0.10 if (@stores[:stealth] || 0) > 0

    return 0.20
  end

  def get_random_battle
    if first_encounter?
      DancingShadow
    else
      World.encounters.select do |h|
        @aesthetics[[x,y]] == h[:aesthetics] and
          in_range_of_swamp(h[:range][0] * @half_width,
                            h[:range][1] * @half_width,
                            h[:range][0] * @half_height,
                            h[:range][1] * @half_height)
      end.map { |h| h[:event] }.first
    end
  end

  def fight_imminent?
    (@fight_move > @fight_delay) || (first_encounter? && @fight_move > 2)
  end

  def check_fight
    return if landmark_at? x, y

    return if @aesthetics[[x,y]] == :road

    @fight_move += 1

    if fight_imminent? and roll <= fight_chance
      encounter = get_random_battle
      if encounter
        @event = encounter.new @game
        @event.god_mode = @god_mode
        move_defector
      end

      @fight_move = 0
    end
  end

  def event_cleared? at_x, at_y
    @cleared[[at_x, at_y]] == true
  end

  def landmark_is_a? x, y, type
    return false if !landmark_at? x, y

    @landmarks[[x,y]] == type
  end

  def distance_to x, y, from_x = @x, from_y = @y
    (from_x - x).abs + (from_y - y).abs
  end

  def distance_to_loc loc
    distance_to loc[:x], loc[:y]
  end

  def move direction, value
    self[[@x, @y]] = :visible
    @x = @x + value if direction == :x
    @y = @y + value if direction == :y
    @y = 39 if @y == -1
    @y = 0 if @y == 40
    @x = 89 if @x == -1
    @x = 0 if @x == 90
    @move_count += 1
    self[[@x, @y]] = :you
    reveal_map
    check_fight
    eat_or_starve
    drink_or_dehydrate
    poison
    move_defector
    init_event
  end

  def poison
    return if !poisoned?

    return if @current_hp <= 0

    @current_hp -= 1 * (@poison_multiplier || 0)

    cause_death :poisoned if @current_hp <= 0
  end

  def punch_level
    return 4 if mine_cleared? :armoury
    return 3 if mine_cleared? :iron_mine
    return 2 if mine_cleared? :tannery
    return 1
  end

  def move_up
    move :y, -1
  end

  def move_down
    move :y, 1
  end

  def move_left
    move :x, -1
  end

  def move_right
    move :x, 1
  end

  def percentage_uncovered
    ((visible_locations.count.to_f / self.count) * 100).ceil
  end

  def ritual_performed?
    @landmarks[[@half_width, @half_height]] == :bottomless_box
  end

  def first_encounter?
    @game.total_defector_deaths == 0 && tutorial_mode?
  end

  def percentage_cleared
    cleared_count = 0
    outpost_count = 0

    unclearable = 9 # master, swamp, merchant, ship cannot be cleared

    @cleared.keys.each do |loc|
      if (@landmarks[loc])
        cleared_count += 1
      end
    end

    @landmarks.keys.each do |loc|
      if (@landmarks[loc] == :outpost)
        outpost_count += 1
      end
    end

    total_cleared = cleared_count + outpost_count
    total_clearable = @landmarks.keys.count - unclearable

    final = ((total_cleared.to_f / total_clearable.to_f) * 100).floor
    if(final > 100)
      return 100
    else
      return final
    end
  end

  def swamp_location_x
    @swamp_location ||= landmark_locations(:swamp)
    return nil if !@swamp_location
    @swamp_location[0]
  end

  def swamp_location_y
    @swamp_location ||= landmark_locations(:swamp)
    return nil if !@swamp_location
    @swamp_location[1]
  end

  def current_admiral_cutscene
    if @game.story_line.cutscene_3_unlocked?
      :destruction_cutscene
    elsif @game.story_line.cutscene_2_unlocked?
      :purging_cutscene
    elsif @game.story_line.cutscene_1_unlocked?
      :star_gazer_cutscene
    else
      :none
    end
  end

  def reveal_entire_map
    self.keys.each do |k|
      self[k] = :visible
    end
  end
end
