# Copyright 2014 Scratchwork Development LLC. All rights reserved.

class WorldEvent < Event
  attr_accessor :title, :combat_tick, :last_damage_given,
                :last_damage_taken, :weapon_stores, :weapons_list,
                :player_dodge_override, :never_miss

  def initialize stores, weapon_stores, weapons_list, damage_multiplier
    super stores
    @durability_multiplier = 1
    @weapon_stores = weapon_stores
    @weapons_list = weapons_list
    @damage_multiplier = (damage_multiplier || 1)
    init_scenes
    expand_scene
    check_scenes
  end

  def expand_scene
    @scenes.each do |k, v|
      if @scenes[k][:options]
        v[:options].each do |k_o, v_o|
          if(v_o.is_a? Symbol)
            v[:options][k_o] = { next_scene: v_o }
          end
        end
      end
    end
  end

  def check_scenes
    @scenes.keys.map do |k|
      if @scenes[k][:options]
        @scenes[k][:options].each { |k_o, v_o| check_scene(k_o, v_o) }
      end
    end
  end

  def check_scene scene_key, scene_value
    return unless scene_value[:next_scene]
    return unless scene_value[:next_scene].is_a? Symbol
    return if @scenes[scene_value[:next_scene]]
    return if scene_key == :leave
    return if scene_key == :continue
    return if scene_key == :run
  end

  def change_scene scene
    if(@scenes[scene] and @scenes[scene][:cutscene])
      @current_scene = scene
    else
      super
    end
  end

  def cutscene?
    scenes[@current_scene][:cutscene]
  end

  def clear
    { :next_scene => :clear }
  end

  def leave
    { :next_scene => :end }
  end

  def aborted
    { :next_scene => :aborted }
  end

  def attack_probablity
    return 1.0 if @never_miss
    0.90 + accuracy_bonus_count * 0.03
  end

  def scene_changed
    @title = scenes[@current_scene][:title] if scenes[@current_scene] and scenes[@current_scene][:title]

    if in_battle?
      scenes[@current_scene][:stunned] ||= false
      scenes[@current_scene][:stun_countdown] = 0
    end
  end

  def player_dodge
    @player_dodge_override ||
      0.20 + evasion_bonus_count * 0.05
  end

  def replenishes_water?
    false
  end

  def becomes_outpost?
    false
  end

  def generates_road?
    false
  end

  def on_clear_replace_with
    nil
  end

  def loot_sample
    return {} if @game.world.first_encounter?

    scene = scenes[@current_scene]

    result = { }

    loot = scene[:loot]

    if(loot.class.to_s == "Stores")
      return loot
    end

    item_loot_keys.each do |key|
      if roll < loot[key][:chance]
        result[key] = (loot[key][:min]..loot[key][:max]).to_a.sample
      end
    end

    result
  end

  def item_loot_keys
    loot_keys.find_all { |k| !weapons_list.include?(k) }
  end

  def weapon_sample
    scene = scenes[@current_scene]

    result = [ ]

    loot = scene[:loot]

    weapon_loot_keys.each do |key|
      if roll < loot[key][:chance]
        amount = (loot[key][:min]..loot[key][:max]).to_a.sample
        amount.times {
          result << weapon_for(key)
        }
      end
    end

    result
  end

  def weapon_for name
    return Knife.new if name == :knife
    return IronSword.new if name == :iron_sword
    return SteelSword.new if name == :steel_sword
    return Katana.new if name == :katana
    return Rifle.new if name == :rifle
    return Laser.new if name == :laser
  end

  def weapon_loot_keys
    loot_keys.find_all { |k| weapons_list.include?(k) }
  end

  def loot_keys
    scenes[@current_scene][:loot].keys
  end

  def in_battle?
    scenes[@current_scene] and scenes[@current_scene][:combat]
  end

  def just_loot?
    !in_battle? and scenes[@current_scene][:loot]
  end

  def battle_complete?
    scenes[@current_scene][:health] <= 0
  end

  def damage_with_perk attack_type
    damage = attack_type.damage

    if attack_type.melee?
      damage =  damage + melee_bonus_count
    end

    if attack_type.action_name == :fists
      # return damage + 109 if @game.world.punch_level == 4 # GODMODE
      # return damage + 104 if @game.world.punch_level == 3 # GODMODE
      # return damage + 102 if @game.world.punch_level == 2 # GODMODE
      # return damage + 100 # GODMODE

      return damage + 9 if @game.world.punch_level == 4
      return damage + 4 if @game.world.punch_level == 3
      return damage + 2 if @game.world.punch_level == 2
      return damage + 0
    end

    return damage
  end

  def melee_bonus_count
    (@game.world.stores[:melee] || 0)
  end

  def accuracy_bonus_count
    (@game.world.stores[:scope] || 0)
  end

  def evasion_bonus_count
    (@game.world.stores[:dodge] || 0)
  end

  def attack attack_type
    attacked = false
    missed = false
    damage = damage_with_perk attack_type
    #damage = 9999 #for testing

    combat_scene = scenes[@current_scene]

    if attack_type.can_use?
      attacked = true
      missed = roll > attack_probablity

      if !missed
        if attack_type.action_name == :bolas
          combat_scene[:stunned] = true
          combat_scene[:stun_countdown] = 5
        else
          combat_scene[:health] -= damage
          @last_damage_given = damage
        end
      end

      if attack_type.action_name == :fists
        @game.world.punch_count += 1
      end

      attack_type.used missed
    end

    {
      name: attack_type.action_name,
      attacked: attacked,
      missed: missed,
      damage: damage
    }
  end

  def dodging?
    @dodging
  end

  def dodge dodge_item
    @dodging = true
    @dodge_frames = 2

    dodge_item.used true

    {
      name: dodge_item.action_name,
      attacked: true,
      missed: false,
      damage: 0
    }
  end

  def heal food_item
    @game.world.eat_meat

    food_item.used true

    {
      name: food_item.action_name,
      attacked: true,
      missed: false,
      damage: -@game.world.meat_heal
    }
  end

  def act action_type
    if action_type.action_name == :stealth
      dodge action_type
    elsif action_type.action_name == :food
      heal action_type
    else
      attack action_type
    end
  end

  def tick_all_actions
    @game.world.actions.each do |action|
      action.tick
    end
  end

  def decay_dodge
    @dodge_frames ||= 0
    @dodge_frames -= 1

    if @dodge_frames < 0
      @dodging = false
      @dodge_frames = 0
    end
  end

  def dodge_frames
    @dodge_frames
  end

  def tick
    return if !in_battle?

    @combat_tick += 1

    decay_dodge

    attack_result = enemy_attack

    tick_all_actions

    attack_result
  end

  def enemy_attack
    combat_scene = scenes[@current_scene]
    attacked = false
    missed = false
    stunned = false
    damage = combat_scene[:damage] * @damage_multiplier
    damage = 0 if @god_mode
    if combat_scene[:stunned]
      combat_scene[:stun_countdown] -= 1
      stunned = true
    end

    if combat_scene[:stun_countdown] <= 0
      combat_scene[:stunned] = false
    end

    if @dodging
      missed = true
      attacked = true
    elsif !stunned
      if cool_down_complete combat_scene[:attack_delay]
        missed = roll < player_dodge

        if !missed
          @game.world.decrement_hp damage
          @last_damage_taken = damage
        end

        attacked = true
      end
    end

    Struct.new(:attacked, :missed, :damage).new(attacked, missed, damage)
  end

  def cool_down_complete attack_delay
    (@combat_tick % scenes[@current_scene][:attack_delay]) == 0
  end

  def on_aborted
    on_clear_base true
  end

  def on_clear
    on_clear_base false
  end

  def replenish_water
    @game.world.water = @game.world.container[:max] if replenishes_water?
  end

  def on_clear_base aborted
    x = @game.world.x
    y = @game.world.y

    if aborted
      @game.world.abort_landmark x, y
    end

    replenish_water if !aborted

    if becomes_outpost? and !aborted
      @game.world.landmarks[[x,y]] = :outpost
      @game.world.draw_road
    elsif on_clear_replace_with and !aborted
      @game.world.landmarks[[x,y]] = on_clear_replace_with
    else
      @game.world.clear_landmark x, y
    end

    if thoughts_message and !aborted
      @game.world.thoughts_message = thoughts_message
    end

    if generates_road? and !aborted
      @game.world.draw_road
    end

    @game.world.nullify_event
  end

  def thoughts_message
    nil
  end
end
