# Copyright 2014 Scratchwork Development LLC. All rights reserved.

class GameOverScene < Scene
  def id
    :game_over_scene
  end

  def will_appear
    stop_music
  end

  def tick
    calc_buttons
    outputs.primitives << title_label
    outputs.primitives << game_over_label
    outputs.primitives << stats_labels
    render_buttons
  end

  def will_appear_a11y_target
    :title_label
  end

  def immediately_after_first_encounter?
    @game.total_defector_deaths == 1 &&
    @game.world.event_on_death == ImpossibleDefectorEvent &&
    @game.world.tutorial_mode? && @game.world.total_deaths == 1
  end

  def title_label
    if @game.world.death_reason == :complete ||
       @game.world.death_reason == :complete_hard ||
       immediately_after_first_encounter?
      Layout.rect(row: 0, col: 0, h: 1, w: Layout.col_count)
            .center
            .merge(text: "...",
                   anchor_x: 0.5,
                   anchor_y: 0.5,
                   font: "fonts/text.ttf",
                   size_px: 30,
                   a11y_id: "title_label",
                   a11y_text: "...",
                   a11y_trait: :label)
    else
      Layout.rect(row: 0, col: 0, h: 1, w: Layout.col_count)
            .center
            .merge(text: "time paradox",
                   anchor_x: 0.5,
                   anchor_y: 0.5,
                   font: "fonts/text.ttf",
                   size_px: 30,
                   a11y_id: "title_label",
                   a11y_text: "time paradox",
                   a11y_trait: :label)
    end
  end

  def stats_labels
    return nil if immediately_after_first_encounter?

    m = @game.world.current_multipliers || @game.world.default_multipliers

    god_mode_string = if(m != @game.world.default_multipliers)
                        "D: #{m[:durability]}, P: #{m[:poison]}, H: #{m[:hunger]}, T: #{m[:thirst]}, V: #{m[:vision]}, E: #{m[:enemy_damage]}"
                      else
                        ""
                      end


    label_rect = if Grid.landscape?
                   Layout.rect(row: 5, col: 0, h: 1, w: Layout.col_count)
                 else
                   Layout.rect(row: 8, col: 0, h: 1, w: Layout.col_count)
                 end

    [
      label_rect.center.merge(text: "world explored: #{@game.world.percentage_uncovered_this_time}%",
                              a11y_id: :world_explored_label,
                              a11y_text: "world explored, #{@game.world.percentage_uncovered_this_time}%",
                              a11y_trait: :label,
                              anchor_x: 0.5,
                              anchor_y: 0.5,
                              size_px: 30,
                              font: "fonts/text.ttf"),
      label_rect.center.merge(text: "areas cleared: #{@game.world.percentage_cleared_this_time}%",
                              a11y_id: :areas_cleared_label,
                              a11y_text: "areas cleared, #{@game.world.percentage_cleared_this_time}%",
                              a11y_trait: :label,
                              anchor_x: 0.5,
                              anchor_y: 1.5,
                              size_px: 30,
                              font: "fonts/text.ttf"),
      label_rect.center.merge(text: "time: #{@game.tick_count} minutes",
                              a11y_id: :time_label,
                              a11y_text: "time, #{@game.tick_count} minutes",
                              a11y_trait: :label,
                              anchor_x: 0.5,
                              anchor_y: 2.5,
                              size_px: 30,
                              font: "fonts/text.ttf"),
      label_rect.center.merge(text: "total played: #{@game.total_tick_count} minutes",
                              a11y_id: :total_played_label,
                              a11y_text: "total played, #{@game.total_tick_count} minutes",
                              a11y_trait: :label,
                              anchor_x: 0.5,
                              anchor_y: 3.5,
                              size_px: 30,
                              font: "fonts/text.ttf"),
      label_rect.center.merge(text: "#{god_mode_string}",
                              anchor_x: 0.5,
                              anchor_y: 4.5,
                              size_px: 30,
                              font: "fonts/text.ttf")
    # world explored: #{percentage_uncovered}%\nareas cleared: #{percentage_cleared}%\ntime: #{@time} minutes\ntotal played: #{@game.total_tick_count} minutes
    ]
  end

  def game_over_label
    label_rect = if Grid.landscape?
                   Layout.rect(row: 1, col: 12)
                 else
                   Layout.rect(row: 3, col: 6)
                 end

    a11y = {
      a11y_id: :game_over_label,
      a11y_text: "",
      a11y_trait: :label
    }

    if immediately_after_first_encounter?
      a11y.a11y_text = "awake. head throbbing. just a nightmare."
      [
        label_rect.merge(text: "awake. head throbbing.",
                         **a11y,
                         font: "fonts/text.ttf",
                         anchor_x: 0.5,
                         anchor_y: 0.5,
                         size_px: 30),
        label_rect.merge(text: "just a nightmare.",
                         font: "fonts/text.ttf",
                         anchor_x: 0.5,
                         anchor_y: 1.5,
                         size_px: 30)
      ]
    elsif @game.world.death_reason == :dehydration
      a11y.a11y_text = "death from dehydration."
      label_rect.merge(text: "death from dehydration.",
                       **a11y,
                       font: "fonts/text.ttf",
                       anchor_x: 0.5,
                       anchor_y: 0.5,
                       size_px: 30)
    elsif @game.world.death_reason == :starvation
      a11y.a11y_text = "starved to death."
      label_rect.merge(text: "starved to death.",
                       **a11y,
                       font: "fonts/text.ttf",
                       anchor_x: 0.5,
                       anchor_y: 0.5,
                       size_px: 30)
    elsif @game.world.death_reason == :battle
      a11y.a11y_text = "defeated in battle."
      label_rect.merge(text: "defeated in battle.",
                       **a11y,
                       font: "fonts/text.ttf",
                       anchor_x: 0.5,
                       anchor_y: 0.5,
                       size_px: 30)
    elsif @game.world.death_reason == :complete
      a11y.a11y_text = "awake. head throbbing. vision blurry. the voices say to survive."
      [
        label_rect.merge(text: "awake. head throbbing. vision blurry.",
                         **a11y,
                         font: "fonts/text.ttf",
                         anchor_x: 0.5,
                         anchor_y: 0.5,
                         size_px: 30),
        label_rect.merge(text: "the voices say to survive.",
                         font: "fonts/text.ttf",
                         anchor_x: 0.5,
                         anchor_y: 1.5,
                         size_px: 30)
      ]
    elsif @game.world.death_reason == :complete_hard
      a11y.a11y_text = "..."
      label_rect.merge(text: "...",
                       **a11y,
                       font: "fonts/text.ttf",
                       anchor_x: 0.5,
                       anchor_y: 0.5,
                       size_px: 30)
    elsif @game.world.death_reason == :poisoned
      a11y.a11y_text = "poisoned with every step."
      label_rect.merge(text: "poisoned with every step.",
                       **a11y,
                       font: "fonts/text.ttf",
                       anchor_x: 0.5,
                       anchor_y: 0.5,
                       size_px: 30)
    elsif @game.world.death_reason == :exploded
      a11y.a11y_text = "engulfed in flames."
      label_rect.merge(text: "engulfed in flames.",
                       **a11y,
                       font: "fonts/text.ttf",
                       anchor_x: 0.5,
                       anchor_y: 0.5,
                       size_px: 30)
    end
  end

  def try_again_button_selected
    if @game.world.death_reason == :complete_hard
      state.next_scene = :cutscene_scene
    else
      state.story_state = :beginning
      state.next_scene = :embark_scene
    end
  end

  def try_again_button
    return nil if state.scene_at.elapsed_time < 90
    r = Layout.rect(row: Layout.row_count - 2,
                    col: Layout.col_count.idiv(2) - 6,
                    w: 12,
                    h: 2).merge(a11y_id: :try_again_button, a11y_trait: :button)

    if immediately_after_first_encounter?
      r.merge(text: "talk to admiral", a11y_text: "talk to admiral")
    elsif @game.world.death_reason == :complete
      r.merge(text: "light fire", a11y_text: "light fire")
    elsif @game.world.death_reason == :complete_hard
      r.merge(text: "the builder", a11y_text: "the builder")
    else
      r.merge(text: "turn back time", a11y_text: "turn back time")
    end
  end

  def button_ids
    [
      :try_again_button
    ]
  end

  def progress_bar_ids
    []
  end
end
