# Copyright 2014 Scratchwork Development LLC. All rights reserved.

class GodModeScene < Scene
  def id
    :god_mode_scene
  end

  def button_ids
    [
      :durability_button,
      :poison_button,
      :hunger_button,
      :thirst_button,
      :vision_button,
      :enemy_damage_button,
      :submit_button,
    ]
  end

  def will_appear_a11y_target

  end

  def will_appear
    args.outputs.a11y[:notify_will_appear] = {
      a11y_trait: :notification,
      a11y_notification_target: "layout_title",
      a11y_notification_debounce: 60
    }

    @durability   ||= @game.world.default_multipliers[:durability]
    @poison       ||= @game.world.default_multipliers[:poison]
    @hunger       ||= @game.world.default_multipliers[:hunger]
    @thirst       ||= @game.world.default_multipliers[:thirst]
    @vision       ||= @game.world.default_multipliers[:vision]
    @enemy_damage ||= @game.world.default_multipliers[:enemy_damage]
  end

  def tick
    calc_buttons
    outputs.primitives << title_label
    outputs.primitives << instructions_label
    outputs.primitives << prefab.button(args, durability_button)
    outputs.primitives << prefab.button(args, poison_button)
    outputs.primitives << prefab.button(args, hunger_button)
    outputs.primitives << prefab.button(args, thirst_button)
    outputs.primitives << prefab.button(args, vision_button)
    outputs.primitives << prefab.button(args, enemy_damage_button)
    outputs.primitives << prefab.button(args, submit_button)
  end

  def enemy_damage_button_selected
    @enemy_damage += 1
    @enemy_damage = 0 if @enemy_damage > 2
  end

  def durability_button_selected
    @durability += 1
    @durability = 0 if @durability > 3
  end

  def poison_button_selected
    @poison += 1
    @poison = 0 if @poison > 2
  end

  def hunger_button_selected
    @hunger += 1
    @hunger = 0 if @hunger > 2
  end

  def thirst_button_selected
    @thirst += 1
    @thirst = 0 if @thirst > 2
  end

  def vision_button_selected
    @vision += 1
    @vision = 1 if @vision > 4
  end

  def option_button_start_row
    if Grid.orientation == :landscape
      3
    else
      10
    end
  end

  def option_button_col
    if Grid.orientation == :landscape
      6
    else
      0
    end
  end

  def option_button_h
    if Grid.orientation == :landscape
      1.25
    else
      2
    end
  end

  def durability_button
    text = if @durability == 0
             "D: #{@durability} (weapons do not break)"
           elsif @durability == 1
             "D: #{@durability} (normal weapon breakage)"
           else
             "D: #{@durability} (weapons break #{@durability}x faster)"
           end

    Layout.rect(row: option_button_start_row + option_button_h * 0, col: option_button_col, w: 12, h: option_button_h)
          .merge(text: text, a11y_id: "durability_button", a11y_text: "durability modifier, #{text}", a11y_trait: :button)
  end

  def poison_button
    text = if @poison == 0
             "P: #{@poison} (no poison, normal)"
           else
             "P: #{@poison} (-#{@poison} hp per step)"
           end

    Layout.rect(row: option_button_start_row + option_button_h * 1, col: option_button_col, w: 12, h: option_button_h)
          .merge(text: text, a11y_id: "poison_button", a11y_text: "poison modifier, #{text}", a11y_trait: :button)
  end

  def hunger_button
    text = if @hunger == 0
             "H: #{@hunger} (no hunger)"
           elsif @hunger == 1
             "H: #{@hunger} (normal hunger)"
           else
             "H: #{@hunger} (-#{@hunger} food per step)"
           end

    Layout.rect(row: option_button_start_row + option_button_h * 2, col: option_button_col, w: 12, h: option_button_h)
          .merge(text: text, a11y_id: "hunger_button", a11y_text: "hunger modifier, #{text}", a11y_trait: :button)
  end

  def thirst_button
    text = if @thirst == 0
             "T: #{@thirst} (no thirst)"
           elsif @thirst == 1
             "T: #{@thirst} (normal thirst)"
           else
             "T: #{@thirst} (-#{@thirst} water per step)"
           end

    Layout.rect(row: option_button_start_row + option_button_h * 3, col: option_button_col, w: 12, h: option_button_h)
          .merge(text: text, a11y_id: "thirst_button", a11y_text: "thirst modifier, #{text}", a11y_trait: :button)
  end


  def vision_button
    text = if @vision == 2
             "V: #{@vision} (normal vision)"
           else
             "V: #{@vision} (reveal radius of #{@vision})"
           end

    Layout.rect(row: option_button_start_row + option_button_h * 4, col: option_button_col, w: 12, h: option_button_h)
          .merge(text: text, a11y_id: "vision_button", a11y_text: "vision modifier, #{text}", a11y_trait: :button)
  end

  def enemy_damage_button
    text = if @enemy_damage == 0
             "E: #{@enemy_damage} (no enemy damage)"
           elsif @enemy_damage == 1
             "E: #{@enemy_damage} (enemies do normal damage)"
           else
             "E: #{@enemy_damage} (enemies do #{@enemy_damage}x damage)"
           end

    Layout.rect(row: option_button_start_row + option_button_h * 5, col: option_button_col, w: 12, h: option_button_h)
          .merge(text: text, a11y_id: "enemy_damage_button", a11y_text: "enemy damage modifier, #{text}", a11y_trait: :button)
  end

  def title_label
    title_rect = if Grid.orientation == :landscape
                   Layout.rect(row: 0, col: [6, 17], h: 1)
                 else
                   Layout.rect(row: 0, col: [0, 11], h: 1)
                  end

    title_rect.center
              .merge(text: "pray",
                     a11y_id: "layout_title",
                     a11y_text: "pray",
                     a11y_trait: :label,
                     font: "fonts/text.ttf",
                     anchor_x: 0.5,
                     anchor_y: 0.5,
                     size_px: 30)
  end

  def instructions_label
    text_with_location = []
    instructions_label_rect = Layout.rect(row: 1, col: [0, 11], h: 3)
    starting_x = instructions_label_rect.x
    starting_y = instructions_label_rect.y

    text = ["what fate will the ensign have, god?"]

    anchor_increment = 0
    text.each_with_index do |t, j|
      long_lines_split(t, 38).each_with_index do |line, i|
        next if line.strip.length == 0
        anchor_increment += 1.2
        text_with_location << { x: instructions_label_rect.x + 4,
                                y: instructions_label_rect.center.y,
                                font: "fonts/text.ttf",
                                size_px: 30,
                                anchor_increment: anchor_increment,
                                text: line.strip }
      end
      anchor_increment += 1
    end

    l = text_with_location.length
    text_with_location.each_with_index do |t, i|
      if l <= 2
        t.x = Grid.w / 2
        t.anchor_x = 0.5
      else
        t.anchor_x = 0
      end

      t.anchor_y = -(l / 2) + t.anchor_increment - 0.5
    end

    if text_with_location[0] && GTK.a11y_enabled?
    text_with_location[0].a11y_id = "world_instructions_text"
    text_with_location[0].a11y_trait = :label
    text_with_location[0].a11y_text = (text_with_location.map { |s| s.text }.join " \n ")
    end

    text_with_location
  end

  def submit_button
    Layout.rect(row: Layout.row_count - option_button_h, col: option_button_col, w: 12, h: option_button_h)
          .merge(text: "submit", a11y_id: "submit_button", a11y_text: "submit", a11y_trait: :button, font: "fonts/text.ttf")
  end

  def submit_button_selected
    state.god_mode_submitted = true

    state.god_mode_multipliers = {
      durability: @durability,
      poison: @poison,
      hunger: @hunger,
      thirst: @thirst,
      vision: @vision,
      enemy_damage: @enemy_damage,
    }

    state.next_scene = :embark_scene
  end
end
