# Copyright 2014 Scratchwork Development LLC. All rights reserved.

class LootSceneBase < Scene
  attr :option_controls,
       :loot_controls,
       :weapon_controls,
       :supply_controls,
       :weapon_loot,
       :item_loot,
       :scroll_view

  def id
    :loot_scene
  end

  def will_appear_a11y_target
    :loot_header
  end

  def weapon_title weapon, equipped
    equipped_string = ""
    rusted = ""

    rusted = "rusted " if(@game.world.rusted?)

    if(weapon.has_durability?)
      "#{rusted}#{weapon.name.gsub("_", " ")}#{equipped_string}"
    else
      "#{rusted}#{weapon.name.gsub("_", " ")}#{equipped_string}"
    end
  end

  def weapon_string weapon, equipped
    equipped_string = ""
    rusted = ""

    rusted = "rusted " if(@game.world.rusted?)

    if(weapon.has_durability?)
      "durability: #{weapon.durability_percentage}%"
    else
      "durability: unbreakable"
    end
  end

  def loot_customizations
    @loot_customizations ||= {}
    @loot_customizations
  end

  def calc_loot_controls
    return if @scroll_view.scroll_page_recently_invoked?
    return if !inputs.mouse.click
    kvp = @supply_controls.find do |k, v|
      inputs.mouse.inside_rect? v[:button]
    end

    kvp ||= @loot_controls.find do |k, v|
      inputs.mouse.inside_rect? v[:button]
    end

    if kvp
      k, controls = kvp
      button = controls[:button]
      if button.id == :drop_button
        queue_flash :drop,
                    button.x,
                    button.y,
                    button.w,
                    button.h,
                    prefab.black.r,
                    prefab.black.g,
                    prefab.black.b,
                    1.0/10.0
        @game.world.drop_stores k, @item_loot
      elsif button.id == :take_button
        queue_flash :take,
                    button.x,
                    button.y,
                    button.w,
                    button.h,
                    prefab.black.r,
                    prefab.black.g,
                    prefab.black.b,
                    1.0/10.0
        @game.world.take_stores k, @item_loot
      else
        raise "unknown button id: #{button}"
      end
    end

    kvp = @weapon_controls.find do |k, v|
      inputs.mouse.inside_rect?(v[:take_button])
    end

    if kvp
      k, controls = kvp
      queue_flash :take,
                  controls[:take_button].x,
                  controls[:take_button].y,
                  controls[:take_button].w,
                  controls[:take_button].h,
                  prefab.black.r,
                  prefab.black.g,
                  prefab.black.b,
                  1.0/10.0

      queue_flash :drop,
                  controls[:drop_button].x,
                  controls[:drop_button].y,
                  controls[:drop_button].w,
                  controls[:drop_button].h,
                  prefab.black.r,
                  prefab.black.g,
                  prefab.black.b,
                  1.0/10.0
      @game.world.take_weapon controls[:weapon], @weapon_loot if @weapon_loot.include? controls[:weapon]
    end

    kvp = @weapon_controls.find do |k, v|
      inputs.mouse.inside_rect?(v[:drop_button])
    end

    if kvp
      k, controls = kvp
      queue_flash :take,
                  controls[:take_button].x,
                  controls[:take_button].y,
                  controls[:take_button].w,
                  controls[:take_button].h,
                  prefab.black.r,
                  prefab.black.g,
                  prefab.black.b,
                  1.0/10.0

      queue_flash :drop,
                  controls[:drop_button].x,
                  controls[:drop_button].y,
                  controls[:drop_button].w,
                  controls[:drop_button].h,
                  prefab.black.r,
                  prefab.black.g,
                  prefab.black.b,
                  1.0/10.0
      @game.world.drop_weapon controls[:weapon], @weapon_loot if @game.world.weapon_stores.include? controls[:weapon]
    end
  end

  def button_ids
    [:continue_button, :run_button]
  end

  def progress_bar_ids
    []
  end

  def event_ended? game
    return true if !current_event

    current_event.current_scene == :end
  end

  def run_button_selected
    carry = @game.world.carry

    if game.world.inventory_weight > carry[:max]
      carry = game.world.carry
      queue_notification :encumbered, "can't carry this much: #{carry[:name]}, can only carry a maximum of #{carry[:max]}."
    else
      current_event.change_scene(:run)
      state.next_scene = world_scene_name
    end
  end

  def continue_button_selected
    carry = @game.world.carry

    if game.world.inventory_weight > carry[:max]
      carry = game.world.carry
      queue_notification :encumbered, "can't carry this much: #{carry[:name]}, can only carry a maximum of #{carry[:max]}."
    else
      if current_scene[:options][:continue]
        current_event.change_scene(:continue)
      else
        current_event.change_scene(:leave)
      end

      if event_ended? game
        set_event_to_nil!
        state.next_scene = world_scene_name
      else
        if current_event && current_event.in_battle?
          state.next_scene = battle_scene_name
        else
          state.next_scene = world_event_scene_name
        end
      end
    end
  end

  def can_run?
    return false if !current_scene
    return false if !current_scene[:options]
    current_scene[:options][:run]
  end

  def run_button
    return nil if changing_scene?
    return nil if !can_run?
    a = if @selected_equipment
          32
        else
          255
        end
    a11y = { a11y_id: "loot_run_button", a11y_trait: :button, a11y_text: "run" }
    if @selected_equipment
      a11y = {}
    end

    carry = @game.world.carry

    color = if game.world.inventory_weight > carry[:max]
              { r: 128, g: 128, b: 128 }
            else
              prefab.black
            end


    r = if Grid.orientation == :landscape
          Layout.rect(row: 11, col: [0, 10.5], h: 1, dy: -4)
        else
          Layout.rect(row: 22, col: [0, 5], h: 2, dy: -4)
        end

    r.merge(text: "← run", a: a, **a11y, **color)
  end

  def continue_button
    return nil if changing_scene?
    a = if @selected_equipment
          32
        else
          255
        end
    a11y = { a11y_id: "loot_continue_button", a11y_trait: :button, a11y_text: "continue" }
    if @selected_equipment
      a11y = {}
    end

    carry = @game.world.carry

    color = if game.world.inventory_weight > carry[:max]
              { r: 128, g: 128, b: 128 }
            else
              prefab.black
            end

    r = if can_run?
          if Grid.orientation == :landscape
            Layout.rect(row: 11, col: [11.5, 23], h: 1, dy: -4)
          else
            Layout.rect(row: 22, col: [6, 11], h: 2, dy: -4)
          end
        else
          if Grid.orientation == :landscape
            Layout.rect(row: 11, col: [0, 23], h: 1, dy: -4)
          else
            Layout.rect(row: 22, col: [0, 11], h: 2, dy: -4)
          end
        end

    r.merge(text: "continue →", a: a, **a11y, **color)
  end

  def title_label
    r = if Grid.orientation == :landscape
          Layout.rect(row: 0, col: 12, w: 24, h: 1)
        else
          Layout.rect(row: 0, col: 6, w: 12, h: 1)
        end

    r.merge(text: "#{current_scene[:loot_title] || "loot"}",
            font: "fonts/text.ttf",
            size_px: 30,
            a11y_id: :loot_header,
            a11y_trait: :text,
            a11y_text: "#{current_scene[:loot_title] || "loot"}",
            anchor_x: 0.5,
            anchor_y: -0.8)
  end

  def a11y_carry_description
    "#{@game.world.carry[:name]}, capacity used: #{@game.world.inventory_weight}, total capacity: #{@game.world.carry[:max]}"
  end

  def weight_label
    carry = @game.world.carry

    color = if game.world.inventory_weight > carry[:max]
              { r: 200, g: 0, b: 0 }
            else
              { r: 0, g: 0, b: 0 }
            end

    r = if Grid.orientation == :landscape
          Layout.rect(row: 0, col: 12, w: 24, h: 1)
        else
          Layout.rect(row: 0, col: 6, w: 12, h: 1)
        end

    r.merge(text: "#{@game.world.carry[:name]}: #{@game.world.inventory_weight}/#{@game.world.carry[:max]}",
            font: "fonts/text.ttf",
            size_px: 30,
            **color,
            a11y_id: :weight_label,
            a11y_trait: :text,
            a11y_text: a11y_carry_description,
            anchor_x: 0.5,
            anchor_y: 0.0)
  end

  def calc_loot_controls
    return if !inputs.mouse.click
    kvp = @supply_controls.find do |k, v|
      inputs.mouse.inside_rect? v[:button]
    end

    kvp ||= @loot_controls.find do |k, v|
      inputs.mouse.inside_rect? v[:button]
    end

    if kvp
      k, controls = kvp
      button = controls[:button]
      if button.id == :drop_button
        queue_flash :drop,
                    button.x,
                    button.y,
                    button.w,
                    button.h,
                    prefab.black.r,
                    prefab.black.g,
                    prefab.black.b,
                    1.0/10.0
        @game.world.drop_stores k, @item_loot
      elsif button.id == :take_button
        queue_flash :take,
                    button.x,
                    button.y,
                    button.w,
                    button.h,
                    prefab.black.r,
                    prefab.black.g,
                    prefab.black.b,
                    1.0/10.0
        @game.world.take_stores k, @item_loot
      else
        raise "unknown button id: #{button}"
      end
    end

    kvp = @weapon_controls.find do |k, v|
      inputs.mouse.inside_rect?(v[:take_button])
    end

    if kvp
      k, controls = kvp
      queue_flash :take,
                  controls[:take_button].x,
                  controls[:take_button].y,
                  controls[:take_button].w,
                  controls[:take_button].h,
                  prefab.black.r,
                  prefab.black.g,
                  prefab.black.b,
                  1.0/10.0

      queue_flash :drop,
                  controls[:drop_button].x,
                  controls[:drop_button].y,
                  controls[:drop_button].w,
                  controls[:drop_button].h,
                  prefab.black.r,
                  prefab.black.g,
                  prefab.black.b,
                  1.0/10.0
      @game.world.take_weapon controls[:weapon], @weapon_loot if @weapon_loot.include? controls[:weapon]
    end

    kvp = @weapon_controls.find do |k, v|
      inputs.mouse.inside_rect?(v[:drop_button])
    end

    if kvp
      k, controls = kvp
      queue_flash :take,
                  controls[:take_button].x,
                  controls[:take_button].y,
                  controls[:take_button].w,
                  controls[:take_button].h,
                  prefab.black.r,
                  prefab.black.g,
                  prefab.black.b,
                  1.0/10.0

      queue_flash :drop,
                  controls[:drop_button].x,
                  controls[:drop_button].y,
                  controls[:drop_button].w,
                  controls[:drop_button].h,
                  prefab.black.r,
                  prefab.black.g,
                  prefab.black.b,
                  1.0/10.0
      @game.world.drop_weapon controls[:weapon], @weapon_loot if @game.world.weapon_stores.include? controls[:weapon]
    end
  end

  def scroll_view_layout
    if Grid.orientation == :landscape
      { row: 1, col: 0, w: 24, h: 10 }
    else
      { row: 1, col: 0, w: 12, h: 21 }
    end
  end

  def will_appear
    @scroll_view = ScrollView.new scroll_view_layout
    @option_controls = []
    @loot_controls = {}
    @weapon_controls = {}
    @supply_controls = {}
    @item_loot = get_item_loot || []
    @weapon_loot = get_weapon_loot || []

    populate_scroll_view
  end

  def populate_scroll_view
    disable_selection_prefab!
    weapons = (@weapon_loot + @game.world.weapon_stores).sort_by { |w| w.order }
    if white_listed_weapons && white_listed_weapons.length > 0
      weapons.reject! do |weapon|
        !white_listed_weapons.include?(weapon.id)
      end
    end

    item_keys = (@item_loot.keys + @game.world.stores.keys).uniq.sort_by { |k| k.to_s }
    if white_listed_loot && white_listed_loot.length > 0
      item_keys.reject! do |key|
        !white_listed_loot.include?(key)
      end
    end

    if Grid.orientation == :landscape
      populate_scroll_view_landscape weapons, item_keys
    else
      populate_scroll_view_portrait weapons, item_keys
    end

    state.loot_scene_state ||= {
      needs_scrolling: false
    }


    if needs_scrolling? != state.loot_scene_state.needs_scrolling
      @scroll_view.scroll_to_bottom
      GTK.schedule_callback Kernel.tick_count + 30 do
        @scroll_view.scroll_to_top
        GTK.schedule_callback Kernel.tick_count + 15 do
          enable_selection_prefab!
          set_selection! "take_all_loot"
        end
      end
    else
      enable_selection_prefab!
      set_selection! "take_all_loot"
    end

    state.loot_scene_state.needs_scrolling = needs_scrolling?
  end

  def take_all_loot_callback
    if !@scroll_view.scroll_page_recently_invoked?
      (@weapon_loot + @game.world.weapon_stores).each do |weapon|
        @game.world.take_weapon weapon, @weapon_loot
      end

      @item_loot.keys.each do |key|
        @item_loot[key].times do
          @game.world.take_stores key, @item_loot
        end
      end

      outputs.a11y_clear_pending_notifications!
      outputs.a11y["take_all_loot_notification"] = {
        a11y_id: "take_all_loot_notification",
        a11y_trait: :notification,
        a11y_text: "all loot taken, #{a11y_carry_description}"
      }
    end
  end

  def populate_scroll_view_landscape weapons, item_keys
    @running_row = 1
    button_size = 1.5
    take_all_rect = Layout.rect(row: @running_row, col: 0, w: 24, h: button_size)
    @scroll_view.add_item take_all_rect.center
                                       .merge(text: "take all loot",
                                              font: "fonts/text.ttf",
                                              size_px: 20,
                                              anchor_x: 0.5,
                                              anchor_y: 0.5)
                                       .with_metadata(control_id: :take_all_loot_label)

    @scroll_view.add_item prefab.thick_border(args, take_all_rect)
                                .merge(a11y_id: :take_all_loot,
                                       a11y_text: "take all loot",
                                       a11y_trait: :button)
                                .with_metadata(control_id: :take_all_loot_rect,
                                               selection_id: "take_all_loot",
                                               callback: lambda {
                                                 take_all_loot_callback
                                               })

    @running_row += button_size

    weapons.each_slice(2) do |weapon_1, weapon_2|
      if weapon_1
        add_weapon_to_scroll_view weapon_1,
                                  row: @running_row,
                                  drop_col: 10.5,
                                  text_col: 2,
                                  take_col: 0,
                                  w: button_size,
                                  h: button_size,
                                  size_px: 20
      end



      if weapon_2
        add_weapon_to_scroll_view weapon_2,
                                  row: @running_row,
                                  drop_col: 10.5 + 12,
                                  text_col: 2 + 12,
                                  take_col: 0 + 12,
                                  w: button_size,
                                  h: button_size,
                                  size_px: 20
      end
      @running_row += button_size
    end

    item_keys.each_slice(2) do |item_key_1, item_key_2|
      if item_key_1
        add_item_to_scroll_view item_key_1,
                                row: @running_row,
                                drop_label: drop_label,
                                drop_desc: drop_desc,
                                take_label: take_label,
                                take_desc: take_desc,
                                text_col_1: button_size,
                                text_col_2: button_size + 8.5,
                                drop_col: 10.5,
                                take_col: 0,
                                w: button_size,
                                h: button_size,
                                size_px: 20,
                                text_anchor_y: -button_size
      end

      if item_key_2
        add_item_to_scroll_view item_key_2,
                                row: @running_row,
                                drop_label: drop_label,
                                drop_desc: drop_desc,
                                take_label: take_label,
                                take_desc: take_desc,
                                text_col_1: button_size + 12,
                                text_col_2: button_size + 8.5 + 12,
                                drop_col: 10.5 + 12,
                                take_col: 0 + 12,
                                w: button_size,
                                h: button_size,
                                size_px: 20,
                                text_anchor_y: -button_size
      end
      @running_row += button_size
    end

    divider_rect = Layout.rect(row: 2.5, col: 12, w: 0, h: @running_row - 1 - button_size, dx: -2.5)
    @scroll_view.add_item prefab.thick_border(args, divider_rect.merge(w: 1))

    @scroll_view.add_item Layout.rect(row: @running_row, col: 0, w: scroll_view_layout.w, h: 0.75)
                                .merge(path: :solid, r: 245, g: 245, b: 245, a: 1)
  end

  def populate_scroll_view_portrait weapons, item_keys
    @running_row = 1
    take_all_rect = Layout.rect(row: @running_row, col: 0, w: 12, h: 2)
    @scroll_view.add_item take_all_rect.center
                                       .merge(text: "take all loot",
                                              font: "fonts/text.ttf",
                                              size_px: 30,
                                              anchor_x: 0.5,
                                              anchor_y: 0.5)
                                       .with_metadata(control_id: :take_all_loot_label)

    @scroll_view.add_item prefab.thick_border(args, take_all_rect)
                                .merge(a11y_id: :take_all_loot,
                                       a11y_text: "take all loot",
                                       a11y_trait: :button)
                                .with_metadata(control_id: :take_all_loot_rect,
                                               selection_id: "take_all_loot",
                                               callback: lambda {
                                                 take_all_loot_callback
                                               })

    @running_row += 2

    weapons.each do |weapon|
      add_weapon_to_scroll_view weapon,
                                row: @running_row,
                                drop_col: 10,
                                text_col: 2,
                                take_col: 0,
                                w: 2,
                                h: 2,
                                size_px: 30
      @running_row += 2
    end

    item_keys.each do |item_key|
      add_item_to_scroll_view item_key,
                              row: @running_row,
                              drop_label: drop_label,
                              drop_desc: drop_desc,
                              take_label: take_label,
                              take_desc: take_desc,
                              text_col_1: 2,
                              text_col_2: 10,
                              drop_col: 10,
                              take_col: 0,
                              w: 2,
                              h: 2,
                              size_px: 30,
                              text_anchor_y: -0.8
      @running_row += 2
    end

    @scroll_view.add_item Layout.rect(row: @running_row, col: 0, w: scroll_view_layout.w, h: 0.75)
                                .merge(path: :solid, r: 245, g: 245, b: 245, a: 1)
  end

  def row_rect drop_col, take_col, row, w, h
    Layout.rect(row: row, col: [take_col, drop_col + w - 1], h: h)
  end

  def add_weapon_to_scroll_view(weapon, row:, drop_col:, text_col:, take_col:, w:, h:, size_px:)
    a11y_weapon_description = "#{weapon_title(weapon,  @game.world.weapon_stores.include?(weapon))}, #{weapon_string(weapon,  @game.world.weapon_stores.include?(weapon))}"

    @scroll_view.add_item Layout.rect(row: row, col: text_col, w: 8, h: h)
                                .center
                                .merge(text: weapon_title(weapon,  @game.world.weapon_stores.include?(weapon)),
                                       anchor_y: 0.0,
                                       anchor_x: 0.5,
                                       size_px: size_px,
                                       font: "fonts/text.ttf")
                                .with_metadata(control_id: :weapon_title_label, weapon: weapon)

    @scroll_view.add_item Layout.rect(row: row, col: text_col, w: 8, h: h)
                                .center
                                .merge(text: weapon_string(weapon,  @game.world.weapon_stores.include?(weapon)),
                                       anchor_y: 1.0,
                                       anchor_x: 0.5,
                                       size_px: size_px,
                                       font: "fonts/text.ttf")
                                .with_metadata(control_id: :weapon_text_label, weapon: weapon)

    take_rect = Layout.rect(row: row, col: take_col, w: w, h: h)
    @scroll_view.add_item take_rect.center
                                   .merge(text: "take",
                                          font: "fonts/text.ttf",
                                          size_px: size_px,
                                          anchor_x: 0.5,
                                          anchor_y: 0.5)
                                   .with_metadata(control_id: :take_weapon_button_label, weapon: weapon)

    @scroll_view.add_item prefab.thick_border(args, take_rect)
                                .merge(a11y_id: "weapon_#{weapon.id}_take",
                                       a11y_text: "take #{a11y_weapon_description}",
                                       a11y_trait: :button)
                                .with_metadata(control_id: :take_weapon_button_rect,
                                               selection_id: "take:#{weapon.id}",
                                               weapon: weapon,
                                               row_rect: row_rect(drop_col, take_col, row, w, h),
                                               callback: lambda {
                                                 @game.world.take_weapon weapon, @weapon_loot if @weapon_loot.include? weapon
                                                 set_selection! "drop:#{weapon.id}"
                                                 outputs.a11y_clear_pending_notifications!
                                                 outputs.a11y["weapon_taken"] = {
                                                   a11y_id: "set_selection",
                                                   a11y_trait: :notification,
                                                   a11y_text: "#{weapon_title(weapon,  @game.world.weapon_stores.include?(weapon))} taken",
                                                 }
                                                 outputs.a11y["set_selection"] = {
                                                   a11y_id: "set_selection",
                                                   a11y_trait: :notification,
                                                   a11y_notification_target: "weapon_#{weapon.id}_drop",
                                                   a11y_notification_debounce: 60
                                                 }
                                               })

    drop_rect = Layout.rect(row: row, col: drop_col, w: w, h: h)
    @scroll_view.add_item drop_rect.center
                                   .merge(text: "drop",
                                          font: "fonts/text.ttf",
                                          size_px: size_px,
                                          anchor_x: 0.5,
                                          anchor_y: 0.5)
                                   .with_metadata(control_id: :drop_weapon_button_label, weapon: weapon)

    @scroll_view.add_item prefab.thick_border(args, drop_rect)
                                .merge(a11y_id: "weapon_#{weapon.id}_drop",
                                       a11y_text: "drop #{a11y_weapon_description}",
                                       a11y_trait: :button)
                                .with_metadata(control_id: :drop_weapon_button_rect,
                                               weapon: weapon,
                                               selection_id: "drop:#{weapon.id}",
                                               row_rect: row_rect(drop_col, take_col, row, w, h),
                                               callback: lambda {
                                                 @game.world.drop_weapon weapon, @weapon_loot if @game.world.weapon_stores.include? weapon
                                                 set_selection! "take:#{weapon.id}"
                                                 outputs.a11y_clear_pending_notifications!
                                                 outputs.a11y["weapon_taken"] = {
                                                   a11y_id: "set_selection",
                                                   a11y_trait: :notification,
                                                   a11y_text: "#{weapon_title(weapon,  @game.world.weapon_stores.include?(weapon))} dropped",
                                                 }

                                                 outputs.a11y["set_selection"] = {
                                                   a11y_id: "set_selection",
                                                   a11y_trait: :notification,
                                                   a11y_notification_target: "weapon_#{weapon.id}_take",
                                                   a11y_notification_debounce: 60
                                                 }
                                               })

  end

  def selection_changed selection_prefab
  end

  def weapon_title weapon, equipped
    equipped_string = ""
    rusted = ""

    rusted = "rusted " if(@game.world.rusted?)

    if(weapon.has_durability?)
      "#{rusted}#{weapon.name.gsub("_", " ")}#{equipped_string}"
    else
      "#{rusted}#{weapon.name.gsub("_", " ")}#{equipped_string}"
    end
  end

  def weapon_string weapon, equipped
    equipped_string = ""
    rusted = ""

    rusted = "rusted " if(@game.world.rusted?)

    if(weapon.has_durability?)
      "durability: #{weapon.durability_percentage}%"
    else
      "durability: unbreakable"
    end
  end

  def loot_customizations
    @loot_customizations ||= {}
    @loot_customizations
  end

  def add_item_to_scroll_view(key,
                              row:,
                              drop_label:,
                              drop_desc:,
                              take_label:,
                              take_desc:,
                              text_col_1:,
                              text_col_2:,
                              drop_col:,
                              take_col:,
                              w:,
                              h:,
                              size_px:,
                              text_anchor_y:)
    drop_desc ||= "ground"
    take_desc ||= "equipped"

    item_name = "#{key.to_s.gsub("_", " ")}"
    item_drop_desc = "#{drop_desc}: #{@item_loot[key] || 0}"
    item_take_desc = "#{take_desc}: #{@game.world.stores[key] || 0}"

    @scroll_view.add_item Layout.rect(row: row, col: text_col_1, w: 1, h: 1, dx: 4)
                                .merge(text: item_name,
                                       anchor_y: 0.0,
                                       size_px: size_px,
                                       font: "fonts/text.ttf")

    @scroll_view.add_item Layout.rect(row: row, col: text_col_1, w: 1, h: 1, dx: 4)
                                .merge(text: "-",
                                       anchor_x: 0.0,
                                       anchor_y: 1.0,
                                       size_px: size_px,
                                       font: "fonts/text.ttf")
                                .with_metadata(control_id: :item_on_ground_label,
                                               control_ground_desc: drop_desc,
                                               item_key: key)

    @scroll_view.add_item Layout.rect(row: row, col: text_col_2, w: 1, h: 1, dx: -8)
                                .merge(text: "-",
                                       anchor_x: 1.0,
                                       anchor_y: 1.0,
                                       size_px: size_px,
                                       font: "fonts/text.ttf")
                                .with_metadata(control_id: :item_equipped_label,
                                               control_equipped_desc: take_desc,
                                               item_key: key)

    take_rect = Layout.rect(row: row, col: take_col, w: w, h: h)
    @scroll_view.add_item take_rect.center
                                   .merge(text: "take",
                                          font: "fonts/text.ttf",
                                          size_px: size_px,
                                          anchor_x: 0.5,
                                          anchor_y: 0.5)
                                   .with_metadata(control_id: :take_item_button_label, item_key: key)

    @scroll_view.add_item prefab.thick_border(args, take_rect)
                                .merge(font: "fonts/text.ttf",
                                       a11y_id: "item_#{key}_name_take",
                                       a11y_trait: :button,
                                       a11y_text: "take one #{item_name}, #{item_drop_desc}, #{item_take_desc}")
                                .with_metadata(control_id: :take_item_button_rect,
                                               selection_id: "take:#{key}",
                                               item_key: key,
                                               row_rect: row_rect(drop_col, take_col, row, w, h),
                                               callback: lambda {
                                                 @game.world.take_stores key, @item_loot
                                                 callback_item_name = "#{key.to_s.gsub("_", " ")}"
                                                 callback_item_drop_desc = "#{drop_desc}: #{@item_loot[key] || 0}"
                                                 callback_item_take_desc = "#{take_desc}: #{@game.world.stores[key] || 0}"
                                                 outputs.a11y_clear_pending_notifications!
                                                 outputs.a11y["take:#{key}"] = {
                                                   a11y_id: "take:#{key}",
                                                   a11y_trait: :notification,
                                                   a11y_text: "#{callback_item_name}, #{callback_item_drop_desc}, #{callback_item_take_desc}, #{a11y_carry_description}"
                                                 }
                                               })

    drop_rect = Layout.rect(row: row, col: drop_col, w: w, h: h)
    @scroll_view.add_item drop_rect.center
                                   .merge(text: "drop",
                                          font: "fonts/text.ttf",
                                          size_px: size_px,
                                          anchor_x: 0.5,
                                          anchor_y: 0.5)
                                   .with_metadata(control_id: :drop_item_button_label, item_key: key)

    @scroll_view.add_item prefab.thick_border(args, drop_rect)
                                .merge(font: "fonts/text.ttf",
                                       a11y_id: "item_#{key}_name_drop",
                                       a11y_trait: :button,
                                       a11y_text: "drop one #{item_name}, #{item_drop_desc}, #{item_take_desc}")
                                .with_metadata(control_id: :drop_item_button_rect, item_key: key,
                                               selection_id: "drop:#{key}",
                                               item_key: key,
                                               row_rect: row_rect(drop_col, take_col, row, w, h),
                                               callback: lambda {
                                                 @game.world.drop_stores key, @item_loot
                                                 callback_item_name = "#{key.to_s.gsub("_", " ")}"
                                                 callback_item_drop_desc = "#{drop_desc}: #{@item_loot[key] || 0}"
                                                 callback_item_take_desc = "#{take_desc}: #{@game.world.stores[key] || 0}"
                                                 outputs.a11y_clear_pending_notifications!
                                                 outputs.a11y["drop:#{key}"] = {
                                                   a11y_id: "drop:#{key}",
                                                   a11y_trait: :notification,
                                                   a11y_text: "#{callback_item_name}, #{callback_item_drop_desc}, #{callback_item_take_desc}, #{a11y_carry_description}"
                                                 }
                                               })
  end

  def needs_scrolling?
    if GTK.platform?(:mobile) && !GTK.a11y_enabled?
      return false
    else
      @running_row > 8.5
    end
  end

  def scroll_up_button_text
    if GTK.platform?(:mobile) || inputs.last_active == :mouse
      "scroll up ↑"
    elsif inputs.last_active == :controller
      "scroll up ↑ (right analog, R1)"
    elsif inputs.last_active == :keyboard
      "scroll up ↑ (page up)"
    else
      "scroll up ↑"
    end
  end

  def scroll_down_button_text
    if GTK.platform?(:mobile) || inputs.last_active == :mouse
      "scroll down ↓"
    elsif inputs.last_active == :controller
      "scroll down ↓ (right analog, R2)"
    elsif inputs.last_active == :keyboard
      "scroll down ↓ (page down)"
    else
      "scroll down ↓"
    end
  end

  def tick_scroll_buttons
    if needs_scrolling?
      scroll_down_rect = Geometry.rect_props(x: Layout.rect(col: 0).x,
                                             y: @scroll_view.rect.y + 1,
                                             w: Layout.rect(scroll_view_layout).w,
                                             h: Layout.rect(h: 0.75).h)

      outputs.primitives << scroll_down_rect.merge(path: :solid, r: 245, g: 245, b: 245, a: 255)
      outputs.primitives << scroll_down_rect.merge(path: :solid, r: 245, g: 245, b: 245, a: 255)
                                            .center
                                            .merge(text: scroll_down_button_text, font: "fonts/text.ttf", size_px: 20, anchor_x: 0.5, anchor_y: 0.5)

      @scroll_down_button_prefab = prefab.thick_border(args, scroll_down_rect)
                                         .merge(a11y_id: :scroll_down_rect,
                                                a11y_text: "next page",
                                                a11y_trait: :button)
                                         .with_metadata(control_id: :scroll_down_rect,
                                                        selection_id: "scroll_down_rect",
                                                        callback: lambda { scroll_down })

      outputs.primitives << @scroll_down_button_prefab
    else
      @scroll_down_button_prefab = nil
    end

    if needs_scrolling? && @scroll_view.y_offset > 5
      outputs.primitives << @scroll_down_button_prefab

      scroll_up_rect = Geometry.rect_props(x: Layout.rect(col: 0).x,
                                           y: @scroll_view.rect.y + @scroll_view.rect.h - Layout.rect(h: 0.75).h,
                                           w: Layout.rect(scroll_view_layout).w,
                                           h: Layout.rect(h: 0.75).h)

      outputs.primitives << scroll_up_rect.merge(path: :solid, r: 245, g: 245, b: 245, a: 255)
      outputs.primitives << scroll_up_rect.merge(path: :solid, r: 245, g: 245, b: 245, a: 255)
                                          .center
                                          .merge(text: scroll_up_button_text,
                                                 font: "fonts/text.ttf",
                                                 size_px: 20,
                                                 anchor_x: 0.5,
                                                 anchor_y: 0.5)

      @scroll_up_button_prefab = prefab.thick_border(args, scroll_up_rect)
                                       .merge(a11y_id: :scroll_up_rect,
                                              a11y_text: "previous page",
                                              a11y_trait: :button)
                                       .with_metadata(control_id: :scroll_up_rect,
                                                      selection_id: "scroll_up_rect",
                                                      callback: lambda { scroll_up })
      outputs.primitives << @scroll_up_button_prefab
    else
      @scroll_up_button_prefab = nil
    end
  end

  def tick
    calc_buttons
    @scroll_view.args = args
    @scroll_view.tick
    if @running_row <= 8.5
      @scrolling = false
      enable_selection_prefab!
    end

    tick_inputs

    filter_mode = if GTK.a11y_enabled?
                    :inside_rect?
                  else
                    :intersect_rect?
                  end

    outputs.primitives << @scroll_view.prefab(filter_mode: filter_mode)
    outputs.primitives << title_label
    outputs.primitives << weight_label

    if GTK.a11y_enabled?
      @scroll_view.offset_items(filter_mode: filter_mode).each do |i|
        if i.a11y_id
          outputs.a11y[i.a11y_id] = {
            x: i.x,
            y: i.y,
            w: i.w,
            h: i.h,
            anchor_x: i.anchor_x,
            anchor_y: i.anchor_y,
            a11y_id: i.a11y_id,
            a11y_text: i.a11y_text,
            a11y_trait: i.a11y_trait,
            a11y_hidden: i.a11y_hidden
          }
        end
      end
    end

    render_buttons

    tick_scroll_buttons

    tick_inputs_scroll
  end

  def tick_inputs_scroll_controller
    # outputs.watch_attrs @scroll_view
    # outputs.watch_ivars self

    # outputs.watch @scroll_view.y_offset
    # outputs.watch @scroll_view.content_height
    # outputs.watch @scroll_view.rect.h
    # outputs.watch @scroll_view.rect.h + @scroll_view.y_offset
    # outputs.watch @scrolling
    return if inputs.last_active != :controller

    if inputs.controller_one.key_down.r1
      scroll_up_page
      set_selection! nil
      disable_selection_prefab!
      @scrolling = true
    elsif inputs.controller_one.key_down.r2
      scroll_down_page
      set_selection! nil
      disable_selection_prefab!
      @scrolling = true
    elsif inputs.controller_one.right_analog_y_perc.abs > 0.20
      @scrolling = true
      @scroll_view.scroll_view_dy -= inputs.controller_one.right_analog_y_perc * 1.5
      set_selection! nil
      disable_selection_prefab!
    elsif @scrolling == true &&
          @scroll_view.scroll_view_dy.abs == 0 &&
          @scroll_view.y_offset.round >= 0 &&
          @scroll_view.y_offset.round <= @scroll_view.y_offset_top_limit - 80
      @scrolling = false
      @scroll_view.y_offset = @scroll_view.y_offset.round
      GTK.scheduled_callbacks[Kernel.tick_count + 15] = lambda {
        enable_selection_prefab!
        set_selection! available_selections_visible.first.id
      }
    elsif @scrolling == true
      @scroll_view.scroll_view_dy *= 0.5
    end
  end

  def tick_inputs_scroll_mouse
    return if inputs.last_active != :mouse

    if @scroll_down_button_prefab && inputs.mouse.click && inputs.mouse.inside_rect?(@scroll_down_button_prefab)
      set_selection! nil
      disable_selection_prefab!
      scroll_down_page
      possible_a11y_focus_points = @scroll_view.offset_items(filter_mode: :inside_rect?)
                                               .find_all { |prim| prim.a11y_id && !prim.a11y_hidden }

      outputs.a11y_clear_pending_notifications!
      GTK.on_tick_count Kernel.tick_count + 10 do
        if possible_a11y_focus_points.length > 0
          outputs.a11y["set_selection"] = {
            a11y_id: "set_selection",
            a11y_trait: :notification,
            a11y_notification_target: possible_a11y_focus_points.last.a11y_id,
          }
        end
      end

      GTK.on_tick_count Kernel.tick_count + 90 do
        if @scroll_view.scroll_perc >= 0.95
          outputs.a11y["last_page"] = {
            a11y_id: "last_page",
            a11y_trait: :notification,
            a11y_text: "you are on the last page"
          }
        end
      end
    elsif @scroll_up_button_prefab && inputs.mouse.click && inputs.mouse.inside_rect?(@scroll_up_button_prefab)
      set_selection! nil
      disable_selection_prefab!
      scroll_up_page
      possible_a11y_focus_points = @scroll_view.offset_items(filter_mode: :inside_rect?)
                                               .find_all { |prim| prim.a11y_id && !prim.a11y_hidden }


      outputs.a11y_clear_pending_notifications!

      GTK.on_tick_count Kernel.tick_count + 90 do
        if possible_a11y_focus_points.length > 0
          outputs.a11y["set_selection"] = {
            a11y_id: "set_selection",
            a11y_trait: :notification,
            a11y_notification_target: possible_a11y_focus_points.first.a11y_id,
          }
        end
      end
    end
  end

  def tick_inputs_scroll_keyboard
    return if inputs.last_active != :keyboard

    if inputs.keyboard.key_down.page_up || inputs.keyboard.key_down.page_down
      @scrolling = true
      set_selection! nil
      disable_selection_prefab!
      if inputs.keyboard.key_down.page_up
        scroll_up
      elsif inputs.keyboard.key_down.page_down
        scroll_down
      end
    elsif @scrolling == true
      should_exit_scrolling = if !@scroll_view.target_y_offset
                                true
                              elsif !@scroll_view.y_offset
                                true
                              elsif @scroll_view.y_offset.round == @scroll_view.target_y_offset.round
                                true
                              else
                                false
                              end
      if should_exit_scrolling
        @scrolling = false
        @scroll_view.y_offset = @scroll_view.y_offset.round
        GTK.scheduled_callbacks[Kernel.tick_count + 30] = lambda {
          enable_selection_prefab!
          set_selection! available_selections_visible.first.id
        }
      end
    end
  end

  def tick_inputs_scroll
    tick_inputs_scroll_controller
    tick_inputs_scroll_mouse
    tick_inputs_scroll_keyboard
  end

  def scroll_down_page
    @scroll_view.scroll_down_page Layout.rect(h: 4).h
  end

  def scroll_up_page
    @scroll_view.scroll_up_page Layout.rect(h: 4).h
    if @scroll_view.target_y_offset < 1
      disable_selection_prefab!
      @scroll_view.target_y_offset = 0
      GTK.scheduled_callbacks[Kernel.tick_count + 40] = lambda {
        enable_selection_prefab!
        set_selection! "take_all_loot"
      }
    end
  end

  def scroll_down target_y_offset_h = 3
    @scroll_view.target_y_offset = @scroll_view.y_offset + Layout.rect(h: target_y_offset_h).h
    if @scroll_view.target_y_offset > @scroll_view.y_offset_top_limit - 80
      @scroll_view.target_y_offset = @scroll_view.y_offset_top_limit - 80
    end
  end

  def scroll_up target_y_offset_h = 3
    @scroll_view.target_y_offset = @scroll_view.y_offset - Layout.rect(h: target_y_offset_h).h
    if @scroll_view.target_y_offset < 1
      disable_selection_prefab!
      @scroll_view.target_y_offset = 0
      GTK.scheduled_callbacks[Kernel.tick_count + 40] = lambda {
        enable_selection_prefab!
        set_selection! "take_all_loot"
      }
    end
  end

  def tick_inputs
    if @scroll_view.clicked_items.length > 0
      button = @scroll_view.clicked_items.first

      if button.metadata && button.metadata.callback
        button.metadata.callback.call
        queue_flash :loot_clicked,
                    button.x,
                    button.y,
                    button.w,
                    button.h,
                    prefab.black.r,
                    prefab.black.g,
                    prefab.black.b,
                    1.0/10.0
      end
    end

    @scroll_view.items.each do |item|
      next unless item.metadata
      if item.metadata.control_id == :item_on_ground_label
        item.text = "#{item.metadata.control_ground_desc}: #{@item_loot[item.metadata.item_key] || 0}"
      elsif item.metadata.control_id == :item_equipped_label
        item.text = "#{item.metadata.control_equipped_desc}: #{@game.world.stores[item.metadata.item_key] || 0}"
      elsif item.metadata.control_id == :take_weapon_button_label || item.metadata.control_id == :take_weapon_button_rect
        if @weapon_loot.include?(item.metadata.weapon)
          item.a11y_hidden = false
          item.a ||= 255
          item.a += 25
        else
          item.a11y_hidden = true
          item.a ||= 0
          item.a -= 25
        end

        item.a = item.a.clamp(0, 255)
      elsif item.metadata.control_id == :drop_weapon_button_label || item.metadata.control_id == :drop_weapon_button_rect
        if @weapon_loot.include?(item.metadata.weapon)
          item.a11y_hidden = true
          item.a ||= 0
          item.a -= 25
        else
          item.a11y_hidden = false
          item.a ||= 255
          item.a += 25
        end

        item.a = item.a.clamp(0, 255)
      end
    end
  end

  def progress_bar_ids
    []
  end

  def available_selections
    # return [] if inputs.last_active != :keyboard && inputs.last_active != :controller && !GTK.a11y_enabled?

    filter_mode = if GTK.a11y_enabled?
                    :inside_rect?
                  else
                    :intersect_rect?
                  end

    items = @scroll_view.offset_items(filter_mode: filter_mode)
                        .find_all { |item| item.metadata && item.metadata.callback && Geometry.inside_rect?(item, @scroll_view.rect) }
                        .map_with_index do |item, i|
                          {
                            id: item.metadata.selection_id,
                            rect: item.slice(:x, :y, :w, :h, :a),
                            metadata: item.metadata,
                          }
                        end

    if @scroll_down_button_prefab
      items << { id: :scroll_down_button,
                 rect: @scroll_down_button_prefab.slice(:x, :y, :w, :h, :a),
                 metadata: @scroll_down_button_prefab.metadata }
    end

    if @scroll_up_button_prefab
      items << { id: :scroll_up_button,
                 rect: @scroll_up_button_prefab.slice(:x, :y, :w, :h, :a),
                 metadata: @scroll_up_button_prefab.metadata }
    end

    items << { id: :run_button, rect: run_button } if run_button
    items << { id: :continue_button, rect: continue_button } if continue_button

    items
  end

  def button_ids
    [:continue_button, :run_button]
  end

  def current_scene
    return { text: [] } if !current_event

    current_event.scenes[current_event.current_scene]
  end

  def white_listed_loot
    current_scene[:white_listed_loot]
  end

  def white_listed_weapons
    current_scene[:white_listed_weapons]
  end

  def drop_label
    current_scene[:loot_drop_command]
  end

  def drop_desc
    current_scene[:loot_drop_desc]
  end

  def take_label
    current_scene[:loot_take_command]
  end

  def take_desc
    current_scene[:loot_take_desc]
  end

  def controller_instructions accept_char
    "#{default_controller_instructions} [RIGHT ANALOG, R1, R2]: scroll."
  end

  def keyboard_instructions
    "#{default_keyboard_instructions} [PAGE UP/PAGE DOWN]: scroll."
  end
end

# def debug_loot_scene
#   $gtk.scheduled_callbacks.clear
#   GTK.reset_next_tick

#   $gtk.scheduled_callbacks[1] = lambda { |args|
#     $game.world.god_mode!
#     $game.world.event = nil
#     purge_class :MiniWorldLootScene
#     purge_class :LootScene
#     purge_class :LootSceneBase
#     require 'app/main.rb'

#     $gtk.scheduled_callbacks[30] = lambda { |args|
#       $gtk.scheduled_callbacks[Kernel.tick_count + 70 * 1] = lambda { |args|
#         puts "setting scene"
#         $state.next_scene = :world_scene
#         $game.world.defector_x = 0
#         $game.world.defector_y = 0
#         $game.world.event = nil
#         $game.world.move_left
#         $game.world.event = nil
#         $game.world.move_right
#       }

#       $gtk.scheduled_callbacks[Kernel.tick_count + 70 * 2] = lambda { |args|
#         puts "moving near thrusters"
#         $game.world.defector_x = $game.world.x - 1
#         $game.world.defector_y = $game.world.y
#         $game.world.current_hp = 9999
#         $game.world.stores.clear
#         $game.world.stores[:food] = 10
#         $game.world.stores[:grenade] = 5
#         $game.world.stores[:stealth] = 1
#         $game.world.stores[:bolas] = 5
#         $game.world.stores[:bullets] = 100
#         $game.world.stores[:battery] = 100
#         $game.world.stores[:torch] = 5
#         $game.world.stores[:eyes] = 5
#         $game.world.stores[:teeth] = 5
#         $game.world.stores[:flesh] = 5
#         $game.world.stores[:torch] = 5
#         $game.world.weapon_stores.clear
#         $game.world.weapon_stores << Knife.new
#         $game.world.weapon_stores << IronSword.new
#         $game.world.weapon_stores << SteelSword.new
#         $game.world.weapon_stores << Katana.new
#         $game.world.weapon_stores << Rifle.new
#         $game.world.weapon_stores << Laser.new
#         $game.world.weapon_stores << Masamune.new
#         $game.world.weapon_stores << Murasame.new
#         $game.world.move_left
#       }

#       $gtk.scheduled_callbacks[Kernel.tick_count + 70 * 3] = lambda { |args|
#         $game.world.god_mode!
#         $scene.option_1_button_selected
#       }

#       $gtk.scheduled_callbacks[Kernel.tick_count + 70 * 4] = lambda { |args|
#         $game.world.event.scenes[$game.world.event.current_scene][:health] = 0
#       }

#       $gtk.scheduled_callbacks[Kernel.tick_count + 70 * 4 + 30] = lambda { |args|
#         $scene.loot_button_selected
#       }

#       $gtk.scheduled_callbacks[Kernel.tick_count + 70 * 5] = lambda { |args|
#         $gtk.simulate_mouse_clicks [
#             { x: 571, y: 458 },
#             # { x: 640, y: 944 },
#             # { x: 640, y: 944 },
#             # { x: 378, y: 1140 },
#             # { x: 414, y: 78 }
#           ]
#       }

#       $gtk.scheduled_callbacks[Kernel.tick_count + 140 * 4] = lambda { |args|
#         # puts "clicking loot"
#         $gtk.simulate_mouse_clicks [
#             { x: 625, y: 1042 },
#             # { x: 640, y: 944 },
#             # { x: 640, y: 944 },
#             # { x: 378, y: 1140 },
#             # { x: 414, y: 78 }
#           ]
#       }

#       # $gtk.scheduled_callbacks[Kernel.tick_count + 140 * 8] = lambda { |args|
#       #   puts "clearing callbacks"
#       #   $gtk.scheduled_callbacks.clear
#       # }
#     }
#   }
# end

# $scene.will_appear_base if $scene
