# Copyright 2014 Scratchwork Development LLC. All rights reserved.

class MiniWorldBattleScene < Scene
  attr :battle_last_attack_result, :actions_hash

  def id
    :mini_world_battle_scene
  end

  def world
    @game.world
  end

  def will_appear
    @death_sound_played = false
    @player_healed = false
    @previous_battle_tick_result = nil
    @battle_last_attack_result = nil
    @end_battle_debounce = 90
    @a11y_notify_percentages = {}
    @start_battle_debounce = 120
    if GTK.a11y_enabled?
      @start_battle_debounce *= 6
    end
    @a11y_debounce ||= 0
    @previous_hp = @game.world.current_hp
    @current_hp = @game.world.current_hp
    world.init_actions
    @actions_hash = @game.world.actions_hash
    world.meat_cool_down = @game.world.meat_delay
    game.world.init_attacks
    if game.world.stores[:grenade] && game.world.stores[:grenade] == 0
      game.world.stores.delete :grenade
    end

    args.outputs.a11y[:notify_will_appear] = {
      a11y_trait: :notification,
      a11y_notification_target: :battle_title
    }

    if game.world.stores[:bolas] && game.world.stores[:bolas] == 0
      game.world.stores.delete :bolas
    end
    gen_spots!
  end

  def left_side_row game, id
    if Grid.orientation == :landscape
      @left_spots[id] + 5
    else
      @left_spots[id] + 9
    end
  end

  def right_side_row game, id
    if Grid.orientation == :landscape
      @right_spots[id] + 5
    else
      @right_spots[id] + 9
    end
  end

  def gen_spots!
    @left_spots = {}
    @right_spots = {}

    [
      :fists,
      :knife,
      :iron_sword,
      :steel_sword,
      :katana,
      :murasame,
      :masamune
    ].each do |id|
      if @actions_hash.keys.include? id
        @left_spots[id] = @left_spots.length * pg_height
      end
    end

    [
      :food,
      :bolas,
      :grenade,
      :rifle,
      :laser,
      :stealth
    ].each do |id|
      if should_show_progress_bar? game, id
        @right_spots[id] = @right_spots.length * pg_height
      end
    end
  end

  def tick
    calc
    render
  end

  def should_tick_battle?
    return false if game.world.event.event.battle_complete?
    return false if game.world.dead?
    return false if GTK.a11y_enabled? && @a11y_debounce > 0
    return true
  end

  def calc
    battle_zmod = if GTK.a11y_enabled?
                    400
                  else
                    60
                  end

    battle_mini_tick_zmod = battle_zmod.idiv(60)

    if @start_battle_debounce > 0
      @start_battle_debounce -= 1
    elsif GTK.a11y_enabled? && @a11y_debounce > 0
      @a11y_debounce -= 1
    elsif args.state.tick_count.zmod? battle_zmod
      if should_tick_battle?
        game.last_battle_tick_result = @game.world.event.event.tick
      end
    end

    if args.state.tick_count.zmod? battle_mini_tick_zmod
      if should_tick_battle?
        @actions_hash.each do |k, v|
          v.mini_tick
        end
      end
    end

    if @previous_hp > 0 && @game.world.current_hp <= 0
      queue_flash :world_dead, Grid.allscreen_x, Grid.allscreen_y, Grid.allscreen_w, Grid.allscreen_h, 255, 0, 0, 1.0 / 10.0
      if !@death_sound_played
        queue_sound "sounds/death.ogg"
        @game.save
        @death_sound_played = true
        args.outputs.a11y[:death_occurred] = {
          a11y_trait: :notification,
          a11y_notification_target: :death_button,
          a11y_notification_debounce: 60
        }
      end
    end

    @previous_hp = @game.world.current_hp

    debounce_end_battle

    calc_battle
    calc_buttons
    calc_progress_bars
    calc_controller_button_input
    calc_keyboard_input
  end

  def calc_keyboard_input
    return nil if !should_show_progress_bar? game, :fists
    if args.inputs.keyboard.key_down.q
      set_selection! :fists_progress_bar
    elsif args.inputs.keyboard.key_down.z
      key = @left_spots.keys.last
      if key != :fists
        progess_bar = "#{key}_progress_bar".to_sym
        set_selection! progess_bar
      end
    elsif args.inputs.keyboard.key_down.e
      key = @right_spots.keys.first
      if key
        progess_bar = "#{key}_progress_bar".to_sym
        set_selection! progess_bar
      end
    elsif args.inputs.keyboard.key_down.c
      key = @right_spots.keys.last
      if key
        progess_bar = "#{key}_progress_bar".to_sym
        set_selection! progess_bar
      end
    end
  end

  def calc_controller_button_input
    return nil if !should_show_progress_bar? game, :fists
    if args.inputs.controller_one.key_down.l1
      set_selection! :fists_progress_bar
    elsif args.inputs.controller_one.key_down.l2
      key = @left_spots.keys.last
      if key != :fists
        progess_bar = "#{key}_progress_bar".to_sym
        set_selection! progess_bar
      end
    elsif args.inputs.controller_one.key_down.r1
      key = @right_spots.keys.first
      if key
        progess_bar = "#{key}_progress_bar".to_sym
        set_selection! progess_bar
      end
    elsif args.inputs.controller_one.key_down.r2
      key = @right_spots.keys.last
      if key
        progess_bar = "#{key}_progress_bar".to_sym
        set_selection! progess_bar
      end
    end
  end

  def debounce_end_battle
    if game.world.event.event.battle_complete? || game.world.dead?
      @end_battle_debounce -= 1
    end
  end

  def death_button_selected
    state.next_scene = :game_over_scene
  end

  def ranged_enemies
    [
      :soldier,
      :sniper,
      :commando,
      :defector
    ]
  end

  def animate_melee_enemy!
    return if @animate_melee_enemy
    @animate_melee_enemy = true
    @animate_melee_enemy_x = nil
    @animate_melee_enemy_direction = nil
  end

  def animate_bullet_enemy!
    return if @animate_bullet_enemy
    @animate_bullet_enemy = true
    @animate_bullet_enemy_x = nil
  end

  def animate_melee_player!
    return if @animate_melee_player
    @animate_melee_player = true
    @animate_melee_player_x = nil
    @animate_melee_player_direction = nil
  end

  def animate_bullet_player!
    return if @animate_bullet_player
    @animate_bullet_player = true
    @animate_bullet_player_x = nil
  end

  def a11y_notify id, message, debounce: 300
    return if !GTK.a11y_enabled?
    $args.outputs.a11y["enemy_battle_result"] = {
      a11y_id: id,
      a11y_trait: :notification,
      a11y_text: message
    }
    @a11y_debounce = debounce
  end

  def calc_battle
    return unless in_battle?

    if @previous_battle_tick_result != game.last_battle_tick_result
      if (game.last_battle_tick_result.attacked)
        if ranged_enemies.include?(current_scene[:enemy])
          if game.last_battle_tick_result.missed
            queue_sound 'sounds/missed.ogg'
            a11y_notify :battle_result, "enemy missed."
          else
            a11y_notify :battle_result, "enemy attacked.\n#{game.last_battle_tick_result.damage} damage taken, hp: #{game.world.current_hp.to_i} of #{game.world.max_hp.to_i}."
            queue_sound 'sounds/enemy_ranged.ogg' if !game.world.dead?
          end
          animate_bullet_enemy!
        else
          if game.last_battle_tick_result.missed
            queue_sound 'sounds/missed.ogg'
            a11y_notify :battle_result, "enemy missed."
          else
            queue_sound 'sounds/enemy_melee.ogg' if !game.world.dead?
            a11y_notify :battle_result, "#{game.last_battle_tick_result.damage} damage taken, hp: #{game.world.current_hp.to_i} of #{game.world.max_hp.to_i}."
          end
          animate_melee_enemy!
        end
      end
      @previous_battle_tick_result = game.last_battle_tick_result
    end
  end

  def in_battle?
    return false unless game.world.event.event
    return false unless game.world.event.event.in_battle?
    return true
  end

  def enemy_bullet_label
    return nil unless game.world.event.event
    return nil unless game.world.event.event.in_battle?
    return nil unless @animate_bullet_enemy

    @animate_bullet_enemy_x ||= enemy_label_x
    diff = (@animate_bullet_enemy_x - player_label_x).abs
    @animate_bullet_enemy_x -= diff * 0.20
    @animate_bullet_enemy_x = @animate_bullet_enemy_x.to_i

    if diff < 40
      @animate_bullet_enemy = false
    end

    {
      x: @animate_bullet_enemy_x + 20,
      y: Grid.h - 100 - 150,
      text: "o",
      size_enum: 5,
      anchor_x: 0.5, anchor_y: 0.5,
      font: "fonts/sprite.ttf",
      **prefab.black
    }
  end

  def render
    outputs.primitives << battle_title
    outputs.primitives << battle_text
    outputs.primitives << player_labels
    outputs.primitives << enemy_labels
    outputs.primitives << enemy_bullet_label
    outputs.primitives << player_bullet_label
    outputs.primitives << controller_jump_labels
    outputs.primitives << keyboard_jump_labels

    render_buttons
    render_progress_bars
  end

  def controller_jump_labels
    return nil if Grid.portrait?
    return nil if inputs.last_active != :controller
    return nil if !should_show_progress_bar? game, :fists
    label_style = { size_px: 22, font: "fonts/text.ttf", anchor_x: 0.5, anchor_y: 0.5 }

    bottom_left = if @left_spots.keys.length > 1
                    Layout.rect(row: left_side_row(game, @left_spots.keys.last), col: 5, w: 1, h: 1)
                          .center
                          .merge(text: "[L2]", **label_style)
                  else
                    nil
                  end

    bottom_right = if @right_spots.keys.length > 1
                     Layout.rect(row: right_side_row(game, @right_spots.keys.last), col: 18, w: 1, h: 1)
                           .center
                           .merge(text: "[R2]", **label_style)
                   else
                     nil
                   end
    [
      Layout.rect(row: 5, col: 5, w: 1, h: 1)
            .center
            .merge(text: "[L1]", **label_style),
      Layout.rect(row: 5, col: 18, w: 1, h: 1)
            .center
            .merge(text: "[R1]", **label_style),
      bottom_left,
      bottom_right
    ]
  end

  def keyboard_jump_labels
    return nil if Grid.portrait?
    return nil if inputs.last_active != :keyboard
    return nil if !should_show_progress_bar? game, :fists
    label_style = { size_px: 22, font: "fonts/text.ttf", anchor_x: 0.5, anchor_y: 0.5 }

    bottom_left = if @left_spots.keys.length > 1
                    Layout.rect(row: left_side_row(game, @left_spots.keys.last), col: 5, w: 1, h: 1)
                          .center
                          .merge(text: "[Z]", **label_style)
                  else
                    nil
                  end

    bottom_right = if @right_spots.keys.length > 1
                     Layout.rect(row: right_side_row(game, @right_spots.keys.last), col: 18, w: 1, h: 1)
                           .center
                           .merge(text: "[C]", **label_style)
                   else
                     nil
                   end
    [
      Layout.rect(row: 5, col: 5, w: 1, h: 1)
            .center
            .merge(text: "[Q]", **label_style),
      Layout.rect(row: 5, col: 18, w: 1, h: 1)
            .center
            .merge(text: "[E]", **label_style),
      bottom_left,
      bottom_right
    ]
  end

  def player_bullet_label
    return nil unless game.world.event.event
    return nil unless game.world.event.event.in_battle?
    return nil unless @animate_bullet_player

    @animate_bullet_player_x ||= player_label_x
    diff = (@animate_bullet_player_x - enemy_label_x).abs
    @animate_bullet_player_x += diff * 0.20
    @animate_bullet_player_x = @animate_bullet_player_x.to_i

    if diff < 40
      @animate_bullet_player = false
    end

    {
      x: @animate_bullet_player_x - 20,
      y: Grid.h - 100 - 150,
      text: "O",
      font: "fonts/sprite.ttf",
      anchor_x: 0.5, anchor_y: 0.5,
      **prefab.black
    }
  end

  def button_ids
    [
      :death_button,
      :loot_button,
      :heal_button
    ]
  end

  def fists_progress_bar_selected
    act_with :fists, "fists"
  end

  def knife_progress_bar_selected
    act_with :knife, "knife"
  end

  def iron_sword_progress_bar_selected
    act_with :iron_sword, "iron sword"
  end

  def steel_sword_progress_bar_selected
    act_with :steel_sword, "steel sword"
  end

  def katana_progress_bar_selected
    act_with :katana, "katana"
  end

  def murasame_progress_bar_selected
    act_with :murasame, "murasame"
  end

  def masamune_progress_bar_selected
    act_with :masamune, "masamune"
  end

  def food_progress_bar_selected
    act_with :food, "food"
  end

  def bolas_progress_bar_selected
    act_with :bolas, "bolas"
  end

  def grenade_progress_bar_selected
    act_with :grenade, "grenade"
  end

  def rifle_progress_bar_selected
    act_with :rifle, "rifle"
  end

  def laser_progress_bar_selected
    act_with :laser, "laser"
  end

  def stealth_progress_bar_selected
    act_with :stealth, "stealth"
  end


  def progress_bar_ids
    [
      :fists_progress_bar,
      :knife_progress_bar,
      :iron_sword_progress_bar,
      :steel_sword_progress_bar,
      :katana_progress_bar,
      :murasame_progress_bar,
      :masamune_progress_bar,

      :food_progress_bar,
      :bolas_progress_bar,
      :grenade_progress_bar,
      :rifle_progress_bar,
      :laser_progress_bar,
      :stealth_progress_bar,
    ]
  end

  def mini_world
    @game.world.event
  end

  def should_show_progress_bar? game, id
    return false unless in_battle?
    return false if mini_world.event.battle_complete?
    return false if game.world.dead?
    !!@actions_hash[id]
  end

  def progress_bar_a11y action, weapon_text
    return {} if !GTK.a11y_enabled?

    a11y_ammo = ""

    needs_quantity = [
      { action: :rifle, stores: :bullets },
      { action: :laser , stores: :battery },
      { action: :grenade , stores: :grenade },
      { action: :bolas , stores: :bolas },
      { action: :food, stores: :food }
    ]

    entry = needs_quantity.find { |n| n[:action] == action }

    if entry
      a11y_ammo = "(#{entry[:stores]} count: #{game.world.stores[entry[:stores]] || 0})"
      if entry[:stores] == :food
        a11y_ammo += ", heal for: #{world.meat_heal} hp."
      end
    end

    action_entry = @actions_hash[action]
    weapon = action_entry.weapon

    a11y_text = if weapon.has_durability?
                  "#{weapon_text} at #{(action_entry.percentage * 100).to_i} percent. durability at #{weapon.durability_percentage} percent. #{a11y_ammo}."
                else
                  "#{weapon_text} at #{(action_entry.percentage * 100).to_i} percent. #{a11y_ammo}."
                end

    {
      a11y_id: "battle_#{action}_progress_bar",
      a11y_text: a11y_text,
      a11y_trait: :button
    }
  end

  def a11y_notify_percentages action, weapon
    @a11y_notify_percentages ||= {}
    @a11y_notify_percentages[action] ||= @actions_hash[action].percentage
    if @a11y_notify_percentages[action] != @actions_hash[action].percentage && @actions_hash[action].percentage == 1.0
      a11y_notify :battle_result, "#{weapon} ready."
    end
    @a11y_notify_percentages[action] = @actions_hash[action].percentage
  end

  def pg_height
    if Grid.orientation == :landscape
      1
    else
      1.5
    end
  end

  def fists_progress_bar
    return nil unless should_show_progress_bar? game, :fists

    a11y_notify_percentages :fists, "fists"
    rect_row = left_side_row(game, :fists)
    rect_col = left_side_column

    rect = Layout.rect(row: rect_row, col: rect_col, h: pg_height)
    {
      x:    rect.x,
      y:    rect.y,
      w:    rect.w,
      h:    rect.h,
      text: "fists (level: #{world.punch_level})",
      text_size_px: progress_bar_text_size,
      **progress_bar_a11y(:fists, "fists"),
      perc: @actions_hash[:fists].percentage
    }
  end

  def knife_progress_bar
    weapon_progress_bar side: :left,
                        key: :knife,
                        text: "knife"
  end

  def left_side_column
    if Grid.orientation == :landscape
      [6, 11]
    else
      [0, 5]
    end
  end

  def right_side_column
    if Grid.orientation == :landscape
      [12, 17]
    else
      [6, 11]
    end
  end

  def weapon_progress_bar side:, key:, text:;
    return nil unless should_show_progress_bar? game, key

    a11y_notify_percentages key, text
    rect_row = if side == :left
                 left_side_row(game, key)
               else
                 right_side_row(game, key)
               end

    rect_col = if side == :left
                 left_side_column
               else
                 right_side_column
               end

    action = @actions_hash[key]
    weapon = action.weapon

    progress_bar_text = if weapon.has_durability?
                          "#{text} (#{@actions_hash[key].durability_percentage}%)"
                        elsif weapon.requires_ammo?
                          "#{text}: #{world.stores[weapon.ammo_key] || 0}"
                        elsif weapon.has_durability?
                          "#{text} (#{@actions_hash[key].durability_percentage}%)"
                        else
                          text
                        end

    progress_bar_text_2 = if weapon.requires_ammo? && weapon.name != weapon.ammo_key
                            "#{weapon.ammo_key}: #{world.stores[weapon.ammo_key] || 0}"
                          else
                            nil
                          end

    text_r_color = if @actions_hash[key].durability_percentage == 100
                     0
                   else
                     200 * (1 - @actions_hash[key].durability_percentage.fdiv(100))
                   end


    text_color = { r: text_r_color,
                   g: 0,
                   b: 0 }

    rect = Layout.rect(row: rect_row, col: rect_col, h: pg_height)
    {
      x:    rect.x,
      y:    rect.y,
      w:    rect.w,
      h:    rect.h,
      text: progress_bar_text,
      text_2: progress_bar_text_2,
      text_size_px: progress_bar_text_size,
      perc: @actions_hash[key].percentage.clamp(0, 1),
      **progress_bar_a11y(key, text),
      text_color: text_color
    }
  end

  def iron_sword_progress_bar
    weapon_progress_bar side: :left,
                        key:  :iron_sword,
                        text: "iron sword"
  end

  def steel_sword_progress_bar
    weapon_progress_bar side: :left,
                        key:  :steel_sword,
                        text: "steel sword"
  end

  def katana_progress_bar
    weapon_progress_bar side: :left,
                        key:  :katana,
                        text: "katana"
  end

  def murasame_progress_bar
    weapon_progress_bar side: :left,
                        key:  :murasame,
                        text: "murasame"
  end

  def masamune_progress_bar
    weapon_progress_bar side: :left,
                        key:  :masamune,
                        text: "masamune"
  end

  def progress_bar_text_size
    if Grid.landscape?
      24
    else
      28
    end
  end

  def food_progress_bar
    return nil unless should_show_progress_bar? game, :food
    a11y_notify_percentages :food, "food"
    rect_row = right_side_row(game, :food)
    rect_col = right_side_column

    rect = Layout.rect(row: rect_row, col: rect_col, h: pg_height)
    {
      x:    rect.x,
      y:    rect.y,
      w:    rect.w,
      h:    rect.h,
      text: "food: #{world.stores[:food] || 0} (+#{world.meat_heal} hp)",
      text_size_px: progress_bar_text_size,
      **progress_bar_a11y(:food, "food"),
      perc: @actions_hash[:food].percentage
    }
  end

  def bolas_progress_bar
    weapon_progress_bar side: :right, key: :bolas, text: "bolas"
  end

  def grenade_progress_bar
    weapon_progress_bar side: :right, key: :grenade, text: "grenade"
  end

  def rifle_progress_bar
    weapon_progress_bar side: :right, key: :rifle, text: "rifle"
  end

  def laser_progress_bar
    weapon_progress_bar side: :right, key: :laser, text: "laser"
  end

  def stealth_progress_bar
    return nil unless should_show_progress_bar? game, :stealth
    a11y_notify_percentages :stealth, "dodge"
    rect_row = right_side_row(game, :stealth)
    rect_col = right_side_column

    rect = Layout.rect(row: rect_row, col: rect_col, h: pg_height)
    {
      x:    rect.x,
      y:    rect.y,
      w:    rect.w,
      h:    rect.h,
      text: "dodge",
      text_size_px: progress_bar_text_size,
      **progress_bar_a11y(:stealth, "dodge"),
      perc: @actions_hash[:stealth].percentage
    }
  end

  def death_button
    return nil unless in_battle?
    return nil unless game.world.dead?
    return nil if @end_battle_debounce > 0

    if @end_battle_debounce == 0
      $args.outputs.a11y["death_button"] = {
        a11y_trait: :notification,
        a11y_notification_target: "death_button",
      }
    end

    r = if Grid.orientation == :landscape
          Layout.rect(row: 6, col: [6, 17], h: 2)
        else
          Layout.rect(row: 11, col: [0, 11], h: 2)
        end

    r.merge(text: "death", a11y_id: "death_button", a11y_text: "death", a11y_trait: :button)
  end

  def loot_button
    return nil unless in_battle?
    return nil if game.world.dead?
    return nil unless game.world.event.event.battle_complete?
    return nil if @end_battle_debounce > 0

    if @end_battle_debounce == 0
      $args.outputs.a11y["death_button"] = {
        a11y_trait: :notification,
        a11y_notification_target: "loot_button",
      }
    end

    r = if Grid.orientation == :landscape
          Layout.rect(row: 6, col: [6, 17], h: 2)
        else
          Layout.rect(row: 11, col: [0, 11], h: 2)
        end

    r.merge(text: "loot",
            a11y_id: "loot_button",
            a11y_text: "loot",
            a11y_trait: :button)
  end

  def heal_button
    return nil unless in_battle?
    return nil if game.world.dead?
    return nil unless game.world.event.event.battle_complete?
    return nil if @end_battle_debounce > 0

    r = if Grid.orientation == :landscape
          Layout.rect(row: 8, col: [6, 17], h: 2)
        else
          Layout.rect(row: 13, col: [0, 11], h: 2)
        end

    r.merge(text: "food: #{world.stores[:food] || 0} (+#{world.meat_heal} hp)",
            a11y_id: "heal",
            a11y_text: "heal. food available: #{world.stores[:food] || 0}, heal for #{world.meat_heal} hp, current hp: #{world.current_hp} of #{world.max_hp} hp.",
            a11y_trait: :button)
  end

  def current_scene
    return { text: [] } if !game.world.event.event

    game.world.event.event.scenes[game.world.event.event.current_scene]
  end

  def heal_button_selected
    game.world.event.event.act(@actions_hash[:food])
    @player_healed = true
    play_attack_sound :food

    outputs.a11y[:heal_button_selected] = {
      a11y_id: :heal_button_selected,
      a11y_text: "healed for #{world.meat_heal} hp, current hp #{world.current_hp} of #{world.max_hp}, food available: #{world.stores[:food] || 0}",
      a11y_trait: :notification
    }
  end

  def battle_text
    return nil unless game.world.event.event
    return nil unless game.world.event.event.in_battle?

    text_with_location = []
    r = if Grid.orientation == :landscape
          Layout.rect(row: 1.5, col: [6, 17], h: 1)
        else
          Layout.rect(row: 1.5, col: [0, 11], h: 1)
        end

    starting_x = r.x
    starting_y = r.y

    size_px = if Grid.orientation == :landscape
                22
              else
                30
              end

    split_length = if Grid.orientation == :landscape
                     50
                   else
                     38
                   end

    anchor_increment = 0
    current_scene[:text].each_with_index do |t, j|
      long_lines_split(t.strip, split_length).each_with_index do |line, i|
        next if line.strip.length == 0
        anchor_increment += 1.1
        text_with_location << { x: r.x + 4,
                                y: r.center.y,
                                font: "fonts/text.ttf",
                                size_px: size_px,
                                anchor_increment: anchor_increment,
                                text: line.strip }
      end
      anchor_increment += 1
    end

    l = text_with_location.length
    text_with_location.each_with_index do |t, i|
      if l <= 2
        t.x = Grid.w / 2
        t.anchor_x = 0.5
      else
        t.anchor_x = 0
      end
      t.anchor_y = -(l / 2) + t.anchor_increment
    end

    if text_with_location[0] && GTK.a11y_enabled?
    text_with_location[0].a11y_id = "battle_text"
    text_with_location[0].a11y_trait = :label
    text_with_location[0].a11y_text = (text_with_location.map { |s| s.text }.join " \n ")
    end

    text_with_location
  end

  def enemy_labels
    return nil unless game.world.event.event
    return nil unless game.world.event.event.in_battle?

    if @animate_melee_enemy
      @animate_melee_enemy_x ||= enemy_label_x
      @animate_melee_enemy_direction ||= -1
      if @animate_melee_enemy_direction == 1
        diff = (@animate_melee_enemy_x - enemy_label_x).abs
      else
        diff = (@animate_melee_enemy_x - player_label_x).abs
      end

      @animate_melee_enemy_x += diff * 0.25 * @animate_melee_enemy_direction
      @animate_melee_enemy_x = @animate_melee_enemy_x.to_i

      if diff < 40 && @animate_melee_enemy_direction == 1
        @animate_melee_enemy = false
      elsif diff < 40
        @animate_melee_enemy_direction = 1
      end
    else
      @animate_melee_enemy_x = enemy_label_x
    end

    char = current_scene[:enemy]

    @battle_last_attack_result_show_ticks ||= 0

    enemy_damage = ""
    if @battle_last_attack_result
      if @battle_last_attack_result.attacked
        @battle_last_attack_result_show_ticks = 60 if @battle_last_attack_result_show_ticks <= 0
        if @battle_last_attack_result.missed
          enemy_damage = "miss"
        elsif @battle_last_attack_result.name == :bolas
          enemy_damage = "stun"
        elsif @battle_last_attack_result.name != :food && @battle_last_attack_result.name != :stealth
          enemy_damage = "`#{@battle_last_attack_result.damage.to_i.to_s}"
        end
      end
    end

    @battle_last_attack_result_show_ticks -= 1 if @battle_last_attack_result_show_ticks > 0

    if @battle_last_attack_result_show_ticks <= 0
      enemy_damage = ""
      @battle_last_attack_result = nil
    end

    if enemy_damage == "" || enemy_damage == "miss" || enemy_damage == "stun"
      enemy_damage_label = {
        x: @animate_melee_enemy_x,
        y: Grid.h - 100 - 110,
        text: enemy_damage,
        font: "fonts/sprite.ttf",
        anchor_x: 0.5, anchor_y: 0.5,
        size_px: 30,
        **text_color
      }
    else
      enemy_damage_label = {
        x: @animate_melee_enemy_x,
        y: Grid.h - 100 - 110,
        text: enemy_damage,
        font: "fonts/sprite.ttf",
        anchor_x: 0.5, anchor_y: 0.5,
        size_px: 30,
        r: 200, g: 0, b: 0
      }
    end

    a11y = { a11y_id: "battle_enemey",
             a11y_text: "enemy: #{current_scene[:health].to_i} of #{current_scene[:max_health].to_i} hp",
             a11y_trait: :label }

    enemy_text_color = if current_scene[:health] <= 0
                         { r: 200, g: 0, b: 0 }
                       elsif current_scene[:stunned]
                         { r: 128, g: 128, b: 128 }
                       else
                         text_color
                       end

    [
      enemy_damage_label,
      {
        x: @animate_melee_enemy_x,
        y: Grid.h - 100 - 150,
        text: enemy_char_map[char] || "B",
        anchor_x: 0.5, anchor_y: 0.5,
        font: "fonts/sprite.ttf",
        size_enum: 10,
        **a11y,
        **enemy_text_color
      },
      {
        x: @animate_melee_enemy_x,
        y: Grid.h - 100 - 185,
        text: "#{current_scene[:health].to_i} / #{current_scene[:max_health].to_i}",
        anchor_x: 0.5, anchor_y: 0.5,
        font: "fonts/sprite.ttf",
        size_enum: 5,
        **enemy_text_color
      }
    ]
  end

  def enemy_char_map
    {
      :beast => "B", :cave_lizard => "L", :lizard => "L",
      :snarling_beast => "B", :strange_bird => "B", :man_eater => "E",
      :gaunt_man => "M", :scavenger => "S", :feral_terror => "F",
      :soldier => "D", :sniper => "H", :thug => "T",
      :vigilante => "V", :frail_man => "M", :bird => "B",
      :rats => "RRR", :veteran => "V", :commando => "C",
      :squatter => "S", :youth => "Y", :man => "M",
      :wraith => "W", :squatter_and_defector => "S &",
      :chief => "C", :beastly_matriarch => "M", :defector => "&"
    }
  end

  def player_dodge_spline
    @player_dodge_spline = [
      [1.0, 0.33, 0.0, 0.0],
      [0.0, 0.0, 0.0, 0.0],
      [0.0, 0.0, 0.66, 1.0]
    ]
  end

  def player_labels
    return nil unless game.world.event.event
    return nil unless game.world.event.event.in_battle?

    @player_dodge_at ||= -1000

    if @animate_melee_player
      @animate_melee_player_x ||= player_label_x
      @animate_melee_player_direction ||= 1
      if @animate_melee_player_direction == 1
        diff = (@animate_melee_player_x - enemy_label_x).abs
      else
        diff = (@animate_melee_player_x - player_label_x).abs
      end

      @animate_melee_player_x += diff * 0.25 * @animate_melee_player_direction
      @animate_melee_player_x = @animate_melee_player_x.to_i

      if diff < 40 && @animate_melee_player_direction == -1
        @animate_melee_player = false
      elsif diff < 40
        @animate_melee_player_direction = -1
      end
    else
      @animate_melee_player_x = player_label_x
    end

    @battle_last_enemy_attack_result_show_ticks ||= 0

    player_damage = ""
    if @previous_battle_tick_result
      if @previous_battle_tick_result.attacked
        @battle_last_enemy_attack_result_show_ticks = 60 if @battle_last_enemy_attack_result_show_ticks <= 0
        if @previous_battle_tick_result.missed
          player_damage = "miss"
        else
          player_damage = "`#{@previous_battle_tick_result.damage.to_i.to_s}"
        end
      end
    end

    if @player_healed
      @battle_last_enemy_attack_result_show_ticks = 60 if @battle_last_enemy_attack_result_show_ticks <= 0
      player_damage = "^#{game.world.meat_heal}"
    end

    @battle_last_enemy_attack_result_show_ticks -= 1 if @battle_last_enemy_attack_result_show_ticks > 0

    if @battle_last_enemy_attack_result_show_ticks <= 0
      player_damage = ""
      @previous_battle_tick_result = nil
      @player_healed = nil
      game.last_battle_tick_result = nil
    end

    if player_damage == "" || player_damage == "miss"
      player_damage_label = {
        x: @animate_melee_player_x,
        y: Grid.h - 100 - 110,
        text: player_damage,
        size_px: 30,
        font: "fonts/sprite.ttf",
        anchor_x: 0.5, anchor_y: 0.5,
      }
    elsif player_damage == "^#{game.world.meat_heal}"
      player_damage_label = {
        x: @animate_melee_player_x,
        y: Grid.h - 100 - 110,
        text: player_damage,
        size_px: 30,
        font: "fonts/sprite.ttf",
        anchor_x: 0.5, anchor_y: 0.5,
        r: 0, g: 128, b: 0
      }
    else
      player_damage_label = {
        x: @animate_melee_player_x,
        y: Grid.h - 100 - 110,
        text: player_damage,
        size_px: 30,
        font: "fonts/sprite.ttf",
        anchor_x: 0.5, anchor_y: 0.5,
        r: 200, g: 0, b: 0
      }
    end

    a11y = { a11y_id: "battle_player", a11y_text: "#{game.world.current_hp.to_i} of #{game.world.max_hp.to_i} hp", a11y_trait: :label }

    player_text_color = if game.world.dead?
                          { r: 200, g: 0, b: 0 }
                        elsif game.world.event.event.dodging?
                          { **text_color,
                            a: 255 * Easing.ease_spline(@player_dodge_at, Kernel.tick_count, 60, player_dodge_spline) }
                        else
                          text_color
                        end

    [
      player_damage_label,
      {
        x: @animate_melee_player_x,
        y: Grid.h - 100 - 150,
        text: "@",
        size_enum: 10,
        font: "fonts/sprite.ttf",
        anchor_x: 0.5, anchor_y: 0.5,
        **a11y,
        **player_text_color
      },
      {
        x: @animate_melee_player_x,
        y: Grid.h - 100 - 185,
        text: "#{game.world.current_hp.to_i} / #{game.world.max_hp.to_i}",
        anchor_x: 0.5, anchor_y: 0.5,
        size_enum: 5,
        font: "fonts/sprite.ttf",
        **player_text_color
      }
    ]
  end

  def player_label_x
    if Grid.orientation == :landscape
      640 - 240
    else
      380 - 240
    end
  end

  def enemy_label_x
    player_label_x + 440
  end

  def battle_title
    return nil unless game.world.event.event
    return nil unless game.world.event.event.in_battle?

    r = if Grid.orientation == :landscape
          Layout.rect(row: 0, col: [6, 17], h: 1)
        else
          Layout.rect(row: 0, col: [0, 11], h: 1)
        end

    r.center
     .merge(text: game.world.event.event.title,
            anchor_x: 0.5,
            anchor_y: 0.5,
            font: "fonts/text.ttf",
            size_px: 30,
            a11y_id: "battle_title",
            a11y_text: game.world.event.event.title,
            a11y_trait: :label)
  end

  def act_with id, weapon
    return if game.world.dead?
    return true if game.world.event.event.battle_complete?
    @start_battle_debounce = 0
    action_type = @actions_hash[id]

    if !action_type.can_use?
      perc = (@actions_hash[id].percentage * 100).to_i
      a11y_notify :battle_result, "#{weapon} at #{perc} percent. not ready."
      return
    end

    @battle_last_attack_result = game.world.event.event.act(action_type)
    @battle_last_attack_result_show_ticks = 60

    if @battle_last_attack_result.name == :food
      @player_healed = true
      play_attack_sound id
    elsif @battle_last_attack_result.missed
      a11y_notify :battle_result, "missed."
      queue_sound 'sounds/missed.ogg'

      if action_type.melee?
        animate_melee_player!
      else
        animate_bullet_player!
      end
    else
      if @battle_last_attack_result.name == :bolas
        a11y_notify :battle_result, "enemy stunned."
      elsif @battle_last_attack_result.name == :stealth
        a11y_notify :battle_result, "dodging attackes."
      else
        a11y_notify :attack_result, "hit for #{@battle_last_attack_result.damage}, enemy hp: #{current_scene[:health].to_i} of #{current_scene[:max_health].to_i}"
      end

      play_attack_sound id

      if id == :stealth
        @player_dodge_at = Kernel.tick_count
      elsif action_type.melee?
        animate_melee_player!
      else
        animate_bullet_player!
      end
    end
  end

  def play_attack_sound id
    return if game.world.dead?
    @attack_sounds = {
      food: 'sounds/food.ogg',
      fists: 'sounds/fists.ogg',
      knife: 'sounds/knife.ogg',
      iron_sword: 'sounds/swing.ogg',
      steel_sword: 'sounds/slash.ogg',
      katana: 'sounds/draw.ogg',
      rifle: 'sounds/shoot.ogg',
      laser: 'sounds/blast.ogg',
      bolas: 'sounds/bolas.ogg',
      grenade: 'sounds/lob.ogg',
      murasame: 'sounds/draw.ogg',
      masamune: 'sounds/draw.ogg',
      stealth: 'sounds/missed.ogg'
    }

    if !@attack_sounds[id]
      puts "warning. attack sound with id: #{id} not found"
    end

    queue_sound @attack_sounds[id] if @attack_sounds[id]
  end

  def loot_button_selected
    state.next_scene = :mini_world_loot_scene
  end

  def debug_weapons
    $game.world.current_hp = 9999
    $game.world.stores.clear
    $game.world.stores[:food] = 10
    $game.world.stores[:grenade] = 5
    $game.world.stores[:stealth] = 1
    $game.world.stores[:bolas] = 5
    $game.world.stores[:bullets] = 100
    $game.world.stores[:battery] = 100
    $game.world.stores[:torch] = 5
    $game.world.weapon_stores.clear
    $game.world.weapon_stores << Knife.new
    $game.world.weapon_stores << IronSword.new
    $game.world.weapon_stores << SteelSword.new
    $game.world.weapon_stores << Katana.new
    $game.world.weapon_stores << Rifle.new
    $game.world.weapon_stores << Laser.new
    $game.world.weapon_stores << Masamune.new
    $game.world.weapon_stores << Murasame.new
    $scene.will_appear
    current_scene[:health] = 100
  end
end
