# Copyright 2014 Scratchwork Development LLC. All rights reserved.

class MiniWorldEventScene < Scene
  def id
    :mini_world_event_scene
  end

  def mini_world
    @game.world.event
  end

  def current_scene
    return { text: [] } if !mini_world.event
    mini_world.event.scenes[mini_world.event.current_scene]
  end

  def tick
    calc
    render
    calc_buttons
  end

  def will_appear
    if mini_world.event.is_a? ObservatoryEvent
      queue_music "sounds/observatory.ogg", true
    else
      # puts mini_world.event.class
    end

    args.outputs.a11y[:notify_will_appear] = {
      a11y_trait: :notification,
      a11y_notification_target: "layout_title",
      a11y_notification_debounce: 60
    }

    @is_leave_ship_event = @game.world.event.is_a? LeaveShipEvent
  end

  def calc
  end

  def option_0_button_selected
    select_option 0
  end

  def option_1_button_selected
    select_option 1
  end

  def option_2_button_selected
    select_option 2
  end

  def option_3_button_selected
    select_option 3
  end

  def option_4_button_selected
    select_option 4
  end

  def button_ids
    [
      :option_0_button,
      :option_1_button,
      :option_2_button,
      :option_3_button,
      :option_4_button
    ]
  end

  def select_option index
    if just_loot?
      state.next_scene = :mini_world_loot_scene
    else
      args.outputs.a11y[:select_option] = {
        a11y_trait: :notification,
        a11y_notification_target: "layout_title",
        a11y_notification_debounce: 60
      }
      key = current_scene[:options].keys[index]
      result = mini_world.event.change_scene key
      if result == false
        queue_notification :no_supplies, "don't have the supplies"
        return
      end

      if !@game.world.event
        state.next_scene = :world_scene
      elsif !mini_world.event
        state.next_scene = :mini_world_scene
      elsif mini_world.event && mini_world.event.in_battle?
        state.next_scene = :mini_world_battle_scene
      elsif mini_world.event && current_scene && current_scene[:cutscene]
        state.next_scene = :cutscene_scene
      elsif mini_world.event && mini_world.event.current_scene == :end
        mini_world.event = nil
        state.next_scene = :mini_world_scene
      end
    end
  end

  def render
    outputs.primitives << event_title
    outputs.primitives << event_text
    render_buttons
    outputs.primitives << selection_prefab
  end

  def current_scene_text
    text = current_scene[:text]
    if text.is_a? Array
      text = text.flatten
    end
    text || []
  end

  def event_text
    return nil unless @game.world.event
    return nil unless mini_world.event

    text_with_location = []
    event_label_rect = if Grid.orientation == :landscape
                         Layout.rect(row: 1, col: [6, 17], h: 3)
                       else
                         Layout.rect(row: 2, col: [0, 11], h: 3)
                       end

    size_px = if Grid.orientation == :landscape
                22
              else
                30
              end

    split_length = if Grid.orientation == :landscape
                     50
                   else
                     38
                   end

    starting_x = event_label_rect.x
    starting_y = event_label_rect.y

    anchor_increment = 0
    current_scene_text.each_with_index do |t, j|
      long_lines_split(t, split_length).each_with_index do |line, i|
        next if line.strip.length == 0
        anchor_increment += 1.2
        text_with_location << { x: event_label_rect.x + 4,
                                y: event_label_rect.center.y,
                                font: "fonts/text.ttf",
                                size_px: size_px,
                                anchor_increment: anchor_increment,
                                text: line.strip }
      end
      anchor_increment += 1
    end

    l = text_with_location.length
    text_with_location.each_with_index do |t, i|
      if l <= 2
        t.x = Grid.w / 2
        t.anchor_x = 0.5
      else
        t.anchor_x = 0
      end

      t.anchor_y = -(l / 2) + t.anchor_increment - 0.5
    end

    if text_with_location[0] && GTK.a11y_enabled?
    text_with_location[0].a11y_id = "world_event_text"
    text_with_location[0].a11y_trait = :label
    text_with_location[0].a11y_text = (text_with_location.map { |s| s.text }.join " \n ")
    text_with_location[0].a11y_text += "\n#{current_scene[:options].keys.length} options"
    end

    text_with_location
  end

  def event_title
    return nil unless @game.world.event
    return nil unless mini_world.event
    r = if Grid.orientation == :landscape
          Layout.rect(row: 0, col: [0, 23], h: 1)
        else
          Layout.rect(row: 0, col: [0, 11], h: 1)
        end

    r.center
     .merge(text: mini_world.event.title,
            anchor_x: 0.5,
            anchor_y: 0.5,
            font: "fonts/text.ttf",
            size_px: 30,
            a11y_id: "layout_title",
            a11y_text: mini_world.event.title,
            a11y_trait: :label)
  end

  def in_battle?
    mini_world.event && mini_world.event.in_battle?
  end

  def just_loot?
    !in_battle? && current_scene[:loot] && current_scene[:options].length == 1
  end

  def option_button_text index
    key = current_scene[:options].keys[index]
    cost = " (#{mini_world.event.cost(key)})"
    cost = "" if cost == " ()"
    "#{key.to_s.gsub("_", " ")}#{cost.to_s.gsub("_", " ")}"
  end

  def option_button index, row:, col:, h:;
    return nil if changing_scene?
    return nil unless mini_world.event
    return nil unless current_scene[:options]
    return nil unless current_scene[:options].keys[index]

    r = Layout.rect(row: row + index * h, col: col, h: h, w: 12)

    if just_loot?
      r.merge(text: "loot",
              a11y_id: "world_event_option_#{index}_button",
              a11y_trait: :button,
              a11y_text: "loot")
    else
      r.merge(text: option_button_text(index),
              a11y_id: "world_event_option_#{index}_button",
              a11y_trait: :button,
              a11y_text: option_button_text(index))
    end
  end

  def option_button_start_row
    if Grid.orientation == :landscape
      5
    else
      11
    end
  end

  def option_button_col
    if Grid.orientation == :landscape
      6
    else
      0
    end
  end

  def option_button_h
    if Grid.orientation == :landscape
      1.5
    else
      2
    end
  end

  def option_0_button
    option_button 0,
                  row: option_button_start_row,
                  col: option_button_col,
                  h:   option_button_h
  end

  def option_1_button
    option_button 1,
                  row: option_button_start_row,
                  col: option_button_col,
                  h:   option_button_h
  end

  def option_2_button
    option_button 2,
                  row: option_button_start_row,
                  col: option_button_col,
                  h:   option_button_h
  end

  def option_3_button
    option_button 3,
                  row: option_button_start_row,
                  col: option_button_col,
                  h:   option_button_h
  end

  def option_4_button
    option_button 4,
                  row: option_button_start_row,
                  col: option_button_col,
                  h:   option_button_h
  end
end
