# Copyright 2014 Scratchwork Development LLC. All rights reserved.

class MiniWorldScene < Scene
  attr :mini_world

  def id
    :mini_world_scene
  end

  def will_appear
    outputs[:ascii_map].background_color = ascii_map_background_color
    outputs[:ascii_map].h = 1280
    outputs[:ascii_map].w = 1280
    outputs[:ascii_map].transient!
    @end_game_ticks = nil
    @world_a11y_describe_entries = nil
    @mini_world = @game.world.event
    $mini_world = @mini_world
    if @mini_world.ticks == 141
      @mini_world.ticks += 1
    end
    update_label_cache

    if @mini_world.end_game_sequence
      stop_music
    else
      queue_music "sounds/hubble.ogg", true
    end

    @game.save
  end

  def tick
    outputs[:ascii_map].background_color = ascii_map_background_color
    outputs[:ascii_map].h = 1280
    outputs[:ascii_map].w = 1280
    outputs[:ascii_map].transient!
    render
    calc
  end

  def calc
    @move_debounce ||= 0
    @move_debounce -= 1
    @move_debounce = 0 if @move_debounce < 0
    calc_buttons
    calc_keyboard_inputs
    calc_controller_inputs
    calc_end_game_notifications
    calc_end_game
  end

  def show_navigation_buttons?
    return false if @mini_world.end_game_sequence_complete?
    return true if GTK.a11y_enabled?
    return true if GTK.platform?(:mobile)
    return true if inputs.last_active == :mouse
    return false
  end

  def calc_keyboard_inputs
    return if inputs.last_active != :keyboard
    return if (@end_game_ticks || 0) > 0

    if inputs.keyboard.key_down.enter || inputs.keyboard.key_down.space
      if prefab.notifications.length > 0
        prefab.dismiss_all_notifications
      else
        state.next_scene = :status_scene
      end
    end

    if inputs.keyboard.key_down.left
      move_left_button_selected
    elsif inputs.keyboard.key_down.right
      move_right_button_selected
    elsif inputs.keyboard.key_down.up
      move_up_button_selected
    elsif inputs.keyboard.key_down.down
      move_down_button_selected
    end

    if inputs.keyboard.key_down.escape
      settings_button_selected
    end
  end

  def calc_controller_inputs
    return if (@end_game_ticks || 0) > 0
    return if inputs.last_active != :controller
    if inputs.controller_one.key_down.left
      move_left_button_selected
    elsif inputs.controller_one.key_down.right
      move_right_button_selected
    elsif inputs.controller_one.key_down.up
      move_up_button_selected
    elsif inputs.controller_one.key_down.down
      move_down_button_selected
    end

    if inputs.controller_one.key_down.b || inputs.controller_one.key_down.a
      if prefab.notifications.length > 0
        prefab.dismiss_all_notifications
      else
        status_button_selected
      end
    end

    if inputs.controller_one.key_down.start
      settings_button_selected
    end
  end

  def settings_button_selected
    state.next_scene = :settings_scene
  end

  def settings_button
    return nil if !show_navigation_buttons?
    if Grid.orientation == :landscape
      bt_rect = Layout.rect(row: 8, col: 18, h: 2, w: 2)
      bt_rect.merge(text: "settings",
                    size_px: 20,
                    a11y_id: "room_settings_button",
                    a11y_text: "settings",
                    a11y_trait: :button)
    else
      if GTK.a11y_enabled?
        Layout.rect(row: 23, col: 8.75, w: 3.75, h: 1)
              .merge(text: "settings",
                     a11y_id: "room_settings_button",
                     a11y_text: "settings",
                     a11y_trait: :button)
      else
        bt_rect = Layout.rect(row: 23, col: 0, h: 1, w: 4)
        bt_rect.merge(text: "settings",
                      a11y_id: "room_settings_button",
                      a11y_text: "settings",
                      a11y_trait: :button)
      end
    end
  end

  def calc_end_game
    return if !@mini_world.end_game_sequence_complete?
    @end_game_ticks ||= 0
    @end_game_ticks += 1

    if @end_game_ticks == 40.seconds
      outputs.a11y[:end_game_notify] = {
        a11y_id: "end_game_notify",
        a11y_trait: :notification,
        a11y_notification_target: :end_game_title,
        a11y_notification_debounce: 60
      }
      food_refused = (@game.stores[:compass] || 0) == 0
      if food_refused
        @game.world.praying_unlocked = true
        @game.world.cleared_game_once = true
        @game.world.cleared_hard_mode = true
        @game.world.cause_death :complete_hard #yes, it's a hack, don't screw with this, you'll cause memory leaks
        @game.save
      else
        @game.world.cleared_game_once = true
        @game.world.cause_death :complete #yes, it's a hack, don't screw with this, you'll cause memory leaks
        @game.save
      end
    elsif @end_game_ticks == 32.seconds
      if !@prefab.notifications[:end_game]
        queue_notification :end_game, "awake. head throbbing. vision blurry."
      end
    elsif @end_game_ticks == 25.seconds
      if !@prefab.notifications[:end_game]
        @mini_world_landmark_labels_cache ||= []
        @mini_world_landmark_labels_cache.clear
        @mini_world_aesthetic_labels_cache ||= []
        @mini_world_aesthetic_labels_cache.clear
        queue_notification :end_game, "the barrier shatters.", text_color: { r: 255, g: 0, b: 0 }
      end
    elsif @end_game_ticks == 18.seconds
      if !@prefab.notifications[:end_game]
        queue_notification :end_game, "roaring fires approach the room from every direction."
      end
    elsif @end_game_ticks == 10.seconds
      if !@prefab.notifications[:end_game]
        queue_notification :end_game, "they smirk, celebrate victory, as they are engulfed in flames."
      end
    elsif @end_game_ticks == 2.seconds
      if !@prefab.notifications[:end_game]
        queue_notification :end_game, "defectors surround the barrier as the detonation countdown hits zero."
      end
    end
  end

  def calc_end_game_notifications
    if @mini_world.end_game_message != @last_end_game_message
      queue_notification :end_game, @mini_world.end_game_message
      @last_end_game_message = @mini_world.end_game_message
      @move_debounce = 60
    end
  end

  def ascii_map_background_color
    [0, 0, 0, 0]
  end

  def end_game_stats_label
    return if (@end_game_ticks || 0) < 40.seconds
    m = @game.world.current_multipliers || @game.world.default_multipliers

    god_mode_string = if(m != @game.world.default_multipliers)
                        "D: #{m[:durability]}, P: #{m[:poison]}, H: #{m[:hunger]}, T: #{m[:thirst]}, V: #{m[:vision]}, E: #{m[:enemy_damage]}"
                      else
                        ""
                      end

    label_rect = Layout.rect(row: 0, col: [0, Layout.col_count - 1], h: 1)

    food_refused_string = "no"
    food_refused_string = "yes!" if (@game.stores[:compass] || 0) == 0

    defeated_final_boss = @game.world.defeated_final_boss
    defeated_final_boss_string = "no"
    defeated_final_boss_string = "yes!" if defeated_final_boss

    ritual = @game.world.ritual_performed?
    ritual_string = "no"
    ritual_string = "yes!" if ritual

    [
      label_rect.center.merge(text: "the ensign",
                              anchor_x: 0.5,
                              anchor_y: 0.5,
                              size_px: 30,
                              a11y_id: "end_game_title",
                              a11y_text: "the ensign",
                              a11y_trait: :label,
                              r: 255, g: 255, b: 255,
                              font: "fonts/text.ttf"),
      label_rect.center.merge(text: "world explored: #{@game.world.percentage_uncovered_this_time}%",
                              a11y_id: "end_game_world_explored",
                              a11y_text: "world explored: #{@game.world.percentage_uncovered_this_time}%",
                              a11y_trait: :label,
                              anchor_x: 0.5,
                              anchor_y: 2.5,
                              size_px: 30,
                              r: 255, g: 255, b: 255,
                              font: "fonts/text.ttf"),
      label_rect.center.merge(text: "areas cleared: #{@game.world.percentage_cleared_this_time}%",
                              a11y_id: "end_game_areas_cleared",
                              a11y_text: "areas cleared: #{@game.world.percentage_cleared_this_time}%",
                              a11y_trait: :label,
                              anchor_x: 0.5,
                              anchor_y: 3.5,
                              size_px: 30,
                              r: 255, g: 255, b: 255,
                              font: "fonts/text.ttf"),
      label_rect.center.merge(text: "time: #{@game.tick_count} minutes",
                              a11y_id: "end_game_time",
                              a11y_text: "time: #{@game.tick_count} minutes",
                              a11y_trait: :label,
                              anchor_x: 0.5,
                              anchor_y: 4.5,
                              size_px: 30,
                              r: 255, g: 255, b: 255,
                              font: "fonts/text.ttf"),
      label_rect.center.merge(text: "total played: #{@game.total_tick_count} minutes",
                              a11y_id: "end_game_total_played",
                              a11y_text: "total played: #{@game.total_tick_count} minutes",
                              a11y_trait: :label,
                              anchor_x: 0.5,
                              anchor_y: 5.5,
                              size_px: 30,
                              r: 255, g: 255, b: 255,
                              font: "fonts/text.ttf"),
      label_rect.center.merge(text: "food refused: #{food_refused_string}",
                              a11y_id: "end_game_food_refused",
                              a11y_text: "food refused: #{food_refused_string}",
                              a11y_trait: :label,
                              anchor_x: 0.5,
                              anchor_y: 7.5,
                              size_px: 30,
                              r: 255, g: 255, b: 255,
                              font: "fonts/text.ttf"),
      label_rect.center.merge(text: "admiral's ritual: #{ritual_string}",
                              a11y_id: "end_game_admirals_ritual",
                              a11y_text: "admiral's ritual: #{ritual_string}",
                              a11y_trait: :label,
                              anchor_x: 0.5,
                              anchor_y: 8.5,
                              size_px: 30,
                              r: 255, g: 255, b: 255,
                              font: "fonts/text.ttf"),
      label_rect.center.merge(text: "accomplished the impossible: #{defeated_final_boss_string}",
                              a11y_id: "end_game_accomplished_the_impossible",
                              a11y_text: "accomplished the impossible: #{defeated_final_boss_string}",
                              a11y_trait: :label,
                              anchor_x: 0.5,
                              anchor_y: 9.5,
                              size_px: 30,
                              r: 255, g: 255, b: 255,
                              font: "fonts/text.ttf"),
      label_rect.center.merge(text: "#{god_mode_string}",
                              anchor_x: 0.5,
                              anchor_y: 11.5,
                              size_px: 30,
                              r: 255, g: 255, b: 255,
                              font: "fonts/text.ttf")
    ]
  end

  def render
    if (@end_game_ticks || 0) > 0
      outputs.primitives << { x: Grid.allscreen_x,
                              y: Grid.allscreen_y,
                              w: Grid.allscreen_w,
                              h: Grid.allscreen_h,
                              path: :solid,
                              r: 0,
                              g: 0,
                              b: 0,
                              a: 255 }
    end

    render_navigation_buttons
    render_ascii_map
    outputs.primitives << world_water_label
    outputs.primitives << world_food_label
    outputs.primitives << world_health_label
    outputs.primitives << end_game_stats_label
  end

  def render_ascii_map
    return if (@end_game_ticks || 0) >= 25.seconds

    @string_size ||= GTK.calcstringbox(@mini_world_landmark_labels_cache.first.text,
                                     size_px: mini_world_label_size_px,
                                     font: "fonts/sprite.ttf")

    string_w, string_h = @string_size
    outputs[:ascii_map].background_color = ascii_map_background_color
    outputs[:ascii_map].h = 1280
    outputs[:ascii_map].w = 1280
    outputs[:ascii_map].transient!
    outputs[:ascii_map].primitives << mini_world_landmark_labels
    outputs[:ascii_map].primitives << mini_world_aesthetic_labels

    screen_w = Grid.w
    screen_h = Grid.h
    map_w = 1280
    map_h = 1280
    x_offset = if Grid.landscape?
                 if $args.inputs.last_active != :mouse && !GTK.a11y_enabled?
                   (screen_w - map_w + string_w / 2.25) / 2
                 else
                   0
                 end
               else
                 (screen_w - map_w + string_w / 2.25) / 2
               end
    y_offset = if Grid.landscape?
                 -510
               else
                 -200
               end

    w = if Grid.landscape?
          1280
        else
          1280
        end

    h = if Grid.landscape?
          1280
        else
          1280
        end

    outputs.primitives << { x: x_offset,
                            y: y_offset,
                            w: w,
                            h: h,
                            path: :ascii_map }

    # outputs.primitives << world_border

    render_a11y_ascii_map
  end

  def world_a11y_describe_world_entries invalidate_cache: false
    if invalidate_cache
      @world_a11y_describe_entries = nil
    end

    if @world_a11y_describe_entries
      return @world_a11y_describe_entries
    end

    @world_a11y_describe_hp = [
      "hp: #{game.world.current_hp} of #{game.world.max_hp.to_i}"
    ]

    @world_a11y_describe_points_of_interest = []
    @world_a11y_describe_points_of_interest << @mini_world.describe_defectors
    @mini_world.describe_landmarks.each do |entry|
      @world_a11y_describe_points_of_interest << entry
    end

    @world_a11y_describe_walls = []
    @mini_world.describe_terrain.each do |entry|
      @world_a11y_describe_walls << entry
    end

    @world_a11y_describe_entries = {
      hp: @world_a11y_describe_hp,
      points_of_interest: @world_a11y_describe_points_of_interest,
      walls: @world_a11y_describe_walls
    }

    @world_a11y_describe_entries
  end


  def will_appear_a11y_target
    :describe_hp
  end

  def render_a11y_ascii_map
    return if !GTK.a11y_enabled?
    return if !show_navigation_buttons?
    # $args.outputs.debug.watch "player location: #{@mini_world.x}, #{@mini_world.y}"
    # @mini_world.landmarks.each do |landmark|
    #   loc, char = landmark
    #   $args.outputs.debug.watch "landmark #{char}: #{loc}"
    #   $args.outputs.debug.watch "description: #{@mini_world.describe_landmark(loc)}"
    # end
    entries = world_a11y_describe_world_entries

    outputs.a11y[:describe_hp] = {
      x: Grid.w / 2,
      y: Grid.h / 2,
      anchor_x: 0.5,
      anchor_y: 1.0,
      a11y_id: "world_a11y_describe_hp",
      a11y_text: "#{entries[:hp].join(",\n")}, \n swipe left for points of interest, swipe right for navigation controls",
      a11y_trait: :label
    }

    outputs.a11y[:describe_points_of_interest] = {
      x: Grid.w / 2,
      y: Grid.h / 2,
      anchor_x: 0.5,
      anchor_y: 0.0,
      a11y_id: "world_a11y_describe_points_of_interest",
      a11y_text: "points of interest, #{entries[:points_of_interest].join(",\n")}, \n swipe right for hp, swipe left for walls",
      a11y_trait: :label
    }

    outputs.a11y[:describe_walls] = {
      x: Grid.w / 2,
      y: Grid.h / 2,
      anchor_x: 0.5,
      anchor_y: -1.0,
      a11y_id: "world_a11y_describe_walls",
      a11y_text: "#{entries[:walls].join(",\n")}, \n swipe left for points of interest",
      a11y_trait: :label
    }
  end

  def world_border
    world_border_rect.merge(primitive_marker: :border)
  end

  def world_border_rect
    if Grid.orientation == :landscape
      Layout.rect(row: 0, col: 0, w: 24, h: 12, include_row_gutter: true, include_col_gutter: true)
    else
      Layout.rect(row: 1, col: 0, w: 12, h: 17, include_row_gutter: true, include_col_gutter: true)
    end
  end

  def world_water_label
    if Grid.orientation == :landscape
      label_rect = Layout.rect(row: 11, col: [0, 1], h: 0)
      { x: label_rect.center.x,
        y: label_rect.center.y,
        font: "fonts/text.ttf",
        anchor_y: 0,
        anchor_x: 0.0,
        size_px: 26,
        text: "water: #{game.world.water.round}" }
    else
      label_rect = Layout.rect(row: 18, col: [0, 5], h: 1)
      { x: label_rect.x,
        y: label_rect.center.y,
        font: "fonts/text.ttf",
        anchor_y: 0.5,
        anchor_x: 0,
        size_px: 30,
        text: "water: #{game.world.water.round}" }
    end
  end

  def world_food_label
    if Grid.orientation == :landscape
      label_rect = Layout.rect(row: 11, col: [0, 1], h: 0)
      { x: label_rect.center.x,
        y: label_rect.center.y,
        font: "fonts/text.ttf",
        anchor_x: 0.0,
        anchor_y: 1.0,
        size_px: 26,
        text: "food: #{game.world.stores[:food]}" }
    else
      label_rect = Layout.rect(row: 18, col: [0, 5], h: 1)
      { x: label_rect.x,
        y: label_rect.center.y,
        font: "fonts/text.ttf",
        anchor_x: 0,
        anchor_y: 1.5,
        size_px: 30,
        text: "food: #{game.world.stores[:food]}" }
    end
  end

  def world_health_label
    if Grid.orientation == :landscape
      label_rect = Layout.rect(row: 11, col: [0, 1], h: 0)
      { x: label_rect.center.x,
        y: label_rect.center.y,
        font: "fonts/text.ttf",
        anchor_x: 0.0,
        anchor_y: 2.0,
        size_px: 26,
        text: "hp: #{game.world.current_hp}/#{game.world.max_hp.to_i}" }
    else
      label_rect = Layout.rect(row: 18, col: [0, 5], h: 1)
      { x: label_rect.x,
        y: label_rect.center.y,
        font: "fonts/text.ttf",
        anchor_y: 2.5,
        anchor_x: 0,
        size_px: 30,
        text: "hp: #{game.world.current_hp}/#{game.world.max_hp.to_i}" }
    end
  end

  def update_label_cache
    @mini_world_landmark_labels_cache ||= []
    @mini_world_landmark_labels_cache.clear
    @mini_world_aesthetic_labels_cache ||= []
    @mini_world_aesthetic_labels_cache.clear
    update_labels
    @world_a11y_describe_entries = nil
  end

  def mini_world_line_height
    25.2
  end

  def mini_world_landmark_labels
    @mini_world_landmark_labels_cache
  end

  def mini_world_aesthetic_labels
    @mini_world_aesthetic_labels_cache
  end

  def render_navigation_buttons
    if (@end_game_ticks || 0) <= 25.seconds
      outputs.primitives << prefab.button(args, move_up_button)
      outputs.primitives << prefab.button(args, move_down_button)
      outputs.primitives << prefab.button(args, move_left_button)
      outputs.primitives << prefab.button(args, move_right_button)
      outputs.primitives << prefab.button(args, status_button)
      outputs.primitives << prefab.button(args, settings_button)
    end
    outputs.primitives << prefab.button(args, end_game_button)
  end

  def status_button_selected
    state.next_scene = :status_scene
  end

  def move_up_button
    return nil if !show_navigation_buttons?
    if Grid.landscape?
      bt_row = 8
      bt_col = 20

      Layout.rect(row: bt_row, col: bt_col, w: 2, h: 2)
            .merge(text: "↑",
                   a11y_id: "world_move_up_button",
                   a11y_text: "move north, swipe left for world information, swipe right for additional navigation controls.",
                   a11y_trait: :button)
    else
      bt_row = 7
      bt_col = 19.5
      if GTK.a11y_enabled?
        bt_row += 11
      else
        bt_row += 12
      end
      bt_col -= 12

      Layout.rect(row: bt_row, col: bt_col, w: 2.5, h: 2.5)
            .merge(text: "↑",
                   a11y_id: "world_move_up_button",
                   a11y_text: "move north, swipe left for world information, swipe right for additional navigation controls.",
                   a11y_trait: :button)
    end
  end

  def move_down_button
    return nil if !show_navigation_buttons?
    if Grid.landscape?
      bt_row = 10
      bt_col = 20

      Layout.rect(row: bt_row, col: bt_col, w: 2, h: 2)
            .merge(text: "↓",
                   a11y_id: "world_move_down_button",
                   a11y_text: "move south",
                   a11y_trait: :button)
    else
      bt_row = 9.5
      bt_col = 19.5
      if GTK.a11y_enabled?
        bt_row += 11
      else
        bt_row += 12
      end
      bt_col -= 12

      Layout.rect(row: bt_row, col: bt_col, w: 2.5, h: 2.5)
            .merge(text: "↓",
                   a11y_id: "world_move_down_button",
                   a11y_text: "move south",
                   a11y_trait: :button)
    end
  end

  def move_right_button
    return nil if !show_navigation_buttons?
    if Grid.landscape?
      bt_row = 10
      bt_col = 22

      Layout.rect(row: bt_row, col: bt_col, w: 2, h: 2)
            .merge(text: "→",
                   a11y_id: "world_move_right_button",
                   a11y_text: "move east",
                   a11y_trait: :button)
    else
      bt_row = 9.5
      bt_col = 22
      if GTK.a11y_enabled?
        bt_row += 11
      else
        bt_row += 12
      end
      bt_col -= 12

      Layout.rect(row: bt_row, col: bt_col, w: 2.5, h: 2.5)
            .merge(text: "→",
                   a11y_id: "world_move_right_button",
                   a11y_text: "move east",
                   a11y_trait: :button)
    end
  end

  def move_left_button
    return nil if !show_navigation_buttons?
    if Grid.landscape?
      bt_row = 10
      bt_col = 18

      Layout.rect(row: bt_row, col: bt_col, w: 2, h: 2)
            .merge(text: "←",
                   a11y_id: "world_move_left_button",
                   a11y_text: "move west",
                   a11y_trait: :button)
    else
      bt_row = 9.5
      bt_col = 17
      if GTK.a11y_enabled?
        bt_row += 11
      else
        bt_row += 12
      end
      bt_col -= 12

      Layout.rect(row: bt_row, col: bt_col, w: 2.5, h: 2.5)
            .merge(text: "←",
                   a11y_id: "world_move_left_button",
                   a11y_text: "move west",
                   a11y_trait: :button)
    end
  end

  def button_ids
    [
      :move_up_button,
      :move_down_button,
      :move_left_button,
      :move_right_button,
      :end_game_button,
      :status_button,
      :settings_button
    ]
  end

  def end_game_button_selected
    state.next_scene = :game_over_scene
  end

  def status_button
    return nil if game.world.is_dream_sequence
    return nil if !show_navigation_buttons?
    if Grid.landscape?
      bt_row = 8
      bt_col = 22
      Layout.rect(row: bt_row, col: bt_col, w: 2, h: 2)
            .merge(text: "items",
                   size_px: 20,
                   a11y_id: "items_button",
                   a11y_text: "items",
                   a11y_trait: :button)
    else
      if GTK.a11y_enabled?
        bt_row = 23
        bt_col = 5

        Layout.rect(row: bt_row, col: bt_col, w: 3.75, h: 1)
              .merge(text: "items",
                     a11y_id: "world_items_button",
                     a11y_text: "items",
                     a11y_trait: :button)
      else
        bt_row = 7
        bt_col = 22
        bt_row += 12
        bt_col -= 12

        Layout.rect(row: bt_row, col: bt_col, w: 2.5, h: 2.5)
              .merge(text: "items",
                     a11y_id: "world_items_button",
                     a11y_text: "items",
                     a11y_trait: :button)
      end
    end
  end

  def end_game_button
    return if (@end_game_ticks || 0) < 40.seconds

    if Grid.landscape?
      Layout.rect(row: 10,
                  col: 6,
                  w: 12,
                  h: 2)
            .merge(text: "...",
                   r: 255,
                   g: 255,
                   b: 255,
                   a11y_id: "end_game_button",
                   a11y_text: "...",
                   a11y_trait: :button)
    else
      Layout.rect(row: 22,
                  col: 0,
                  w: 12,
                  h: 2)
            .merge(text: "...",
                   r: 255,
                   g: 255,
                   b: 255,
                   a11y_id: "end_game_button",
                   a11y_text: "...",
                   a11y_trait: :button)
    end
  end

  def determine_event
    if @mini_world.has_event?
      state.next_scene = :mini_world_event_scene
    elsif @mini_world.end_game_sequence
      @mini_world.game_tick
    end
  end

  def update_labels
    current_map = @mini_world

    if Grid.landscape?
      start_y = 0
    else
      start_y = (@mini_world.y - (max_visible_rows / 2)).to_i

      if start_y < 0
        start_y = 0
      elsif start_y + max_visible_rows > current_map.height
        start_y = current_map.height - max_visible_rows
      end
    end

    if Grid.landscape?
      start_x = 0
    else
      start_x = (@mini_world.x - (max_visible_columns / 2)).to_i
      if start_x < 0
        start_x = 0
      elsif start_x + (max_visible_columns) > current_map.width
        start_x = current_map.width - max_visible_columns
      end
    end

    (max_visible_rows + 1).times do |row|
      landmark_str = ""
      aesthetic_str = ""
      current_y = start_y + row
      max_visible_columns.times do |x|
        current_x = start_x + x
        enemy_at_location = current_map.enemies.find { |e| e.x == current_x && e.y == current_y }
        if current_map.x == current_x && current_map.y == current_y
          landmark_str.concat "@"
          aesthetic_str.concat " "
        else
          if enemy_at_location
            landmark_str.concat enemy_at_location[:char]
            aesthetic_str.concat " "
          elsif current_map.landmark_at? current_x, current_y
            char = current_map.landmarks[[current_x, current_y]]
            if char
              landmark_str.concat char
            else
              landmark_str.concat " "
            end
            aesthetic_str.concat " "
          else
            char = current_map.map[[current_x, current_y]]
            landmark_str.concat " "
            if char
              aesthetic_str.concat char
            else
              aesthetic_str.concat " "
            end
          end
        end

        @mini_world_aesthetic_labels_cache[row] =
          { x: 58,
            y: 1240 - mini_world_line_height * (row + 1) - 24,
            text: aesthetic_str,
            **aesthetic_color,
            size_px: mini_world_label_size_px,
            font: "fonts/sprite.ttf",
            anchor_x: 0 }

        @mini_world_landmark_labels_cache[row] =
          { x: 58,
            y: 1240 - mini_world_line_height * (row + 1) - 24,
            text: landmark_str,
            **landmark_color,
            size_px: mini_world_label_size_px,
            font: "fonts/sprite.ttf",
            anchor_x: 0 }
      end
    end
  end

  def aesthetic_color
    if @mini_world.end_game_sequence_complete?
      { r: 255, g: 255, b: 255 }
    else
      prefab.kindof_light_gray
    end
  end

  def landmark_color
    if @mini_world.end_game_sequence_complete?
      { r: 255, g: 255, b: 255 }
    else
      text_color
    end
  end

  def a11y_notify_cant_move_there_if_needed dx, dy
    next_x = @mini_world.x + dx
    next_y = @mini_world.y + dy
    if @mini_world.wall_at? next_x, next_y
      args.outputs.a11y[:cant_move_there] = {
        a11y_trait: :notification,
        a11y_text: "can't move there, wall or debris occupies location."
      }
    end
  end

  def move_up_button_selected
    return if @move_debounce > 0
    a11y_notify_cant_move_there_if_needed 0, -1
    @mini_world.move_up
    update_label_cache
    determine_event
  end

  def move_down_button_selected
    return if @move_debounce > 0
    a11y_notify_cant_move_there_if_needed 0, 1
    @mini_world.move_down
    update_label_cache
    determine_event
  end

  def move_left_button_selected
    return if @move_debounce > 0
    a11y_notify_cant_move_there_if_needed -1, 0
    @mini_world.move_left
    update_label_cache
    determine_event
  end

  def move_right_button_selected
    return if @move_debounce > 0
    a11y_notify_cant_move_there_if_needed 1, 0
    @mini_world.move_right
    update_label_cache
    determine_event
  end

  def mini_world_label_size_px
    20
  end

  def max_visible_columns
    @mini_world.width
  end

  def max_visible_rows
    @mini_world.height
  end

  def keyboard_instructions
    "[arrow keys/WASD]: move around. [SPACE/ENTER]: open status screen. [ESC]: settings."
  end

  def controller_instructions accept_char
    "[DIRECTIONAL]: move around. [#{accept_char}]: open status screen. [START]: settings."
  end
end
