# Copyright 2014 Scratchwork Development LLC. All rights reserved.

class RootScene < Scene
  attr :game, :loot_scene, :prefab

  def initialize
    @game = Game.new
    $game = @game
    @world_scene = WorldScene.new
    @embark_scene = EmbarkScene.new
    @world_event_scene = WorldEventScene.new
    @battle_scene = BattleScene.new
    @prefab = Prefab.new
    @game_over_scene = GameOverScene.new
    @loot_scene = LootScene.new
    @cutscene_scene = CutsceneScene.new
    @alert_scene = AlertScene.new
    @mini_world_scene = MiniWorldScene.new
    @mini_world_event_scene = MiniWorldEventScene.new
    @mini_world_loot_scene = MiniWorldLootScene.new
    @mini_world_battle_scene = MiniWorldBattleScene.new
    @status_scene = StatusScene.new
    @settings_scene = SettingsScene.new
    @god_mode_scene = GodModeScene.new
    @modal_scene = ModalScene.new
    $root_scene = self
  end

  def defaults
    state.scene ||= :embark_scene
    state.scene_at ||= 0
    state.fade_at ||= 0
  end

  def bg_prefab
    [
    { x: Grid.x,
      y: Grid.y,
      w: Grid.w,
      h: Grid.h,
      path: :solid,
      r: 245,
      g: 245,
      b: 245,
      a: 255 }
    ]
  end

  def fade_prefab
    start_time = state.fade_at
    duration = 30
    spline = [
      [  0, 0.25, 0.75, 1.0],
      [1.0, 0.75, 0.25,   0]
    ]

    if Kernel.tick_count < 15
      current_progress = 1
    else
      current_progress = Easing.ease_spline(start_time, Kernel.tick_count, duration, spline)
    end

    a = 255 * current_progress
    a = a.clamp(0, 255)
    [
      {
        x: Grid.allscreen_x,
        y: Grid.h,
        w: Grid.allscreen_w + 10,
        h: Grid.allscreen_offset_y + 10,
        path: :solid,
        r: 0,
        g: 0,
        b: 0,
        a: a
      },
      {
        x: Grid.allscreen_x,
        y: Grid.y,
        w: Grid.allscreen_w + 10,
        h: Grid.h,
        path: :solid,
        r: 0,
        g: 0,
        b: 0,
        a: a
      },
      {
        x: Grid.allscreen_x,
        y: Grid.allscreen_y - 4,
        w: Grid.allscreen_w + 10,
        h: Grid.allscreen_offset_y + 4,
        path: :solid,
        r: 0,
        g: 0,
        b: 0,
        a: a
      },
    ]
  end

  def tick
    defaults

    tick_bg_music

    if state.queued_scene
      inputs.clear
      set_selection! nil
    end

    if prefab.notifications.length > 0 && inputs.mouse.click && inputs.mouse.y > (Grid.h - prefab.notification_h)
      prefab.dismiss_current_notification
      inputs.clear
    end

    @prefab.tick_flashes
    @prefab.tick_notifications

    if Kernel.tick_count != 0 && Kernel.tick_count.zmod?(60.seconds)
      @game.one_minute_passed!
    end

    scene_before_tick = state.scene

    outputs.background_color = [245, 245, 245]
    outputs.primitives << bg_prefab

    scene_tick
    outputs.primitives << scene_current.selection_prefab
    outputs.primitives << fade_prefab
    outputs.primitives << scene_current.instructions_prefab

    if state.scene != scene_before_tick
      raise "Scene change is not allowed during tick, set next_scene instead"
    end

    if state.next_scene
      if state.queued_scene
        state.next_scene = nil
        state.next_scene_metadata = nil
      else
        state.queued_scene = state.next_scene
        state.queued_scene_at = Kernel.tick_count
        state.queued_scene_metadata = state.next_scene_metadata
        state.queued_scene_callback = state.next_scene_callback
        state.next_scene = nil
        state.fade_at = Kernel.tick_count
      end
    end

    if state.queued_scene_at && state.queued_scene_at.elapsed_time > 13 && state.queued_scene
      state.previous_scene = state.scene
      state.scene = state.queued_scene
      state.scene_at = Kernel.tick_count
      state.scene_metadata = state.queued_scene_metadata
      state.queued_scene_callback.call if state.queued_scene_callback
      state.queued_scene = nil
      state.queued_scene_at = nil
      state.queued_scene_metadata = nil
      state.queued_scene_callback = nil
    end
    @prefab.render_flash_primitives(args)
    outputs.primitives << @prefab.notification_primitives

    # labels = outputs.primitives.find_all { |l| l.text && !l.font && !l.h && !l.w }
    # if labels.length > 0
    #   puts labels
    # end

    # outputs.primitives << {
    #   x: Grid.x,
    #   y: Grid.y,
    #   w: Grid.w,
    #   h: Grid.h,
    #   primitive_marker: :border,
    #   r: 255,
    #   g: 255,
    #   b: 255
    # }

    # outputs.primitives << { x: 0, y: 0, w: Grid.w, h: Grid.h, primitive_marker: :border }

    if GTK.a11y_enabled?
      a11y_labels = Array.find_all(outputs.primitives) do |primitive|
        primitive.text && primitive.a11y_id
      end

      a11y_labels.each do |label|
        outputs.a11y[label.a11y_id] = {
          x: label.x,
          y: label.y,
          anchor_x: label.anchor_x,
          anchor_y: label.anchor_y,
          a11y_id: label.a11y_id,
          a11y_text: label.a11y_text,
          a11y_trait: :label
        }
      end

      a11y_buttons = Array.find_all(outputs.primitives) do |primitive|
        primitive.a11y_id && primitive.a11y_trait == :button
      end

      a11y_buttons.each do |button|
        args.outputs.a11y[button.a11y_id] = {
          a11y_id: button.a11y_id,
          a11y_text: button.a11y_text,
          a11y_trait: :button,
          x: button.x,
          y: button.y,
          w: button.w,
          h: button.h,
          anchor_x: button.anchor_x,
          anchor_y: button.anchor_y
        }
      end
    end

    # args.outputs.watch "#{GTK.current_framerate.to_sf}"
    render_landscape_letter_box if Grid.orientation == :landscape
  end

  def render_landscape_letter_box
    # right side
    outputs.primitives << {
      x: Grid.w,
      y: 0,
      w: Grid.allscreen_offset_x,
      h: Grid.allscreen_h,
      r: 0,
      g: 0,
      b: 0,
      a: 255,
      path: :solid
    }

    # left side
    outputs.primitives << {
      x: -Grid.allscreen_offset_x,
      y: 0,
      w: Grid.allscreen_offset_x,
      h: Grid.allscreen_h,
      r: 0,
      g: 0,
      b: 0,
      a: 255,
      path: :solid
    }

    # top side
    outputs.primitives << {
      x: -Grid.allscreen_offset_x,
      y: Grid.h,
      w: Grid.allscreen_w,
      h: Grid.allscreen_offset_y + 1,
      r: 0,
      g: 0,
      b: 0,
      a: 255,
      path: :solid
    }

    # bottom side
    outputs.primitives << {
      x: -Grid.allscreen_offset_x,
      y: -Grid.allscreen_offset_y,
      w: Grid.allscreen_w,
      h: Grid.allscreen_offset_y,
      r: 0,
      g: 0,
      b: 0,
      a: 255,
      path: :solid
    }
  end

  def scenes
    [
      @embark_scene,
      @world_scene,
      @world_event_scene,
      @battle_scene,
      @game_over_scene,
      @loot_scene,
      @cutscene_scene,
      @alert_scene,
      @mini_world_scene,
      @mini_world_event_scene,
      @mini_world_loot_scene,
      @mini_world_battle_scene,
      @status_scene,
      @settings_scene,
      @god_mode_scene,
      @modal_scene
    ]
  end

  def tick_bg_music
    if audio["bg_music"] && audio["bg_music"].gain < 1.0
      audio["bg_music"].gain += 0.01
      audio["bg_music"].gain = 1.0 if audio["bg_music"].gain > 1.0
    end

    if audio["bg_music_fade"] && audio["bg_music_fade"].gain > 0.0
      audio["bg_music_fade"].gain -= 0.01
      if audio["bg_music_fade"].gain <= 0.0
        audio["bg_music_fade"] = nil
      end
    end
  end

  def scene_current
    scenes.find { |s| s.id == state.scene }
  end

  def scene_set_args
    scene = scene_current

    if !scene
      raise "Scene with id of #{state.scene} was not found."
    end

    scene.game = @game
    scene.args = args
    scene.prefab = @prefab
    scene.metadata = state.scene_metadata || {}
    $scene = scene_current
  end

  def scene_tick
    scene_set_args

    if state.scene_at == Kernel.tick_count - 1
      scene_current.will_appear_base
    end

    scene_current.calc_selection_prefab
    scene_current.tick
  end

  def manual_ui_regression!
    center_x = $game.world.half_width
    center_y = $game.world.half_height
    lambdas = [
      -> { state.next_scene = :embark_scene },
      -> { $scene.reset_current_journey_button_selected },
      -> { $scene.reset_current_journey_button_selected },
      -> { state.next_scene = :embark_scene },
      -> { $scene.ellipses_button_selected },
      -> { state.next_scene = :embark_scene },
      -> { $scene.its_never_over_button_selected },
      -> { $scene.fill_flask_button_selected },
      -> { $scene.refuse_food_button_selected },
      # -> { $scene.take_compass_button_selected },
      -> { $scene.embark_button_selected },
      -> {
        $game.world.god_mode!
        $game.world.stores[:alien_alloy] = 20
        $game.world.x = center_x
        $game.world.y = center_y
        $game.world.move_down
        $game.world.landmarks.clear

        # boreholes
        $game.world.landmarks[[center_x - 12, center_y - 12]]  = :borehole
        $game.world.landmarks[[center_x - 12, center_y - 9]]   = :borehole
        $game.world.landmarks[[center_x - 12, center_y - 6]]   = :borehole
        $game.world.landmarks[[center_x - 12, center_y - 3]]   = :borehole
        $game.world.landmarks[[center_x - 9,  center_y - 12]]  = :borehole
        $game.world.landmarks[[center_x - 9,  center_y - 9]]   = :borehole
        $game.world.landmarks[[center_x - 9,  center_y - 6]]   = :borehole
        $game.world.landmarks[[center_x - 9,  center_y - 3]]   = :borehole
        $game.world.landmarks[[center_x + 12, center_y - 12]]  = :borehole
        $game.world.landmarks[[center_x + 12, center_y - 9]]   = :borehole
        $game.world.landmarks[[center_x + 12, center_y - 6]]   = :borehole
        $game.world.landmarks[[center_x + 12, center_y - 3]]   = :borehole
        $game.world.landmarks[[center_x + 9,  center_y - 12]]  = :borehole
        $game.world.landmarks[[center_x + 9,  center_y - 9]]   = :borehole
        $game.world.landmarks[[center_x + 9,  center_y - 6]]   = :borehole
        $game.world.landmarks[[center_x + 9,  center_y - 3]]   = :borehole

        # regular houses
        $game.world.landmarks[[center_x - 6, center_y - 12]]  = :house
        $game.world.landmarks[[center_x - 6, center_y - 9]]   = :house
        $game.world.landmarks[[center_x - 6, center_y - 6]]   = :house
        $game.world.landmarks[[center_x - 6, center_y - 3]]   = :house
        $game.world.landmarks[[center_x - 3, center_y - 12]]  = :house
        $game.world.landmarks[[center_x - 3, center_y - 9]]   = :house
        $game.world.landmarks[[center_x - 3, center_y - 6]]   = :house
        $game.world.landmarks[[center_x - 3, center_y - 3]]   = :house

        # morality houses
        $game.world.landmarks[[center_x + 6, center_y - 12]]  = :cowering_family
        $game.world.landmarks[[center_x + 6, center_y - 9]]   = :mother_and_baby
        $game.world.landmarks[[center_x + 6, center_y - 6]]   = :man_and_boy
        $game.world.landmarks[[center_x + 6, center_y - 3]]   = :mansion
        $game.world.landmarks[[center_x + 3, center_y - 12]]  = :a_small_child
        $game.world.landmarks[[center_x + 3, center_y - 9]]   = :firelit_room
        $game.world.landmarks[[center_x + 3, center_y - 6]]   = :suicide_house
        $game.world.landmarks[[center_x + 3, center_y - 3]]   = :ritual

        # set pieces
        $game.world.landmarks[[center_x, center_y]]           = :swamp
        $game.world.landmarks[[center_x, center_y + 3]]       = :tannery
        $game.world.landmarks[[center_x, center_y + 6]]       = :iron_mine
        $game.world.landmarks[[center_x, center_y + 9]]       = :town
        $game.world.landmarks[[center_x, center_y + 12]]      = :city
        $game.world.landmarks[[center_x, center_y + 15]]      = :armoury
        $game.world.landmarks[[center_x, center_y + 18]]      = :ship
        $game.world.landmarks[[0, 0]] = :zero_zero
        $scene.will_appear_base
      },
      -> {
        $scene.move_down_button_selected
      }
    ]

    running_tick = Kernel.tick_count + 30
    lambdas.each do |lambda|
      GTK.on_tick_count running_tick do
        lambda.call
      end
      running_tick += 30
    end
  end
end
