# Copyright 2014 Scratchwork Development LLC. All rights reserved.

class Hash
  def with_metadata h
    self.merge metadata: h
  end
end

class ScrollView
  attr_gtk

  attr :y_offset,
       :rect,
       :clicked_items,
       :target_y_offset,
       :scroll_view_dy,
       :scroll_view_dy_fall_rate,
       :lowest_item,
       :lowest_a11y_item

  def initialize row:, col:, w:, h:;
    @items = []
    @clicked_items = []
    @y_offset = 0
    @scroll_view_dy = 0
    @rect = Layout.rect row: row,
                        col: col,
                        w: w,
                        h: h,
                        include_row_gutter: true,
                        include_col_gutter: true
    @primitives = []
  end

  def overflow?
    @lowest_item && @lowest_item.y.abs > @rect.h
  end

  def at_bottom?
    y_offset > content_height - @rect.h
  end

  def page_count
    (content_height / rect.h)
  end

  def current_page
    (page_count - ((content_height - (y_offset + rect.h)) / rect.h))
  end

  def scroll_perc
    y_offset / content_bottom
  end

  def scroll_down_page offset_h
    @scroll_page_invoked_at = Kernel.tick_count
    self.target_y_offset = self.y_offset + self.rect.h - offset_h
    if self.target_y_offset > self.y_offset_top_limit - 80
      self.target_y_offset = self.y_offset_top_limit - 80
    end
  end

  def scroll_up_page offset_h
    @scroll_page_invoked_at = Kernel.tick_count
    self.target_y_offset = self.y_offset - self.rect.h + offset_h
    if self.target_y_offset < 1
      self.target_y_offset = 0
    end
  end

  def add_item prefab
    raise "prefab must be a Hash" unless prefab.is_a? Hash
    @items << prefab
    @lowest_item ||= prefab
    if prefab.y < @lowest_item.y
      @lowest_item = prefab
    end

    if prefab.a11y_id
      @lowest_a11y_item ||= prefab
      if prefab.y < @lowest_a11y_item.y
        @lowest_a11y_item = prefab
      end
    end
  end

  def content_height
    lowest_item = @items.min_by { |primitive| primitive.y } || { x: 0, y: 0 }
    h = @rect.h

    if lowest_item
      h -= lowest_item.y - Layout.gutter
    end

    h
  end

  def y_offset_bottom_limit
    -80
  end

  def y_offset_top_limit
    content_height - @rect.h + @rect.y + 80
  end

  def content_bottom
    content_height - @rect.h + @rect.y
  end

  def scroll_to_bottom
    @target_y_offset = content_height - @rect.h + @rect.y
  end

  def scroll_to_top
    @target_y_offset = 0
  end

  def tick_inputs_mouse_wheel
    return if !inputs.mouse.intersect_rect?(@rect)

    if inputs.mouse.wheel
      if inputs.mouse.wheel.inverted
        @scroll_view_dy -= inputs.mouse.wheel.y
      else
        @scroll_view_dy += inputs.mouse.wheel.y
      end
    end
  end

  def tick_inputs_keyboard
    if inputs.keyboard.key_down.page_down
      if @scroll_view_dy >= 0
        @scroll_view_dy += 5
      else
        @scroll_view_dy = @scroll_view_dy.lerp(0, 1)
      end
    elsif inputs.keyboard.key_down.page_up
      if @scroll_view_dy <= 0
        @scroll_view_dy -= 5
      else
        @scroll_view_dy = @scroll_view_dy.lerp(0, 1)
      end
    end
  end

  def scroll_page_recently_invoked?
    (@scroll_page_invoked_at || 0).elapsed_time < 45
  end

  def tick_inputs_mouse
    @clicked_items = []

    if inputs.mouse.down && inputs.mouse.intersect_rect?(@rect)
      @last_mouse_held_y = inputs.mouse.y
      @last_mouse_held_y_diff = 0
      @mouse_down_at = state.tick_count
      @mouse_down_y = inputs.mouse.y
      @scroll_view_dy = 0
      @maybe_click = if @scroll_view_dy.abs < 12
                       true
                     else
                       false
                     end
    elsif inputs.mouse.down
      @last_mouse_held_y = nil
      @last_mouse_held_y_diff = nil
      @mouse_down_at = nil
      @mouse_down_y = nil
    end

    return if !@mouse_down_at

    if inputs.mouse.held && inputs.mouse.held_at.elapsed_time >= 2 && !scroll_page_recently_invoked?
      @last_mouse_held_y ||= inputs.mouse.y
      @last_mouse_held_y_diff ||= 0
      @last_mouse_held_y_diff = inputs.mouse.y - @last_mouse_held_y
      @last_mouse_held_y = inputs.mouse.y
      @target_y_offset = @y_offset + (inputs.mouse.y - @mouse_down_y) * 2
      @mouse_down_y = inputs.mouse.y
    elsif inputs.mouse.up
      @target_y_offset = nil
      @mouse_up_at = state.tick_count
      @mouse_up_y = inputs.mouse.y

      if @maybe_click &&
         (@last_mouse_held_y_diff).abs <= 1 &&
         (@mouse_down_at - @mouse_up_at).abs < 12
        if inputs.mouse.y - 20 > @rect.y && inputs.mouse.y < (@rect.y + @rect.h - 20)
          @clicked_items = offset_items.reject { |primitive| !primitive.w || !primitive.h || (primitive.a || 255) == 0 }
                                       .find_all { |primitive| inputs.mouse.inside_rect? primitive }
        end
      else
        @scroll_view_dy += @last_mouse_held_y_diff
      end

      @last_mouse_held_y = nil
      @last_mouse_held_y_diff = nil
      @mouse_down_at = nil
      @mouse_down_ = nil
      @mouse_up_at = nil
    end
  end

  def tick_inputs
    tick_inputs_mouse_wheel
    tick_inputs_mouse
    tick_inputs_keyboard
  end

  def is_near_bottom?

  end

  def tick
    outputs[:scroll_view].w = Grid.w
    outputs[:scroll_view].h = Grid.h
    outputs[:scroll_view].background_color = [245, 245, 245, 245]
    outputs[:scroll_view].transient!

    outputs[:scroll_view_content].w = Grid.w
    outputs[:scroll_view_content].h = Grid.h
    outputs[:scroll_view_content].background_color = [245, 245, 245, 245]
    outputs[:scroll_view_content].transient!

    if @target_y_offset
      if @target_y_offset < y_offset_bottom_limit
        @y_offset = @y_offset.lerp @target_y_offset, 0.05
      elsif @target_y_offset > y_offset_top_limit
        @y_offset = @y_offset.lerp @target_y_offset, 0.05
      else
        @y_offset = @y_offset.lerp @target_y_offset, 0.5
      end
      @target_y_offset = nil if @y_offset.round == @target_y_offset.round
      @scroll_view_dy = 0
    end

    tick_inputs

    @y_offset += @scroll_view_dy

    if @y_offset < 0
      if inputs.mouse.held
      else
        @y_offset = @y_offset.lerp(0, 0.2)
      end
    end

    if content_height <= (@rect.h - @rect.y)
      @y_offset = 0
      @scroll_view_dy = 0
    elsif @y_offset > content_height - @rect.h + @rect.y
      if inputs.mouse.held
      else
        @y_offset = @y_offset.lerp(content_height - @rect.h + @rect.y, 0.2)
      end
    end
    @scroll_view_dy *= 0.95
    @scroll_view_dy = @scroll_view_dy.round(2)
    if @scroll_view_dy.abs <= 0.1 && @scroll_view_dy != 0
      @scroll_view_dy = 0
      @y_offset = @y_offset.round
    end
  end

  def items
    @items
  end

  def a11y_items
    @items.find_all do |primitive|
      primitive.a11y_id
    end
  end

  def offset_items filter_mode: :intersect_rect?
    @items.map do |primitive|
      primitive.merge(y: primitive.y + @y_offset)
    end.find_all do |primitive|
      if filter_mode == :inside_rect?
        Geometry.inside_rect? Geometry.rect_props(primitive), @rect
      else
        Geometry.intersect_rect? Geometry.rect_props(primitive), @rect
      end
    end
  end

  def offset_a11y_items filter_mode: :intersect_rect?
    offset_items(filter_mode: filter_mode).find_all do |primitive|
      primitive.a11y_id
    end
  end

  def prefab(filter_mode: :intersect_rect?)
    outputs[:scroll_view_content].background_color = [245, 245, 245, 255]
    outputs[:scroll_view_content].transient!

    outputs[:scroll_view].background_color = [245, 245, 245, 255]
    outputs[:scroll_view].transient!

    outputs[:scroll_view_content].primitives << offset_items(filter_mode: filter_mode)

    outputs[:scroll_view].primitives << {
      x: @rect.x,
      y: @rect.y,
      w: @rect.w,
      h: @rect.h,
      source_x: @rect.x,
      source_y: @rect.y,
      source_w: @rect.w,
      source_h: @rect.h,
      path: :scroll_view_content
    }

    outputs[:scroll_view].primitives << [
      { x: @rect.x,
        y: @rect.y,
        w: @rect.w,
        h: @rect.h,
        primitive_marker: :border,
        r: 128,
        g: 128,
        b: 128 },
    ]

    { x: 0,
      y: 0,
      w: Grid.w,
      h: Grid.h,
      path: :scroll_view }
  end
end
