# Copyright 2014 Scratchwork Development LLC. All rights reserved.

class SettingsScene < Scene
  def id
    :settings_scene
  end

  def will_appear
    @game_version = $cvars["game_metadata.version"].value
    @manual_regression_taps = 0
    @manual_regression_rect = manual_regression_button
    @god_mode_taps = 0
    @god_mode_rect = god_mode_button
    stop_music
  end

  def will_appear_a11y_target
    :layout_title
  end

  def button_ids
    [
      :back_button,
      :toggle_fullscreen_button,
      :volume_decrease_button,
      :volume_increase_button,
      :exit_game_button,
    ]
  end

  def back_button_id
    :back_button
  end

  def allows_fullscreen_toggle?
    return false if GTK.platform? :steamdeck
    return false if GTK.platform? :ios
    return false if GTK.platform? :android
    return true
  end

  def progress_bar_ids
    if @game.cleared_game_once?
      [
        :dev_commentary_01_progress_bar,
        :dev_commentary_02_progress_bar,
        :dev_commentary_03_progress_bar,
        :dev_commentary_04_progress_bar,
        :dev_commentary_05_progress_bar,
        :dev_commentary_06_progress_bar,
        :dev_commentary_07_progress_bar,
        :dev_commentary_08_progress_bar,
        :dev_commentary_09_progress_bar,
        :dev_commentary_10_progress_bar,
        :dev_commentary_11_progress_bar,
        :dev_commentary_12_progress_bar,
      ]
    else
      []
    end
  end

  def volume_decrease_button_selected
    args.audio.volume -= 0.1
    args.audio.volume = 0 if args.audio.volume < 0
    queue_sound 'sounds/draw.ogg'
  end

  def volume_increase_button_selected
    args.audio.volume += 0.1
    args.audio.volume = 1 if args.audio.volume > 1
    queue_sound 'sounds/draw.ogg'
  end

  def back_button_selected
    state.next_scene = state.previous_scene
  end

  def tick
    outputs.primitives << title_label
    outputs.primitives << volume_label
    outputs.primitives << version_label
    outputs.primitives << dev_commentary_label
    outputs.primitives << prefab.button(args, back_button)
    outputs.primitives << prefab.button(args, toggle_fullscreen_button)
    outputs.primitives << prefab.button(args, volume_decrease_button)
    outputs.primitives << prefab.button(args, volume_increase_button)
    outputs.primitives << prefab.button(args, exit_game_button)

    if inputs.mouse.click && inputs.mouse.intersect_rect?(@manual_regression_rect)
      @manual_regression_taps += 1
      if @manual_regression_taps >= 10
        outputs.primitives << manual_regression_button.merge(path: :solid, r: 0, g: 0, b: 0, primitive_marker: :sprite)
      end

      if @manual_regression_taps == 15
        state.next_scene = state.previous_scene
        GTK.on_tick_count Kernel.tick_count + 60 do
          $root_scene.manual_ui_regression!
        end
      end
    end

    if inputs.mouse.click && inputs.mouse.intersect_rect?(@god_mode_rect)
      @god_mode_taps += 1
      if @god_mode_taps >= 10
        outputs.primitives << god_mode_button.merge(path: :solid, r: 0, g: 0, b: 0, primitive_marker: :sprite)
      end

      if @god_mode_taps == 15
        state.next_scene = state.previous_scene
        GTK.on_tick_count Kernel.tick_count + 60 do
          $game.world.toggle_god_mode!
        end
      end
    end

    calc_buttons
    calc_progress_bars
    render_progress_bars
  end

  def toggle_fullscreen_button_selected
    GTK.toggle_window_fullscreen
  end

  def dev_commentary_progress_bar text, path, row: 0, col: 0, w: 3, h: 2
    @dev_commentary_files ||= {}
    @dev_commentary_files[text] = path
    perc = if audio["bg_music"] &&
              audio["bg_music"].input == path &&
              audio["bg_music"].playtime &&
              audio["bg_music"].playlength &&
              audio["bg_music"].playlength != 0
             audio["bg_music"].playtime.fdiv(audio["bg_music"].playlength).round(2)
           else
             0
           end

    rect = Layout.rect(row: row, col: col, w: w, h: h)
                 .merge(text: text,
                        perc: perc,
                        a11y_id: "dev_commentary_#{text}",
                        a11y_trait: :button,
                        a11y_text: "developer commentary ##{text}")
  end

  def commentary_col_position pos
    if Grid.landscape?
      6 + pos * 3
    else
      pos * 3
    end
  end

  def dev_commentary_01_progress_bar
    dev_commentary_progress_bar "1",
                                "sounds/intro.ogg",
                                row: 6,
                                col: commentary_col_position(0)
  end

  def dev_commentary_02_progress_bar
    dev_commentary_progress_bar "2",
                                "sounds/the_casual_gamer.ogg",
                                row: 6,
                                col: commentary_col_position(1)
  end

  def dev_commentary_03_progress_bar
    dev_commentary_progress_bar "3",
                                "sounds/early_development.ogg",
                                row: 6,
                                col: commentary_col_position(2)
  end

  def dev_commentary_04_progress_bar
    dev_commentary_progress_bar "4",
                                "sounds/training.ogg",
                                row: 6,
                                col: commentary_col_position(3)
  end

  def dev_commentary_05_progress_bar
    dev_commentary_progress_bar "5",
                                "sounds/difficult_vs_punishing.ogg",
                                row: 8,
                                col: commentary_col_position(0)
  end

  def dev_commentary_06_progress_bar
    dev_commentary_progress_bar "6",
                                "sounds/cog_dis.ogg",
                                row: 8,
                                col: commentary_col_position(1)
  end

  def dev_commentary_07_progress_bar
    dev_commentary_progress_bar "7",
                                "sounds/narrative.ogg",
                                row: 8,
                                col: commentary_col_position(2)
  end

  def dev_commentary_08_progress_bar
    dev_commentary_progress_bar "8",
                                "sounds/the_defector.ogg",
                                row: 8,
                                col: commentary_col_position(3)
  end

  def dev_commentary_09_progress_bar
    dev_commentary_progress_bar "9",
                                "sounds/the_ritual.ogg",
                                row: 10,
                                col: commentary_col_position(0)
  end

  def dev_commentary_10_progress_bar
    dev_commentary_progress_bar "10",
                                "sounds/the_world.ogg",
                                row: 10,
                                col: commentary_col_position(1)
  end

  def dev_commentary_11_progress_bar
    dev_commentary_progress_bar "11",
                                "sounds/fun_stuff.ogg",
                                row: 10,
                                col: commentary_col_position(2)
  end

  def dev_commentary_12_progress_bar
    dev_commentary_progress_bar "12",
                                "sounds/inspiration.ogg",
                                row: 10,
                                col: commentary_col_position(3)
  end

  def dev_commentary_01_progress_bar_selected
    queue_music @dev_commentary_files["1"], false
  end

  def dev_commentary_02_progress_bar_selected
    queue_music @dev_commentary_files["2"], false
  end

  def dev_commentary_03_progress_bar_selected
    queue_music @dev_commentary_files["3"], false
  end

  def dev_commentary_04_progress_bar_selected
    queue_music @dev_commentary_files["4"], false
  end

  def dev_commentary_05_progress_bar_selected
    queue_music @dev_commentary_files["5"], false
  end

  def dev_commentary_06_progress_bar_selected
    queue_music @dev_commentary_files["6"], false
  end

  def dev_commentary_07_progress_bar_selected
    queue_music @dev_commentary_files["7"], false
  end

  def dev_commentary_08_progress_bar_selected
    queue_music @dev_commentary_files["8"], false
  end

  def dev_commentary_09_progress_bar_selected
    queue_music @dev_commentary_files["9"], false
  end

  def dev_commentary_10_progress_bar_selected
    queue_music @dev_commentary_files["10"], false
  end

  def dev_commentary_11_progress_bar_selected
    queue_music @dev_commentary_files["11"], false
  end

  def dev_commentary_12_progress_bar_selected
    queue_music @dev_commentary_files["12"], false
  end

  def tick_progress_bars
  end

  def toggle_fullscreen_button
    return nil if !GTK.can_resize_window?
    return nil if Grid.portrait?

    bt_rect_row = 1
    bt_rect_col = if GTK.can_resize_window? && GTK.can_close_window?
                    10
                  elsif GTK.can_close_window?
                    10
                  else
                    12
                  end

    bt_w = if GTK.can_resize_window? && GTK.can_close_window?
             4
           elsif GTK.can_close_window?
             6
           else
             12
           end

    Layout.rect(row: bt_rect_row, col: bt_rect_col, h: 2, w: bt_w)
          .merge(text: (GTK.window_fullscreen? ? "windowed" : "fullscreen"),
                 a11y_id: :pause_back,
                 a11y_text: "resume",
                 a11y_trait: :button)
  end

  def exit_game_button
    return nil if !GTK.can_close_window?
    return nil if Grid.portrait?

    bt_rect_row = 1
    bt_rect_col = if GTK.can_resize_window? && GTK.can_close_window?
                    14
                  elsif GTK.can_close_window?
                    12
                  else
                    12
                  end

    bt_w = if GTK.can_resize_window? && GTK.can_close_window?
             4
           elsif GTK.can_close_window?
             6
           else
             12
           end

    Layout.rect(row: bt_rect_row, col: bt_rect_col, h: 2, w: bt_w)
          .merge(text: "quit game",
                 a11y_id: :pause_back,
                 a11y_text: "quit game",
                 a11y_trait: :button)
  end

  def exit_game_button_selected
    GameState.save $game
    GTK.request_quit
  end

  def god_mode_button
    bt_rect_row = 1
    bt_rect_col = if Grid.landscape?
                    14
                  else
                    8
                  end
    Layout.rect(row: 0, col: bt_rect_col, h: 1, w: 4)
  end

  def manual_regression_button
    bt_rect_row = 1
    bt_rect_col = if Grid.landscape?
                    6
                  else
                    0
                  end

    Layout.rect(row: 0, col: bt_rect_col, h: 1, w: 4)
  end

  def back_button
    bt_rect_row = 1
    bt_rect_col = if Grid.landscape?
                    6
                  else
                    0
                  end

    bt_w = if GTK.can_resize_window? && GTK.can_close_window?
             4
           elsif GTK.can_close_window? && !GTK.platform?(:mobile)
             6
           else
             12
           end

    Layout.rect(row: bt_rect_row, col: bt_rect_col, h: 2, w: bt_w)
          .merge(text: "resume game",
                 a11y_id: :pause_back,
                 a11y_text: "resume",
                 a11y_trait: :button)
  end

  def allows_fullscreen_toggle?
    return false if $gtk.platform? :steamdeck
    return false if $gtk.platform? :ios
    return false if $gtk.platform? :android
    return true
  end

  def allows_quit?
    return false if $gtk.platform? :ios
    return false if $gtk.platform? :web
    return true
  end

  def volume_decrease_button
    bt_rect_row = 3
    bt_rect_col = if Grid.landscape?
                    6
                  else
                    0
                  end

    Layout.rect(row: bt_rect_row, col: bt_rect_col, h: 2, w: 4)
          .merge(text: "↓",
                 a11y_id: :pause_volume_decrease,
                 a11y_text: "decrease volume",
                 a11y_trait: :button)
  end

  def volume_label
    volume_label_rect = if Grid.orientation == :landscape
                          Layout.rect(row: 3, col: [6, 17], h: 2)
                        else
                          Layout.rect(row: 3, col: [3, 8], h: 2)
                        end

    volume_label_rect.center.merge(text: "volume: #{($args.audio.volume * 100).to_i}%",
                                   anchor_x: 0.5,
                                   anchor_y: 0.5,
                                   a11y_id: :pause_volume,
                                   a11y_text: "volume: #{($args.audio.volume * 100).to_i}%",
                                   font: "fonts/text.ttf",
                                   size_px: 30,
                                   a11y_trait: :label)
  end

  def title_label
    title_rect = if Grid.orientation == :landscape
                   Layout.rect(row: 0, col: [6, 17], h: 1)
                 else
                   Layout.rect(row: 0, col: [0, 11], h: 1)
                  end

    title_rect.center.merge(
      text: "settings",
      anchor_x: 0.5,
      anchor_y: 0.5,
      size_px: 30,
      a11y_id: "layout_title",
      a11y_text: "settings",
      font: "fonts/text.ttf",
      a11y_trait: :label)
  end

  def dev_commentary_label
    return if !@game.cleared_game_once?
    title_rect = if Grid.orientation == :landscape
                   Layout.rect(row: 5, col: [6, 17], h: 1)
                 else
                   Layout.rect(row: 5, col: [0, 11], h: 1)
                  end

    title_rect.center.merge(
      text: "developer commentary",
      anchor_x: 0.5,
      anchor_y: 1.0,
      size_px: 26,
      a11y_id: "dev_commentary",
      a11y_text: "developer commentary",
      font: "fonts/text.ttf",
      a11y_trait: :label)
  end

  def version_label
    version_rect = if Grid.orientation == :landscape
                   Layout.rect(row: 0, col: [6, 17], h: 1)
                 else
                   Layout.rect(row: 0, col: 0, w: 1, h: 1)
                  end

    version_rect.center.merge(x: version_rect.x,
                              text: "v#{@game_version}",
                              anchor_x: 0,
                              anchor_y: 0.5,
                              a11y_id: :pause_version,
                              a11y_text: "version #{@game_version}",
                              font: "fonts/text.ttf",
                              a11y_trait: :label)
  end

  def manual_regression_button_selected
    $root_scene.manual_ui_regression!
  end

  def volume_increase_button
    bt_rect_row = 3
    bt_rect_col = if Grid.landscape?
                    14
                  else
                    8
                  end
    Layout.rect(row: bt_rect_row, col: bt_rect_col, h: 2, w: 4)
          .merge(text: "↑",
                 a11y_id: :pause_volume_increase,
                 a11y_text: "increase volume",
                 a11y_trait: :button)
  end

  def keyboard_instructions
    default_keyboard_instructions
  end

  def controller_instructions accept_char
    default_controller_instructions
  end
end
