# Copyright 2014 Scratchwork Development LLC. All rights reserved.

class StatusScene < Scene
  def id
    :status_scene
  end

  def button_ids
    [
      :close_button,
      :heal_button,
      :scroll_down_button,
      :scroll_up_button
    ]
  end

  def scroll_view_layout
    if Grid.orientation == :landscape
      { row: 1, col: 0, w: 24, h: scroll_view_layout_h }
    else
      { row: 1, col: 0, w: 12, h: scroll_view_layout_h }
    end
  end

  def scroll_view_layout_h
    if Grid.landscape?
      6
    else
      16
    end
  end

  def will_appear
    @heal_at = -1000
    @scroll_view = ScrollView.new scroll_view_layout
    @running_row = 1.5

    entries = (@game.world.stores.keys + @game.world.weapon_stores)

    slice_count = if Grid.landscape?
                    4
                  else
                    2
                  end

    entries.each_slice(slice_count)
           .each do |items|
      running_col = 1
      items.each do |o|
        if o.is_a?(Symbol)
          add_item_to_scroll_view row: @running_row, col: running_col, w: 5, key: o
        elsif o
          add_weapon_to_scroll_view row: @running_row, col: running_col, w: 5, weapon: o
        end
        running_col += 6
      end

      @running_row += 1.5
    end

    (slice_count - 1).times do |i|
      border_rect = if (@running_row - 1.5) < scroll_view_layout_h
                      Layout.rect(row: 1, col: 6 * (i + 1), w: 0, h: scroll_view_layout_h)
                    else
                      Layout.rect(row: 1, col: 6 * (i + 1), w: 0, h: @running_row - 0.5)
                    end

      @scroll_view.add_item prefab.thick_border(args, border_rect.merge(w: 1))
    end

    @scroll_view.scroll_to_bottom

    GTK.schedule_callback Kernel.tick_count + 30 do
      @scroll_view.scroll_to_top
    end
  end

  def tick
    # outputs.watch "content_height: #{@scroll_view.content_height}"
    # outputs.watch "target_offset_y: #{@scroll_view.target_y_offset}"
    # outputs.watch "offset_y: #{@scroll_view.y_offset}"
    # outputs.watch "scrollview h: #{@scroll_view.rect.h}"
    # outputs.watch "page count: #{@scroll_view.page_count}"
    # outputs.watch "current page: #{@scroll_view.current_page}"
    # outputs.watch "scroll_view_dy: #{@scroll_view.scroll_view_dy}"
    # outputs.watch "scroll_perc: #{@scroll_view.scroll_perc}"
    # outputs.watch "content_bottom: #{@scroll_view.content_bottom}"

    calc_buttons
    @scroll_view.args = args
    @scroll_view.tick
    if @running_row <= scroll_view_layout_h
      @scrolling = false
    end

    @scroll_view.items.each do |item|
      next unless item.metadata
      if item.metadata.control_id == :item_equipped_label
        item.text = "#{item.metadata.control_equipped_desc}: #{@game.world.stores[item.metadata.item_key] || 0}"
      end

      if GTK.a11y_enabled?
        if item.metadata.control_id == :item_equipped_label
          item.a11y_id = "#{item.metadata.item_key}"
          item.a11y_text = "#{item.metadata.item_key.to_s.gsub("_", " ")}: #{item.metadata.control_equipped_desc}: #{@game.world.stores[item.metadata.item_key] || 0}"
          item.a11y_trait = :label
        elsif item.metadata.control_id == :weapon_text_label
          item.a11y_id = "#{item.metadata.weapon.id}"
          item.a11y_text = "#{item.metadata.weapon.name},\n#{weapon_string(item.metadata.weapon,  @game.world.weapon_stores.include?(item.metadata.weapon))}"
          item.a11y_trait = :label
        end
      end
    end

    if GTK.a11y_enabled?
      @scroll_view.offset_items(filter_mode: :inside_rect?).each do |i|
        if i.a11y_id
          outputs.a11y[i.a11y_id] = {
            x: i.x,
            y: i.y,
            w: i.w,
            h: i.h,
            anchor_x: i.anchor_x,
            anchor_y: i.anchor_y,
            a11y_id: i.a11y_id,
            a11y_text: i.a11y_text,
            a11y_trait: i.a11y_trait,
            a11y_hidden: i.a11y_hidden
          }
        end
      end
    end

    if inputs.last_active == :controller
      if inputs.controller_one.right_analog_y_perc.abs > 0.20
        @scrolling = true
        @scroll_view.scroll_view_dy -= inputs.controller_one.right_analog_y_perc
      elsif @scrolling == true &&
            @scroll_view.scroll_view_dy.abs == 0 &&
            @scroll_view.y_offset.round >= 0 &&
            @scroll_view.y_offset.round <= @scroll_view.y_offset_top_limit - 80
        @scrolling = false
        @scroll_view.y_offset = @scroll_view.y_offset.round
      elsif @scrolling == true
        @scroll_view.scroll_view_dy *= 0.5
      end
    elsif GTK.a11y_enabled? || inputs.last_active == :mouse
      if scroll_down_button && inputs.mouse.click && inputs.mouse.inside_rect?(scroll_down_button)
        @scroll_view.scroll_down_page Layout.rect(h: 4).h

        GTK.on_tick_count Kernel.tick_count + 10 do
          if GTK.a11y_enabled?
            possible_a11y_focus_points = @scroll_view.offset_items(filter_mode: :inside_rect?)
                                                     .find_all { |prim| prim.a11y_id && !prim.a11y_hidden }
                                                     .sort_by { |prim| [-prim.y, prim.x] }

            if possible_a11y_focus_points.length > 0
              outputs.a11y["set_selection"] = {
                a11y_id: "set_selection",
                a11y_trait: :notification,
                a11y_notification_target: possible_a11y_focus_points.last.a11y_id,
              }
            end
          end
        end

        GTK.on_tick_count Kernel.tick_count + 90 do
          if GTK.a11y_enabled?
            if @scroll_view.scroll_perc >= 0.95
              outputs.a11y["last_page"] = {
                a11y_id: "last_page",
                a11y_trait: :notification,
                a11y_text: "you are on the last page"
              }
            end
          end
        end
      elsif scroll_up_button && inputs.mouse.click && inputs.mouse.inside_rect?(scroll_up_button)
        @scroll_view.scroll_up_page Layout.rect(h: 4).h

        GTK.on_tick_count Kernel.tick_count + 10 do
          if GTK.a11y_enabled?
            possible_a11y_focus_points = @scroll_view.offset_items(filter_mode: :inside_rect?)
                                                     .find_all { |prim| prim.a11y_id && !prim.a11y_hidden }
                                                     .sort_by { |prim| [-prim.y, prim.x] }

            if possible_a11y_focus_points.length > 0
              outputs.a11y["set_selection"] = {
                a11y_id: "set_selection",
                a11y_trait: :notification,
                a11y_notification_target: possible_a11y_focus_points.first.a11y_id,
              }
            end
          end
        end

        GTK.on_tick_count Kernel.tick_count + 90 do
          if GTK.a11y_enabled?
            if @scroll_view.scroll_perc <= 0.05
              outputs.a11y["first_page"] = {
                a11y_id: "first_page",
                a11y_trait: :notification,
                a11y_text: "you are on the first page"
              }
            end
          end
        end
      end
    elsif inputs.last_active == :keyboard
      if inputs.keyboard.key_down.page_up || inputs.keyboard.key_down.page_down
        @scrolling = true
        if inputs.keyboard.key_down.page_up
          scroll_up_button_selected
        elsif inputs.keyboard.key_down.page_down
          scroll_down_button_selected
        end
      elsif @scrolling == true &&
            @scroll_view.scroll_view_dy.abs == 0 &&
            @scroll_view.y_offset.round >= 0 &&
            @scroll_view.y_offset.round <= @scroll_view.y_offset_top_limit - 80
        @scrolling = false
        @scroll_view.y_offset = @scroll_view.y_offset.round
      end
    end

    if @scrolling &&
       (@scroll_view.y_offset.round + @scroll_view.scroll_view_dy) <= 0 &&
       @selection.current &&
       @selection.current.id == :scroll_up_button
      set_selection! :scroll_down_button
    end

    filter_mode = if GTK.a11y_enabled?
                    :inside_rect?
                  else
                    :intersect_rect?
                  end

    outputs.primitives << @scroll_view.prefab(filter_mode: filter_mode)
    outputs.primitives << player_labels
    outputs.primitives << title_label
    outputs.primitives << weight_label
    outputs.primitives << prefab.button(args, scroll_up_button)
    outputs.primitives << prefab.button(args, scroll_down_button)
    outputs.primitives << prefab.button(args, close_button)
    outputs.primitives << prefab.button(args, heal_button)
  end

  def close_button_selected
    state.next_scene = state.previous_scene
  end

  def heal_button_selected
    if (game.world.stores[:food] || 0) >= 1
      @heal_at = Kernel.tick_count
      game.world.current_hp += game.world.meat_heal
      game.world.current_hp = game.world.current_hp.clamp(0, game.world.max_hp)
      game.world.stores[:food] -= 1
      queue_sound "sounds/food.ogg"

      outputs.a11y[:heal_button_selected] = {
        a11y_id: :heal_button_selected,
        a11y_text: "healed for #{game.world.meat_heal} hp. current hp: #{game.world.current_hp} of #{game.world.max_hp}. food available: #{game.world.stores[:food] || 0}",
        a11y_trait: :notification
      }
    end
  end

  def player_labels
    player_rect = if Grid.landscape?
                    Layout.rect(row: 7.5, col: 0, w: Layout.col_count, h: 1)
                  else
                    Layout.rect(row: 18, col: 0, w: Layout.col_count, h: 1)
                  end

    a = if @heal_at.elapsed_time < 60
          255
        else
          0
        end
    [
      {
        x: player_rect.center.x,
        y: player_rect.center.y,
        text: "^#{game.world.meat_heal}",
        size_px: 30,
        font: "fonts/sprite.ttf",
        anchor_x: 0.5, anchor_y: -0.5,
        r: 0, g: 128, b: 0, a: a
      },
      {
        x: player_rect.center.x,
        y: player_rect.center.y,
        text: "@",
        size_enum: 10,
        font: "fonts/sprite.ttf",
        anchor_x: 0.5, anchor_y: 0.5,
        **prefab.black
      },
      {
        x: player_rect.center.x,
        y: player_rect.center.y,
        text: "#{game.world.current_hp.to_i} / #{game.world.max_hp.to_i}",
        anchor_x: 0.5, anchor_y: 1.5,
        size_enum: 5,
        font: "fonts/sprite.ttf",
        **prefab.black
      }
    ]
  end

  def scroll_up_button_selected
    @scroll_view.target_y_offset = @scroll_view.y_offset - Layout.rect(h: 1.5).h
    if @scroll_view.target_y_offset < 1
      disable_selection_prefab!
      @scroll_view.target_y_offset = 0
      GTK.scheduled_callbacks[Kernel.tick_count + 30] = lambda {
        enable_selection_prefab!
        if @selection && @selection.current.id == :scroll_up_button
          set_selection! :scroll_down_button
        end
      }
    end
  end

  def scroll_down_button_selected
    @scroll_view.target_y_offset = @scroll_view.y_offset + Layout.rect(h: 1.5).h
    if @scroll_view.target_y_offset > @scroll_view.y_offset_top_limit - 80
      @scroll_view.target_y_offset = @scroll_view.y_offset_top_limit - 80
    end
  end

  def scroll_up_button
    return nil if inputs.last_active != :keyboard && inputs.last_active != :controller && !GTK.a11y_enabled?
    return nil if @running_row <= scroll_view_layout_h
    return nil if @scroll_view.y_offset <= 5

    { x: Layout.rect(col: 0).x,
      y: @scroll_view.rect.y + @scroll_view.rect.h - Layout.rect(h: 0.75).h,
      w: Layout.rect(scroll_view_layout).w,
      h: Layout.rect(h: 0.75).h,
      bg_color: { r: 245, g: 245, b: 245 },
      a11y_id: :scroll_up_button,
      a11y_trait: :button,
      a11y_text: "previous page",
      text: "scroll up ↑" }
  end

  def scroll_down_button
    return nil if inputs.last_active != :keyboard && inputs.last_active != :controller && !GTK.a11y_enabled?
    return nil if @running_row <= scroll_view_layout_h

    { x: Layout.rect(col: 0).x,
      y: @scroll_view.rect.y + 1,
      w: Layout.rect(scroll_view_layout).w,
      h: Layout.rect(h: 0.75).h,
      bg_color: { r: 245, g: 245, b: 245 },
      a11y_id: :scroll_down_button,
      a11y_trait: :button,
      a11y_text: "next page",
      text: "scroll down ↓" }
  end

  def close_button
    close_rect = if Grid.landscape?
                    Layout.rect(row: 10.5, col: 6, w: 12, h: 1.5)
                  else
                    Layout.rect(row: 22, col: 0, w: 12, h: 2)
                  end

    close_rect.merge(text: "close",
                     a11y_id: :close_button,
                     a11y_text: "close",
                     a11y_trait: :button)
  end

  def heal_button
    color = if (game.world.stores[:food] || 0) >= 1
              prefab.black
            else
              { r: 128, g: 128, b: 128 }
            end

    heal_rect = if Grid.landscape?
                    Layout.rect(row: 9, col: 6, w: 12, h: 1.5)
                  else
                    Layout.rect(row: 20, col: 0, w: 12, h: 2)
                  end

    heal_rect.merge(text: "food: #{game.world.stores[:food] || 0} (+#{game.world.meat_heal} hp)",
                    a11y_id: "heal",
                    a11y_text: "heal, food available: #{game.world.stores[:food] || 0}, heal for: #{game.world.meat_heal} hp, current hp: #{game.world.current_hp} of #{game.world.max_hp} hp.",
                    a11y_trait: :button,
                    **color)
  end

  def add_weapon_to_scroll_view row:, col:, w:, weapon:;
    return if !weapon
    @scroll_view.add_item Layout.rect(row: row, col: col, w: w, h: 1)
                                .merge(text: weapon_title(weapon, @game.world.weapon_stores.include?(weapon)),
                                       anchor_y: -0.5,
                                       anchor_x: 0.0,
                                       size_px: 30,
                                       font: "fonts/text.ttf")
                                .with_metadata(control_id: :weapon_title_label, weapon: weapon)

    @scroll_view.add_item Layout.rect(row: row, col: col, w: w, h: 1)
                                .merge(text: weapon_string(weapon,  @game.world.weapon_stores.include?(weapon)),
                                       anchor_y: 0.5,
                                       anchor_x: 0.0,
                                       size_px: 25,
                                       font: "fonts/text.ttf")
                                .with_metadata(control_id: :weapon_text_label, weapon: weapon)
  end

  def weapon_title weapon, equipped
    equipped_string = ""
    rusted = ""

    rusted = "rusted " if(@game.world.rusted?)

    if(weapon.has_durability?)
      "#{rusted}#{weapon.name.gsub("_", " ")}#{equipped_string}"
    else
      "#{rusted}#{weapon.name.gsub("_", " ")}#{equipped_string}"
    end
  end

  def weapon_string weapon, equipped
    equipped_string = ""
    rusted = ""

    rusted = "rusted " if(@game.world.rusted?)

    if(weapon.has_durability?)
      "durability: #{weapon.durability_percentage}%"
    else
      "durability: unbreakable"
    end
  end

  def add_item_to_scroll_view row:, col:, w:, key:;
    return if !key
    @scroll_view.add_item Layout.rect(row: row, col: col, w: w, h: 1)
                                .merge(text: "#{key.to_s.gsub("_", " ")}",
                                       anchor_y: -0.5,
                                       size_px: 30,
                                       font: "fonts/text.ttf")

    @scroll_view.add_item Layout.rect(row: row, col: col, w: w, h: 1)
                                .merge(text: "-",
                                       anchor_y: 0.5,
                                       size_px: 25,
                                       font: "fonts/text.ttf")
                                .with_metadata(control_id: :item_equipped_label,
                                               control_equipped_desc: "count",
                                               item_key: key)
  end

  def will_appear_a11y_target
    :items_header
  end

  def title_label
    Layout.rect(row: 0, col: 0, w: Layout.col_count, h: 1)
          .center
          .merge(text: "items",
                 font: "fonts/text.ttf",
                 a11y_text: "items",
                 a11y_id: :items_header,
                 a11y_trait: :label,
                 size_px: 30,
                 anchor_x: 0.5,
                 anchor_y: 0.0)
  end

  def a11y_carry_description
    "#{@game.world.carry[:name]}, capacity used: #{@game.world.inventory_weight}, total capacity: #{@game.world.carry[:max]}"
  end

  def weight_label
    carry = @game.world.carry

    color = if game.world.inventory_weight > carry[:max]
              { r: 200, g: 0, b: 0 }
            else
              { r: 0, g: 0, b: 0 }
            end

    Layout.rect(row: 0, col: 0, w: Layout.col_count, h: 1)
          .center
          .merge(text: "#{@game.world.carry[:name]}: #{@game.world.inventory_weight}/#{@game.world.carry[:max]}",
                 font: "fonts/text.ttf",
                 size_px: 30,
                 a11y_id: :weight_label,
                 a11y_trait: :text,
                 a11y_text: a11y_carry_description,
                 **color,
                 anchor_x: 0.5,
                 anchor_y: 0.75)
  end
end
