# Copyright 2014 Scratchwork Development LLC. All rights reserved.

class WorldScene < Scene
  attr :last_punch_level, :camera_zoom, :camera_offset_x, :camera_offset_y

  def id
    :world_scene
  end

  def world
    @game.world
  end

  def will_appear
    outputs[:world_map].w = 1280
    outputs[:world_map].h = 720
    outputs[:world_map].transient!
    outputs[:world_map].background_color = [245, 245, 245, 0]

    args.outputs.a11y[:notify_will_appear] = {
      a11y_trait: :notification,
      a11y_notification_target: :describe_food_water
    }

    @game.save

    @world_a11y_describe_entries = nil

    @focal_point_x = @game.world.x
    @focal_point_y = @game.world.y
    @camera_target_x = focal_point_in_world_space.x - 640
    @camera_target_y = focal_point_in_world_space.y - 360
    @camera_offset_x = @camera_target_x
    @camera_offset_y = @camera_target_y
    @navigation_color_fade_out_at = nil
    @navigation_color = if Grid.landscape?
                          { r: 0, g: 0, b: 0, a: 255 }
                        else
                          { r: 0, g: 0, b: 0, a: 255 }
                        end

    invalidate_label_cache
    update_label_cache
    if !@game.world.defector_x
      @defector_horizon_notification = false
      @defector_near_notification = false
    end
    @water_zero_notification = false
    @food_zero_notification = false
    @last_story_line = nil
    prefab.notification_callbacks[:world_scene] = lambda do |key, notification|
      if key == :defector_horizon
        @game.world.defector_visible = false
      end
    end
    calc_notifications

    queue_music "sounds/world.ogg", true

    if @game.world.dead?
      transition_to_game_over
    end

    flash_player_position
  end

  def update_label_cache
    @camera_zoom = if Grid.orientation == :landscape
                     0.75
                   else
                     1.5
                   end
    @camera_offset_x ||= 0
    @camera_offset_y ||= 0
    # game.world.height.times do |i|
      @world_aesthetics_label_cache.clear
      @world_landmark_labels_cache.clear
    # end

    @player_row = nil
    @player_col = nil
    @defector_row = nil
    @defector_col = nil
    @swamp_row = nil
    @swamp_col = nil
    @world_a11y_describe_entries = nil

    update_aesthetics_labels
    update_landmark_labels
  end

  def update_landmark_labels
    start_y = -1
    max_visible_rows.times
      .map { |i| world_landmarks_for start_y + i, start_y }
      .each_with_index.map do |l, i|
      if game.world.y == start_y + i
        @player_row = i
      end
      if game.world.defector_y == start_y + i
        @defector_row = i
      end
      if game.world.swamp_location_y == start_y + i
        @swamp_row = i
      end

      if l && !@world_landmark_labels_cache[i]
        @world_landmark_labels_cache[i] =
          { x: 0,
            y: 720 - world_line_height * i,
            text: l,
            **text_color,
            size_px: world_label_size_px,
            font: "fonts/sprite.ttf",
            anchor_x: 0 }
      end
    end
  end

  def world_landmarks_for row, world_y
    str = ""
    return nil if @world_landmark_labels_cache[row]
    start_x = -1
    max_visible_columns.times do |i|
      landmark = game.world.landmarks[[start_x + i, row]]
      if landmark == :swamp
        @swamp_col = i
      end

      visible = game.world[[start_x + i, row]]
      cleared = game.world.cleared[[start_x + i, row]]
      if visible == :visible && world.defector_x == (start_x + i) && world.defector_y == row && !world.is_defector_dead
        str.concat "&"
        @defector_col = i
      elsif landmark && visible == :visible && !cleared
        if !landmark_map[landmark]
          GTK.notify! "not in map: #{landmark}"
        end
        if landmark == :swamp && game.world.last_admiral_cutscene_visited == game.world.current_admiral_cutscene
          str.concat "#{landmark_map[landmark]}"
        elsif landmark == :swamp
          str.concat "!"
        else
          str.concat "#{landmark_map[landmark]}"
        end
      elsif visible == :you
        if game.world.dead?
          str.concat "X"
        else
          @player_col = i
          if game.world.x == game.world.defector_x && game.world.y == game.world.defector_y
            str.concat "!"
          else
            str.concat "@"
          end
        end
      else
        str.concat " "
      end
    end
    str
  end

  def update_aesthetics_labels
    square = 40
    aesthetics_text_color = prefab.really_light_gray

    start_y = -1

    max_visible_rows.times
                    .map { |i| world_aesthetics_for start_y + i }
                    .each_with_index.map do |l, i|
      if l && !@world_aesthetics_label_cache[i]
        @world_aesthetics_label_cache[i] =
          { x: 0,
            y: 720 - world_line_height * i,
            text: l,
            size_px: world_label_size_px,
            font: "fonts/sprite.ttf",
            anchor_x: 0, **aesthetics_text_color }
      end
    end
  end

  def aesthetic_map
    {
      :forest => "+",
      :field => "|",
      :barrens => ".",
      :road => "#"
    }
  end

  def landmark_map
    {
      morality_house: "H",
      cowering_family: "H",
      mother_and_baby: "H",
      mansion: "H",
      man_and_boy: "H",
      a_small_child: "H",
      zero_zero: "H",
      lozenge_lounge: "H",
      suicide_house: "H",
      firelit_room: "H",
      outpost: "P",
      iron_mine: "I",
      merchant: "M",
      master: "D",
      thieves: "E",
      tannery: "T",
      armoury: "R",
      house: "H",
      ritual: "H",
      close_house: "H",
      close_cave: "C",
      cave: "C",
      town: "O",
      city: "Y",
      ship: "W",
      builder: "&",
      borehole: "B",
      battlefield: "F",
      swamp: "S",
      bottomless_box: "S",
      defector: "&"
    }
  end

  def world_aesthetics_for row
    str = ""
    return nil if @world_aesthetics_label_cache[row]
    start_x = -1
    max_visible_columns.times do |i|
      aesthetic = game.world.aesthetics[[start_x + i, row]]
      has_landmark = game.world.landmark_at?(start_x + i, row)
      visible = game.world[[start_x + i, row]] == :visible
      cleared = game.world.cleared[[start_x + i, row]]
      if visible && !world.is_defector_dead && world.defector_x == start_x + i && world.defector_y == row
        str.concat " "
      elsif aesthetic && !has_landmark && visible
        str.concat "#{aesthetic_map[aesthetic]}"
      elsif has_landmark && cleared && game.world[[start_x + i, row]] != :you
        str.concat "#{landmark_map[game.world.landmarks[[start_x + i, row]]]}"
      else
        str.concat " "
      end
    end
    str
  end

  def max_visible_columns
    92
  end

  def max_visible_rows
    42
  end

  def invalidate_label_cache
    @world_aesthetics_label_cache ||= []
    @world_aesthetics_label_cache.clear
    @world_landmark_labels_cache ||= []
    @world_landmark_labels_cache.clear
  end

  def world_line_height
    # 27.2
    17.2
  end

  def world_label_size_px
    # 30
    20.85
  end

  def defaults
  end

  def tick
    outputs[:world_map].w = 1280
    outputs[:world_map].h = 720
    outputs[:world_map].transient!
    outputs[:world_map].background_color = [245, 245, 245, 0]
    tick_flash_player_position
    tick_flash_defector_position
    defaults
    calc
    render
  end

  def flash_player_position
    world_x = @game.world.x
    world_y = @game.world.y
    x = (world_x * 13.9) + 13.0
    y = 720 - (world_y * world_line_height) - 16 - 16 - 2
    prefab.queue_flash GTK.create_uuid, x, y, 16, 16, 0, 128, 0, 1/60.0, rt_name: :world_map, a: 232
  end

  def tick_flash_player_position
    return if Grid.portrait?
    if Kernel.tick_count.zmod? 200
      flash_player_position
    end
  end

  def tick_flash_defector_position
    return if Grid.portrait?
    return if !@game.world.defector_x
    return if @game.world.is_defector_dead
    return if @game.world[[@game.world.defector_x, @game.world.defector_y]] != :visible
    if Kernel.tick_count.zmod? 200
      world_x = @game.world.defector_x
      world_y = @game.world.defector_y
      x = (world_x * 13.9) + 13.0
      y = 720 - (world_y * world_line_height) - 16 - 16 - 2
      prefab.queue_flash GTK.create_uuid, x, y, 16, 16, 128, 0, 0, 1/60.0, rt_name: :world_map, a: 232
    end
  end

  def render
    render_ascii_flashes
    render_status_labels
    render_map
    render_navigation_buttons
  end

  def calc
    @move_debounce ||= 0
    @move_debounce -= 1
    @move_debounce = 0 if @move_debounce < 0
    calc_buttons
    calc_keyboard_inputs
    calc_controller_inputs
    calc_camera_pan
    calc_camera
    if world.has_event? && !world.dead?
      clear_notifications
      if world.event.is_a? WorldEvent
        state.next_scene = :world_event_scene
      elsif world.event.is_a? MiniWorld
        state.next_scene = :mini_world_scene
      end
    end

    if inputs.mouse.click && inputs.mouse.y > Grid.allscreen_h * 0.66
      prefab.dismiss_current_notification
    end

    if inputs.keyboard.key_down.ac_back
      @game.save
      GTK.request_quit
    end
  end

  def calc_keyboard_inputs
    return if inputs.last_active != :keyboard
    return if inputs.last_active_at.elapsed_time <= 1

    if inputs.keyboard.key_down.enter || inputs.keyboard.key_down.space
      if prefab.notifications.length > 0
        prefab.dismiss_all_notifications
      else
        state.next_scene = :status_scene
      end
    end

    if inputs.keyboard.key_down.left
      move_left_button_selected
    elsif inputs.keyboard.key_down.right
      move_right_button_selected
    elsif inputs.keyboard.key_down.up
      move_up_button_selected
    elsif inputs.keyboard.key_down.down
      move_down_button_selected
    end

    if inputs.keyboard.key_down.escape || inputs.keyboard.key_down.p
      settings_button_selected
    end
  end

  def calc_controller_inputs
    return if inputs.last_active != :controller
    return if inputs.last_active_at.elapsed_time <= 1

    if inputs.controller_one.key_down.left
      move_left_button_selected
    elsif inputs.controller_one.key_down.right
      move_right_button_selected
    elsif inputs.controller_one.key_down.up
      move_up_button_selected
    elsif inputs.controller_one.key_down.down
      move_down_button_selected
    end

    if inputs.controller_one.key_down.a
      if prefab.notifications.length > 0
        prefab.dismiss_all_notifications
      else
        items_button_selected
      end
    end

    if inputs.controller_one.key_down.start
      settings_button_selected
    end
  end

  def calc_camera_pan
    return if Grid.orientation == :landscape
    if inputs.mouse.down && inputs.mouse.intersect_rect?(world_border_rect)
      @pan_camera_start_x = inputs.mouse.x
      @pan_camera_start_y = inputs.mouse.y
    elsif inputs.mouse.down
      @pan_camera_start_x = nil
      @pan_camera_start_y = nil
    end

    if inputs.mouse.up
      @pan_camera_start_x = nil
      @pan_camera_start_y = nil
    end

    return if !@pan_camera_start_x

    if (inputs.mouse.held)
      @focal_point_x -= (inputs.mouse.x - @pan_camera_start_x) * 0.07
      @focal_point_y += (inputs.mouse.y - @pan_camera_start_y) * 0.07
      @focal_point_y = 17 if @focal_point_y < 17
      @focal_point_x = 15 if @focal_point_x < 15
      @focal_point_x = 75 if @focal_point_x > 75
      @focal_point_y = 23 if @focal_point_y > 23
      @pan_camera_start_x = inputs.mouse.x
      @pan_camera_start_y = inputs.mouse.y
    end
  end

  def calc_camera
    @camera_target_x = focal_point_in_world_space.x - 640
    @camera_target_y = focal_point_in_world_space.y - 360
    @camera_offset_x ||= @camera_target_x
    @camera_offset_y ||= @camera_target_y
    @camera_offset_x = @camera_offset_x.lerp(@camera_target_x, 0.1).round(2)
    @camera_offset_y = @camera_offset_y.lerp(@camera_target_y, 0.1).round(2)
  end

  def calc_notifications
    @last_move_count ||= @game.world.move_count

    if @last_move_count > @game.world.move_count
      @last_story_line = nil
      @last_thought = nil
      @last_punch_level = @game.world.punch_level
      @last_gear = @game.world.carry[:name]
      @last_water = @game.world.water
      @water_zero_notification = false
      @food_zero_notification = false
      @defector_near_notification = false
      @defector_horizon_notification = false
    end
    @last_move_count = @game.world.move_count

    begin
      calc_storyline_notifications  ||
      calc_thoughts_notification    ||
      calc_gear_notification        ||
      calc_punch_level_notification ||
      calc_food_zero_notification   ||
      calc_water_zero_notification  ||
      calc_defector_notifications   ||
      calc_water_replenished_notification
    end
  end

  def calc_gear_notification
    occurred = false
    if @last_gear != current_gear && current_gear[0] != :bag && state.previous_scene != :embark_scene
      queue_notification :equipment, "gear upgraded. #{current_gear[0].gsub("_", " ")}, #{current_gear[1].gsub("_", " ")}, #{current_gear[2].gsub("_", " ")}."
      occurred = true
    end
    @last_gear = current_gear
    occurred
  end

  def current_gear
    [@game.world.carry[:name],
     @game.world.container[:name],
     @game.world.armour]
  end

  def calc_storyline_notifications
    return false if @game.world.event
    return false if @game.world.is_dream_sequence

    occurred = false

    if @game.story_line.current_message &&
       @last_story_line != @game.story_line.current_message &&
       @game.story_line.current_message.strip.length > 0

      if !prefab.notifications[:story_line]
        prefab.dismiss_all_notifications
        queue_notification :story_line, @game.story_line.current_message
        @last_story_line = @game.story_line.current_message
        @move_debounce = 60
        @game.story_line.mark_message_as_seen
        occurred = true
      end
    end

    occurred
  end

  def calc_thoughts_notification
    occurred = false
    if game.world.thoughts_message &&
       @last_thought != game.world.thoughts_message &&
       game.world.thoughts_message.strip.length > 0
      queue_notification :thought, game.world.thoughts_message
      occurred = true
    end

    @last_thought = game.world.thoughts_message
    occurred
  end

  def calc_defector_notifications
    occurred = false
    if !@defector_near_notification && game.world.defector_distance <= 2
      @defector_near_notification = true
      @defector_horizon_notification = true
      queue_notification :defector_near, "a defector closes in."
      occurred = true
    elsif @game.world.defector_visible
      queue_notification :defector_horizon, "a defector. visible in the distance."
      @game.world.defector_visible = false
      @defector_horizon_notification = true
      @focal_point_x = @game.world.defector_x
      @focal_point_y = @game.world.defector_y
      occurred = true
    end

    occurred
  end

  def calc_punch_level_notification
    occurred = false
    @last_punch_level ||= @game.world.punch_level

    if @game.world.punch_level != @last_punch_level
      queue_notification :food, "battle brings strength. fist damage increased."
      occurred = true
    end

    @last_punch_level = @game.world.punch_level
    occurred
  end

  def calc_water_replenished_notification
    occurred = false
    @last_water ||= game.world.water

    if @last_water < game.world.water && game.world.move_count > 10
      queue_notification :water, "water replenished."
      occurred = true
    end

    @last_water = game.world.water
    occurred
  end

  def calc_water_zero_notification
    occurred = false
    if !@water_zero_notification && game.world.water <= 3
      queue_notification :water, "water's almost gone. death awaits."
      @water_zero_notification = true
      @move_debounce = 60
      occurred = true
    end
    occurred
  end

  def calc_food_zero_notification
    occurred = false
    if !@food_zero_notification && game.world.stores[:food] <= 1
      queue_notification :food, "food's almost gone. death awaits."
      @food_zero_notification = true
      @move_debounce = 60
      occurred = true
    end
    occurred
  end

  def button_ids
    if inputs.last_active == :controller
      []
    elsif inputs.last_active == :keyboard
      []
    else
      [
        :move_up_button,
        :move_down_button,
        :move_left_button,
        :move_right_button,
        :items_button,
        :settings_button
      ]
    end
  end

  def items_button_selected
    state.next_scene = :status_scene
  end

  def settings_button_selected
    state.next_scene = :settings_scene
  end

  def items_button
    return nil if !show_navigation_buttons?
    if Grid.landscape?
      bt_row = 8
      bt_col = 22
      Layout.rect(row: bt_row, col: bt_col, w: 2, h: 2)
            .merge(text: "items",
                   **@navigation_color,
                   size_px: 20,
                   a11y_id: "world_items_button",
                   a11y_text: "items",
                   a11y_trait: :button)
    else
      if GTK.a11y_enabled?
        bt_row = 23
        bt_col = 5

        Layout.rect(row: bt_row, col: bt_col, w: 3.75, h: 1)
              .merge(text: "items",
                     a11y_id: "world_items_button",
                     a11y_text: "items",
                     a11y_trait: :button)
      else
        bt_row = 7
        bt_col = 22
        bt_row += 12
        bt_col -= 12

        Layout.rect(row: bt_row, col: bt_col, w: 2.5, h: 2.5)
              .merge(text: "items",
                     a11y_id: "world_items_button",
                     a11y_text: "items",
                     a11y_trait: :button)
      end
    end
  end

  def move_left_button_selected
    return if @move_debounce > 0
    return if @world_disable_controls
    @world_player_location_debounce = 100
    game.world.move_left
    @focal_point_x = @game.world.x
    @focal_point_y = @game.world.y
    update_label_cache
    determine_world_event
  end

  def move_right_button_selected
    return if @move_debounce > 0
    return if @world_disable_controls
    @world_player_location_debounce = 100
    game.world.move_right
    @focal_point_x = @game.world.x
    @focal_point_y = @game.world.y
    update_label_cache
    determine_world_event
  end

  def move_down_button_selected
    return if @move_debounce > 0
    return if @world_disable_controls
    @world_player_location_debounce = 100
    game.world.move_down
    @focal_point_x = @game.world.x
    @focal_point_y = @game.world.y
    update_label_cache
    determine_world_event
  end

  def move_up_button_selected
    return if @move_debounce > 0
    return if @world_disable_controls
    @world_player_location_debounce = 100
    game.world.move_up
    @focal_point_x = @game.world.x
    @focal_point_y = @game.world.y
    update_label_cache
    determine_world_event
  end

  def transition_to_game_over
    queue_flash :world_dead, Grid.allscreen_x, Grid.allscreen_y, Grid.allscreen_w, Grid.allscreen_h, 255, 0, 0, 1.0 / 10.0
    clear_notifications
    state.next_scene = :game_over_scene
    queue_sound "sounds/death.ogg"
  end

  def determine_world_event
    if Grid.landscape?
      @navigation_color = { r: 0, g: 0, b: 0, a: 128 }
    end

    if GTK.a11y_enabled?
      new_landmarks = world.revealed_locs_last_move.find_all do |loc|
        world.landmarks[loc]
      end.map do |loc|
        { x: loc[0], y: loc[1] }
      end

      if new_landmarks.length > 0

        args.outputs.a11y[:new_landmarks] = {
          a11y_trait: :notification,
          a11y_text: "new landmarks discovered: #{(world.describe_landmarks new_landmarks).join(", ")}"
        }
      end
    end

    if world.dead?
      transition_to_game_over
      return
    elsif world.has_event?
      if world.event.is_a? WorldEvent
        if world.defector_event?
          # reset_defector_notifications
        end
      elsif world.event.is_a? MiniWorld
      end
      return
    else
      calc_notifications
      if @game.world.defector_visible
        @game.world.defector_visible = false
      end
    end
  end

  def render_status_labels
    outputs.primitives << world_water_label
    outputs.primitives << world_food_label
    outputs.primitives << world_health_label
    outputs.primitives << world_compass_label
  end

  def world_water_label
    return nil if @game.world.is_dream_sequence
    if Grid.orientation == :landscape
      label_rect = if has_compass?
                     Layout.rect(row: 0, col: [0, 5], h: 0)
                   else
                     Layout.rect(row: 0, col: [5, 6], h: 0)
                   end
      { x: label_rect.center.x,
        y: label_rect.center.y,
        font: "fonts/text.ttf",
        anchor_y: 0,
        anchor_x: 0.5,
        size_px: 26,
        text: "water: #{game.world.water.round}" }
    else
      label_rect = Layout.rect(row: 18, col: [0, 5], h: 1)
      { x: label_rect.x,
        y: label_rect.center.y,
        font: "fonts/text.ttf",
        anchor_y: 0.5,
        anchor_x: 0,
        size_px: 30,
        text: "water: #{game.world.water.round}" }
    end
  end

  def world_food_label
    return nil if @game.world.is_dream_sequence
    if Grid.orientation == :landscape
      label_rect = if has_compass?
                     Layout.rect(row: 0, col: [6, 11], h: 0)
                   else
                     Layout.rect(row: 0, col: [11, 12], h: 0)
                   end
      { x: label_rect.center.x,
        y: label_rect.center.y,
        font: "fonts/text.ttf",
        anchor_y: 0,
        anchor_x: 0.5,
        size_px: 26,
        text: "food: #{game.world.stores[:food]}" }
    else
      label_rect = Layout.rect(row: 18, col: [0, 5], h: 1)
      { x: label_rect.x,
        y: label_rect.center.y,
        font: "fonts/text.ttf",
        anchor_y: 1.5,
        anchor_x: 0,
        size_px: 30,
        text: "food: #{game.world.stores[:food]}" }
    end
  end

  def render_ascii_flashes
    @world_player_location_debounce ||= 60
    if @world_player_location_debounce > 0
      @world_player_location_debounce -= 1
    else
      @world_player_location_debounce = 200
    end
  end

  def queue_swamp_flash
    return if game.world.last_admiral_cutscene_visited == game.world.current_admiral_cutscene
    return if !@swamp_col
    return if !@swamp_row
    flash_x = Layout.rect(row: 0, col: 0, h: 0).x + 4 + (@swamp_col) * 20.0
    flash_y = Layout.rect(row: 1, col: 0, h: 0).y - ((@swamp_row + 1) * world_line_height) - world_line_height + 4
    tc = prefab.black
    queue_flash :swamp_location, flash_x, flash_y, 26, 26, 0, 80, 0, 1/10.0
  end

  def world_health_label
    return nil if @game.world.is_dream_sequence
    if Grid.orientation == :landscape
      label_rect = if has_compass?
                     Layout.rect(row: 0, col: [12, 17], h: 0)
                   else
                     Layout.rect(row: 0, col: [17, 18], h: 0)
                   end
      { x: label_rect.center.x,
        y: label_rect.center.y,
        font: "fonts/text.ttf",
        anchor_y: 0,
        anchor_x: 0.5,
        size_px: 30,
        text: "hp: #{game.world.current_hp}/#{game.world.max_hp.to_i}" }
    else
      label_rect = Layout.rect(row: 18, col: [0, 5], h: 1)
      { x: label_rect.x,
        y: label_rect.center.y,
        font: "fonts/text.ttf",
        anchor_y: 2.5,
        anchor_x: 0,
        size_px: 30,
        text: "hp: #{game.world.current_hp}/#{game.world.max_hp.to_i}" }
    end
  end

  def has_compass?
    game.stores[:compass] == 1
  end

  def world_compass_label
    return nil if game.stores[:compass] != 1

    unicode_arrow = case game.world.compass_direction
                    when "NE"
                      "↗"
                    when "NW"
                      "↖"
                    when "SE"
                      "↘"
                    when "SW"
                      "↙"
                    when "N"
                      "↑"
                    when "S"
                      "↓"
                    when "E"
                      "→"
                    when "W"
                      "←"
                    else
                      "spins"
                    end

    if Grid.orientation == :landscape
      label_rect = Layout.rect(row: 0, col: [18, 23], h: 0)
      { x: label_rect.center.x,
        y: label_rect.y,
        font: "fonts/text.ttf",
        anchor_y: 0.0,
        anchor_x: 0.5,
        size_px: 30,
        text: "compass: #{unicode_arrow} (#{game.world.compass_direction})" }
    else
      label_rect = Layout.rect(row: 18, col: [0, 5], h: 1)

      { x: label_rect.x,
        y: label_rect.center.y,
        font: "fonts/text.ttf",
        anchor_y: 3.5,
        anchor_x: 0,
        size_px: 30,
        text: "compass: #{unicode_arrow} (#{game.world.compass_direction})" }
    end
  end

  def render_map
    outputs[:world_map].primitives << world_landmarks_label
    outputs[:world_map].primitives << world_aesthetics_label
    if Grid.portrait?
      outputs[:world_map].primitives << prefab.thick_border(args, { x: 10, y: 13, w: 1260, h: 694, primitive_marker: :border })
    end
    dst_x = world_border_rect.center.x
    dst_y = world_border_rect.center.y
    dst_w = world_border_rect.w
    dst_h = world_border_rect.h
    source_x = nil
    source_y = nil
    source_w = nil
    source_h = nil
    if Grid.portrait?
      source_x = (1280 - world_border_rect.w / camera_zoom) / 2 + @camera_offset_x
      source_y = (720 - world_border_rect.h / camera_zoom) / 2 - @camera_offset_y
      source_w = world_border_rect.w / camera_zoom
      source_h = world_border_rect.h / camera_zoom
      if (source_y + source_h) > 720
        source_y -= (source_y + source_h) - 720
        source_h = 720 - source_y
      elsif (source_y + source_h) < 0
        source_y = 0
        source_h = world_border_rect.h / camera_zoom
      end
    end
    outputs.primitives << { x: dst_x,
                            y: dst_y,
                            w: dst_w,
                            h: dst_h,
                            source_x: source_x,
                            source_y: source_y,
                            source_w: source_w,
                            source_h: source_h,
                            anchor_x: 0.5,
                            anchor_y: 0.5,
                            path: :world_map }
    outputs.primitives << world_border

    # outputs.primitives << { x: Grid.allscreen_x,
    #                         y: Grid.allscreen_y,
    #                         w: Grid.allscreen_w,
    #                         h: Grid.allscreen_h,
    #                         r: 255,
    #                         g: 0,
    #                         b: 0,
    #                         path: :solid }

    # outputs.primitives << { x: 0,
    #                         y: args.grid.center_y,
    #                         text: "allscreen_px #{Grid.allscreen_x_px}, #{Grid.allscreen_y_px}, #{Grid.allscreen_w_px}, #{Grid.allscreen_h_px}",
    #                         anchor_x: 0,
    #                         anchor_y: -0.5 }

    # outputs.primitives << { x: 0,
    #                         y: args.grid.center_y,
    #                         text: "allscreen    #{args.grid.allscreen_x}, #{Grid.allscreen_y}, #{Grid.allscreen_w}, #{Grid.allscreen_h}",
    #                         anchor_x: 0,
    #                         anchor_y: 0.5 }

    # outputs.primitives << { x: 0,
    #                         y: Grid.center_y,
    #                         text: "logical      #{Grid.x}, #{Grid.y}, #{Grid.w}, #{Grid.h}",
    #                         anchor_x: 0,
    #                         anchor_y: 1.5 }

    # outputs.primitives << { x: 0,
    #                         y: Grid.center_y,
    #                         text: "scale        #{Grid.texture_scale} #{Grid.texture_scale_enum}",
    #                         anchor_x: 0,
    #                         anchor_y: 2.5 }
    render_a11y_world_map
  end

  def render_a11y_world_map
    return if !GTK.a11y_enabled?
    entries = world_a11y_describe_world_entries
    outputs.a11y[:describe_food_water] = {
      x: Grid.w / 2,
      y: Grid.h / 2,
      anchor_x: 0.5,
      anchor_y: 0.5,
      a11y_id: "world_a11y_describe_food_water",
      a11y_text: "#{entries[:food_water].join(",\n")}, \n swipe left for defector location, swipe right for navigation controls",
      a11y_trait: :label
    }

    outputs.a11y[:describe_defector] = {
      x: Grid.w / 2,
      y: Grid.h / 2,
      anchor_x: 0.5,
      anchor_y: -0.5,
      a11y_id: "world_a11y_describe_defector",
      a11y_text: "#{entries[:defector].join(",\n")}, \n swipe left for landmarks, swipe right for status information",
      a11y_trait: :label
    }

    outputs.a11y[:describe_landmarks] = {
      x: Grid.w / 2,
      y: Grid.h / 2,
      anchor_x: 0.5,
      anchor_y: -1.5,
      a11y_id: "world_a11y_describe_landmarks",
      a11y_text: "#{entries[:landmarks].join(",\n")}, \n swipe left for unexplored locations, swipe right for defector location",
      a11y_trait: :label
    }

    outputs.a11y[:describe_unexplored] = {
      x: Grid.w / 2,
      y: Grid.h / 2,
      anchor_x: 0.5,
      anchor_y: -2.5,
      a11y_id: "world_a11y_describe_unexplored",
      a11y_text: "#{entries[:unexplored].join(",\n")}, \n swipe left for terrain information, swipe right for landmarks",
      a11y_trait: :label
    }

    outputs.a11y[:describe_terrain] = {
      x: Grid.w / 2,
      y: Grid.h / 2,
      anchor_x: 0.5,
      anchor_y: -3.5,
      a11y_id: "world_a11y_describe_terrain",
      a11y_text: "#{entries[:terrain].join(",\n")}",
      a11y_trait: :label
    }
  end

  def meat_low? game

  end

  def world_a11y_describe_world_entries invalidate_cache: false
    if invalidate_cache
      @world_a11y_describe_entries = nil
    end

    if @world_a11y_describe_entries
      return {
        all: @world_a11y_describe_entries,
        food_water: @world_a11y_describe_food_water,
        compass: @world_a11y_describe_compass,
        defector: @world_a11y_describe_defector,
        landmarks: @world_a11y_describe_landmarks,
        unexplored: @world_a11y_describe_unexplored,
        terrain: @world_a11y_describe_terrain
      }
    end

    @world_a11y_describe_entries = []

    compass_direction = {
      "N" => "points north",
      "W" => "points west",
      "E" => "points east",
      "S" => "points south",
      "NW" => "points northwest",
      "NE" => "points northeast",
      "SW" => "points southwest",
      "SE" => "points southeast",
      "spins" => "spins",
    }

    compass_a11y_descriptione = if game.world.is_dream_sequence
                                  "compass: spins"
                                elsif has_compass?
                                  "compass: #{compass_direction[game.world.compass_direction]}"
                                else
                                  "compass: left behind at murky swamp."
                                end


    @world_a11y_describe_food_water = [
      "water: #{game.world.water} units",
      "food: #{game.world.stores[:food]} units",
      "hp: #{game.world.current_hp} of #{game.world.max_hp.to_i}",
      compass_a11y_descriptione
    ]

    @world_a11y_describe_entries.concat @world_a11y_describe_food_water

    @world_a11y_describe_defector = [game.world.describe_defector]
    @world_a11y_describe_entries.concat @world_a11y_describe_defector

    @world_a11y_describe_landmarks = []
    @world_a11y_describe_landmarks << "landmarks:"
    if game.world.visible_landmarks.length > 0
      game.world.describe_landmarks.each { |s| @world_a11y_describe_landmarks << s if s }
    else
      @world_a11y_describe_landmarks << "no landmarks visible"
    end
    @world_a11y_describe_entries.concat @world_a11y_describe_landmarks

    @world_a11y_describe_unexplored = []
    @world_a11y_describe_unexplored << "unexplored:"
    unexplored = game.world.describe_unexplored
    unexplored.each { |s| @world_a11y_describe_unexplored << s if s }
    @world_a11y_describe_entries.concat @world_a11y_describe_unexplored

    @world_a11y_describe_terrain = []
    @world_a11y_describe_terrain << "terrain:"
    game.world.describe_terrain.each { |s| @world_a11y_describe_terrain << s if s }
    @world_a11y_describe_entries.concat @world_a11y_describe_terrain

    return {
      all: @world_a11y_describe_entries,
      food_water: @world_a11y_describe_food_water,
      defector: @world_a11y_describe_defector,
      landmarks: @world_a11y_describe_landmarks,
      unexplored: @world_a11y_describe_unexplored,
      terrain: @world_a11y_describe_terrain
    }
  end

  def world_border_rect
    if Grid.orientation == :landscape
      Layout.rect(row: 0, col: 0, w: 24, h: 12, include_row_gutter: true, include_col_gutter: true)
    else
      Layout.rect(row: 1, col: 0, w: 12, h: 17, include_row_gutter: true, include_col_gutter: true)
    end
  end

  def world_border
    world_border_rect.merge(primitive_marker: :border)
  end

  def world_landmarks_label
    @world_landmark_labels_cache
  end

  def world_aesthetics_label
    # if @world_current_fire_state_for_world != game.room.fire
    #   @world_current_fire_state_for_world = game.room.fire
    #   update_cache game, :force
    # end
    @world_aesthetics_label_cache
  end

  def render_navigation_buttons
    outputs.primitives << prefab.button(args, move_up_button)
    outputs.primitives << prefab.button(args, move_down_button)
    outputs.primitives << prefab.button(args, move_left_button)
    outputs.primitives << prefab.button(args, move_right_button)
    outputs.primitives << prefab.button(args, items_button)
    outputs.primitives << prefab.button(args, settings_button)
  end

  def show_navigation_buttons?
    return true if GTK.a11y_enabled?
    return true if GTK.platform?(:mobile)
    return true if inputs.last_active == :mouse
    return false
  end

  def keyboard_instructions
    "[arrow keys/WASD]: move around. [SPACE/ENTER]: open status screen. [ESC]: settings."
  end

  def controller_instructions accept_char
    "[DIRECTIONAL]: move around. [#{accept_char}]: open status screen. [START]: settings."
  end

  def move_up_button
    return nil if !show_navigation_buttons?
    if Grid.landscape?
      bt_row = 8
      bt_col = 20

      Layout.rect(row: bt_row, col: bt_col, w: 2, h: 2)
            .merge(text: "↑",
                   **@navigation_color,
                   a11y_id: "world_move_up_button",
                   a11y_text: "move north, swipe left for world information, swipe right for additional navigation controls.",
                   a11y_trait: :button)
    else
      bt_row = 7
      bt_col = 19.5
      if GTK.a11y_enabled?
        bt_row += 11
      else
        bt_row += 12
      end

      bt_col -= 12

      Layout.rect(row: bt_row, col: bt_col, w: 2.5, h: 2.5)
            .merge(text: "↑",
                   a11y_id: "world_move_up_button",
                   a11y_text: "move north, swipe left for world information, swipe right for additional navigation controls.",
                   a11y_trait: :button)
    end
  end

  def move_down_button
    return nil if !show_navigation_buttons?
    if Grid.landscape?
      bt_row = 10
      bt_col = 20

      Layout.rect(row: bt_row, col: bt_col, w: 2, h: 2)
            .merge(text: "↓",
                   **@navigation_color,
                   a11y_id: "world_move_down_button",
                   a11y_text: "move south",
                   a11y_trait: :button)
    else
      bt_row = 9.5
      bt_col = 19.5
      if GTK.a11y_enabled?
        bt_row += 11
      else
        bt_row += 12
      end
      bt_col -= 12

      Layout.rect(row: bt_row, col: bt_col, w: 2.5, h: 2.5)
            .merge(text: "↓",
                   a11y_id: "world_move_down_button",
                   a11y_text: "move south",
                   a11y_trait: :button)
    end
  end

  def move_right_button
    return nil if !show_navigation_buttons?
    if Grid.landscape?
      bt_row = 10
      bt_col = 22

      Layout.rect(row: bt_row, col: bt_col, w: 2, h: 2)
            .merge(text: "→",
                   **@navigation_color,
                   a11y_id: "world_move_right_button",
                   a11y_text: "move east",
                   a11y_trait: :button)
    else
      bt_row = 9.5
      bt_col = 22
      if GTK.a11y_enabled?
        bt_row += 11
      else
        bt_row += 12
      end
      bt_col -= 12

      Layout.rect(row: bt_row, col: bt_col, w: 2.5, h: 2.5)
            .merge(text: "→",
                   a11y_id: "world_move_right_button",
                   a11y_text: "move east",
                   a11y_trait: :button)
    end
  end

  def move_left_button
    return nil if !show_navigation_buttons?
    if Grid.landscape?
      bt_row = 10
      bt_col = 18

      Layout.rect(row: bt_row, col: bt_col, w: 2, h: 2)
            .merge(text: "←",
                   **@navigation_color,
                   a11y_id: "world_move_left_button",
                   a11y_text: "move west",
                   a11y_trait: :button)
    else
      bt_row = 9.5
      bt_col = 17
      if GTK.a11y_enabled?
        bt_row += 11
      else
        bt_row += 12
      end
      bt_col -= 12

      Layout.rect(row: bt_row, col: bt_col, w: 2.5, h: 2.5)
            .merge(text: "←",
                   a11y_id: "world_move_left_button",
                   a11y_text: "move west",
                   a11y_trait: :button)
    end
  end

  def settings_button
    return nil if !show_navigation_buttons?
    if Grid.orientation == :landscape
      bt_rect = Layout.rect(row: 8, col: 18, h: 2, w: 2)
      bt_rect.merge(text: "settings",
                   **@navigation_color,
                    size_px: 20,
                    a11y_id: "room_settings_button",
                    a11y_text: "settings",
                    a11y_trait: :button)
    else
      if GTK.a11y_enabled?
        Layout.rect(row: 23, col: 8.75, w: 3.75, h: 1)
              .merge(text: "settings",
                     a11y_id: "room_settings_button",
                     a11y_text: "settings",
                     a11y_trait: :button)
      else
        bt_rect = Layout.rect(row: 23, col: 0, h: 1, w: 4)
        bt_rect.merge(text: "settings",
                      a11y_id: "room_settings_button",
                      a11y_text: "settings",
                      a11y_trait: :button)
      end
    end
  end

  def loc_in_world_space loc_x, loc_y
    effective_x = loc_x
    effective_y = loc_y
    effective_x = 15 if effective_x < 15
    effective_x = 75 if effective_x > 75
    effective_y = 17 if effective_y < 17
    effective_y = 23 if effective_y > 23

    { x: effective_x * (1280.fdiv(90)),
      y: effective_y * (720.fdiv(40)) }
  end

  def focal_point_in_world_space
    loc_in_world_space @focal_point_x, @focal_point_y
  end
end
